#include "static_data.h"
#include "z64save.h"
#include "context.h"
#include "dungeon.h"

std::array<Rando::Location, RC_MAX> Rando::StaticData::locationTable;
std::multimap<std::tuple<s16, s16, s32>, RandomizerCheck> Rando::StaticData::CheckFromActorMultimap;

std::vector<RandomizerCheck> Rando::StaticData::dungeonRewardLocations = {
    // Bosses
    RC_QUEEN_GOHMA, RC_KING_DODONGO, RC_BARINADE,    RC_PHANTOM_GANON, RC_VOLVAGIA,
    RC_MORPHA,      RC_TWINROVA,     RC_BONGO_BONGO, RC_LINKS_POCKET,
};

using namespace Rando;

std::vector<RandomizerCheck> Rando::StaticData::GetPondFishLocations() {
    std::vector<RandomizerCheck> pondFishLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_FISH && location.GetScene() == SCENE_FISHING_POND && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            pondFishLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return pondFishLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetOverworldFishLocations() {
    std::vector<RandomizerCheck> overworldFishLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_FISH && location.GetScene() != SCENE_FISHING_POND && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            overworldFishLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return overworldFishLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetOverworldPotLocations() {
    std::vector<RandomizerCheck> overworldPotLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_POT && location.IsOverworld() && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            overworldPotLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return overworldPotLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetStaticHintLocations() {
    std::vector<RandomizerCheck> staticHintLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_STATIC_HINT && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            staticHintLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return staticHintLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetScrubLocations() {
    std::vector<RandomizerCheck> scrubLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_SCRUB && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            scrubLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return scrubLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetMerchantLocations() {
    std::vector<RandomizerCheck> scrubLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_MERCHANT && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            scrubLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return scrubLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetAdultTradeLocations() {
    std::vector<RandomizerCheck> tradeLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_ADULT_TRADE && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            tradeLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return tradeLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetGossipStoneLocations() {
    std::vector<RandomizerCheck> gossipStoneLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_GOSSIP_STONE && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            gossipStoneLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return gossipStoneLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetShopLocations() {
    std::vector<RandomizerCheck> shopLocations = {};
    for (Location& location : locationTable) {
        if (location.GetRCType() == RCTYPE_SHOP && location.GetRandomizerCheck() != RC_UNKNOWN_CHECK) {
            shopLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return shopLocations;
}


std::vector<RandomizerCheck> Rando::StaticData::GetOverworldLocations() {
    //RANDOTODO better way of filling the initial location pool, among other things. 
    std::vector<RandomizerCheck> overworldLocations = {};
    auto ctx = Rando::Context::GetInstance();
    for (Location& location : locationTable) {
        if (
            location.IsOverworld() &&
            location.GetRandomizerCheck() != RC_UNKNOWN_CHECK &&
            location.GetRandomizerCheck() != RC_TRIFORCE_COMPLETED && //not really an overworld check
            location.GetRCType() != RCTYPE_FISH && // temp fix while locations are properly sorted out
            location.GetRCType() != RCTYPE_POT &&  // Same as fish
            location.GetRCType() != RCTYPE_CHEST_GAME && //this is supposed to be excluded
            (ctx->GetOption(RSK_SHUFFLE_ADULT_TRADE) || location.GetRCType() != RCTYPE_ADULT_TRADE) && //trade is handled elsewhere in location pool
            location.GetRCType() != RCTYPE_STATIC_HINT && 
            location.GetRCType() != RCTYPE_GOSSIP_STONE  //don't put items on hints
        ) {
            overworldLocations.push_back(location.GetRandomizerCheck());
        }
    }
    return overworldLocations;
}

std::vector<RandomizerCheck> Rando::StaticData::GetAllDungeonLocations() {
    auto ctx = Rando::Context::GetInstance();
    std::vector<RandomizerCheck> dungeonLocations;
    for (const auto dungeon : ctx->GetDungeons()->GetDungeonList()) {
        std::vector<RandomizerCheck> dungeonLoc = dungeon->GetDungeonLocations();
        dungeonLocations.insert(dungeonLocations.end(), dungeonLoc.begin(), dungeonLoc.end());
    }
    return dungeonLocations;
}

void Rando::StaticData::InitLocationTable() { //                                                      Randomizer Check                                                 Quest            Type                                Area                                 Actor ID              Scene ID                            Params                        Flags Short Name                                     Hint Text Key                                                    Vanilla Item                                                        Spoiler Collection Check                                                                                                      Vanilla Progression  Price
    // clang-format off
    locationTable[RC_UNKNOWN_CHECK] =                                                  Location::Base(RC_UNKNOWN_CHECK,                                                RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_INVALID,                      ACTOR_ID_MAX,         SCENE_ID_MAX,                       0x00,                               "Invalid Location", "Invalid Location",        RHT_NONE,                                                        RG_NONE);
    // Kokiri Forest
    locationTable[RC_KF_KOKIRI_SWORD_CHEST] =                                         Location::Chest(RC_KF_KOKIRI_SWORD_CHEST,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_KOKIRI_FOREST,                1248,                         0x00, "Kokiri Sword Chest",                          RHT_KF_KOKIRI_SWORD_CHEST,                                       RG_KOKIRI_SWORD,                                                                                                                                                                                  true);
    locationTable[RC_KF_MIDOS_TOP_LEFT_CHEST] =                                       Location::Chest(RC_KF_MIDOS_TOP_LEFT_CHEST,                                      RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_MIDOS_HOUSE,                  22944,                        0x00, "Mido Top Left Chest",                         RHT_KF_MIDOS_TOP_LEFT_CHEST,                                     RG_BLUE_RUPEE);
    locationTable[RC_KF_MIDOS_TOP_RIGHT_CHEST] =                                      Location::Chest(RC_KF_MIDOS_TOP_RIGHT_CHEST,                                     RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_MIDOS_HOUSE,                  22945,                        0x01, "Mido Top Right Chest",                        RHT_KF_MIDOS_TOP_RIGHT_CHEST,                                    RG_BLUE_RUPEE);
    locationTable[RC_KF_MIDOS_BOTTOM_LEFT_CHEST] =                                    Location::Chest(RC_KF_MIDOS_BOTTOM_LEFT_CHEST,                                   RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_MIDOS_HOUSE,                  22914,                        0x02, "Mido Bottom Left Chest",                      RHT_KF_MIDOS_BOTTOM_LEFT_CHEST,                                  RG_GREG_RUPEE);
    locationTable[RC_KF_MIDOS_BOTTOM_RIGHT_CHEST] =                                   Location::Chest(RC_KF_MIDOS_BOTTOM_RIGHT_CHEST,                                  RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_MIDOS_HOUSE,                  22787,                        0x03, "Mido Bottom Right Chest",                     RHT_KF_MIDOS_BOTTOM_RIGHT_CHEST,                                 RG_RECOVERY_HEART);
    locationTable[RC_KF_STORMS_GROTTO_CHEST] =                                        Location::Chest(RC_KF_STORMS_GROTTO_CHEST,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_KOKIRI_FOREST,                ACTOR_EN_BOX,         SCENE_GROTTOS,                      22988,                        0x0C, "Storms Grotto Chest",                         RHT_KF_STORMS_GROTTO_CHEST,                                      RG_RED_RUPEE);
    // Lost Woods
    locationTable[RC_LW_NEAR_SHORTCUTS_GROTTO_CHEST] =                                Location::Chest(RC_LW_NEAR_SHORTCUTS_GROTTO_CHEST,                               RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_LOST_WOODS,                   ACTOR_EN_BOX,         SCENE_GROTTOS,                      22964,                        0x14, "Near Shortcuts Grotto Chest",                 RHT_KF_STORMS_GROTTO_CHEST,                                      RG_RED_RUPEE);
    locationTable[RC_LW_SKULL_KID] =                                                   Location::Base(RC_LW_SKULL_KID,                                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LOST_WOODS,                   0x00,                               "Skull Kid",                                   RHT_LW_SKULL_KID,                                                RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::ItemGetInf(22),                                                                                       true);
    locationTable[RC_LW_TRADE_COJIRO] =                                                Location::Base(RC_LW_TRADE_COJIRO,                                              RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_LOST_WOODS,                   0x00,                               "Trade Cojiro",                                RHT_LW_TRADE_COJIRO,                                             RG_ODD_MUSHROOM,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_LW_TRADE_COJIRO),                                                 true);
    locationTable[RC_LW_TRADE_ODD_POTION] =                                            Location::Base(RC_LW_TRADE_ODD_POTION,                                          RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_LOST_WOODS,                   0x00,                               "Trade Odd Potion",                            RHT_LW_TRADE_COJIRO,                                             RG_POACHERS_SAW,                                                    SpoilerCollectionCheck::ItemGetInf(49),                                                                                       true);
    locationTable[RC_LW_OCARINA_MEMORY_GAME] =                                         Location::Base(RC_LW_OCARINA_MEMORY_GAME,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LOST_WOODS,                   0x00,                               "Ocarina Memory Game",                         RHT_LW_OCARINA_MEMORY_GAME,                                      RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::ItemGetInf(23),                                                                                       true);
    locationTable[RC_LW_TARGET_IN_WOODS] =                                             Location::Base(RC_LW_TARGET_IN_WOODS,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LOST_WOODS,                   0x00,                               "Target in Woods",                             RHT_LW_TARGET_IN_WOODS,                                          RG_PROGRESSIVE_SLINGSHOT,                                           SpoilerCollectionCheck::ItemGetInf(29),                                                                                       true);
    locationTable[RC_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_RIGHT] =                          Location::Base(RC_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_RIGHT,                        RCQUEST_BOTH,    RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_LOST_WOODS,                   0x00,                               "Deku Scrub Near Deku Theater Right",          RHT_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_RIGHT,                       RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_RIGHT),                       false, 20);
    locationTable[RC_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_LEFT] =                           Location::Base(RC_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_LEFT,                         RCQUEST_BOTH,    RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_LOST_WOODS,                   0x01,                               "Deku Scrub Near Deku Theater Left",           RHT_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_LEFT,                        RG_BUY_DEKU_STICK_1,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LW_DEKU_SCRUB_NEAR_DEKU_THEATER_LEFT),                        false, 15);
    locationTable[RC_LW_DEKU_SCRUB_NEAR_BRIDGE] =                                      Location::Base(RC_LW_DEKU_SCRUB_NEAR_BRIDGE,                                    RCQUEST_BOTH,    RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_LOST_WOODS,                   0x09,                               "Deku Scrub Near Bridge",                      RHT_LW_DEKU_SCRUB_NEAR_BRIDGE,                                   RG_PROGRESSIVE_STICK_UPGRADE,                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LW_DEKU_SCRUB_NEAR_BRIDGE),                                   true,  40);
    locationTable[RC_LW_DEKU_SCRUB_GROTTO_REAR] =                                      Location::Base(RC_LW_DEKU_SCRUB_GROTTO_REAR,                                    RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LOST_WOODS,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x03, 0xF5),       "Deku Scrub Grotto Rear",                      RHT_LW_DEKU_SCRUB_GROTTO_REAR,                                   RG_BUY_DEKU_SEEDS_30,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LW_DEKU_SCRUB_GROTTO_REAR),                                   false, 40);
    locationTable[RC_LW_DEKU_SCRUB_GROTTO_FRONT] =                                     Location::Base(RC_LW_DEKU_SCRUB_GROTTO_FRONT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LOST_WOODS,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x0A, 0xF5),       "Deku Scrub Grotto Front",                     RHT_LW_DEKU_SCRUB_GROTTO_FRONT,                                  RG_PROGRESSIVE_NUT_UPGRADE,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LW_DEKU_SCRUB_GROTTO_FRONT),                                  false, 40);
    locationTable[RC_DEKU_THEATER_SKULL_MASK] =                                        Location::Base(RC_DEKU_THEATER_SKULL_MASK,                                      RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_LOST_WOODS,                   ACTOR_ID_MAX,         SCENE_GROTTOS,                      0x00,                               "Deku Theater Skull Mask",                     RHT_DEKU_THEATER_SKULL_MASK,                                     RG_PROGRESSIVE_STICK_UPGRADE,                                       SpoilerCollectionCheck::ItemGetInf(ITEMGETINF_OBTAINED_STICK_UPGRADE_FROM_STAGE),                                             true);
    locationTable[RC_DEKU_THEATER_MASK_OF_TRUTH] =                                     Location::Base(RC_DEKU_THEATER_MASK_OF_TRUTH,                                   RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_LOST_WOODS,                   ACTOR_ID_MAX,         SCENE_GROTTOS,                      0x00,                               "Deku Theater Mask of Truth",                  RHT_DEKU_THEATER_MASK_OF_TRUTH,                                  RG_PROGRESSIVE_NUT_UPGRADE,                                         SpoilerCollectionCheck::ItemGetInf(ITEMGETINF_OBTAINED_NUT_UPGRADE_FROM_STAGE),                                               true);
    // Sacred Forest Meadow
    locationTable[RC_SFM_WOLFOS_GROTTO_CHEST] =                                       Location::Chest(RC_SFM_WOLFOS_GROTTO_CHEST,                                      RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_SACRED_FOREST_MEADOW,         ACTOR_EN_BOX,         SCENE_GROTTOS,                      31409,                        0x11, "Wolfos Grotto Chest",                         RHT_SFM_WOLFOS_GROTTO_CHEST,                                     RG_PURPLE_RUPEE);
    locationTable[RC_SFM_DEKU_SCRUB_GROTTO_REAR] =                                     Location::Base(RC_SFM_DEKU_SCRUB_GROTTO_REAR,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_SACRED_FOREST_MEADOW,         ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x07, 0xEE),       "Deku Scrub Grotto Rear",                      RHT_SFM_DEKU_SCRUB_GROTTO_REAR,                                  RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_SFM_DEKU_SCRUB_GROTTO_REAR),                                  false, 40);
    locationTable[RC_SFM_DEKU_SCRUB_GROTTO_FRONT] =                                    Location::Base(RC_SFM_DEKU_SCRUB_GROTTO_FRONT,                                  RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_SACRED_FOREST_MEADOW,         ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x08, 0xEE),       "Deku Scrub Grotto Front",                     RHT_SFM_DEKU_SCRUB_GROTTO_FRONT,                                 RG_BUY_GREEN_POTION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_SFM_DEKU_SCRUB_GROTTO_FRONT),                                 false, 40);
    // Hyrule Field
    locationTable[RC_HF_SOUTHEAST_GROTTO_CHEST] =                                     Location::Chest(RC_HF_SOUTHEAST_GROTTO_CHEST,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_HYRULE_FIELD,                 ACTOR_EN_BOX,         SCENE_GROTTOS,                      22978,                        0x02, "Southeast Grotto Chest",                      RHT_HF_SOUTHEAST_GROTTO_CHEST,                                   RG_RED_RUPEE);
    locationTable[RC_HF_OPEN_GROTTO_CHEST] =                                          Location::Chest(RC_HF_OPEN_GROTTO_CHEST,                                         RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_HYRULE_FIELD,                 ACTOR_EN_BOX,         SCENE_GROTTOS,                      22947,                        0x03, "Open Grotto Chest",                           RHT_HF_OPEN_GROTTO_CHEST,                                        RG_BLUE_RUPEE);
    locationTable[RC_HF_NEAR_MARKET_GROTTO_CHEST] =                                   Location::Chest(RC_HF_NEAR_MARKET_GROTTO_CHEST,                                  RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_HYRULE_FIELD,                 ACTOR_EN_BOX,         SCENE_GROTTOS,                      22944,                        0x00, "Near Market Grotto Chest",                    RHT_HF_NEAR_MARKET_GROTTO_CHEST,                                 RG_BLUE_RUPEE);
    locationTable[RC_HF_OCARINA_OF_TIME_ITEM] =                                        Location::Base(RC_HF_OCARINA_OF_TIME_ITEM,                                      RCQUEST_BOTH,    RCTYPE_OCARINA,                                                          ACTOR_ID_MAX,         SCENE_HYRULE_FIELD,                 0x00,                               "Ocarina of Time Item",                        RHT_HF_OCARINA_OF_TIME_ITEM,                                     RG_PROGRESSIVE_OCARINA,                                             SpoilerCollectionCheck::EventChkInf(0x43));
    locationTable[RC_HF_TEKTITE_GROTTO_FREESTANDING_POH] =                      Location::Collectable(RC_HF_TEKTITE_GROTTO_FREESTANDING_POH,                           RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_HYRULE_FIELD,                 ACTOR_EN_ITEM00,      SCENE_GROTTOS,                      262,                          0x01, "Tektite Grotto Freestanding PoH",             RHT_HF_TEKTITE_GROTTO_FREESTANDING_POH,                          RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_HF_DEKU_SCRUB_GROTTO] =                                           Location::Base(RC_HF_DEKU_SCRUB_GROTTO,                                         RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_HYRULE_FIELD,                 ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x02, 0xE6),       "Deku Scrub Grotto",                           RHT_HF_DEKU_SCRUB_GROTTO,                                        RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_HF_DEKU_SCRUB_GROTTO),                                        true,  10);
    // Lake Hylia
    locationTable[RC_LH_CHILD_FISHING] =                                               Location::Base(RC_LH_CHILD_FISHING,                                             RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_FISHING_POND,                 0x00,                               "Child Fishing",                               RHT_LH_CHILD_FISHING,                                            RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::RandomizerInf(RAND_INF_CHILD_FISHING),                                                                true);
    locationTable[RC_LH_ADULT_FISHING] =                                               Location::Base(RC_LH_ADULT_FISHING,                                             RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_FISHING_POND,                 0x00,                               "Adult Fishing",                               RHT_LH_ADULT_FISHING,                                            RG_PROGRESSIVE_SCALE,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_FISHING),                                                                true);
    locationTable[RC_LH_HYRULE_LOACH] =                                                Location::Base(RC_LH_HYRULE_LOACH,                                              RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_FISHING_POND,                 0x00,                               "Hyrule Loach Reward",                         RHT_LH_HYRULE_LOACH,                                             RG_PURPLE_RUPEE,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_CAUGHT_LOACH));
    locationTable[RC_LH_LAB_DIVE] =                                                    Location::Base(RC_LH_LAB_DIVE,                                                  RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LAKESIDE_LABORATORY,          0x00,                               "Lab Dive",                                    RHT_LH_LAB_DIVE,                                                 RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::ItemGetInf(16),                                                                                       true);
    locationTable[RC_LH_TRADE_FROG] =                                                  Location::Base(RC_LH_TRADE_FROG,                                                RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_LAKESIDE_LABORATORY,          0x00,                               "Lab Trade Eyeball Frog",                      RHT_LH_TRADE_FROG,                                               RG_EYEDROPS,                                                        SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_LH_TRADE_FROG),                                                   true);
    locationTable[RC_LH_UNDERWATER_ITEM] =                                             Location::Base(RC_LH_UNDERWATER_ITEM,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LAKE_HYLIA,                   0x00,                               "Underwater Item",                             RHT_LH_UNDERWATER_ITEM,                                          RG_RUTOS_LETTER,                                                    SpoilerCollectionCheck::EventChkInf(0x31),                                                                                    true);
    locationTable[RC_LH_SUN] =                                                         Location::Base(RC_LH_SUN,                                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LAKE_HYLIA,                   0x00,                               "Sun",                                         RHT_LH_SUN,                                                      RG_FIRE_ARROWS,                                                     SpoilerCollectionCheck::Chest(0x57, 0x1F),                                                                                    true);
    locationTable[RC_LH_FREESTANDING_POH] =                                     Location::Collectable(RC_LH_FREESTANDING_POH,                                          RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_LAKE_HYLIA,                   7686,                         0x1E, "Freestanding PoH",                            RHT_LH_FREESTANDING_POH,                                         RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_LH_DEKU_SCRUB_GROTTO_LEFT] =                                      Location::Base(RC_LH_DEKU_SCRUB_GROTTO_LEFT,                                    RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LAKE_HYLIA,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x00, 0xEF),       "Deku Scrub Grotto Left",                      RHT_LH_DEKU_SCRUB_GROTTO_LEFT,                                   RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LH_DEKU_SCRUB_GROTTO_LEFT),                                   false, 20);
    locationTable[RC_LH_DEKU_SCRUB_GROTTO_RIGHT] =                                     Location::Base(RC_LH_DEKU_SCRUB_GROTTO_RIGHT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LAKE_HYLIA,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x05, 0xEF),       "Deku Scrub Grotto Right",                     RHT_LH_DEKU_SCRUB_GROTTO_RIGHT,                                  RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LH_DEKU_SCRUB_GROTTO_RIGHT),                                  false, 40);
    locationTable[RC_LH_DEKU_SCRUB_GROTTO_CENTER] =                                    Location::Base(RC_LH_DEKU_SCRUB_GROTTO_CENTER,                                  RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LAKE_HYLIA,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x03, 0xEF),       "Deku Scrub Grotto Center",                    RHT_LH_DEKU_SCRUB_GROTTO_CENTER,                                 RG_BUY_DEKU_SEEDS_30,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LH_DEKU_SCRUB_GROTTO_CENTER),                                 false, 40);
    // Gerudo Valley
    locationTable[RC_GV_CHEST] =                                                      Location::Chest(RC_GV_CHEST,                                                     RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_VALLEY,                23200,                        0x00, "Chest",                                       RHT_GV_CHEST,                                                    RG_PURPLE_RUPEE);
    locationTable[RC_GV_TRADE_SAW] =                                                   Location::Base(RC_GV_TRADE_SAW,                                                 RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_GERUDO_VALLEY,                0x00,                               "Trade Saw",                                   RHT_GV_TRADE_SAW,                                                RG_BROKEN_SWORD,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_GV_TRADE_SAW),                                                    true);
    locationTable[RC_GV_WATERFALL_FREESTANDING_POH] =                           Location::Collectable(RC_GV_WATERFALL_FREESTANDING_POH,                                RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_GERUDO_VALLEY,                262,                          0x01, "Waterfall Freestanding PoH",                  RHT_GV_WATERFALL_FREESTANDING_POH,                               RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GV_CRATE_FREESTANDING_POH] =                               Location::Collectable(RC_GV_CRATE_FREESTANDING_POH,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_GERUDO_VALLEY,                518,                          0x02, "Crate Freestanding PoH",                      RHT_GV_CRATE_FREESTANDING_POH,                                   RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GV_DEKU_SCRUB_GROTTO_REAR] =                                      Location::Base(RC_GV_DEKU_SCRUB_GROTTO_REAR,                                    RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_GERUDO_VALLEY,                ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x07, 0xF0),       "Deku Scrub Grotto Rear",                      RHT_GV_DEKU_SCRUB_GROTTO_REAR,                                   RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GV_DEKU_SCRUB_GROTTO_REAR),                                   false, 40);
    locationTable[RC_GV_DEKU_SCRUB_GROTTO_FRONT] =                                     Location::Base(RC_GV_DEKU_SCRUB_GROTTO_FRONT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_GERUDO_VALLEY,                ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x08, 0xF0),       "Deku Scrub Grotto Front",                     RHT_GV_DEKU_SCRUB_GROTTO_FRONT,                                  RG_BUY_GREEN_POTION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GV_DEKU_SCRUB_GROTTO_FRONT),                                  false, 40);
    // Gerudo Fortress
    locationTable[RC_GF_CHEST] =                                                      Location::Chest(RC_GF_CHEST,                                                     RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDOS_FORTRESS,             1984,                         0x00, "Chest",                                       RHT_GF_CHEST,                                                    RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GF_HBA_1000_POINTS] =                                             Location::Base(RC_GF_HBA_1000_POINTS,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_GERUDOS_FORTRESS,             0x00,                               "GF HBA 1000 Points",                          RHT_GF_HBA_1000_POINTS,                                          RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::InfTable(INFTABLE_190),                                                                               true);
    locationTable[RC_GF_HBA_1500_POINTS] =                                             Location::Base(RC_GF_HBA_1500_POINTS,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_GERUDOS_FORTRESS,             0x00,                               "GF HBA 1500 Points",                          RHT_GF_HBA_1500_POINTS,                                          RG_PROGRESSIVE_BOW,                                                 SpoilerCollectionCheck::ItemGetInf(15),                                                                                       true);
    // RandoTodo: Do we replace these with the RC_HIDEOUT keys or keep these?
    locationTable[RC_GF_GERUDO_MEMBERSHIP_CARD] =                                      Location::Base(RC_GF_GERUDO_MEMBERSHIP_CARD,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_THIEVES_HIDEOUT,              0x00,                               "GF Gerudo Membership Card",                   RHT_GF_GERUDO_MEMBERSHIP_CARD,                                   RG_GERUDO_MEMBERSHIP_CARD,                                          SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_ITEM_FROM_LEADER_OF_FORTRESS),                                              true);
    locationTable[RC_GF_NORTH_F1_CARPENTER] =                                   Location::Collectable(RC_GF_NORTH_F1_CARPENTER,                                        RCQUEST_BOTH,    RCTYPE_GF_KEY,                                                           ACTOR_EN_ITEM00,      SCENE_THIEVES_HIDEOUT,              3089,                         0x0C, "GF North F1 Carpenter",                       RHT_GF_NORTH_F1_CARPENTER,                                       RG_GERUDO_FORTRESS_SMALL_KEY,                                                                                                                                                                     true);
    locationTable[RC_GF_NORTH_F2_CARPENTER] =                                   Location::Collectable(RC_GF_NORTH_F2_CARPENTER,                                        RCQUEST_BOTH,    RCTYPE_GF_KEY,                                                           ACTOR_EN_ITEM00,      SCENE_THIEVES_HIDEOUT,              2577,                         0x0A, "GF North F2 Carpenter",                       RHT_GF_NORTH_F2_CARPENTER,                                       RG_GERUDO_FORTRESS_SMALL_KEY,                                                                                                                                                                     true);
    locationTable[RC_GF_SOUTH_F1_CARPENTER] =                                   Location::Collectable(RC_GF_SOUTH_F1_CARPENTER,                                        RCQUEST_BOTH,    RCTYPE_GF_KEY,                                                           ACTOR_EN_ITEM00,      SCENE_THIEVES_HIDEOUT,              3601,                         0x0E, "GF South F1 Carpenter",                       RHT_GF_SOUTH_F1_CARPENTER,                                       RG_GERUDO_FORTRESS_SMALL_KEY,                                                                                                                                                                     true);
    locationTable[RC_GF_SOUTH_F2_CARPENTER] =                                   Location::Collectable(RC_GF_SOUTH_F2_CARPENTER,                                        RCQUEST_BOTH,    RCTYPE_GF_KEY,                                                           ACTOR_EN_ITEM00,      SCENE_THIEVES_HIDEOUT,              3857,                         0x0F, "GF South F2 Carpenter",                       RHT_GF_SOUTH_F2_CARPENTER,                                       RG_GERUDO_FORTRESS_SMALL_KEY,                                                                                                                                                                     true);
    // Haunted Wasteland
    locationTable[RC_WASTELAND_CHEST] =                                               Location::Chest(RC_WASTELAND_CHEST,                                              RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_WASTELAND,                    ACTOR_EN_BOX,         SCENE_HAUNTED_WASTELAND,            -30048,                       0x00, "Chest",                                       RHT_WASTELAND_CHEST,                                             RG_PURPLE_RUPEE);
    locationTable[RC_WASTELAND_BOMBCHU_SALESMAN] =                                     Location::Base(RC_WASTELAND_BOMBCHU_SALESMAN,                                   RCQUEST_BOTH,    RCTYPE_MERCHANT,                    RCAREA_WASTELAND,                    ACTOR_ID_MAX,         SCENE_HAUNTED_WASTELAND,            0x00,                               "Carpet Salesman",                             RHT_WASTELAND_BOMBCHU_SALESMAN,                                  RG_BUY_BOMBCHUS_10,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_MERCHANTS_CARPET_SALESMAN),                                                    false, 200);
    // Desert Colossus
    locationTable[RC_COLOSSUS_FREESTANDING_POH] =                               Location::Collectable(RC_COLOSSUS_FREESTANDING_POH,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_DESERT_COLOSSUS,              3334,                         0x0D, "Freestanding PoH",                            RHT_COLOSSUS_FREESTANDING_POH,                                   RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_COLOSSUS_DEKU_SCRUB_GROTTO_REAR] =                                Location::Base(RC_COLOSSUS_DEKU_SCRUB_GROTTO_REAR,                              RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_DESERT_COLOSSUS,              ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x07, 0xFD),       "Deku Scrub Grotto Rear",                      RHT_COLOSSUS_DEKU_SCRUB_GROTTO_REAR,                             RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_COLOSSUS_DEKU_SCRUB_GROTTO_REAR),                             false, 40);
    locationTable[RC_COLOSSUS_DEKU_SCRUB_GROTTO_FRONT] =                               Location::Base(RC_COLOSSUS_DEKU_SCRUB_GROTTO_FRONT,                             RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_DESERT_COLOSSUS,              ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x08, 0xFD),       "Deku Scrub Grotto Front",                     RHT_COLOSSUS_DEKU_SCRUB_GROTTO_FRONT,                            RG_BUY_GREEN_POTION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_COLOSSUS_DEKU_SCRUB_GROTTO_FRONT),                            false, 40);
    // Market
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_REWARD] =                             Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_REWARD,                            RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Treasure Chest Game Reward",                  RHT_MARKET_TREASURE_CHEST_GAME_REWARD,                           RG_TREASURE_GAME_HEART,                                             SpoilerCollectionCheck::ItemGetInf(27),                                                                                       true);
    locationTable[RC_MARKET_BOMBCHU_BOWLING_FIRST_PRIZE] =                             Location::Base(RC_MARKET_BOMBCHU_BOWLING_FIRST_PRIZE,                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_BOMBCHU_BOWLING_ALLEY,        0x00,                               "Bombchu Bowling First Prize",                 RHT_MARKET_BOMBCHU_BOWLING_FIRST_PRIZE,                          RG_PROGRESSIVE_BOMB_BAG,                                            SpoilerCollectionCheck::ItemGetInf(17),                                                                                       true);
    locationTable[RC_MARKET_BOMBCHU_BOWLING_SECOND_PRIZE] =                            Location::Base(RC_MARKET_BOMBCHU_BOWLING_SECOND_PRIZE,                          RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_BOMBCHU_BOWLING_ALLEY,        0x00,                               "Bombchu Bowling Second Prize",                RHT_MARKET_BOMBCHU_BOWLING_SECOND_PRIZE,                         RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::ItemGetInf(18),                                                                                       true);
    locationTable[RC_MARKET_LOST_DOG] =                                                Location::Base(RC_MARKET_LOST_DOG,                                              RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_DOG_LADY_HOUSE,               0x00,                               "Lost Dog",                                    RHT_MARKET_LOST_DOG,                                             RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::InfTable(INFTABLE_191),                                                                               true);
    locationTable[RC_MARKET_SHOOTING_GALLERY_REWARD] =                                 Location::Base(RC_MARKET_SHOOTING_GALLERY_REWARD,                               RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_MARKET,                       ACTOR_ID_MAX,         SCENE_SHOOTING_GALLERY,             0x00,                               "Shooting Gallery",                            RHT_MARKET_SHOOTING_GALLERY_REWARD,                              RG_PROGRESSIVE_SLINGSHOT,                                           SpoilerCollectionCheck::ItemGetInf(ITEMGETINF_0D),                                                                            true);
    locationTable[RC_MARKET_10_BIG_POES] =                                             Location::Base(RC_MARKET_10_BIG_POES,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_MARKET_GUARD_HOUSE,           0x00,                               "10 Big Poes",                                 RHT_MARKET_10_BIG_POES,                                          RG_EMPTY_BOTTLE,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_10_BIG_POES),                                                                  true);
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_ITEM_1] =                             Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_ITEM_1,                            RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game First Room Item",                  RHT_MARKET_TREASURE_CHEST_GAME_ITEM_1,                           RG_GREEN_RUPEE,                                                     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_ITEM_1));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_ITEM_2] =                             Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_ITEM_2,                            RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Second Room Item",                 RHT_MARKET_TREASURE_CHEST_GAME_ITEM_2,                           RG_GREEN_RUPEE,                                                     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_ITEM_2));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_ITEM_3] =                             Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_ITEM_3,                            RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Third Room Item",                  RHT_MARKET_TREASURE_CHEST_GAME_ITEM_3,                           RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_ITEM_3));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_ITEM_4] =                             Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_ITEM_4,                            RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Fourth Room Item",                 RHT_MARKET_TREASURE_CHEST_GAME_ITEM_4,                           RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_ITEM_4));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_ITEM_5] =                             Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_ITEM_5,                            RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Fifth Room Item",                  RHT_MARKET_TREASURE_CHEST_GAME_ITEM_5,                           RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_ITEM_5));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_KEY_1] =                              Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_KEY_1,                             RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game First Room Key",                   RHT_MARKET_TREASURE_CHEST_GAME_KEY_1,                            RG_TREASURE_GAME_SMALL_KEY,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_KEY_1));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_KEY_2] =                              Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_KEY_2,                             RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Second Room Key",                  RHT_MARKET_TREASURE_CHEST_GAME_KEY_2,                            RG_TREASURE_GAME_SMALL_KEY,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_KEY_2));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_KEY_3] =                              Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_KEY_3,                             RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Third Room Key",                   RHT_MARKET_TREASURE_CHEST_GAME_KEY_3,                            RG_TREASURE_GAME_SMALL_KEY,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_KEY_3));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_KEY_4] =                              Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_KEY_4,                             RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Fourth Room Key",                  RHT_MARKET_TREASURE_CHEST_GAME_KEY_4,                            RG_TREASURE_GAME_SMALL_KEY,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_KEY_4));
    locationTable[RC_MARKET_TREASURE_CHEST_GAME_KEY_5] =                              Location::Chest(RC_MARKET_TREASURE_CHEST_GAME_KEY_5,                             RCQUEST_BOTH,    RCTYPE_CHEST_GAME,                                                       ACTOR_EN_BOX,         SCENE_TREASURE_BOX_SHOP,            0x00,                               "Chest Game Fifth Room Key",                   RHT_MARKET_TREASURE_CHEST_GAME_KEY_5,                            RG_TREASURE_GAME_SMALL_KEY,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_MARKET_TREASURE_CHEST_GAME_KEY_5));
    // Hyrule Castle
    locationTable[RC_HC_MALON_EGG] =                                                   Location::Base(RC_HC_MALON_EGG,                                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HYRULE_CASTLE,                0x00,                               "Malon Egg",                                   RHT_HC_MALON_EGG,                                                RG_WEIRD_EGG,                                                       SpoilerCollectionCheck::EventChkInf(0x12),                                                                                    true);
    locationTable[RC_HC_ZELDAS_LETTER] =                                               Location::Base(RC_HC_ZELDAS_LETTER,                                             RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_CASTLE_COURTYARD_ZELDA,       0x00,                               "Zeldas Letter",                               RHT_HC_ZELDAS_LETTER,                                            RG_ZELDAS_LETTER,                                                   SpoilerCollectionCheck::EventChkInf(0x40),                                                                                    true);
    // Kakariko
    locationTable[RC_KAK_REDEAD_GROTTO_CHEST] =                                       Location::Chest(RC_KAK_REDEAD_GROTTO_CHEST,                                      RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_KAKARIKO_VILLAGE,             ACTOR_EN_BOX,         SCENE_GROTTOS,                      31434,                        0x0A, "Redead Grotto Chest",                         RHT_KAK_REDEAD_GROTTO_CHEST,                                     RG_HUGE_RUPEE);
    locationTable[RC_KAK_OPEN_GROTTO_CHEST] =                                         Location::Chest(RC_KAK_OPEN_GROTTO_CHEST,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_KAKARIKO_VILLAGE,             ACTOR_EN_BOX,         SCENE_GROTTOS,                      22984,                        0x08, "Open Grotto Chest",                           RHT_KAK_OPEN_GROTTO_CHEST,                                       RG_RED_RUPEE);
    locationTable[RC_KAK_10_GOLD_SKULLTULA_REWARD] =                                   Location::Base(RC_KAK_10_GOLD_SKULLTULA_REWARD,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HOUSE_OF_SKULLTULA,           0x00,                               "10 Gold Skulltula Reward",                    RHT_KAK_10_GOLD_SKULLTULA_REWARD,                                RG_PROGRESSIVE_WALLET,                                              SpoilerCollectionCheck::EventChkInf(0xDA),                                                                                    true);
    locationTable[RC_KAK_20_GOLD_SKULLTULA_REWARD] =                                   Location::Base(RC_KAK_20_GOLD_SKULLTULA_REWARD,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HOUSE_OF_SKULLTULA,           0x00,                               "20 Gold Skulltula Reward",                    RHT_KAK_20_GOLD_SKULLTULA_REWARD,                                RG_STONE_OF_AGONY,                                                  SpoilerCollectionCheck::EventChkInf(0xDB),                                                                                    true);
    locationTable[RC_KAK_30_GOLD_SKULLTULA_REWARD] =                                   Location::Base(RC_KAK_30_GOLD_SKULLTULA_REWARD,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HOUSE_OF_SKULLTULA,           0x00,                               "30 Gold Skulltula Reward",                    RHT_KAK_30_GOLD_SKULLTULA_REWARD,                                RG_PROGRESSIVE_WALLET,                                              SpoilerCollectionCheck::EventChkInf(0xDC),                                                                                    true);
    locationTable[RC_KAK_40_GOLD_SKULLTULA_REWARD] =                                   Location::Base(RC_KAK_40_GOLD_SKULLTULA_REWARD,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HOUSE_OF_SKULLTULA,           0x00,                               "40 Gold Skulltula Reward",                    RHT_KAK_40_GOLD_SKULLTULA_REWARD,                                RG_BOMBCHU_10,                                                      SpoilerCollectionCheck::EventChkInf(0xDD));
    locationTable[RC_KAK_50_GOLD_SKULLTULA_REWARD] =                                   Location::Base(RC_KAK_50_GOLD_SKULLTULA_REWARD,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HOUSE_OF_SKULLTULA,           0x00,                               "50 Gold Skulltula Reward",                    RHT_KAK_50_GOLD_SKULLTULA_REWARD,                                RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::EventChkInf(0xDE),                                                                                    true);
    locationTable[RC_KAK_100_GOLD_SKULLTULA_REWARD] =                                  Location::Base(RC_KAK_100_GOLD_SKULLTULA_REWARD,                                RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_HOUSE_OF_SKULLTULA,           0x00,                               "100 Gold Skulltula Reward",                   RHT_KAK_100_GOLD_SKULLTULA_REWARD,                               RG_HUGE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_100_GOLD_SKULLTULA_REWARD));
    locationTable[RC_KAK_MAN_ON_ROOF] =                                                Location::Base(RC_KAK_MAN_ON_ROOF,                                              RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_KAKARIKO_VILLAGE,             0x00,                               "Man on Roof",                                 RHT_KAK_MAN_ON_ROOF,                                             RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::ItemGetInf(21),                                                                                       true);
    locationTable[RC_KAK_SHOOTING_GALLERY_REWARD] =                                    Location::Base(RC_KAK_SHOOTING_GALLERY_REWARD,                                  RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_KAKARIKO_VILLAGE,             ACTOR_ID_MAX,         SCENE_SHOOTING_GALLERY,             0x00,                               "Shooting Gallery Reward",                     RHT_KAK_SHOOTING_GALLERY_REWARD,                                 RG_PROGRESSIVE_BOW,                                                 SpoilerCollectionCheck::ItemGetInf(ITEMGETINF_0E),                                                                            true);
    locationTable[RC_KAK_TRADE_ODD_MUSHROOM] =                                         Location::Base(RC_KAK_TRADE_ODD_MUSHROOM,                                       RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_POTION_SHOP_GRANNY,           0x00,                               "Trade Odd Mushroom",                          RHT_KAK_TRADE_ODD_MUSHROOM,                                      RG_ODD_POTION,                                                      SpoilerCollectionCheck::ItemGetInf(48),                                                                                       true);
    locationTable[RC_KAK_GRANNYS_SHOP] =                                               Location::Base(RC_KAK_GRANNYS_SHOP,                                             RCQUEST_BOTH,    RCTYPE_MERCHANT,                                                         ACTOR_ID_MAX,         SCENE_POTION_SHOP_GRANNY,           0x00,                               "Granny's Shop",                               RHT_KAK_GRANNYS_SHOP,                                            RG_BUY_BLUE_POTION,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_MERCHANTS_GRANNYS_SHOP),                                                       true,  100);
    locationTable[RC_KAK_ANJU_AS_ADULT] =                                              Location::Base(RC_KAK_ANJU_AS_ADULT,                                            RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_KAKARIKO_VILLAGE,             0x00,                               "Anju as Adult",                               RHT_KAK_ANJU_AS_ADULT,                                           RG_CLAIM_CHECK,                                                     SpoilerCollectionCheck::ItemGetInf(44),                                                                                       true);
    locationTable[RC_KAK_ANJU_AS_CHILD] =                                              Location::Base(RC_KAK_ANJU_AS_CHILD,                                            RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_KAKARIKO_VILLAGE,             0x00,                               "Anju as Child",                               RHT_KAK_ANJU_AS_CHILD,                                           RG_EMPTY_BOTTLE,                                                    SpoilerCollectionCheck::ItemGetInf(12),                                                                                       true);
    locationTable[RC_KAK_TRADE_POCKET_CUCCO] =                                         Location::Base(RC_KAK_TRADE_POCKET_CUCCO,                                       RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_KAKARIKO_VILLAGE,             0x00,                               "Trade Pocket Cucco",                          RHT_KAK_TRADE_POCKET_CUCCO,                                      RG_COJIRO,                                                          SpoilerCollectionCheck::ItemGetInf(46),                                                                                       true);
    locationTable[RC_KAK_IMPAS_HOUSE_FREESTANDING_POH] =                        Location::Collectable(RC_KAK_IMPAS_HOUSE_FREESTANDING_POH,                             RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_IMPAS_HOUSE,                  262,                          0x01, "Impas House Freestanding PoH",                RHT_KAK_IMPAS_HOUSE_FREESTANDING_POH,                            RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_KAK_WINDMILL_FREESTANDING_POH] =                           Location::Collectable(RC_KAK_WINDMILL_FREESTANDING_POH,                                RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_KAKARIKO_VILLAGE,             ACTOR_EN_ITEM00,      SCENE_WINDMILL_AND_DAMPES_GRAVE,    262,                          0x01, "Windmill Freestanding PoH",                   RHT_KAK_WINDMILL_FREESTANDING_POH,                               RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    // Graveyard
    locationTable[RC_GRAVEYARD_SHIELD_GRAVE_CHEST] =                                  Location::Chest(RC_GRAVEYARD_SHIELD_GRAVE_CHEST,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_GRAVEYARD,                    ACTOR_EN_BOX,         SCENE_GRAVE_WITH_FAIRYS_FOUNTAIN,   21824,                        0x00, "Shield Grave Chest",                          RHT_GRAVEYARD_SHIELD_GRAVE_CHEST,                                RG_HYLIAN_SHIELD);
    locationTable[RC_GRAVEYARD_HEART_PIECE_GRAVE_CHEST] =                             Location::Chest(RC_GRAVEYARD_HEART_PIECE_GRAVE_CHEST,                            RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_GRAVEYARD,                    ACTOR_EN_BOX,         SCENE_REDEAD_GRAVE,                 -22592,                       0x00, "Heart Piece Grave Chest",                     RHT_GRAVEYARD_HEART_PIECE_GRAVE_CHEST,                           RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GRAVEYARD_ROYAL_FAMILYS_TOMB_CHEST] =                            Location::Chest(RC_GRAVEYARD_ROYAL_FAMILYS_TOMB_CHEST,                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_ROYAL_FAMILYS_TOMB,           -32736,                       0x00, "Composers Grave Chest",                       RHT_GRAVEYARD_ROYAL_FAMILYS_TOMB_CHEST,                          RG_BOMBS_5);
    locationTable[RC_GRAVEYARD_HOOKSHOT_CHEST] =                                      Location::Chest(RC_GRAVEYARD_HOOKSHOT_CHEST,                                     RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_GRAVEYARD,                    ACTOR_EN_BOX,         SCENE_WINDMILL_AND_DAMPES_GRAVE,    4352,                         0x00, "Hookshot Chest",                              RHT_GRAVEYARD_HOOKSHOT_CHEST,                                    RG_PROGRESSIVE_HOOKSHOT,                                                                                                                                                                          true);
    locationTable[RC_GRAVEYARD_FREESTANDING_POH] =                              Location::Collectable(RC_GRAVEYARD_FREESTANDING_POH,                                   RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_GRAVEYARD,                    1030,                         0x04, "Freestanding PoH",                            RHT_GRAVEYARD_FREESTANDING_POH,                                  RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GRAVEYARD_DAMPE_RACE_FREESTANDING_POH] =                   Location::Collectable(RC_GRAVEYARD_DAMPE_RACE_FREESTANDING_POH,                        RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_GRAVEYARD,                    ACTOR_EN_ITEM00,      SCENE_WINDMILL_AND_DAMPES_GRAVE,    1798,                         0x07, "Dampe Race Freestanding PoH",                 RHT_GRAVEYARD_DAMPE_RACE_FREESTANDING_POH,                       RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GRAVEYARD_DAMPE_GRAVEDIGGING_TOUR] =                       Location::Collectable(RC_GRAVEYARD_DAMPE_GRAVEDIGGING_TOUR,                            RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_GRAVEYARD,                    6406,                         0x19, "Dampe Gravedigging Tour",                     RHT_GRAVEYARD_DAMPE_GRAVEDIGGING_TOUR,                           RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    // Death Mountain
    locationTable[RC_DMT_CHEST] =                                                     Location::Chest(RC_DMT_CHEST,                                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEATH_MOUNTAIN_TRAIL,         23201,                        0x01, "Chest",                                       RHT_DMT_CHEST,                                                   RG_PURPLE_RUPEE);
    locationTable[RC_DMT_STORMS_GROTTO_CHEST] =                                       Location::Chest(RC_DMT_STORMS_GROTTO_CHEST,                                      RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_DEATH_MOUNTAIN_TRAIL,         ACTOR_EN_BOX,         SCENE_GROTTOS,                      23255,                        0x17, "Storms Grotto Chest",                         RHT_DMT_STORMS_GROTTO_CHEST,                                     RG_HUGE_RUPEE);
    locationTable[RC_DMT_TRADE_BROKEN_SWORD] =                                         Location::Base(RC_DMT_TRADE_BROKEN_SWORD,                                       RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_DEATH_MOUNTAIN_TRAIL,         0x00,                               "Trade Broken Sword",                          RHT_DMT_TRADE_BROKEN_SWORD,                                      RG_PRESCRIPTION,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_DMT_TRADE_BROKEN_SWORD),                                          true);
    locationTable[RC_DMT_TRADE_EYEDROPS] =                                             Location::Base(RC_DMT_TRADE_EYEDROPS,                                           RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_DEATH_MOUNTAIN_TRAIL,         0x00,                               "Trade Eyedrops",                              RHT_DMT_TRADE_EYEDROPS,                                          RG_CLAIM_CHECK,                                                     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_DMT_TRADE_EYEDROPS),                                              true);
    locationTable[RC_DMT_TRADE_CLAIM_CHECK] =                                          Location::Base(RC_DMT_TRADE_CLAIM_CHECK,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_DEATH_MOUNTAIN_TRAIL,         0x00,                               "Trade Claim Check",                           RHT_DMT_TRADE_CLAIM_CHECK,                                       RG_BIGGORON_SWORD,                                                  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_DMT_TRADE_CLAIM_CHECK),                                           true);
    locationTable[RC_DMT_FREESTANDING_POH] =                                    Location::Collectable(RC_DMT_FREESTANDING_POH,                                         RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_DEATH_MOUNTAIN_TRAIL,         7686,                         0x1E, "Freestanding PoH",                            RHT_DMT_FREESTANDING_POH,                                        RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    // Goron City
    locationTable[RC_GC_MAZE_LEFT_CHEST] =                                            Location::Chest(RC_GC_MAZE_LEFT_CHEST,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GORON_CITY,                   23232,                        0x00, "Maze Left Chest",                             RHT_GC_MAZE_LEFT_CHEST,                                          RG_HUGE_RUPEE);
    locationTable[RC_GC_MAZE_RIGHT_CHEST] =                                           Location::Chest(RC_GC_MAZE_RIGHT_CHEST,                                          RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GORON_CITY,                   23201,                        0x01, "Maze Right Chest",                            RHT_GC_MAZE_RIGHT_CHEST,                                         RG_PURPLE_RUPEE);
    locationTable[RC_GC_MAZE_CENTER_CHEST] =                                          Location::Chest(RC_GC_MAZE_CENTER_CHEST,                                         RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GORON_CITY,                   23202,                        0x02, "Maze Center Chest",                           RHT_GC_MAZE_CENTER_CHEST,                                        RG_PURPLE_RUPEE);
    locationTable[RC_GC_ROLLING_GORON_AS_CHILD] =                                      Location::Base(RC_GC_ROLLING_GORON_AS_CHILD,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_GORON_CITY,                   0x00,                               "Rolling Goron as Child",                      RHT_GC_ROLLING_GORON_AS_CHILD,                                   RG_PROGRESSIVE_BOMB_BAG,                                            SpoilerCollectionCheck::InfTable(INFTABLE_11E),                                                                               true);
    locationTable[RC_GC_ROLLING_GORON_AS_ADULT] =                                      Location::Base(RC_GC_ROLLING_GORON_AS_ADULT,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_GORON_CITY,                   0x00,                               "Rolling Goron as Adult",                      RHT_GC_ROLLING_GORON_AS_ADULT,                                   RG_GORON_TUNIC,                                                     SpoilerCollectionCheck::InfTable(INFTABLE_GORON_CITY_DOORS_UNLOCKED),                                                         true);
    locationTable[RC_GC_DARUNIAS_JOY] =                                                Location::Base(RC_GC_DARUNIAS_JOY,                                              RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_GORON_CITY,                   0x00,                               "Darunias Joy",                                RHT_GC_DARUNIAS_JOY,                                             RG_PROGRESSIVE_STRENGTH,                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_DARUNIAS_JOY),                                                                 true);
    locationTable[RC_GC_POT_FREESTANDING_POH] =                                 Location::Collectable(RC_GC_POT_FREESTANDING_POH,                                      RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_GORON_CITY,                   7942,                         0x1F, "Pot Freestanding PoH",                        RHT_GC_POT_FREESTANDING_POH,                                     RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_GC_DEKU_SCRUB_GROTTO_LEFT] =                                      Location::Base(RC_GC_DEKU_SCRUB_GROTTO_LEFT,                                    RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_GORON_CITY,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x00, 0xFB),       "Deku Scrub Grotto Left",                      RHT_GC_DEKU_SCRUB_GROTTO_LEFT,                                   RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GC_DEKU_SCRUB_GROTTO_LEFT),                                   false, 20);
    locationTable[RC_GC_DEKU_SCRUB_GROTTO_RIGHT] =                                     Location::Base(RC_GC_DEKU_SCRUB_GROTTO_RIGHT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_GORON_CITY,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x05, 0xFB),       "Deku Scrub Grotto Right",                     RHT_GC_DEKU_SCRUB_GROTTO_RIGHT,                                  RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GC_DEKU_SCRUB_GROTTO_RIGHT),                                  false, 40);
    locationTable[RC_GC_DEKU_SCRUB_GROTTO_CENTER] =                                    Location::Base(RC_GC_DEKU_SCRUB_GROTTO_CENTER,                                  RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_GORON_CITY,                   ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x03, 0xFB),       "Deku Scrub Grotto Center",                    RHT_GC_DEKU_SCRUB_GROTTO_CENTER,                                 RG_BUY_ARROWS_30,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GC_DEKU_SCRUB_GROTTO_CENTER),                                 false, 70);
    locationTable[RC_GC_MEDIGORON] =                                                   Location::Base(RC_GC_MEDIGORON,                                                 RCQUEST_BOTH,    RCTYPE_MERCHANT,                                                         ACTOR_ID_MAX,         SCENE_GORON_CITY,                   0x00,                               "Medigoron",                                   RHT_GC_MEDIGORON,                                                RG_GIANTS_KNIFE,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_MERCHANTS_MEDIGORON),                                                          false, 200);
    // Death Mountain Crater
    locationTable[RC_DMC_UPPER_GROTTO_CHEST] =                                        Location::Chest(RC_DMC_UPPER_GROTTO_CHEST,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_EN_BOX,         SCENE_GROTTOS,                      23802,                        0x1A, "Upper Grotto Chest",                          RHT_DMC_UPPER_GROTTO_CHEST,                                      RG_BOMBS_20);
    locationTable[RC_DMC_WALL_FREESTANDING_POH] =                               Location::Collectable(RC_DMC_WALL_FREESTANDING_POH,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_DEATH_MOUNTAIN_CRATER,        518,                          0x02, "Wall Freestanding PoH",                       RHT_DMC_WALL_FREESTANDING_POH,                                   RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_DMC_VOLCANO_FREESTANDING_POH] =                            Location::Collectable(RC_DMC_VOLCANO_FREESTANDING_POH,                                 RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_DEATH_MOUNTAIN_CRATER,        2054,                         0x08, "Volcano Freestanding PoH",                    RHT_DMC_WALL_FREESTANDING_POH,                                   RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_DMC_DEKU_SCRUB] =                                                 Location::Base(RC_DMC_DEKU_SCRUB,                                               RCQUEST_BOTH,    RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DEATH_MOUNTAIN_CRATER,        0x05,                               "Deku Scrub",                                  RHT_DMC_DEKU_SCRUB,                                              RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DMC_DEKU_SCRUB),                                              false, 40);
    locationTable[RC_DMC_DEKU_SCRUB_GROTTO_LEFT] =                                     Location::Base(RC_DMC_DEKU_SCRUB_GROTTO_LEFT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x00, 0xF9),       "Deku Scrub Grotto Left",                      RHT_DMC_DEKU_SCRUB_GROTTO_LEFT,                                  RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DMC_DEKU_SCRUB_GROTTO_LEFT),                                  false, 20);
    locationTable[RC_DMC_DEKU_SCRUB_GROTTO_RIGHT] =                                    Location::Base(RC_DMC_DEKU_SCRUB_GROTTO_RIGHT,                                  RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x05, 0xF9),       "Deku Scrub Grotto Right",                     RHT_DMC_DEKU_SCRUB_GROTTO_RIGHT,                                 RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DMC_DEKU_SCRUB_GROTTO_RIGHT),                                 false, 40);
    locationTable[RC_DMC_DEKU_SCRUB_GROTTO_CENTER] =                                   Location::Base(RC_DMC_DEKU_SCRUB_GROTTO_CENTER,                                 RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x03, 0xF9),       "Deku Scrub Grotto Center",                    RHT_DMC_DEKU_SCRUB_GROTTO_CENTER,                                RG_BUY_ARROWS_30,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DMC_DEKU_SCRUB_GROTTO_CENTER),                                false, 70);
    // Zoras River
    locationTable[RC_ZR_OPEN_GROTTO_CHEST] =                                           Location::Chest(RC_ZR_OPEN_GROTTO_CHEST,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_ZORAS_RIVER,                  ACTOR_EN_BOX,         SCENE_GROTTOS,                      22985,                        0x09, "Open Grotto Chest",                           RHT_ZR_OPEN_GROTTO_CHEST,                                        RG_RED_RUPEE);
    locationTable[RC_ZR_MAGIC_BEAN_SALESMAN] =                                         Location::Base(RC_ZR_MAGIC_BEAN_SALESMAN,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_ZORAS_RIVER,                  0x00,                               "Magic Bean Salesman",                         RHT_ZR_MAGIC_BEAN_SALESMAN,                                      RG_MAGIC_BEAN,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MERCHANTS_MAGIC_BEAN_SALESMAN),                                                true,  60);
    locationTable[RC_ZR_FROGS_ZELDAS_LULLABY] =                                        Location::Base(RC_ZR_FROGS_ZELDAS_LULLABY,                                      RCQUEST_BOTH,    RCTYPE_FROG_SONG,                                                        ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs Zelda's Lullaby",                       RHT_ZR_FROGS_ZELDAS_LULLABY,                                     RG_PURPLE_RUPEE,                                                    SpoilerCollectionCheck::EventChkInf(0xD1));
    locationTable[RC_ZR_FROGS_EPONAS_SONG] =                                           Location::Base(RC_ZR_FROGS_EPONAS_SONG,                                         RCQUEST_BOTH,    RCTYPE_FROG_SONG,                                                        ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs Epona's Song",                          RHT_ZR_FROGS_EPONAS_SONG,                                        RG_PURPLE_RUPEE,                                                    SpoilerCollectionCheck::EventChkInf(0xD2));
    locationTable[RC_ZR_FROGS_SARIAS_SONG] =                                           Location::Base(RC_ZR_FROGS_SARIAS_SONG,                                         RCQUEST_BOTH,    RCTYPE_FROG_SONG,                                                        ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs Saria's Song",                          RHT_ZR_FROGS_SARIAS_SONG,                                        RG_PURPLE_RUPEE,                                                    SpoilerCollectionCheck::EventChkInf(0xD4));
    locationTable[RC_ZR_FROGS_SUNS_SONG] =                                             Location::Base(RC_ZR_FROGS_SUNS_SONG,                                           RCQUEST_BOTH,    RCTYPE_FROG_SONG,                                                        ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs Sun's Song",                            RHT_ZR_FROGS_SUNS_SONG,                                          RG_PURPLE_RUPEE,                                                    SpoilerCollectionCheck::EventChkInf(0xD3));
    locationTable[RC_ZR_FROGS_SONG_OF_TIME] =                                          Location::Base(RC_ZR_FROGS_SONG_OF_TIME,                                        RCQUEST_BOTH,    RCTYPE_FROG_SONG,                                                        ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs Song of Time",                          RHT_ZR_FROGS_SONG_OF_TIME,                                       RG_PURPLE_RUPEE,                                                    SpoilerCollectionCheck::EventChkInf(0xD5));
    locationTable[RC_ZR_FROGS_IN_THE_RAIN] =                                           Location::Base(RC_ZR_FROGS_IN_THE_RAIN,                                         RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs in the Rain",                           RHT_ZR_FROGS_IN_THE_RAIN,                                        RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::EventChkInf(0xD6),                                                                                    true);
    locationTable[RC_ZR_FROGS_OCARINA_GAME] =                                          Location::Base(RC_ZR_FROGS_OCARINA_GAME,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_FR,          SCENE_ZORAS_RIVER,                  0x00,                               "Frogs Ocarina Game",                          RHT_ZR_FROGS_OCARINA_GAME,                                       RG_PIECE_OF_HEART,                                                  SpoilerCollectionCheck::EventChkInf(0xD0),                                                                                    true);
    locationTable[RC_ZR_NEAR_OPEN_GROTTO_FREESTANDING_POH] =                    Location::Collectable(RC_ZR_NEAR_OPEN_GROTTO_FREESTANDING_POH,                         RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_ZORAS_RIVER,                  1030,                         0x04, "Near Open Grotto Freestanding PoH",           RHT_ZR_NEAR_OPEN_GROTTO_FREESTANDING_POH,                        RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_ZR_NEAR_DOMAIN_FREESTANDING_POH] =                         Location::Collectable(RC_ZR_NEAR_DOMAIN_FREESTANDING_POH,                              RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_ZORAS_RIVER,                  2822,                         0x0B, "Near Domain Freestanding PoH",                RHT_ZR_NEAR_DOMAIN_FREESTANDING_POH,                             RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_ZR_DEKU_SCRUB_GROTTO_REAR] =                                      Location::Base(RC_ZR_DEKU_SCRUB_GROTTO_REAR,                                    RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_ZORAS_RIVER,                  ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x07, 0xEB),       "Deku Scrub Grotto Rear",                      RHT_ZR_DEKU_SCRUB_GROTTO_REAR,                                   RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_ZR_DEKU_SCRUB_GROTTO_REAR),                                   false, 40);
    locationTable[RC_ZR_DEKU_SCRUB_GROTTO_FRONT] =                                     Location::Base(RC_ZR_DEKU_SCRUB_GROTTO_FRONT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_ZORAS_RIVER,                  ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x08, 0xEB),       "Deku Scrub Grotto Front",                     RHT_ZR_DEKU_SCRUB_GROTTO_FRONT,                                  RG_BUY_GREEN_POTION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_ZR_DEKU_SCRUB_GROTTO_FRONT),                                  false, 40);
    // Zoras Domain
    locationTable[RC_ZD_CHEST] =                                                      Location::Chest(RC_ZD_CHEST,                                                     RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_ZORAS_DOMAIN,                 -18496,                       0x00, "Chest",                                       RHT_ZD_CHEST,                                                    RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_ZD_DIVING_MINIGAME] =                                             Location::Base(RC_ZD_DIVING_MINIGAME,                                           RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_ZORAS_DOMAIN,                 0x00,                               "Diving Minigame",                             RHT_ZD_DIVING_MINIGAME,                                          RG_PROGRESSIVE_SCALE,                                               SpoilerCollectionCheck::EventChkInf(0x38),                                                                                    true);
    locationTable[RC_ZD_KING_ZORA_THAWED] =                                            Location::Base(RC_ZD_KING_ZORA_THAWED,                                          RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_ZORAS_DOMAIN,                 0x00,                               "King Zora Thawed",                            RHT_ZD_KING_ZORA_THAWED,                                         RG_ZORA_TUNIC,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_KING_ZORA_THAWED),                                                             true);
    locationTable[RC_ZD_TRADE_PRESCRIPTION] =                                          Location::Base(RC_ZD_TRADE_PRESCRIPTION,                                        RCQUEST_BOTH,    RCTYPE_ADULT_TRADE,                                                      ACTOR_ID_MAX,         SCENE_ZORAS_DOMAIN,                 0x00,                               "Trade Prescription",                          RHT_ZD_TRADE_PRESCRIPTION,                                       RG_EYEBALL_FROG,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ADULT_TRADES_ZD_TRADE_PRESCRIPTION),                                           true);
    // Zora's Fountain
    locationTable[RC_ZF_ICEBERC_FREESTANDING_POH] =                             Location::Collectable(RC_ZF_ICEBERC_FREESTANDING_POH,                                  RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_ZORAS_FOUNTAIN,               262,                          0x01, "Iceberg Freestanding PoH",                    RHT_ZF_ICEBERG_FREESTANDING_POH,                                 RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_ZF_BOTTOM_FREESTANDING_POH] =                              Location::Collectable(RC_ZF_BOTTOM_FREESTANDING_POH,                                   RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_ZORAS_FOUNTAIN,               5126,                         0x14, "Bottom Freestanding PoH",                     RHT_ZF_BOTTOM_FREESTANDING_POH,                                  RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    // Lon Lon Ranch
    locationTable[RC_LLR_TALONS_CHICKENS] =                                            Location::Base(RC_LLR_TALONS_CHICKENS,                                          RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_LON_LON_BUILDINGS,            0x00,                               "Talons Chickens",                             RHT_LLR_TALONS_CHICKENS,                                         RG_BOTTLE_WITH_MILK,                                                SpoilerCollectionCheck::ItemGetInf(2),                                                                                        true);
    locationTable[RC_LLR_FREESTANDING_POH] =                                    Location::Collectable(RC_LLR_FREESTANDING_POH,                                         RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_LON_LON_BUILDINGS,            262,                          0x01, "Freestanding PoH",                            RHT_LLR_FREESTANDING_POH,                                        RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    locationTable[RC_LLR_DEKU_SCRUB_GROTTO_LEFT] =                                     Location::Base(RC_LLR_DEKU_SCRUB_GROTTO_LEFT,                                   RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LON_LON_RANCH,                ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x00, 0xFC),       "Deku Scrub Grotto Left",                      RHT_LLR_DEKU_SCRUB_GROTTO_LEFT,                                  RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LLR_DEKU_SCRUB_GROTTO_LEFT),                                  false, 20);
    locationTable[RC_LLR_DEKU_SCRUB_GROTTO_RIGHT] =                                    Location::Base(RC_LLR_DEKU_SCRUB_GROTTO_RIGHT,                                  RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LON_LON_RANCH,                ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x05, 0xFC),       "Deku Scrub Grotto Right",                     RHT_LLR_DEKU_SCRUB_GROTTO_RIGHT,                                 RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LLR_DEKU_SCRUB_GROTTO_RIGHT),                                 false, 40);
    locationTable[RC_LLR_DEKU_SCRUB_GROTTO_CENTER] =                                   Location::Base(RC_LLR_DEKU_SCRUB_GROTTO_CENTER,                                 RCQUEST_BOTH,    RCTYPE_SCRUB,                       RCAREA_LON_LON_RANCH,                ACTOR_EN_DNS,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(0x03, 0xFC),       "Deku Scrub Grotto Center",                    RHT_LLR_DEKU_SCRUB_GROTTO_CENTER,                                RG_BUY_DEKU_SEEDS_30,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_LLR_DEKU_SCRUB_GROTTO_CENTER),                                false, 40);

    // Dungeons
    // Deku Tree Vanilla
    locationTable[RC_DEKU_TREE_MAP_CHEST] =                                           Location::Chest(RC_DEKU_TREE_MAP_CHEST,                                          RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    2083,                         0x03, "Map Chest",                                   RHT_DEKU_TREE_MAP_CHEST,                                         RG_DEKU_TREE_MAP,                                                                                                                                                                                 true);
    locationTable[RC_DEKU_TREE_COMPASS_CHEST] =                                       Location::Chest(RC_DEKU_TREE_COMPASS_CHEST,                                      RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    2050,                         0x02, "Compass Chest",                               RHT_DEKU_TREE_COMPASS_CHEST,                                     RG_DEKU_TREE_COMPASS,                                                                                                                                                                             true);
    locationTable[RC_DEKU_TREE_COMPASS_ROOM_SIDE_CHEST] =                             Location::Chest(RC_DEKU_TREE_COMPASS_ROOM_SIDE_CHEST,                            RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    22790,                        0x06, "Compass Room Side Chest",                     RHT_DEKU_TREE_COMPASS_ROOM_SIDE_CHEST,                           RG_RECOVERY_HEART);
    locationTable[RC_DEKU_TREE_BASEMENT_CHEST] =                                      Location::Chest(RC_DEKU_TREE_BASEMENT_CHEST,                                     RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    22788,                        0x04, "Basement Chest",                              RHT_DEKU_TREE_BASEMENT_CHEST,                                    RG_RECOVERY_HEART);
    locationTable[RC_DEKU_TREE_SLINGSHOT_CHEST] =                                     Location::Chest(RC_DEKU_TREE_SLINGSHOT_CHEST,                                    RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    161,                          0x01, "Slingshot Chest",                             RHT_DEKU_TREE_SLINGSHOT_CHEST,                                   RG_PROGRESSIVE_SLINGSHOT,                                                                                                                                                                         true);
    locationTable[RC_DEKU_TREE_SLINGSHOT_ROOM_SIDE_CHEST] =                           Location::Chest(RC_DEKU_TREE_SLINGSHOT_ROOM_SIDE_CHEST,                          RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    22789,                        0x05, "Slingshot Room Side Chest",                   RHT_DEKU_TREE_SLINGSHOT_ROOM_SIDE_CHEST,                         RG_RECOVERY_HEART);
    // Deku Tree MQ
    locationTable[RC_DEKU_TREE_MQ_MAP_CHEST] =                                        Location::Chest(RC_DEKU_TREE_MQ_MAP_CHEST,                                       RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    2083,                         0x03, "MQ Map Chest",                                RHT_DEKU_TREE_MQ_MAP_CHEST,                                      RG_DEKU_TREE_MAP,                                                                                                                                                                                 true);
    locationTable[RC_DEKU_TREE_MQ_COMPASS_CHEST] =                                    Location::Chest(RC_DEKU_TREE_MQ_COMPASS_CHEST,                                   RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    2049,                         0x01, "MQ Compass Chest",                            RHT_DEKU_TREE_MQ_COMPASS_CHEST,                                  RG_DEKU_TREE_COMPASS,                                                                                                                                                                             true);
    locationTable[RC_DEKU_TREE_MQ_SLINGSHOT_CHEST] =                                  Location::Chest(RC_DEKU_TREE_MQ_SLINGSHOT_CHEST,                                 RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    4262,                         0x06, "MQ Slingshot Chest",                          RHT_DEKU_TREE_MQ_SLINGSHOT_CHEST,                                RG_PROGRESSIVE_SLINGSHOT,                                                                                                                                                                         true);
    locationTable[RC_DEKU_TREE_MQ_SLINGSHOT_ROOM_BACK_CHEST] =                        Location::Chest(RC_DEKU_TREE_MQ_SLINGSHOT_ROOM_BACK_CHEST,                       RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    -31454,                       0x02, "MQ Slingshot Room Back Chest",                RHT_DEKU_TREE_MQ_SLINGSHOT_ROOM_BACK_CHEST,                      RG_DEKU_SHIELD);
    locationTable[RC_DEKU_TREE_MQ_BASEMENT_CHEST] =                                   Location::Chest(RC_DEKU_TREE_MQ_BASEMENT_CHEST,                                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    -31452,                       0x04, "MQ Basement Chest",                           RHT_DEKU_TREE_MQ_BASEMENT_CHEST,                                 RG_DEKU_SHIELD);
    locationTable[RC_DEKU_TREE_MQ_BEFORE_SPINNING_LOG_CHEST] =                        Location::Chest(RC_DEKU_TREE_MQ_BEFORE_SPINNING_LOG_CHEST,                       RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    22789,                        0x05, "MQ Before Spinning Log Chest",                RHT_DEKU_TREE_MQ_BEFORE_SPINNING_LOG_CHEST,                      RG_RECOVERY_HEART);
    locationTable[RC_DEKU_TREE_MQ_AFTER_SPINNING_LOG_CHEST] =                         Location::Chest(RC_DEKU_TREE_MQ_AFTER_SPINNING_LOG_CHEST,                        RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DEKU_TREE,                    23200,                        0x00, "MQ After Spinning Log Chest",                 RHT_DEKU_TREE_MQ_AFTER_SPINNING_LOG_CHEST,                       RG_PURPLE_RUPEE);
    locationTable[RC_DEKU_TREE_MQ_DEKU_SCRUB] =                                        Location::Base(RC_DEKU_TREE_MQ_DEKU_SCRUB,                                      RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DEKU_TREE,                    0x04,                               "MQ Deku Scrub",                               RHT_DEKU_TREE_MQ_DEKU_SCRUB,                                     RG_BUY_DEKU_SHIELD,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DEKU_TREE_MQ_DEKU_SCRUB),                                     false, 50);

    // Dodongo's Cavern Shared
    locationTable[RC_DODONGOS_CAVERN_BOSS_ROOM_CHEST] =                               Location::Chest(RC_DODONGOS_CAVERN_BOSS_ROOM_CHEST,                              RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN_BOSS,         20512,                        0x00, "Boss Room Chest",                             RHT_DODONGOS_CAVERN_BOSS_ROOM_CHEST,                             RG_BOMBS_5);
    // Dodongo's Cavern Vanilla
    locationTable[RC_DODONGOS_CAVERN_MAP_CHEST] =                                     Location::Chest(RC_DODONGOS_CAVERN_MAP_CHEST,                                    RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              2088,                         0x08, "Map Chest",                                   RHT_DODONGOS_CAVERN_MAP_CHEST,                                   RG_DODONGOS_CAVERN_MAP,                                                                                                                                                                           true);
    locationTable[RC_DODONGOS_CAVERN_COMPASS_CHEST] =                                 Location::Chest(RC_DODONGOS_CAVERN_COMPASS_CHEST,                                RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              2053,                         0x05, "Compass Chest",                               RHT_DODONGOS_CAVERN_COMPASS_CHEST,                               RG_DODONGOS_CAVERN_COMPASS,                                                                                                                                                                       true);
    locationTable[RC_DODONGOS_CAVERN_BOMB_FLOWER_PLATFORM_CHEST] =                    Location::Chest(RC_DODONGOS_CAVERN_BOMB_FLOWER_PLATFORM_CHEST,                   RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              22982,                        0x06, "Bomb Flower Platform Chest",                  RHT_DODONGOS_CAVERN_BOMB_FLOWER_PLATFORM_CHEST,                  RG_RED_RUPEE);
    locationTable[RC_DODONGOS_CAVERN_BOMB_BAG_CHEST] =                                Location::Chest(RC_DODONGOS_CAVERN_BOMB_BAG_CHEST,                               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              1604,                         0x04, "Bomb Bag Chest",                              RHT_DODONGOS_CAVERN_BOMB_BAG_CHEST,                              RG_PROGRESSIVE_BOMB_BAG,                                                                                                                                                                          true);
    locationTable[RC_DODONGOS_CAVERN_END_OF_BRIDGE_CHEST] =                           Location::Chest(RC_DODONGOS_CAVERN_END_OF_BRIDGE_CHEST,                          RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              21802,                        0x0A, "End Of Bridge Chest",                         RHT_DODONGOS_CAVERN_END_OF_BRIDGE_CHEST,                         RG_DEKU_SHIELD);
    locationTable[RC_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_LEFT] =                  Location::Base(RC_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_LEFT,                RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x00,                               "Deku Scrub Near Bomb Bag Left",               RHT_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_LEFT,               RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_LEFT),               false, 20);
    locationTable[RC_DODONGOS_CAVERN_DEKU_SCRUB_SIDE_ROOM_NEAR_DODONGOS] =             Location::Base(RC_DODONGOS_CAVERN_DEKU_SCRUB_SIDE_ROOM_NEAR_DODONGOS,           RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x01,                               "Deku Scrub Side Room Near Dodongos",          RHT_DODONGOS_CAVERN_DEKU_SCRUB_SIDE_ROOM_NEAR_DODONGOS,          RG_BUY_DEKU_STICK_1,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_DEKU_SCRUB_SIDE_ROOM_NEAR_DODONGOS),          false, 15);
    locationTable[RC_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_RIGHT] =                 Location::Base(RC_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_RIGHT,               RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x03,                               "Deku Scrub Near Bomb Bag Right",              RHT_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_RIGHT,              RG_BUY_DEKU_SEEDS_30,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_DEKU_SCRUB_NEAR_BOMB_BAG_RIGHT),              false, 40);
    locationTable[RC_DODONGOS_CAVERN_DEKU_SCRUB_LOBBY] =                               Location::Base(RC_DODONGOS_CAVERN_DEKU_SCRUB_LOBBY,                             RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x04,                               "Deku Scrub Lobby",                            RHT_DODONGOS_CAVERN_DEKU_SCRUB_LOBBY,                            RG_BUY_DEKU_SHIELD,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_DEKU_SCRUB_LOBBY),                            false, 50);
    // Dodongo's Cavern MQ
    locationTable[RC_DODONGOS_CAVERN_MQ_MAP_CHEST] =                                  Location::Chest(RC_DODONGOS_CAVERN_MQ_MAP_CHEST,                                 RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              2080,                         0x00, "MQ Map Chest",                                RHT_DODONGOS_CAVERN_MQ_MAP_CHEST,                                RG_DODONGOS_CAVERN_MAP,                                                                                                                                                                           true);
    locationTable[RC_DODONGOS_CAVERN_MQ_BOMB_BAG_CHEST] =                             Location::Chest(RC_DODONGOS_CAVERN_MQ_BOMB_BAG_CHEST,                            RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              1604,                         0x04, "MQ Bomb Bag Chest",                           RHT_DODONGOS_CAVERN_MQ_BOMB_BAG_CHEST,                           RG_PROGRESSIVE_BOMB_BAG,                                                                                                                                                                          true);
    locationTable[RC_DODONGOS_CAVERN_MQ_COMPASS_CHEST] =                              Location::Chest(RC_DODONGOS_CAVERN_MQ_COMPASS_CHEST,                             RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              6149,                         0x05, "MQ Compass Chest",                            RHT_DODONGOS_CAVERN_MQ_COMPASS_CHEST,                            RG_DODONGOS_CAVERN_COMPASS,                                                                                                                                                                       true);
    locationTable[RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CHEST] =                          Location::Chest(RC_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CHEST,                         RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              29986,                        0x02, "MQ Larvae Room Chest",                        RHT_DODONGOS_CAVERN_MQ_LARVAE_ROOM_CHEST,                        RG_DEKU_SHIELD);
    locationTable[RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_ROOM_CHEST] =                    Location::Chest(RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_ROOM_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              22947,                        0x03, "MQ Torch Puzzle Room Chest",                  RHT_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_ROOM_CHEST,                  RG_BLUE_RUPEE);
    locationTable[RC_DODONGOS_CAVERN_MQ_UNDER_GRAVE_CHEST] =                          Location::Chest(RC_DODONGOS_CAVERN_MQ_UNDER_GRAVE_CHEST,                         RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_DODONGOS_CAVERN,              21825,                        0x01, "MQ Under Grave Chest",                        RHT_DODONGOS_CAVERN_MQ_UNDER_GRAVE_CHEST,                        RG_HYLIAN_SHIELD);
    locationTable[RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_REAR] =                       Location::Base(RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_REAR,                     RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x01,                               "MQ Deku Scrub Lobby Rear",                    RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_REAR,                    RG_BUY_DEKU_STICK_1,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_REAR),                    false, 15);
    locationTable[RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_FRONT] =                      Location::Base(RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_FRONT,                    RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x03,                               "MQ Deku Scrub Lobby Front",                   RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_FRONT,                   RG_BUY_DEKU_SEEDS_30,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_MQ_DEKU_SCRUB_LOBBY_FRONT),                   false, 40);
    locationTable[RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_STAIRCASE] =                        Location::Base(RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_STAIRCASE,                      RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x04,                               "MQ Deku Scrub Staircase",                     RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_STAIRCASE,                     RG_BUY_DEKU_SHIELD,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_MQ_DEKU_SCRUB_STAIRCASE),                     false, 50);
    locationTable[RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_SIDE_ROOM_NEAR_LOWER_LIZALFOS] =    Location::Base(RC_DODONGOS_CAVERN_MQ_DEKU_SCRUB_SIDE_ROOM_NEAR_LOWER_LIZALFOS,  RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_DODONGOS_CAVERN,              0x07,                               "MQ Deku Scrub Side Room Near Lower Lizalfos", RHT_DODONGOS_CAVERN_MQ_DEKU_SCRUB_SIDE_ROOM_NEAR_LOWER_LIZALFOS, RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_DODONGOS_CAVERN_MQ_DEKU_SCRUB_SIDE_ROOM_NEAR_LOWER_LIZALFOS), false, 40);

    // Jabu-Jabu's Belly Vanilla
    locationTable[RC_JABU_JABUS_BELLY_MAP_CHEST] =                                    Location::Chest(RC_JABU_JABUS_BELLY_MAP_CHEST,                                   RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_JABU_JABU,                    6178,                         0x02, "Map Chest",                                   RHT_JABU_JABUS_BELLY_MAP_CHEST,                                  RG_JABU_JABUS_BELLY_MAP,                                                                                                                                                                          true);
    locationTable[RC_JABU_JABUS_BELLY_COMPASS_CHEST] =                                Location::Chest(RC_JABU_JABUS_BELLY_COMPASS_CHEST,                               RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -18428,                       0x04, "Compass Chest",                               RHT_JABU_JABUS_BELLY_COMPASS_CHEST,                              RG_JABU_JABUS_BELLY_COMPASS,                                                                                                                                                                      true);
    locationTable[RC_JABU_JABUS_BELLY_BOOMERANG_CHEST] =                              Location::Chest(RC_JABU_JABUS_BELLY_BOOMERANG_CHEST,                             RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    4289,                         0x01, "Boomerang Chest",                             RHT_JABU_JABUS_BELLY_BOOMERANG_CHEST,                            RG_BOOMERANG,                                                                                                                                                                                     true);
    locationTable[RC_JABU_JABUS_BELLY_DEKU_SCRUB] =                                    Location::Base(RC_JABU_JABUS_BELLY_DEKU_SCRUB,                                  RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_JABU_JABU,                    0x00,                               "Deku Scrub",                                  RHT_JABU_JABUS_BELLY_DEKU_SCRUB,                                 RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_JABU_JABUS_BELLY_DEKU_SCRUB),                                 false, 20);
    // Jabu-Jabu's Belly MQ
    locationTable[RC_JABU_JABUS_BELLY_MQ_FIRST_ROOM_SIDE_CHEST] =                     Location::Chest(RC_JABU_JABUS_BELLY_MQ_FIRST_ROOM_SIDE_CHEST,                    RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -32699,                       0x05, "MQ First Room Side Chest",                    RHT_JABU_JABUS_BELLY_MQ_FIRST_ROOM_SIDE_CHEST,                   RG_DEKU_NUTS_5);
    locationTable[RC_JABU_JABUS_BELLY_MQ_MAP_CHEST] =                                 Location::Chest(RC_JABU_JABUS_BELLY_MQ_MAP_CHEST,                                RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -18397,                       0x03, "MQ Map Chest",                                RHT_JABU_JABUS_BELLY_MQ_MAP_CHEST,                               RG_JABU_JABUS_BELLY_MAP,                                                                                                                                                                          true);
    locationTable[RC_JABU_JABUS_BELLY_MQ_SECOND_ROOM_LOWER_CHEST] =                   Location::Chest(RC_JABU_JABUS_BELLY_MQ_SECOND_ROOM_LOWER_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    20546,                        0x02, "MQ Second Room Lower Chest",                  RHT_JABU_JABUS_BELLY_MQ_SECOND_ROOM_LOWER_CHEST,                 RG_DEKU_NUTS_5);
    locationTable[RC_JABU_JABUS_BELLY_MQ_COMPASS_CHEST] =                             Location::Chest(RC_JABU_JABUS_BELLY_MQ_COMPASS_CHEST,                            RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -18432,                       0x00, "MQ Compass Chest",                            RHT_JABU_JABUS_BELLY_MQ_COMPASS_CHEST,                           RG_JABU_JABUS_BELLY_COMPASS,                                                                                                                                                                      true);
    locationTable[RC_JABU_JABUS_BELLY_MQ_SECOND_ROOM_UPPER_CHEST] =                   Location::Chest(RC_JABU_JABUS_BELLY_MQ_SECOND_ROOM_UPPER_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -30457,                       0x07, "MQ Second Room Upper Chest",                  RHT_JABU_JABUS_BELLY_MQ_SECOND_ROOM_UPPER_CHEST,                 RG_RECOVERY_HEART);
    locationTable[RC_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_SWITCHES_CHEST] =              Location::Chest(RC_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_SWITCHES_CHEST,             RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -32696,                       0x08, "MQ Basement Near Switches Chest",             RHT_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_SWITCHES_CHEST,            RG_DEKU_NUTS_5);
    locationTable[RC_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_VINES_CHEST] =                 Location::Chest(RC_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_VINES_CHEST,                RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -32668,                       0x04, "MQ Basement Near Vines Chest",                RHT_JABU_JABUS_BELLY_MQ_BASEMENT_NEAR_VINES_CHEST,               RG_BOMBCHU_10);
    locationTable[RC_JABU_JABUS_BELLY_MQ_NEAR_BOSS_CHEST] =                           Location::Chest(RC_JABU_JABUS_BELLY_MQ_NEAR_BOSS_CHEST,                          RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    -31446,                       0x0A, "MQ Near Boss Chest",                          RHT_JABU_JABUS_BELLY_MQ_NEAR_BOSS_CHEST,                         RG_DEKU_SHIELD);
    locationTable[RC_JABU_JABUS_BELLY_MQ_FALLING_LIKE_LIKE_ROOM_CHEST] =              Location::Chest(RC_JABU_JABUS_BELLY_MQ_FALLING_LIKE_LIKE_ROOM_CHEST,             RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    28905,                        0x09, "MQ Falling Like Like Room Chest",             RHT_JABU_JABUS_BELLY_MQ_FALLING_LIKE_LIKE_ROOM_CHEST,            RG_DEKU_STICK_1);
    locationTable[RC_JABU_JABUS_BELLY_MQ_BOOMERANG_ROOM_SMALL_CHEST] =                Location::Chest(RC_JABU_JABUS_BELLY_MQ_BOOMERANG_ROOM_SMALL_CHEST,               RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    20545,                        0x01, "MQ Boomerang Room Small Chest",               RHT_JABU_JABUS_BELLY_MQ_BOOMERANG_ROOM_SMALL_CHEST,              RG_DEKU_NUTS_5);
    locationTable[RC_JABU_JABUS_BELLY_MQ_BOOMERANG_CHEST] =                           Location::Chest(RC_JABU_JABUS_BELLY_MQ_BOOMERANG_CHEST,                          RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_JABU_JABU,                    4294,                         0x06, "MQ Boomerang Chest",                          RHT_JABU_JABUS_BELLY_MQ_BOOMERANG_CHEST,                         RG_BOOMERANG,                                                                                                                                                                                     true);

    // Forest Temple Vanilla
    locationTable[RC_FOREST_TEMPLE_FIRST_ROOM_CHEST] =                                Location::Chest(RC_FOREST_TEMPLE_FIRST_ROOM_CHEST,                               RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22595,                        0x03, "First Room Chest",                            RHT_FOREST_TEMPLE_FIRST_ROOM_CHEST,                              RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_FIRST_STALFOS_CHEST] =                             Location::Chest(RC_FOREST_TEMPLE_FIRST_STALFOS_CHEST,                            RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                30784,                        0x00, "First Stalfos Chest",                         RHT_FOREST_TEMPLE_FIRST_STALFOS_CHEST,                           RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_RAISED_ISLAND_COURTYARD_CHEST] =                   Location::Chest(RC_FOREST_TEMPLE_RAISED_ISLAND_COURTYARD_CHEST,                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22789,                        0x05, "Raised Island Courtyard Chest",               RHT_FOREST_TEMPLE_RAISED_ISLAND_COURTYARD_CHEST,                 RG_RECOVERY_HEART);
    locationTable[RC_FOREST_TEMPLE_MAP_CHEST] =                                       Location::Chest(RC_FOREST_TEMPLE_MAP_CHEST,                                      RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                6177,                         0x01, "Map Chest",                                   RHT_FOREST_TEMPLE_MAP_CHEST,                                     RG_FOREST_TEMPLE_MAP,                                                                                                                                                                             true);
    locationTable[RC_FOREST_TEMPLE_WELL_CHEST] =                                      Location::Chest(RC_FOREST_TEMPLE_WELL_CHEST,                                     RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22601,                        0x09, "Well Chest",                                  RHT_FOREST_TEMPLE_WELL_CHEST,                                    RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_FALLING_CEILING_ROOM_CHEST] =                      Location::Chest(RC_FOREST_TEMPLE_FALLING_CEILING_ROOM_CHEST,                     RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22855,                        0x07, "Falling Ceiling Room Chest",                  RHT_FOREST_TEMPLE_FALLING_CEILING_ROOM_CHEST,                    RG_ARROWS_10);
    locationTable[RC_FOREST_TEMPLE_EYE_SWITCH_CHEST] =                                Location::Chest(RC_FOREST_TEMPLE_EYE_SWITCH_CHEST,                               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                -30364,                       0x04, "Eye Switch Chest",                            RHT_FOREST_TEMPLE_EYE_SWITCH_CHEST,                              RG_ARROWS_30);
    locationTable[RC_FOREST_TEMPLE_BOSS_KEY_CHEST] =                                  Location::Chest(RC_FOREST_TEMPLE_BOSS_KEY_CHEST,                                 RCQUEST_VANILLA, RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                10222,                        0x0E, "Boss Key Chest",                              RHT_FOREST_TEMPLE_BOSS_KEY_CHEST,                                RG_FOREST_TEMPLE_BOSS_KEY,                                                                                                                                                                        true);
    locationTable[RC_FOREST_TEMPLE_FLOORMASTER_CHEST] =                               Location::Chest(RC_FOREST_TEMPLE_FLOORMASTER_CHEST,                              RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                30786,                        0x02, "Floormaster Chest",                           RHT_FOREST_TEMPLE_FLOORMASTER_CHEST,                             RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_BOW_CHEST] =                                       Location::Chest(RC_FOREST_TEMPLE_BOW_CHEST,                                      RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                -20340,                       0x0C, "Bow Chest",                                   RHT_FOREST_TEMPLE_BOW_CHEST,                                     RG_PROGRESSIVE_BOW,                                                                                                                                                                               true);
    locationTable[RC_FOREST_TEMPLE_RED_POE_CHEST] =                                   Location::Chest(RC_FOREST_TEMPLE_RED_POE_CHEST,                                  RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                30797,                        0x0D, "Red Poe Chest",                               RHT_FOREST_TEMPLE_RED_POE_CHEST,                                 RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_CHEST] =                                  Location::Chest(RC_FOREST_TEMPLE_BLUE_POE_CHEST,                                 RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                6159,                         0x0F, "Blue Poe Chest",                              RHT_FOREST_TEMPLE_BLUE_POE_CHEST,                                RG_FOREST_TEMPLE_COMPASS,                                                                                                                                                                         true);
    locationTable[RC_FOREST_TEMPLE_BASEMENT_CHEST] =                                  Location::Chest(RC_FOREST_TEMPLE_BASEMENT_CHEST,                                 RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22827,                        0x0B, "Basement Chest",                              RHT_FOREST_TEMPLE_BASEMENT_CHEST,                                RG_ARROWS_5);
    // Forest Temple MQ
    locationTable[RC_FOREST_TEMPLE_MQ_FIRST_ROOM_CHEST] =                             Location::Chest(RC_FOREST_TEMPLE_MQ_FIRST_ROOM_CHEST,                            RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                -30653,                       0x03, "MQ First Room Chest",                         RHT_FOREST_TEMPLE_MQ_FIRST_ROOM_CHEST,                           RG_FOREST_TEMPLE_SMALL_KEY);
    locationTable[RC_FOREST_TEMPLE_MQ_WOLFOS_CHEST] =                                 Location::Chest(RC_FOREST_TEMPLE_MQ_WOLFOS_CHEST,                                RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                30784,                        0x00, "MQ Wolfos Chest",                             RHT_FOREST_TEMPLE_MQ_WOLFOS_CHEST,                               RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_MQ_BOW_CHEST] =                                    Location::Chest(RC_FOREST_TEMPLE_MQ_BOW_CHEST,                                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                -20340,                       0x0C, "MQ Bow Chest",                                RHT_FOREST_TEMPLE_MQ_BOW_CHEST,                                  RG_PROGRESSIVE_BOW,                                                                                                                                                                               true);
    locationTable[RC_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_LOWER_CHEST] =          Location::Chest(RC_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_LOWER_CHEST,         RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22593,                        0x01, "MQ Raised Island Courtyard Lower Chest",      RHT_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_LOWER_CHEST,        RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_UPPER_CHEST] =          Location::Chest(RC_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_UPPER_CHEST,         RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22597,                        0x05, "MQ Raised Island Courtyard Upper Chest",      RHT_FOREST_TEMPLE_MQ_RAISED_ISLAND_COURTYARD_UPPER_CHEST,        RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_MQ_WELL_CHEST] =                                   Location::Chest(RC_FOREST_TEMPLE_MQ_WELL_CHEST,                                  RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22601,                        0x09, "MQ Well Chest",                               RHT_FOREST_TEMPLE_MQ_WELL_CHEST,                                 RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_MQ_MAP_CHEST] =                                    Location::Chest(RC_FOREST_TEMPLE_MQ_MAP_CHEST,                                   RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                6189,                         0x0D, "MQ Map Chest",                                RHT_FOREST_TEMPLE_MQ_MAP_CHEST,                                  RG_FOREST_TEMPLE_MAP,                                                                                                                                                                             true);
    locationTable[RC_FOREST_TEMPLE_MQ_COMPASS_CHEST] =                                Location::Chest(RC_FOREST_TEMPLE_MQ_COMPASS_CHEST,                               RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                6159,                         0x0F, "MQ Compass Chest",                            RHT_FOREST_TEMPLE_MQ_COMPASS_CHEST,                              RG_FOREST_TEMPLE_COMPASS,                                                                                                                                                                         true);
    locationTable[RC_FOREST_TEMPLE_MQ_FALLING_CEILING_ROOM_CHEST] =                   Location::Chest(RC_FOREST_TEMPLE_MQ_FALLING_CEILING_ROOM_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                -30426,                       0x06, "MQ Falling Ceiling Room Chest",               RHT_FOREST_TEMPLE_MQ_FALLING_CEILING_ROOM_CHEST,                 RG_ARROWS_5);
    locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_CHEST] =                               Location::Chest(RC_FOREST_TEMPLE_MQ_BASEMENT_CHEST,                              RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                22827,                        0x0B, "MQ Basement Chest",                           RHT_FOREST_TEMPLE_MQ_BASEMENT_CHEST,                             RG_ARROWS_5);
    locationTable[RC_FOREST_TEMPLE_MQ_REDEAD_CHEST] =                                 Location::Chest(RC_FOREST_TEMPLE_MQ_REDEAD_CHEST,                                RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                30786,                        0x02, "MQ Redead Chest",                             RHT_FOREST_TEMPLE_MQ_REDEAD_CHEST,                               RG_FOREST_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_FOREST_TEMPLE_MQ_BOSS_KEY_CHEST] =                               Location::Chest(RC_FOREST_TEMPLE_MQ_BOSS_KEY_CHEST,                              RCQUEST_MQ,      RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_FOREST_TEMPLE,                10222,                        0x0E, "MQ Boss Key Chest",                           RHT_FOREST_TEMPLE_MQ_BOSS_KEY_CHEST,                             RG_FOREST_TEMPLE_BOSS_KEY,                                                                                                                                                                        true);

    // Fire Temple Vanilla
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_CHEST] =                                   Location::Chest(RC_FIRE_TEMPLE_NEAR_BOSS_CHEST,                                  RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22593,                        0x01, "Near Boss Chest",                             RHT_FIRE_TEMPLE_NEAR_BOSS_CHEST,                                 RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_FLARE_DANCER_CHEST] =                                Location::Chest(RC_FIRE_TEMPLE_FLARE_DANCER_CHEST,                               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  31936,                        0x00, "Flare Dancer Chest",                          RHT_FIRE_TEMPLE_FLARE_DANCER_CHEST,                              RG_BOMBS_10);
    locationTable[RC_FIRE_TEMPLE_BOSS_KEY_CHEST] =                                    Location::Chest(RC_FIRE_TEMPLE_BOSS_KEY_CHEST,                                   RCQUEST_VANILLA, RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  10220,                        0x0C, "Boss Key Chest",                              RHT_FIRE_TEMPLE_BOSS_KEY_CHEST,                                  RG_FIRE_TEMPLE_BOSS_KEY,                                                                                                                                                                          true);
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST] =                  Location::Chest(RC_FIRE_TEMPLE_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,                 RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22594,                        0x02, "Big Lava Room Blocked Door Chest",            RHT_FIRE_TEMPLE_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,                RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_ROOM_LOWER_OPEN_DOOR_CHEST] =               Location::Chest(RC_FIRE_TEMPLE_BIG_LAVA_ROOM_LOWER_OPEN_DOOR_CHEST,              RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22596,                        0x04, "Big Lava Room Lower Open Door Chest",         RHT_FIRE_TEMPLE_BIG_LAVA_ROOM_LOWER_OPEN_DOOR_CHEST,             RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_BOULDER_MAZE_LOWER_CHEST] =                          Location::Chest(RC_FIRE_TEMPLE_BOULDER_MAZE_LOWER_CHEST,                         RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22595,                        0x03, "Boulder Maze Lower Chest",                    RHT_FIRE_TEMPLE_BOULDER_MAZE_LOWER_CHEST,                        RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_BOULDER_MAZE_UPPER_CHEST] =                          Location::Chest(RC_FIRE_TEMPLE_BOULDER_MAZE_UPPER_CHEST,                         RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22598,                        0x06, "Boulder Maze Upper Chest",                    RHT_FIRE_TEMPLE_BOULDER_MAZE_UPPER_CHEST,                        RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_BOULDER_MAZE_SIDE_ROOM_CHEST] =                      Location::Chest(RC_FIRE_TEMPLE_BOULDER_MAZE_SIDE_ROOM_CHEST,                     RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22600,                        0x08, "Boulder Maze Side Room Chest",                RHT_FIRE_TEMPLE_BOULDER_MAZE_SIDE_ROOM_CHEST,                    RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_BOULDER_MAZE_SHORTCUT_CHEST] =                       Location::Chest(RC_FIRE_TEMPLE_BOULDER_MAZE_SHORTCUT_CHEST,                      RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22603,                        0x0B, "Boulder Maze Shortcut Chest",                 RHT_FIRE_TEMPLE_BOULDER_MAZE_SHORTCUT_CHEST,                     RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_SCARECROW_CHEST] =                                   Location::Chest(RC_FIRE_TEMPLE_SCARECROW_CHEST,                                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  23245,                        0x0D, "Scarecrow Chest",                             RHT_FIRE_TEMPLE_SCARECROW_CHEST,                                 RG_HUGE_RUPEE);
    locationTable[RC_FIRE_TEMPLE_MAP_CHEST] =                                         Location::Chest(RC_FIRE_TEMPLE_MAP_CHEST,                                        RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  2090,                         0x0A, "Map Chest",                                   RHT_FIRE_TEMPLE_MAP_CHEST,                                       RG_FIRE_TEMPLE_MAP,                                                                                                                                                                               true);
    locationTable[RC_FIRE_TEMPLE_COMPASS_CHEST] =                                     Location::Chest(RC_FIRE_TEMPLE_COMPASS_CHEST,                                    RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  2055,                         0x07, "Compass Chest",                               RHT_FIRE_TEMPLE_COMPASS_CHEST,                                   RG_FIRE_TEMPLE_COMPASS,                                                                                                                                                                           true);
    locationTable[RC_FIRE_TEMPLE_HIGHEST_GORON_CHEST] =                               Location::Chest(RC_FIRE_TEMPLE_HIGHEST_GORON_CHEST,                              RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22601,                        0x09, "Highest Goron Chest",                         RHT_FIRE_TEMPLE_HIGHEST_GORON_CHEST,                             RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_MEGATON_HAMMER_CHEST] =                              Location::Chest(RC_FIRE_TEMPLE_MEGATON_HAMMER_CHEST,                             RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  421,                          0x05, "Megaton Hammer Chest",                        RHT_FIRE_TEMPLE_MEGATON_HAMMER_CHEST,                            RG_MEGATON_HAMMER,                                                                                                                                                                                true);
    // Fire Temple MQ
    locationTable[RC_FIRE_TEMPLE_MQ_NEAR_BOSS_CHEST] =                                Location::Chest(RC_FIRE_TEMPLE_MQ_NEAR_BOSS_CHEST,                               RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22599,                        0x07, "MQ Near Boss Chest",                          RHT_FIRE_TEMPLE_MQ_NEAR_BOSS_CHEST,                              RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_MQ_MEGATON_HAMMER_CHEST] =                           Location::Chest(RC_FIRE_TEMPLE_MQ_MEGATON_HAMMER_CHEST,                          RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  4512,                         0x00, "MQ Megaton Hammer Chest",                     RHT_FIRE_TEMPLE_MQ_MEGATON_HAMMER_CHEST,                         RG_MEGATON_HAMMER,                                                                                                                                                                                true);
    locationTable[RC_FIRE_TEMPLE_MQ_COMPASS_CHEST] =                                  Location::Chest(RC_FIRE_TEMPLE_MQ_COMPASS_CHEST,                                 RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  2059,                         0x0B, "MQ Compass Chest",                            RHT_FIRE_TEMPLE_MQ_COMPASS_CHEST,                                RG_FIRE_TEMPLE_COMPASS,                                                                                                                                                                           true);
    locationTable[RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CHEST] =                      Location::Chest(RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CHEST,                     RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  23747,                        0x03, "MQ Lizalfos Maze Lower Chest",                RHT_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_LOWER_CHEST,                    RG_BOMBS_10);
    locationTable[RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CHEST] =                      Location::Chest(RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CHEST,                     RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  23782,                        0x06, "MQ Lizalfos Maze Upper Chest",                RHT_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_UPPER_CHEST,                    RG_BOMBS_10);
    locationTable[RC_FIRE_TEMPLE_MQ_CHEST_ON_FIRE] =                                  Location::Chest(RC_FIRE_TEMPLE_MQ_CHEST_ON_FIRE,                                 RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22597,                        0x05, "MQ Chest on Fire",                            RHT_FIRE_TEMPLE_MQ_CHEST_ON_FIRE,                                RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_MQ_MAP_ROOM_SIDE_CHEST] =                            Location::Chest(RC_FIRE_TEMPLE_MQ_MAP_ROOM_SIDE_CHEST,                           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  30018,                        0x02, "MQ Map Room Side Chest",                      RHT_FIRE_TEMPLE_MQ_MAP_ROOM_SIDE_CHEST,                          RG_HYLIAN_SHIELD);
    locationTable[RC_FIRE_TEMPLE_MQ_MAP_CHEST] =                                      Location::Chest(RC_FIRE_TEMPLE_MQ_MAP_CHEST,                                     RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  2092,                         0x0C, "MQ Map Chest",                                RHT_FIRE_TEMPLE_MQ_MAP_CHEST,                                    RG_FIRE_TEMPLE_MAP,                                                                                                                                                                               true);
    locationTable[RC_FIRE_TEMPLE_MQ_BOSS_KEY_CHEST] =                                 Location::Chest(RC_FIRE_TEMPLE_MQ_BOSS_KEY_CHEST,                                RCQUEST_MQ,      RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  10212,                        0x04, "MQ Boss Key Chest",                           RHT_FIRE_TEMPLE_MQ_BOSS_KEY_CHEST,                               RG_FIRE_TEMPLE_BOSS_KEY,                                                                                                                                                                          true);
    locationTable[RC_FIRE_TEMPLE_MQ_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST] =               Location::Chest(RC_FIRE_TEMPLE_MQ_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,              RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22593,                        0x01, "MQ Big Lava Room Blocked Door Chest",         RHT_FIRE_TEMPLE_MQ_BIG_LAVA_ROOM_BLOCKED_DOOR_CHEST,             RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_SIDE_ROOM_CHEST] =                  Location::Chest(RC_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_SIDE_ROOM_CHEST,                 RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_FIRE_TEMPLE,                  22600,                        0x08, "MQ Lizalfos Maze Side Room Chest",            RHT_FIRE_TEMPLE_MQ_LIZALFOS_MAZE_SIDE_ROOM_CHEST,                RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);
    locationTable[RC_FIRE_TEMPLE_MQ_FREESTANDING_KEY] =                         Location::Collectable(RC_FIRE_TEMPLE_MQ_FREESTANDING_KEY,                              RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_FIRE_TEMPLE,                  7185,                         0x1C, "MQ Freestanding Key",                         RHT_FIRE_TEMPLE_MQ_FREESTANDING_KEY,                             RG_FIRE_TEMPLE_SMALL_KEY,                                                                                                                                                                         true);

    // Water Temple Vanilla
    locationTable[RC_WATER_TEMPLE_MAP_CHEST] =                                        Location::Chest(RC_WATER_TEMPLE_MAP_CHEST,                                       RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 6178,                         0x02, "Map Chest",                                   RHT_WATER_TEMPLE_MAP_CHEST,                                      RG_WATER_TEMPLE_MAP,                                                                                                                                                                              true);
    locationTable[RC_WATER_TEMPLE_COMPASS_CHEST] =                                    Location::Chest(RC_WATER_TEMPLE_COMPASS_CHEST,                                   RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 2057,                         0x09, "Compass Chest",                               RHT_WATER_TEMPLE_COMPASS_CHEST,                                  RG_WATER_TEMPLE_COMPASS,                                                                                                                                                                          true);
    locationTable[RC_WATER_TEMPLE_TORCHES_CHEST] =                                    Location::Chest(RC_WATER_TEMPLE_TORCHES_CHEST,                                   RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 30785,                        0x01, "Torches Chest",                               RHT_WATER_TEMPLE_TORCHES_CHEST,                                  RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    locationTable[RC_WATER_TEMPLE_DRAGON_CHEST] =                                     Location::Chest(RC_WATER_TEMPLE_DRAGON_CHEST,                                    RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 22602,                        0x0A, "Dragon Chest",                                RHT_WATER_TEMPLE_DRAGON_CHEST,                                   RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    locationTable[RC_WATER_TEMPLE_CENTRAL_BOW_TARGET_CHEST] =                         Location::Chest(RC_WATER_TEMPLE_CENTRAL_BOW_TARGET_CHEST,                        RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 22600,                        0x08, "Central Bow Target Chest",                    RHT_WATER_TEMPLE_CENTRAL_BOW_TARGET_CHEST,                       RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    locationTable[RC_WATER_TEMPLE_CENTRAL_PILLAR_CHEST] =                             Location::Chest(RC_WATER_TEMPLE_CENTRAL_PILLAR_CHEST,                            RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 22598,                        0x06, "Central Pillar Chest",                        RHT_WATER_TEMPLE_CENTRAL_PILLAR_CHEST,                           RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    locationTable[RC_WATER_TEMPLE_CRACKED_WALL_CHEST] =                               Location::Chest(RC_WATER_TEMPLE_CRACKED_WALL_CHEST,                              RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 22592,                        0x00, "Cracked Wall Chest",                          RHT_WATER_TEMPLE_CRACKED_WALL_CHEST,                             RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    locationTable[RC_WATER_TEMPLE_BOSS_KEY_CHEST] =                                   Location::Chest(RC_WATER_TEMPLE_BOSS_KEY_CHEST,                                  RCQUEST_VANILLA, RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 10213,                        0x05, "Boss Key Chest",                              RHT_WATER_TEMPLE_BOSS_KEY_CHEST,                                 RG_WATER_TEMPLE_BOSS_KEY,                                                                                                                                                                         true);
    locationTable[RC_WATER_TEMPLE_LONGSHOT_CHEST] =                                   Location::Chest(RC_WATER_TEMPLE_LONGSHOT_CHEST,                                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 295,                          0x07, "Longshot Chest",                              RHT_WATER_TEMPLE_LONGSHOT_CHEST,                                 RG_PROGRESSIVE_HOOKSHOT,                                                                                                                                                                          true);
    locationTable[RC_WATER_TEMPLE_RIVER_CHEST] =                                      Location::Chest(RC_WATER_TEMPLE_RIVER_CHEST,                                     RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 22595,                        0x03, "River Chest",                                 RHT_WATER_TEMPLE_RIVER_CHEST,                                    RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    // Water Temple MQ
    locationTable[RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_CHEST] =                          Location::Chest(RC_WATER_TEMPLE_MQ_CENTRAL_PILLAR_CHEST,                         RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 -30650,                       0x06, "MQ Central Pillar Chest",                     RHT_WATER_TEMPLE_MQ_CENTRAL_PILLAR_CHEST,                        RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);
    locationTable[RC_WATER_TEMPLE_MQ_BOSS_KEY_CHEST] =                                Location::Chest(RC_WATER_TEMPLE_MQ_BOSS_KEY_CHEST,                               RCQUEST_MQ,      RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 10213,                        0x05, "MQ Boss Key Chest",                           RHT_WATER_TEMPLE_MQ_BOSS_KEY_CHEST,                              RG_WATER_TEMPLE_BOSS_KEY,                                                                                                                                                                         true);
    locationTable[RC_WATER_TEMPLE_MQ_LONGSHOT_CHEST] =                                Location::Chest(RC_WATER_TEMPLE_MQ_LONGSHOT_CHEST,                               RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 -20192,                       0x00, "MQ Longshot Chest",                           RHT_WATER_TEMPLE_MQ_LONGSHOT_CHEST,                              RG_PROGRESSIVE_HOOKSHOT,                                                                                                                                                                          true);
    locationTable[RC_WATER_TEMPLE_MQ_COMPASS_CHEST] =                                 Location::Chest(RC_WATER_TEMPLE_MQ_COMPASS_CHEST,                                RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 6145,                         0x01, "MQ Compass Chest",                            RHT_WATER_TEMPLE_MQ_COMPASS_CHEST,                               RG_WATER_TEMPLE_COMPASS,                                                                                                                                                                          true);
    locationTable[RC_WATER_TEMPLE_MQ_MAP_CHEST] =                                     Location::Chest(RC_WATER_TEMPLE_MQ_MAP_CHEST,                                    RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_WATER_TEMPLE,                 -18398,                       0x02, "MQ Map Chest",                                RHT_WATER_TEMPLE_MQ_MAP_CHEST,                                   RG_WATER_TEMPLE_MAP,                                                                                                                                                                              true);
    locationTable[RC_WATER_TEMPLE_MQ_FREESTANDING_KEY] =                        Location::Collectable(RC_WATER_TEMPLE_MQ_FREESTANDING_KEY,                             RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_WATER_TEMPLE,                 273,                          0x01, "MQ Freestanding Key",                         RHT_WATER_TEMPLE_MQ_FREESTANDING_KEY,                            RG_WATER_TEMPLE_SMALL_KEY,                                                                                                                                                                        true);

    // Spirit Temple Shared
    locationTable[RC_SPIRIT_TEMPLE_SILVER_GAUNTLETS_CHEST] =                          Location::Chest(RC_SPIRIT_TEMPLE_SILVER_GAUNTLETS_CHEST,                         RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_SPIRIT_TEMPLE,                ACTOR_EN_BOX,         SCENE_DESERT_COLOSSUS,              1707,                         0x0B, "Silver Gauntlets Chest",                      RHT_SPIRIT_TEMPLE_SILVER_GAUNTLETS_CHEST,                        RG_PROGRESSIVE_STRENGTH,                                                                                                                                                                          true);
    locationTable[RC_SPIRIT_TEMPLE_MIRROR_SHIELD_CHEST] =                             Location::Chest(RC_SPIRIT_TEMPLE_MIRROR_SHIELD_CHEST,                            RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_SPIRIT_TEMPLE,                ACTOR_EN_BOX,         SCENE_DESERT_COLOSSUS,              13673,                        0x09, "Mirror Shield Chest",                         RHT_SPIRIT_TEMPLE_MIRROR_SHIELD_CHEST,                           RG_MIRROR_SHIELD,                                                                                                                                                                                 true);
    // Spirit Temple Vanilla
    locationTable[RC_SPIRIT_TEMPLE_CHILD_BRIDGE_CHEST] =                              Location::Chest(RC_SPIRIT_TEMPLE_CHILD_BRIDGE_CHEST,                             RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                21800,                        0x08, "Child Bridge Chest",                          RHT_SPIRIT_TEMPLE_CHILD_BRIDGE_CHEST,                            RG_DEKU_SHIELD);
    locationTable[RC_SPIRIT_TEMPLE_CHILD_EARLY_TORCHES_CHEST] =                       Location::Chest(RC_SPIRIT_TEMPLE_CHILD_EARLY_TORCHES_CHEST,                      RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30656,                       0x00, "Child Early Torches Chest",                   RHT_SPIRIT_TEMPLE_CHILD_EARLY_TORCHES_CHEST,                     RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_COMPASS_CHEST] =                                   Location::Chest(RC_SPIRIT_TEMPLE_COMPASS_CHEST,                                  RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                14340,                        0x04, "Compass Chest",                               RHT_SPIRIT_TEMPLE_COMPASS_CHEST,                                 RG_SPIRIT_TEMPLE_COMPASS,                                                                                                                                                                         true);
    locationTable[RC_SPIRIT_TEMPLE_EARLY_ADULT_RIGHT_CHEST] =                         Location::Chest(RC_SPIRIT_TEMPLE_EARLY_ADULT_RIGHT_CHEST,                        RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                22599,                        0x07, "Early Adult Right Chest",                     RHT_SPIRIT_TEMPLE_EARLY_ADULT_RIGHT_CHEST,                       RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_FIRST_MIRROR_LEFT_CHEST] =                         Location::Chest(RC_SPIRIT_TEMPLE_FIRST_MIRROR_LEFT_CHEST,                        RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30451,                       0x0D, "First Mirror Left Chest",                     RHT_SPIRIT_TEMPLE_FIRST_MIRROR_LEFT_CHEST,                       RG_ICE_TRAP);
    locationTable[RC_SPIRIT_TEMPLE_FIRST_MIRROR_RIGHT_CHEST] =                        Location::Chest(RC_SPIRIT_TEMPLE_FIRST_MIRROR_RIGHT_CHEST,                       RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -28786,                       0x0E, "First Mirror Right Chest",                    RHT_SPIRIT_TEMPLE_FIRST_MIRROR_RIGHT_CHEST,                      RG_RECOVERY_HEART);
    locationTable[RC_SPIRIT_TEMPLE_MAP_CHEST] =                                       Location::Chest(RC_SPIRIT_TEMPLE_MAP_CHEST,                                      RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -18397,                       0x03, "Map Chest",                                   RHT_SPIRIT_TEMPLE_MAP_CHEST,                                     RG_SPIRIT_TEMPLE_MAP,                                                                                                                                                                             true);
    locationTable[RC_SPIRIT_TEMPLE_CHILD_CLIMB_NORTH_CHEST] =                         Location::Chest(RC_SPIRIT_TEMPLE_CHILD_CLIMB_NORTH_CHEST,                        RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -32666,                       0x06, "Child Climb North Chest",                     RHT_SPIRIT_TEMPLE_CHILD_CLIMB_NORTH_CHEST,                       RG_BOMBCHU_10);
    locationTable[RC_SPIRIT_TEMPLE_CHILD_CLIMB_EAST_CHEST] =                          Location::Chest(RC_SPIRIT_TEMPLE_CHILD_CLIMB_EAST_CHEST,                         RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -31444,                       0x0C, "Child Climb East Chest",                      RHT_SPIRIT_TEMPLE_CHILD_CLIMB_EAST_CHEST,                        RG_DEKU_SHIELD);
    locationTable[RC_SPIRIT_TEMPLE_SUN_BLOCK_ROOM_CHEST] =                            Location::Chest(RC_SPIRIT_TEMPLE_SUN_BLOCK_ROOM_CHEST,                           RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30655,                       0x01, "Sun Block Room Chest",                        RHT_SPIRIT_TEMPLE_SUN_BLOCK_ROOM_CHEST,                          RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_STATUE_ROOM_HAND_CHEST] =                          Location::Chest(RC_SPIRIT_TEMPLE_STATUE_ROOM_HAND_CHEST,                         RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30654,                       0x02, "Statue Room Hand Chest",                      RHT_SPIRIT_TEMPLE_STATUE_ROOM_HAND_CHEST,                        RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_STATUE_ROOM_NORTHEAST_CHEST] =                     Location::Chest(RC_SPIRIT_TEMPLE_STATUE_ROOM_NORTHEAST_CHEST,                    RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30577,                       0x0F, "Statue Room Northeast Chest",                 RHT_SPIRIT_TEMPLE_STATUE_ROOM_NORTHEAST_CHEST,                   RG_BLUE_RUPEE);
    locationTable[RC_SPIRIT_TEMPLE_NEAR_FOUR_ARMOS_CHEST] =                           Location::Chest(RC_SPIRIT_TEMPLE_NEAR_FOUR_ARMOS_CHEST,                          RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                22597,                        0x05, "Near Four Armos Chest",                       RHT_SPIRIT_TEMPLE_NEAR_FOUR_ARMOS_CHEST,                         RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_HALLWAY_RIGHT_INVISIBLE_CHEST] =                   Location::Chest(RC_SPIRIT_TEMPLE_HALLWAY_RIGHT_INVISIBLE_CHEST,                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                26900,                        0x14, "Hallway Right Invisible Chest",               RHT_SPIRIT_TEMPLE_HALLWAY_RIGHT_INVISIBLE_CHEST,                 RG_RECOVERY_HEART);
    locationTable[RC_SPIRIT_TEMPLE_HALLWAY_LEFT_INVISIBLE_CHEST] =                    Location::Chest(RC_SPIRIT_TEMPLE_HALLWAY_LEFT_INVISIBLE_CHEST,                   RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                26901,                        0x15, "Hallway Left Invisible Chest",                RHT_SPIRIT_TEMPLE_HALLWAY_LEFT_INVISIBLE_CHEST,                  RG_RECOVERY_HEART);
    locationTable[RC_SPIRIT_TEMPLE_BOSS_KEY_CHEST] =                                  Location::Chest(RC_SPIRIT_TEMPLE_BOSS_KEY_CHEST,                                 RCQUEST_VANILLA, RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                10218,                        0x0A, "Boss Key Chest",                              RHT_SPIRIT_TEMPLE_BOSS_KEY_CHEST,                                RG_SPIRIT_TEMPLE_BOSS_KEY,                                                                                                                                                                        true);
    locationTable[RC_SPIRIT_TEMPLE_TOPMOST_CHEST] =                                   Location::Chest(RC_SPIRIT_TEMPLE_TOPMOST_CHEST,                                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -29454,                       0x12, "Topmost Chest",                               RHT_SPIRIT_TEMPLE_TOPMOST_CHEST,                                 RG_BOMBS_20,                                                                                                                                                                                      true);
    // Spirit Temple MQ
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_LEFT_CHEST] =                    Location::Chest(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_LEFT_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                20602,                        0x1A, "MQ Entrance Front Left Chest",                RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_LEFT_CHEST,                  RG_BOMBCHU_10);
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_RIGHT_CHEST] =                    Location::Chest(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_RIGHT_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -32641,                       0x1F, "MQ Entrance Back Right Chest",                RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_RIGHT_CHEST,                  RG_BOMBCHU_10);
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_RIGHT_CHEST] =                   Location::Chest(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_RIGHT_CHEST,                  RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30629,                       0x1B, "MQ Entrance Front Right Chest",               RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_FRONT_RIGHT_CHEST,                 RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_LEFT_CHEST] =                     Location::Chest(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_LEFT_CHEST,                    RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30626,                       0x1E, "MQ Entrance Back Left Chest",                 RHT_SPIRIT_TEMPLE_MQ_ENTRANCE_BACK_LEFT_CHEST,                   RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_HAMMER_SWITCH_CHEST] =                    Location::Chest(RC_SPIRIT_TEMPLE_MQ_CHILD_HAMMER_SWITCH_CHEST,                   RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30627,                       0x1D, "MQ Child Hammer Switch Chest",                RHT_SPIRIT_TEMPLE_MQ_CHILD_HAMMER_SWITCH_CHEST,                  RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_MAP_CHEST] =                                    Location::Chest(RC_SPIRIT_TEMPLE_MQ_MAP_CHEST,                                   RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                2080,                         0x00, "MQ Map Chest",                                RHT_SPIRIT_TEMPLE_MQ_MAP_CHEST,                                  RG_SPIRIT_TEMPLE_MAP,                                                                                                                                                                             true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_MAP_ROOM_ENEMY_CHEST] =                         Location::Chest(RC_SPIRIT_TEMPLE_MQ_MAP_ROOM_ENEMY_CHEST,                        RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                30792,                        0x08, "MQ Map Room Enemy Chest",                     RHT_SPIRIT_TEMPLE_MQ_MAP_ROOM_ENEMY_CHEST,                       RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_NORTH_CHEST] =                      Location::Chest(RC_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_NORTH_CHEST,                     RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                28774,                        0x06, "MQ Child Climb North Chest",                  RHT_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_NORTH_CHEST,                    RG_BOMBCHU_10);
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_SOUTH_CHEST] =                      Location::Chest(RC_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_SOUTH_CHEST,                     RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30644,                       0x0C, "MQ Child Climb South Chest",                  RHT_SPIRIT_TEMPLE_MQ_CHILD_CLIMB_SOUTH_CHEST,                    RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_COMPASS_CHEST] =                                Location::Chest(RC_SPIRIT_TEMPLE_MQ_COMPASS_CHEST,                               RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -18429,                       0x03, "MQ Compass Chest",                            RHT_SPIRIT_TEMPLE_MQ_COMPASS_CHEST,                              RG_SPIRIT_TEMPLE_COMPASS,                                                                                                                                                                         true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_ROOM_LULLABY_CHEST] =                    Location::Chest(RC_SPIRIT_TEMPLE_MQ_STATUE_ROOM_LULLABY_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30577,                       0x0F, "MQ Statue Room Lullaby Chest",                RHT_SPIRIT_TEMPLE_MQ_STATUE_ROOM_LULLABY_CHEST,                  RG_BLUE_RUPEE);
    locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_ROOM_INVISIBLE_CHEST] =                  Location::Chest(RC_SPIRIT_TEMPLE_MQ_STATUE_ROOM_INVISIBLE_CHEST,                 RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                26882,                        0x02, "MQ Statue Room Invisible Chest",              RHT_SPIRIT_TEMPLE_MQ_STATUE_ROOM_INVISIBLE_CHEST,                RG_RECOVERY_HEART);
    locationTable[RC_SPIRIT_TEMPLE_MQ_SILVER_BLOCK_HALLWAY_CHEST] =                   Location::Chest(RC_SPIRIT_TEMPLE_MQ_SILVER_BLOCK_HALLWAY_CHEST,                  RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30628,                       0x1C, "MQ Silver Block Hallway Chest",               RHT_SPIRIT_TEMPLE_MQ_SILVER_BLOCK_HALLWAY_CHEST,                 RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_SUN_BLOCK_ROOM_CHEST] =                         Location::Chest(RC_SPIRIT_TEMPLE_MQ_SUN_BLOCK_ROOM_CHEST,                        RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                -30463,                       0x01, "MQ Sun Block Room Chest",                     RHT_SPIRIT_TEMPLE_MQ_SUN_BLOCK_ROOM_CHEST,                       RG_RECOVERY_HEART);
    locationTable[RC_SPIRIT_TEMPLE_MQ_SYMPHONY_ROOM_CHEST] =                          Location::Chest(RC_SPIRIT_TEMPLE_MQ_SYMPHONY_ROOM_CHEST,                         RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                23207,                        0x07, "MQ Symphony Room Chest",                      RHT_SPIRIT_TEMPLE_MQ_SYMPHONY_ROOM_CHEST,                        RG_PURPLE_RUPEE);
    locationTable[RC_SPIRIT_TEMPLE_MQ_LEEVER_ROOM_CHEST] =                            Location::Chest(RC_SPIRIT_TEMPLE_MQ_LEEVER_ROOM_CHEST,                           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                31396,                        0x04, "MQ Leever Room Chest",                        RHT_SPIRIT_TEMPLE_MQ_LEEVER_ROOM_CHEST,                          RG_PURPLE_RUPEE);
    locationTable[RC_SPIRIT_TEMPLE_MQ_BEAMOS_ROOM_CHEST] =                            Location::Chest(RC_SPIRIT_TEMPLE_MQ_BEAMOS_ROOM_CHEST,                           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                24472,                        0x19, "MQ Beamos Room Chest",                        RHT_SPIRIT_TEMPLE_MQ_BEAMOS_ROOM_CHEST,                          RG_RECOVERY_HEART);
    locationTable[RC_SPIRIT_TEMPLE_MQ_CHEST_SWITCH_CHEST] =                           Location::Chest(RC_SPIRIT_TEMPLE_MQ_CHEST_SWITCH_CHEST,                          RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                31097,                        0x18, "MQ Chest Switch Chest",                       RHT_SPIRIT_TEMPLE_MQ_CHEST_SWITCH_CHEST,                         RG_ICE_TRAP);
    locationTable[RC_SPIRIT_TEMPLE_MQ_BOSS_KEY_CHEST] =                               Location::Chest(RC_SPIRIT_TEMPLE_MQ_BOSS_KEY_CHEST,                              RCQUEST_MQ,      RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                10213,                        0x05, "MQ Boss Key Chest",                           RHT_SPIRIT_TEMPLE_MQ_BOSS_KEY_CHEST,                             RG_SPIRIT_TEMPLE_BOSS_KEY,                                                                                                                                                                        true);
    locationTable[RC_SPIRIT_TEMPLE_MQ_MIRROR_PUZZLE_INVISIBLE_CHEST] =                Location::Chest(RC_SPIRIT_TEMPLE_MQ_MIRROR_PUZZLE_INVISIBLE_CHEST,               RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SPIRIT_TEMPLE,                26706,                        0x12, "MQ Mirror Puzzle Invisible Chest",            RHT_SPIRIT_TEMPLE_MQ_MIRROR_PUZZLE_INVISIBLE_CHEST,              RG_SPIRIT_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);

    // Shadow Temple Vanilla
    locationTable[RC_SHADOW_TEMPLE_MAP_CHEST] =                                       Location::Chest(RC_SHADOW_TEMPLE_MAP_CHEST,                                      RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                6177,                         0x01, "Map Chest",                                   RHT_SHADOW_TEMPLE_MAP_CHEST,                                     RG_SHADOW_TEMPLE_MAP,                                                                                                                                                                             true);
    locationTable[RC_SHADOW_TEMPLE_HOVER_BOOTS_CHEST] =                               Location::Chest(RC_SHADOW_TEMPLE_HOVER_BOOTS_CHEST,                              RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                5607,                         0x07, "Hover Boots Chest",                           RHT_SHADOW_TEMPLE_HOVER_BOOTS_CHEST,                             RG_HOVER_BOOTS,                                                                                                                                                                                   true);
    locationTable[RC_SHADOW_TEMPLE_COMPASS_CHEST] =                                   Location::Chest(RC_SHADOW_TEMPLE_COMPASS_CHEST,                                  RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                6147,                         0x03, "Compass Chest",                               RHT_SHADOW_TEMPLE_COMPASS_CHEST,                                 RG_SHADOW_TEMPLE_COMPASS,                                                                                                                                                                         true);
    locationTable[RC_SHADOW_TEMPLE_EARLY_SILVER_RUPEE_CHEST] =                        Location::Chest(RC_SHADOW_TEMPLE_EARLY_SILVER_RUPEE_CHEST,                       RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22594,                        0x02, "Early Silver Rupee Chest",                    RHT_SHADOW_TEMPLE_EARLY_SILVER_RUPEE_CHEST,                      RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_INVISIBLE_BLADES_VISIBLE_CHEST] =                  Location::Chest(RC_SHADOW_TEMPLE_INVISIBLE_BLADES_VISIBLE_CHEST,                 RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22668,                        0x0C, "Invisible Blades Visible Chest",              RHT_SHADOW_TEMPLE_INVISIBLE_BLADES_VISIBLE_CHEST,                RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_INVISIBLE_BLADES_INVISIBLE_CHEST] =                Location::Chest(RC_SHADOW_TEMPLE_INVISIBLE_BLADES_INVISIBLE_CHEST,               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26998,                        0x16, "Invisible Blades Invisible Chest",            RHT_SHADOW_TEMPLE_INVISIBLE_BLADES_INVISIBLE_CHEST,              RG_ARROWS_30);
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_LOWER_CHEST] =                      Location::Chest(RC_SHADOW_TEMPLE_FALLING_SPIKES_LOWER_CHEST,                     RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22853,                        0x05, "Falling Spikes Lower Chest",                  RHT_SHADOW_TEMPLE_FALLING_SPIKES_LOWER_CHEST,                    RG_ARROWS_10);
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_UPPER_CHEST] =                      Location::Chest(RC_SHADOW_TEMPLE_FALLING_SPIKES_UPPER_CHEST,                     RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22662,                        0x06, "Falling Spikes Upper Chest",                  RHT_SHADOW_TEMPLE_FALLING_SPIKES_UPPER_CHEST,                    RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_SWITCH_CHEST] =                     Location::Chest(RC_SHADOW_TEMPLE_FALLING_SPIKES_SWITCH_CHEST,                    RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                -30652,                       0x04, "Falling Spikes Switch Chest",                 RHT_SHADOW_TEMPLE_FALLING_SPIKES_SWITCH_CHEST,                   RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_INVISIBLE_SPIKES_CHEST] =                          Location::Chest(RC_SHADOW_TEMPLE_INVISIBLE_SPIKES_CHEST,                         RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                30857,                        0x09, "Invisible Spikes Chest",                      RHT_SHADOW_TEMPLE_INVISIBLE_SPIKES_CHEST,                        RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_WIND_HINT_CHEST] =                                 Location::Chest(RC_SHADOW_TEMPLE_WIND_HINT_CHEST,                                RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26965,                        0x15, "Wind Hint Chest",                             RHT_SHADOW_TEMPLE_WIND_HINT_CHEST,                               RG_ARROWS_10);
    locationTable[RC_SHADOW_TEMPLE_AFTER_WIND_ENEMY_CHEST] =                          Location::Chest(RC_SHADOW_TEMPLE_AFTER_WIND_ENEMY_CHEST,                         RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                30856,                        0x08, "After Wind Enemy Chest",                      RHT_SHADOW_TEMPLE_AFTER_WIND_ENEMY_CHEST,                        RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_AFTER_WIND_HIDDEN_CHEST] =                         Location::Chest(RC_SHADOW_TEMPLE_AFTER_WIND_HIDDEN_CHEST,                        RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26708,                        0x14, "After Wind Hidden Chest",                     RHT_SHADOW_TEMPLE_AFTER_WIND_HIDDEN_CHEST,                       RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_SPIKE_WALLS_LEFT_CHEST] =                          Location::Chest(RC_SHADOW_TEMPLE_SPIKE_WALLS_LEFT_CHEST,                         RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22666,                        0x0A, "Spike Walls Left Chest",                      RHT_SHADOW_TEMPLE_SPIKE_WALLS_LEFT_CHEST,                        RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_BOSS_KEY_CHEST] =                                  Location::Chest(RC_SHADOW_TEMPLE_BOSS_KEY_CHEST,                                 RCQUEST_VANILLA, RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                10219,                        0x0B, "Boss Key Chest",                              RHT_SHADOW_TEMPLE_BOSS_KEY_CHEST,                                RG_SHADOW_TEMPLE_BOSS_KEY,                                                                                                                                                                        true);
    locationTable[RC_SHADOW_TEMPLE_INVISIBLE_FLOORMASTER_CHEST] =                     Location::Chest(RC_SHADOW_TEMPLE_INVISIBLE_FLOORMASTER_CHEST,                    RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                30797,                        0x0D, "Invisible Floormaster Chest",                 RHT_SHADOW_TEMPLE_INVISIBLE_FLOORMASTER_CHEST,                   RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_FREESTANDING_KEY] =                          Location::Collectable(RC_SHADOW_TEMPLE_FREESTANDING_KEY,                               RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_SHADOW_TEMPLE,                273,                          0x01, "Freestanding Key",                            RHT_SHADOW_TEMPLE_FREESTANDING_KEY,                              RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    // Shadow Temple MQ
    locationTable[RC_SHADOW_TEMPLE_MQ_COMPASS_CHEST] =                                Location::Chest(RC_SHADOW_TEMPLE_MQ_COMPASS_CHEST,                               RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                6145,                         0x01, "MQ Compass Chest",                            RHT_SHADOW_TEMPLE_MQ_COMPASS_CHEST,                              RG_SHADOW_TEMPLE_COMPASS,                                                                                                                                                                         true);
    locationTable[RC_SHADOW_TEMPLE_MQ_HOVER_BOOTS_CHEST] =                            Location::Chest(RC_SHADOW_TEMPLE_MQ_HOVER_BOOTS_CHEST,                           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                5607,                         0x07, "MQ Hover Boots Chest",                        RHT_SHADOW_TEMPLE_MQ_HOVER_BOOTS_CHEST,                          RG_HOVER_BOOTS,                                                                                                                                                                                   true);
    locationTable[RC_SHADOW_TEMPLE_MQ_EARLY_GIBDOS_CHEST] =                           Location::Chest(RC_SHADOW_TEMPLE_MQ_EARLY_GIBDOS_CHEST,                          RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                30787,                        0x03, "MQ Early Gibdos Chest",                       RHT_SHADOW_TEMPLE_MQ_EARLY_GIBDOS_CHEST,                         RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_MQ_MAP_CHEST] =                                    Location::Chest(RC_SHADOW_TEMPLE_MQ_MAP_CHEST,                                   RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                2082,                         0x02, "MQ Map Chest",                                RHT_SHADOW_TEMPLE_MQ_MAP_CHEST,                                  RG_SHADOW_TEMPLE_MAP,                                                                                                                                                                             true);
    locationTable[RC_SHADOW_TEMPLE_MQ_BEAMOS_SILVER_RUPEES_CHEST] =                   Location::Chest(RC_SHADOW_TEMPLE_MQ_BEAMOS_SILVER_RUPEES_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                -30417,                       0x0F, "MQ Beamos Silver Rupees Chest",               RHT_SHADOW_TEMPLE_MQ_BEAMOS_SILVER_RUPEES_CHEST,                 RG_ARROWS_5);
    locationTable[RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_SWITCH_CHEST] =                  Location::Chest(RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_SWITCH_CHEST,                 RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22662,                        0x04, "MQ Falling Spikes Switch Chest",              RHT_SHADOW_TEMPLE_MQ_FALLING_SPIKES_SWITCH_CHEST,                RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_LOWER_CHEST] =                   Location::Chest(RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_LOWER_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22853,                        0x05, "MQ Falling Spikes Lower Chest",               RHT_SHADOW_TEMPLE_MQ_FALLING_SPIKES_LOWER_CHEST,                 RG_ARROWS_10);
    locationTable[RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_UPPER_CHEST] =                   Location::Chest(RC_SHADOW_TEMPLE_MQ_FALLING_SPIKES_UPPER_CHEST,                  RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                -30652,                       0x06, "MQ Falling Spikes Upper Chest",               RHT_SHADOW_TEMPLE_MQ_FALLING_SPIKES_UPPER_CHEST,                 RG_ARROWS_5);
    locationTable[RC_SHADOW_TEMPLE_MQ_INVISIBLE_SPIKES_CHEST] =                       Location::Chest(RC_SHADOW_TEMPLE_MQ_INVISIBLE_SPIKES_CHEST,                      RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                30857,                        0x09, "MQ Invisible Spikes Chest",                   RHT_SHADOW_TEMPLE_MQ_INVISIBLE_SPIKES_CHEST,                     RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_MQ_BOSS_KEY_CHEST] =                               Location::Chest(RC_SHADOW_TEMPLE_MQ_BOSS_KEY_CHEST,                              RCQUEST_MQ,      RCTYPE_BOSS_KEY,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                10219,                        0x0B, "MQ Boss Key Chest",                           RHT_SHADOW_TEMPLE_MQ_BOSS_KEY_CHEST,                             RG_SHADOW_TEMPLE_BOSS_KEY,                                                                                                                                                                        true);
    locationTable[RC_SHADOW_TEMPLE_MQ_SPIKE_WALLS_LEFT_CHEST] =                       Location::Chest(RC_SHADOW_TEMPLE_MQ_SPIKE_WALLS_LEFT_CHEST,                      RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22666,                        0x0A, "MQ Spike Walls Left Chest",                   RHT_SHADOW_TEMPLE_MQ_SPIKE_WALLS_LEFT_CHEST,                     RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_MQ_STALFOS_ROOM_CHEST] =                           Location::Chest(RC_SHADOW_TEMPLE_MQ_STALFOS_ROOM_CHEST,                          RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                31184,                        0x10, "MQ Stalfos Room Chest",                       RHT_SHADOW_TEMPLE_MQ_STALFOS_ROOM_CHEST,                         RG_RED_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_INVISIBLE_CHEST] =             Location::Chest(RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_INVISIBLE_CHEST,            RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26710,                        0x16, "MQ Invisible Blades Invisible Chest",         RHT_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_INVISIBLE_CHEST,           RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_VISIBLE_CHEST] =               Location::Chest(RC_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_VISIBLE_CHEST,              RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                22668,                        0x0C, "MQ Invisible Blades Visible Chest",           RHT_SHADOW_TEMPLE_MQ_INVISIBLE_BLADES_VISIBLE_CHEST,             RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_MQ_BOMB_FLOWER_CHEST] =                            Location::Chest(RC_SHADOW_TEMPLE_MQ_BOMB_FLOWER_CHEST,                           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                31053,                        0x0D, "MQ Bomb Flower Chest",                        RHT_SHADOW_TEMPLE_MQ_BOMB_FLOWER_CHEST,                          RG_ARROWS_10);
    locationTable[RC_SHADOW_TEMPLE_MQ_WIND_HINT_CHEST] =                              Location::Chest(RC_SHADOW_TEMPLE_MQ_WIND_HINT_CHEST,                             RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26709,                        0x15, "MQ Wind Hint Chest",                          RHT_SHADOW_TEMPLE_MQ_WIND_HINT_CHEST,                            RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_MQ_AFTER_WIND_HIDDEN_CHEST] =                      Location::Chest(RC_SHADOW_TEMPLE_MQ_AFTER_WIND_HIDDEN_CHEST,                     RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26932,                        0x14, "MQ After Wind Hidden Chest",                  RHT_SHADOW_TEMPLE_MQ_AFTER_WIND_HIDDEN_CHEST,                    RG_ARROWS_5);
    locationTable[RC_SHADOW_TEMPLE_MQ_AFTER_WIND_ENEMY_CHEST] =                       Location::Chest(RC_SHADOW_TEMPLE_MQ_AFTER_WIND_ENEMY_CHEST,                      RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                30856,                        0x08, "MQ After Wind Enemy Chest",                   RHT_SHADOW_TEMPLE_MQ_AFTER_WIND_ENEMY_CHEST,                     RG_BLUE_RUPEE);
    locationTable[RC_SHADOW_TEMPLE_MQ_NEAR_SHIP_INVISIBLE_CHEST] =                    Location::Chest(RC_SHADOW_TEMPLE_MQ_NEAR_SHIP_INVISIBLE_CHEST,                   RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_SHADOW_TEMPLE,                26702,                        0x0E, "MQ Near Ship Invisible Chest",                RHT_SHADOW_TEMPLE_MQ_NEAR_SHIP_INVISIBLE_CHEST,                  RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_SHADOW_TEMPLE_MQ_FREESTANDING_KEY] =                       Location::Collectable(RC_SHADOW_TEMPLE_MQ_FREESTANDING_KEY,                            RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_SHADOW_TEMPLE,                1553,                         0x06, "MQ Freestanding Key",                         RHT_SHADOW_TEMPLE_MQ_FREESTANDING_KEY,                           RG_SHADOW_TEMPLE_SMALL_KEY,                                                                                                                                                                       true);

    // Bottom of the Well Vanilla
    locationTable[RC_BOTTOM_OF_THE_WELL_FRONT_LEFT_FAKE_WALL_CHEST] =                 Location::Chest(RC_BOTTOM_OF_THE_WELL_FRONT_LEFT_FAKE_WALL_CHEST,                RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           22600,                        0x08, "Front Left Fake Wall Chest",                  RHT_BOTTOM_OF_THE_WELL_FRONT_LEFT_FAKE_WALL_CHEST,               RG_BOTTOM_OF_THE_WELL_SMALL_KEY,                                                                                                                                                                  true);
    locationTable[RC_BOTTOM_OF_THE_WELL_FRONT_CENTER_BOMBABLE_CHEST] =                Location::Chest(RC_BOTTOM_OF_THE_WELL_FRONT_CENTER_BOMBABLE_CHEST,               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           20578,                        0x02, "Front Center Bombable Chest",                 RHT_BOTTOM_OF_THE_WELL_FRONT_CENTER_BOMBABLE_CHEST,              RG_BOMBCHU_10);
    locationTable[RC_BOTTOM_OF_THE_WELL_RIGHT_BOTTOM_FAKE_WALL_CHEST] =               Location::Chest(RC_BOTTOM_OF_THE_WELL_RIGHT_BOTTOM_FAKE_WALL_CHEST,              RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           22597,                        0x05, "Right Bottom Fake Wall Chest",                RHT_BOTTOM_OF_THE_WELL_RIGHT_BOTTOM_FAKE_WALL_CHEST,             RG_BOTTOM_OF_THE_WELL_SMALL_KEY,                                                                                                                                                                  true);
    locationTable[RC_BOTTOM_OF_THE_WELL_COMPASS_CHEST] =                              Location::Chest(RC_BOTTOM_OF_THE_WELL_COMPASS_CHEST,                             RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           2049,                         0x01, "Compass Chest",                               RHT_BOTTOM_OF_THE_WELL_COMPASS_CHEST,                            RG_BOTTOM_OF_THE_WELL_COMPASS,                                                                                                                                                                    true);
    locationTable[RC_BOTTOM_OF_THE_WELL_CENTER_SKULLTULA_CHEST] =                     Location::Chest(RC_BOTTOM_OF_THE_WELL_CENTER_SKULLTULA_CHEST,                    RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           20558,                        0x0E, "Center Skulltula Chest",                      RHT_BOTTOM_OF_THE_WELL_CENTER_SKULLTULA_CHEST,                   RG_DEKU_NUTS_5);
    locationTable[RC_BOTTOM_OF_THE_WELL_BACK_LEFT_BOMBABLE_CHEST] =                   Location::Chest(RC_BOTTOM_OF_THE_WELL_BACK_LEFT_BOMBABLE_CHEST,                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           23684,                        0x04, "Back Left Bombable Chest",                    RHT_BOTTOM_OF_THE_WELL_BACK_LEFT_BOMBABLE_CHEST,                 RG_DEKU_NUTS_10);
    locationTable[RC_BOTTOM_OF_THE_WELL_LENS_OF_TRUTH_CHEST] =                        Location::Chest(RC_BOTTOM_OF_THE_WELL_LENS_OF_TRUTH_CHEST,                       RCQUEST_VANILLA, RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           4419,                         0x03, "Lens of Truth Chest",                         RHT_BOTTOM_OF_THE_WELL_LENS_OF_TRUTH_CHEST,                      RG_LENS_OF_TRUTH,                                                                                                                                                                                 true);
    locationTable[RC_BOTTOM_OF_THE_WELL_INVISIBLE_CHEST] =                            Location::Chest(RC_BOTTOM_OF_THE_WELL_INVISIBLE_CHEST,                           RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           27348,                        0x14, "Invisible Chest",                             RHT_BOTTOM_OF_THE_WELL_INVISIBLE_CHEST,                          RG_HUGE_RUPEE);
    locationTable[RC_BOTTOM_OF_THE_WELL_UNDERWATER_FRONT_CHEST] =                     Location::Chest(RC_BOTTOM_OF_THE_WELL_UNDERWATER_FRONT_CHEST,                    RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           23760,                        0x10, "Underwater Front Chest",                      RHT_BOTTOM_OF_THE_WELL_UNDERWATER_FRONT_CHEST,                   RG_BOMBS_10);
    locationTable[RC_BOTTOM_OF_THE_WELL_UNDERWATER_LEFT_CHEST] =                      Location::Chest(RC_BOTTOM_OF_THE_WELL_UNDERWATER_LEFT_CHEST,                     RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           22793,                        0x09, "Underwater Left Chest",                       RHT_BOTTOM_OF_THE_WELL_UNDERWATER_LEFT_CHEST,                    RG_RECOVERY_HEART);
    locationTable[RC_BOTTOM_OF_THE_WELL_MAP_CHEST] =                                  Location::Chest(RC_BOTTOM_OF_THE_WELL_MAP_CHEST,                                 RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           2087,                         0x07, "Map Chest",                                   RHT_BOTTOM_OF_THE_WELL_MAP_CHEST,                                RG_BOTTOM_OF_THE_WELL_MAP,                                                                                                                                                                        true);
    locationTable[RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_CHEST] =                           Location::Chest(RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_CHEST,                          RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           21802,                        0x0A, "Fire Keese Chest",                            RHT_BOTTOM_OF_THE_WELL_FIRE_KEESE_CHEST,                         RG_DEKU_SHIELD);
    locationTable[RC_BOTTOM_OF_THE_WELL_LIKE_LIKE_CHEST] =                            Location::Chest(RC_BOTTOM_OF_THE_WELL_LIKE_LIKE_CHEST,                           RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           21836,                        0x0C, "Like Like Chest",                             RHT_BOTTOM_OF_THE_WELL_LIKE_LIKE_CHEST,                          RG_HYLIAN_SHIELD);
    locationTable[RC_BOTTOM_OF_THE_WELL_FREESTANDING_KEY] =                     Location::Collectable(RC_BOTTOM_OF_THE_WELL_FREESTANDING_KEY,                          RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_BOTTOM_OF_THE_WELL,           273,                          0x01, "Freestanding Key",                            RHT_BOTTOM_OF_THE_WELL_FREESTANDING_KEY,                         RG_BOTTOM_OF_THE_WELL_SMALL_KEY,                                                                                                                                                                  true);
    // Bottom of the Well Master Quest
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_MAP_CHEST] =                               Location::Chest(RC_BOTTOM_OF_THE_WELL_MQ_MAP_CHEST,                              RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           2083,                         0x03, "MQ Map Chest",                                RHT_BOTTOM_OF_THE_WELL_MQ_MAP_CHEST,                             RG_BOTTOM_OF_THE_WELL_MAP,                                                                                                                                                                        true);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_LENS_OF_TRUTH_CHEST] =                     Location::Chest(RC_BOTTOM_OF_THE_WELL_MQ_LENS_OF_TRUTH_CHEST,                    RCQUEST_MQ,      RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           -20159,                       0x01, "MQ Lens of Truth Chest",                      RHT_BOTTOM_OF_THE_WELL_MQ_LENS_OF_TRUTH_CHEST,                   RG_LENS_OF_TRUTH,                                                                                                                                                                                 true);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_COMPASS_CHEST] =                           Location::Chest(RC_BOTTOM_OF_THE_WELL_MQ_COMPASS_CHEST,                          RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_BOTTOM_OF_THE_WELL,           6146,                         0x02, "MQ Compass Chest",                            RHT_BOTTOM_OF_THE_WELL_MQ_COMPASS_CHEST,                         RG_BOTTOM_OF_THE_WELL_COMPASS,                                                                                                                                                                    true);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_FREESTANDING_KEY] =        Location::Collectable(RC_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_FREESTANDING_KEY,             RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_BOTTOM_OF_THE_WELL,           529,                          0x02, "MQ Dead Hand Freestanding Key",               RHT_BOTTOM_OF_THE_WELL_MQ_DEAD_HAND_FREESTANDING_KEY,            RG_BOTTOM_OF_THE_WELL_SMALL_KEY,                                                                                                                                                                  true);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_FREESTANDING_KEY] =  Location::Collectable(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_FREESTANDING_KEY,       RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_BOTTOM_OF_THE_WELL,           273,                          0x01, "MQ East Inner Room Freestanding Key",         RHT_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_FREESTANDING_KEY,      RG_BOTTOM_OF_THE_WELL_SMALL_KEY,                                                                                                                                                                  true);

    // Ice Cavern Vanilla
    locationTable[RC_ICE_CAVERN_MAP_CHEST] =                                          Location::Chest(RC_ICE_CAVERN_MAP_CHEST,                                         RCQUEST_VANILLA, RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_ICE_CAVERN,                   2080,                         0x00, "Map Chest",                                   RHT_ICE_CAVERN_MAP_CHEST,                                        RG_ICE_CAVERN_MAP,                                                                                                                                                                                true);
    locationTable[RC_ICE_CAVERN_COMPASS_CHEST] =                                      Location::Chest(RC_ICE_CAVERN_COMPASS_CHEST,                                     RCQUEST_VANILLA, RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_ICE_CAVERN,                   2049,                         0x01, "Compass Chest",                               RHT_ICE_CAVERN_COMPASS_CHEST,                                    RG_ICE_CAVERN_COMPASS,                                                                                                                                                                            true);
    locationTable[RC_ICE_CAVERN_IRON_BOOTS_CHEST] =                                   Location::Chest(RC_ICE_CAVERN_IRON_BOOTS_CHEST,                                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_ICE_CAVERN,                   5570,                         0x02, "Iron Boots Chest",                            RHT_ICE_CAVERN_IRON_BOOTS_CHEST,                                 RG_IRON_BOOTS,                                                                                                                                                                                    true);
    locationTable[RC_ICE_CAVERN_FREESTANDING_POH] =                             Location::Collectable(RC_ICE_CAVERN_FREESTANDING_POH,                                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_ICE_CAVERN,                   262,                          0x01, "Freestanding PoH",                            RHT_ICE_CAVERN_FREESTANDING_POH,                                 RG_PIECE_OF_HEART,                                                                                                                                                                                true);
    // Ice Cavern Master Quest
    locationTable[RC_ICE_CAVERN_MQ_IRON_BOOTS_CHEST] =                                Location::Chest(RC_ICE_CAVERN_MQ_IRON_BOOTS_CHEST,                               RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_ICE_CAVERN,                   5570,                         0x02, "MQ Iron Boots Chest",                         RHT_ICE_CAVERN_MQ_IRON_BOOTS_CHEST,                              RG_IRON_BOOTS,                                                                                                                                                                                    true);
    locationTable[RC_ICE_CAVERN_MQ_COMPASS_CHEST] =                                   Location::Chest(RC_ICE_CAVERN_MQ_COMPASS_CHEST,                                  RCQUEST_MQ,      RCTYPE_COMPASS,                                                          ACTOR_EN_BOX,         SCENE_ICE_CAVERN,                   2048,                         0x00, "MQ Compass Chest",                            RHT_ICE_CAVERN_MQ_COMPASS_CHEST,                                 RG_ICE_CAVERN_COMPASS,                                                                                                                                                                            true);
    locationTable[RC_ICE_CAVERN_MQ_MAP_CHEST] =                                       Location::Chest(RC_ICE_CAVERN_MQ_MAP_CHEST,                                      RCQUEST_MQ,      RCTYPE_MAP,                                                              ACTOR_EN_BOX,         SCENE_ICE_CAVERN,                   -18399,                       0x01, "MQ Map Chest",                                RHT_ICE_CAVERN_MQ_MAP_CHEST,                                     RG_ICE_CAVERN_MAP,                                                                                                                                                                                true);
    locationTable[RC_ICE_CAVERN_MQ_FREESTANDING_POH] =                          Location::Collectable(RC_ICE_CAVERN_MQ_FREESTANDING_POH,                               RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_ITEM00,      SCENE_ICE_CAVERN,                   262,                          0x01, "MQ Freestanding PoH",                         RHT_ICE_CAVERN_MQ_FREESTANDING_POH,                              RG_PIECE_OF_HEART,                                                                                                                                                                                true);

    // Gerudo Training Grounds Vanilla
    locationTable[RC_GERUDO_TRAINING_GROUND_LOBBY_LEFT_CHEST] =                       Location::Chest(RC_GERUDO_TRAINING_GROUND_LOBBY_LEFT_CHEST,                      RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30573,                       0x13, "Lobby Left Chest",                            RHT_GERUDO_TRAINING_GROUND_LOBBY_LEFT_CHEST,                     RG_BLUE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_LOBBY_RIGHT_CHEST] =                      Location::Chest(RC_GERUDO_TRAINING_GROUND_LOBBY_RIGHT_CHEST,                     RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30393,                       0x07, "Lobby Right Chest",                           RHT_GERUDO_TRAINING_GROUND_LOBBY_RIGHT_CHEST,                    RG_ARROWS_10);
    locationTable[RC_GERUDO_TRAINING_GROUND_STALFOS_CHEST] =                          Location::Chest(RC_GERUDO_TRAINING_GROUND_STALFOS_CHEST,                         RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30656,                       0x00, "Stalfos Chest",                               RHT_GERUDO_TRAINING_GROUND_STALFOS_CHEST,                        RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_BEAMOS_CHEST] =                           Location::Chest(RC_GERUDO_TRAINING_GROUND_BEAMOS_CHEST,                          RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30655,                       0x01, "Beamos Chest",                                RHT_GERUDO_TRAINING_GROUND_BEAMOS_CHEST,                         RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_HIDDEN_CEILING_CHEST] =                   Location::Chest(RC_GERUDO_TRAINING_GROUND_HIDDEN_CEILING_CHEST,                  RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22603,                        0x0B, "Hidden Ceiling Chest",                        RHT_GERUDO_TRAINING_GROUND_HIDDEN_CEILING_CHEST,                 RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_MAZE_PATH_FIRST_CHEST] =                  Location::Chest(RC_GERUDO_TRAINING_GROUND_MAZE_PATH_FIRST_CHEST,                 RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       23206,                        0x06, "Maze Path First Chest",                       RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_FIRST_CHEST,                RG_PURPLE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MAZE_PATH_SECOND_CHEST] =                 Location::Chest(RC_GERUDO_TRAINING_GROUND_MAZE_PATH_SECOND_CHEST,                RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22986,                        0x0A, "Maze Path Second Chest",                      RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_SECOND_CHEST,               RG_RED_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MAZE_PATH_THIRD_CHEST] =                  Location::Chest(RC_GERUDO_TRAINING_GROUND_MAZE_PATH_THIRD_CHEST,                 RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22889,                        0x09, "Maze Path Third Chest",                       RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_THIRD_CHEST,                RG_ARROWS_30);
    locationTable[RC_GERUDO_TRAINING_GROUND_MAZE_PATH_FINAL_CHEST] =                  Location::Chest(RC_GERUDO_TRAINING_GROUND_MAZE_PATH_FINAL_CHEST,                 RCQUEST_VANILLA, RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       2860,                         0x0C, "Maze Path Final Chest",                       RHT_GERUDO_TRAINING_GROUND_MAZE_PATH_FINAL_CHEST,                RG_ICE_ARROWS,                                                                                                                                                                                    true);
    locationTable[RC_GERUDO_TRAINING_GROUND_MAZE_RIGHT_CENTRAL_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_MAZE_RIGHT_CENTRAL_CHEST,              RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       23877,                        0x05, "Maze Right Central Chest",                    RHT_GERUDO_TRAINING_GROUND_MAZE_RIGHT_CENTRAL_CHEST,             RG_BOMBCHU_5);
    locationTable[RC_GERUDO_TRAINING_GROUND_MAZE_RIGHT_SIDE_CHEST] =                  Location::Chest(RC_GERUDO_TRAINING_GROUND_MAZE_RIGHT_SIDE_CHEST,                 RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22888,                        0x08, "Maze Right Side Chest",                       RHT_GERUDO_TRAINING_GROUND_MAZE_RIGHT_SIDE_CHEST,                RG_ARROWS_30);
    locationTable[RC_GERUDO_TRAINING_GROUND_UNDERWATER_SILVER_RUPEE_CHEST] =          Location::Chest(RC_GERUDO_TRAINING_GROUND_UNDERWATER_SILVER_RUPEE_CHEST,         RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30643,                       0x0D, "Underwater Silver Rupee Chest",               RHT_GERUDO_TRAINING_GROUND_UNDERWATER_SILVER_RUPEE_CHEST,        RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_HAMMER_ROOM_CLEAR_CHEST] =                Location::Chest(RC_GERUDO_TRAINING_GROUND_HAMMER_ROOM_CLEAR_CHEST,               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       31058,                        0x12, "Hammer Room Clear Chest",                     RHT_GERUDO_TRAINING_GROUND_HAMMER_ROOM_CLEAR_CHEST,              RG_ARROWS_10);
    locationTable[RC_GERUDO_TRAINING_GROUND_HAMMER_ROOM_SWITCH_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_HAMMER_ROOM_SWITCH_CHEST,              RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22608,                        0x10, "Hammer Room Switch Chest",                    RHT_GERUDO_TRAINING_GROUND_HAMMER_ROOM_SWITCH_CHEST,             RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_EYE_STATUE_CHEST] =                       Location::Chest(RC_GERUDO_TRAINING_GROUND_EYE_STATUE_CHEST,                      RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30653,                       0x03, "Eye Statue Chest",                            RHT_GERUDO_TRAINING_GROUND_EYE_STATUE_CHEST,                     RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_NEAR_SCARECROW_CHEST] =                   Location::Chest(RC_GERUDO_TRAINING_GROUND_NEAR_SCARECROW_CHEST,                  RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22596,                        0x04, "Near Scarecrow Chest",                        RHT_GERUDO_TRAINING_GROUND_NEAR_SCARECROW_CHEST,                 RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_BEFORE_HEAVY_BLOCK_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_BEFORE_HEAVY_BLOCK_CHEST,              RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       31089,                        0x11, "Before Heavy Block Chest",                    RHT_GERUDO_TRAINING_GROUND_BEFORE_HEAVY_BLOCK_CHEST,             RG_ARROWS_30);
    locationTable[RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FIRST_CHEST] =                Location::Chest(RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FIRST_CHEST,               RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       31439,                        0x0F, "Heavy Block First Chest",                     RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FIRST_CHEST,              RG_HUGE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_SECOND_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_SECOND_CHEST,              RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       30862,                        0x0E, "Heavy Block Second Chest",                    RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_SECOND_CHEST,             RG_BLUE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_THIRD_CHEST] =                Location::Chest(RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_THIRD_CHEST,               RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       26708,                        0x14, "Heavy Block Third Chest",                     RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_THIRD_CHEST,              RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FOURTH_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FOURTH_CHEST,              RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       24450,                        0x02, "Heavy Block Fourth Chest",                    RHT_GERUDO_TRAINING_GROUND_HEAVY_BLOCK_FOURTH_CHEST,             RG_ICE_TRAP);
    locationTable[RC_GERUDO_TRAINING_GROUND_FREESTANDING_KEY] =                 Location::Collectable(RC_GERUDO_TRAINING_GROUND_FREESTANDING_KEY,                      RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_GERUDO_TRAINING_GROUND,       273,                          0x01, "Freestanding Key",                            RHT_GERUDO_TRAINING_GROUND_FREESTANDING_KEY,                     RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    // Gerudo Training Grounds Master Quest
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_CHEST] =                   Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       23879,                        0x07, "MQ Lobby Right Chest",                        RHT_GERUDO_TRAINING_GROUND_MQ_LOBBY_RIGHT_CHEST,                 RG_BOMBCHU_5);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_CHEST] =                    Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22867,                        0x13, "MQ Lobby Left Chest",                         RHT_GERUDO_TRAINING_GROUND_MQ_LOBBY_LEFT_CHEST,                  RG_ARROWS_10);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_FIRST_IRON_KNUCKLE_CHEST] =            Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_FIRST_IRON_KNUCKLE_CHEST,           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30304,                       0x00, "MQ First Iron Knuckle Chest",                 RHT_GERUDO_TRAINING_GROUND_MQ_FIRST_IRON_KNUCKLE_CHEST,          RG_BLUE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_BEFORE_HEAVY_BLOCK_CHEST] =            Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_BEFORE_HEAVY_BLOCK_CHEST,           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       31057,                        0x11, "MQ Before Heavy Block Chest",                 RHT_GERUDO_TRAINING_GROUND_MQ_BEFORE_HEAVY_BLOCK_CHEST,          RG_ARROWS_10);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_EYE_STATUE_CHEST] =                    Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_EYE_STATUE_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -32669,                       0x03, "MQ Eye Statue Chest",                         RHT_GERUDO_TRAINING_GROUND_MQ_EYE_STATUE_CHEST,                  RG_BOMBCHU_10);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_FLAME_CIRCLE_CHEST] =                  Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_FLAME_CIRCLE_CHEST,                 RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30642,                       0x0E, "MQ Flame Circle Chest",                       RHT_GERUDO_TRAINING_GROUND_MQ_FLAME_CIRCLE_CHEST,                RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_SECOND_IRON_KNUCKLE_CHEST] =           Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_SECOND_IRON_KNUCKLE_CHEST,          RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       31058,                        0x12, "MQ Second Iron Knuckle Chest",                RHT_GERUDO_TRAINING_GROUND_MQ_SECOND_IRON_KNUCKLE_CHEST,         RG_ARROWS_10);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_DINOLFOS_CHEST] =                      Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_DINOLFOS_CHEST,                     RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30655,                       0x01, "MQ Dinolfos Chest",                           RHT_GERUDO_TRAINING_GROUND_MQ_DINOLFOS_CHEST,                    RG_GERUDO_TRAINING_GROUNDS_SMALL_KEY,                                                                                                                                                             true);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_ICE_ARROWS_CHEST] =                    Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_ICE_ARROWS_CHEST,                   RCQUEST_MQ,      RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -17628,                       0x04, "MQ Ice Arrows Chest",                         RHT_GERUDO_TRAINING_GROUND_MQ_ICE_ARROWS_CHEST,                  RG_ICE_ARROWS,                                                                                                                                                                                    true);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_CENTRAL_CHEST] =            Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_CENTRAL_CHEST,           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22661,                        0x05, "MQ Maze Right Central Chest",                 RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_CENTRAL_CHEST,          RG_BLUE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_FIRST_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_FIRST_CHEST,              RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22918,                        0x06, "MQ Maze Path First Chest",                    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_FIRST_CHEST,             RG_GREEN_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_SIDE_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_SIDE_CHEST,              RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       24136,                        0x08, "MQ Maze Right Side Chest",                    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_RIGHT_SIDE_CHEST,             RG_TREASURE_GAME_GREEN_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_THIRD_CHEST] =               Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_THIRD_CHEST,              RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       24137,                        0x09, "MQ Maze Path Third Chest",                    RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_THIRD_CHEST,             RG_TREASURE_GAME_GREEN_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_SECOND_CHEST] =              Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_SECOND_CHEST,             RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       22986,                        0x0A, "MQ Maze Path Second Chest",                   RHT_GERUDO_TRAINING_GROUND_MQ_MAZE_PATH_SECOND_CHEST,            RG_RED_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_HIDDEN_CEILING_CHEST] =                Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_HIDDEN_CEILING_CHEST,               RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       23211,                        0x0B, "MQ Hidden Ceiling Chest",                     RHT_GERUDO_TRAINING_GROUND_MQ_HIDDEN_CEILING_CHEST,              RG_PURPLE_RUPEE);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_UNDERWATER_SILVER_RUPEE_CHEST] =       Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_UNDERWATER_SILVER_RUPEE_CHEST,      RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       -30643,                       0x0D, "MQ Underwater Silver Rupee Chest",            RHT_GERUDO_TRAINING_GROUND_MQ_UNDERWATER_SILVER_RUPEE_CHEST,     RG_TREASURE_GAME_GREEN_RUPEE,                                                                                                                                                                     true);
    locationTable[RC_GERUDO_TRAINING_GROUND_MQ_HEAVY_BLOCK_CHEST] =                   Location::Chest(RC_GERUDO_TRAINING_GROUND_MQ_HEAVY_BLOCK_CHEST,                  RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_GERUDO_TRAINING_GROUND,       31394,                        0x02, "MQ Heavy Block Chest",                        RHT_GERUDO_TRAINING_GROUND_MQ_HEAVY_BLOCK_CHEST,                 RG_PURPLE_RUPEE);

    // Ganon's Castle Shared
    locationTable[RC_GANONS_TOWER_BOSS_KEY_CHEST] =                                   Location::Chest(RC_GANONS_TOWER_BOSS_KEY_CHEST,                                  RCQUEST_BOTH,    RCTYPE_GANON_BOSS_KEY,                                                   ACTOR_EN_BOX,         SCENE_GANONS_TOWER,                 10219,                        0x0B, "Tower Boss Key Chest",                        RHT_GANONS_TOWER_BOSS_KEY_CHEST,                                 RG_GANONS_CASTLE_BOSS_KEY,                                                                                                                                                                        true);
    // Ganon's Castle Vanilla
    locationTable[RC_GANONS_CASTLE_FOREST_TRIAL_CHEST] =                              Location::Chest(RC_GANONS_CASTLE_FOREST_TRIAL_CHEST,                             RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         30857,                        0x09, "Forest Trial Chest",                          RHT_GANONS_CASTLE_FOREST_TRIAL_CHEST,                            RG_BLUE_RUPEE);
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_LEFT_CHEST] =                          Location::Chest(RC_GANONS_CASTLE_WATER_TRIAL_LEFT_CHEST,                         RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         24455,                        0x07, "Water Trial Left Chest",                      RHT_GANONS_CASTLE_WATER_TRIAL_LEFT_CHEST,                        RG_ICE_TRAP);
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_RIGHT_CHEST] =                         Location::Chest(RC_GANONS_CASTLE_WATER_TRIAL_RIGHT_CHEST,                        RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         22790,                        0x06, "Water Trial Right Chest",                     RHT_GANONS_CASTLE_WATER_TRIAL_RIGHT_CHEST,                       RG_RECOVERY_HEART);
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_FRONT_CHEST] =                        Location::Chest(RC_GANONS_CASTLE_SHADOW_TRIAL_FRONT_CHEST,                       RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         22664,                        0x08, "Shadow Trial Front Chest",                    RHT_GANONS_CASTLE_SHADOW_TRIAL_FRONT_CHEST,                      RG_BLUE_RUPEE);
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_GOLDEN_GAUNTLETS_CHEST] =             Location::Chest(RC_GANONS_CASTLE_SHADOW_TRIAL_GOLDEN_GAUNTLETS_CHEST,            RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         14021,                        0x05, "Shadow Trial Golden Gauntlets Chest",         RHT_GANONS_CASTLE_SHADOW_TRIAL_GOLDEN_GAUNTLETS_CHEST,           RG_PROGRESSIVE_STRENGTH,                                                                                                                                                                          true);
    locationTable[RC_GANONS_CASTLE_SPIRIT_TRIAL_CRYSTAL_SWITCH_CHEST] =               Location::Chest(RC_GANONS_CASTLE_SPIRIT_TRIAL_CRYSTAL_SWITCH_CHEST,              RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -29326,                       0x12, "Spirit Trial Crystal Switch Chest",           RHT_GANONS_CASTLE_SPIRIT_TRIAL_CRYSTAL_SWITCH_CHEST,             RG_BOMBCHU_20);
    locationTable[RC_GANONS_CASTLE_SPIRIT_TRIAL_INVISIBLE_CHEST] =                    Location::Chest(RC_GANONS_CASTLE_SPIRIT_TRIAL_INVISIBLE_CHEST,                   RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         26964,                        0x14, "Spirit Trial Invisible Chest",                RHT_GANONS_CASTLE_SPIRIT_TRIAL_INVISIBLE_CHEST,                  RG_ARROWS_10);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_FIRST_LEFT_CHEST] =                    Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_FIRST_LEFT_CHEST,                   RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         22668,                        0x0C, "Light Trial First Left Chest",                RHT_GANONS_CASTLE_LIGHT_TRIAL_FIRST_LEFT_CHEST,                  RG_BLUE_RUPEE);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_SECOND_LEFT_CHEST] =                   Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_SECOND_LEFT_CHEST,                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         24459,                        0x0B, "Light Trial Second Left Chest",               RHT_GANONS_CASTLE_LIGHT_TRIAL_SECOND_LEFT_CHEST,                 RG_ICE_TRAP);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_THIRD_LEFT_CHEST] =                    Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_THIRD_LEFT_CHEST,                   RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         22797,                        0x0D, "Light Trial Third Left Chest",                RHT_GANONS_CASTLE_LIGHT_TRIAL_THIRD_LEFT_CHEST,                  RG_RECOVERY_HEART);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_FIRST_RIGHT_CHEST] =                   Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_FIRST_RIGHT_CHEST,                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         24462,                        0x0E, "Light Trial First Right Chest",               RHT_GANONS_CASTLE_LIGHT_TRIAL_FIRST_RIGHT_CHEST,                 RG_ICE_TRAP);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_SECOND_RIGHT_CHEST] =                  Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_SECOND_RIGHT_CHEST,                 RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         22890,                        0x0A, "Light Trial Second Right Chest",              RHT_GANONS_CASTLE_LIGHT_TRIAL_SECOND_RIGHT_CHEST,                RG_ARROWS_30);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_THIRD_RIGHT_CHEST] =                   Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_THIRD_RIGHT_CHEST,                  RCQUEST_VANILLA, RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         24463,                        0x0F, "Light Trial Third Right Chest",               RHT_GANONS_CASTLE_LIGHT_TRIAL_THIRD_RIGHT_CHEST,                 RG_ICE_TRAP);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_INVISIBLE_ENEMIES_CHEST] =             Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_INVISIBLE_ENEMIES_CHEST,            RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         30800,                        0x10, "Light Trial Invisible Enemies Chest",         RHT_GANONS_CASTLE_LIGHT_TRIAL_INVISIBLE_ENEMIES_CHEST,           RG_GANONS_CASTLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_LULLABY_CHEST] =                       Location::Chest(RC_GANONS_CASTLE_LIGHT_TRIAL_LULLABY_CHEST,                      RCQUEST_VANILLA, RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30639,                       0x11, "Light Trial Lullaby Chest",                   RHT_GANONS_CASTLE_LIGHT_TRIAL_LULLABY_CHEST,                     RG_GANONS_CASTLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_GANONS_CASTLE_DEKU_SCRUB_CENTER_LEFT] =                           Location::Base(RC_GANONS_CASTLE_DEKU_SCRUB_CENTER_LEFT,                         RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x05,                               "Deku Scrub Center-Left",                      RHT_GANONS_CASTLE_DEKU_SCRUB_CENTER_LEFT,                        RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_DEKU_SCRUB_CENTER_LEFT),                        false, 40);
    locationTable[RC_GANONS_CASTLE_DEKU_SCRUB_CENTER_RIGHT] =                          Location::Base(RC_GANONS_CASTLE_DEKU_SCRUB_CENTER_RIGHT,                        RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x03,                               "Deku Scrub Center-Right",                     RHT_GANONS_CASTLE_DEKU_SCRUB_CENTER_RIGHT,                       RG_BUY_ARROWS_30,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_DEKU_SCRUB_CENTER_RIGHT),                       false, 70);
    locationTable[RC_GANONS_CASTLE_DEKU_SCRUB_RIGHT] =                                 Location::Base(RC_GANONS_CASTLE_DEKU_SCRUB_RIGHT,                               RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x07,                               "Deku Scrub Right",                            RHT_GANONS_CASTLE_DEKU_SCRUB_RIGHT,                              RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_DEKU_SCRUB_RIGHT),                              false, 20);
    locationTable[RC_GANONS_CASTLE_DEKU_SCRUB_LEFT] =                                  Location::Base(RC_GANONS_CASTLE_DEKU_SCRUB_LEFT,                                RCQUEST_VANILLA, RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x08,                               "Deku Scrub Left",                             RHT_GANONS_CASTLE_DEKU_SCRUB_LEFT,                               RG_BUY_GREEN_POTION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_DEKU_SCRUB_LEFT),                               false, 40);
    // Ganon's Castle MQ
    locationTable[RC_GANONS_CASTLE_MQ_WATER_TRIAL_CHEST] =                            Location::Chest(RC_GANONS_CASTLE_MQ_WATER_TRIAL_CHEST,                           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         22977,                        0x01, "MQ Water Trial Chest",                        RHT_GANONS_CASTLE_MQ_WATER_TRIAL_CHEST,                          RG_RED_RUPEE);
    locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_EYE_SWITCH_CHEST] =                Location::Chest(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_EYE_SWITCH_CHEST,               RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30398,                       0x02, "MQ Forest Trial Eye Switch Chest",            RHT_GANONS_CASTLE_MQ_FOREST_TRIAL_EYE_SWITCH_CHEST,              RG_ARROWS_10);
    locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_FROZEN_EYE_SWITCH_CHEST] =         Location::Chest(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_FROZEN_EYE_SWITCH_CHEST,        RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -32733,                       0x03, "MQ Forest Trial Frozen Eye Switch Chest",     RHT_GANONS_CASTLE_MQ_FOREST_TRIAL_FROZEN_EYE_SWITCH_CHEST,       RG_BOMBS_5);
    locationTable[RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_LULLABY_CHEST] =                    Location::Chest(RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_LULLABY_CHEST,                   RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30460,                       0x04, "MQ Light Trial Lullaby Chest",                RHT_GANONS_CASTLE_MQ_LIGHT_TRIAL_LULLABY_CHEST,                  RG_RECOVERY_HEART);
    locationTable[RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_BOMB_FLOWER_CHEST] =               Location::Chest(RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_BOMB_FLOWER_CHEST,              RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30400,                       0x00, "MQ Shadow Trial Bomb Flower Chest",           RHT_GANONS_CASTLE_MQ_SHADOW_TRIAL_BOMB_FLOWER_CHEST,             RG_ARROWS_10);
    locationTable[RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_EYE_SWITCH_CHEST] =                Location::Chest(RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_EYE_SWITCH_CHEST,               RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30651,                       0x05, "MQ Shadow Trial Eye Switch Chest",            RHT_GANONS_CASTLE_MQ_SHADOW_TRIAL_EYE_SWITCH_CHEST,              RG_GANONS_CASTLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_GOLDEN_GAUNTLETS_CHEST] =          Location::Chest(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_GOLDEN_GAUNTLETS_CHEST,         RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -18746,                       0x06, "MQ Spirit Trial Golden Gauntlets Chest",      RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_GOLDEN_GAUNTLETS_CHEST,        RG_PROGRESSIVE_STRENGTH,                                                                                                                                                                          true);
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_RIGHT_CHEST] =            Location::Chest(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_RIGHT_CHEST,           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30457,                       0x07, "MQ Spirit Trial Sun Back Right Chest",        RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_RIGHT_CHEST,          RG_RECOVERY_HEART);
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_LEFT_CHEST] =             Location::Chest(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_LEFT_CHEST,            RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30648,                       0x08, "MQ Spirit Trial Sun Back Left Chest",         RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_BACK_LEFT_CHEST,           RG_GANONS_CASTLE_SMALL_KEY);
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_FRONT_LEFT_CHEST] =            Location::Chest(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_FRONT_LEFT_CHEST,           RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         -30455,                       0x09, "MQ Spirit Trial Sun Front Left Chest",        RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_SUN_FRONT_LEFT_CHEST,          RG_RECOVERY_HEART);
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_FIRST_CHEST] =                     Location::Chest(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_FIRST_CHEST,                    RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         20586,                        0x0A, "MQ Spirit Trial First Chest",                 RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_FIRST_CHEST,                   RG_BOMBCHU_10);
    locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_INVISIBLE_CHEST] =                 Location::Chest(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_INVISIBLE_CHEST,                RCQUEST_MQ,      RCTYPE_STANDARD,                                                         ACTOR_EN_BOX,         SCENE_INSIDE_GANONS_CASTLE,         26964,                        0x14, "MQ Spirit Trial Invisible Chest",             RHT_GANONS_CASTLE_MQ_SPIRIT_TRIAL_INVISIBLE_CHEST,               RG_ARROWS_10);
    locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_FREESTANDING_KEY] =          Location::Collectable(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_FREESTANDING_KEY,               RCQUEST_MQ,      RCTYPE_SMALL_KEY,                                                        ACTOR_EN_ITEM00,      SCENE_INSIDE_GANONS_CASTLE,         273,                          0x01, "MQ Forest Trial Freestanding Key",            RHT_GANONS_CASTLE_MQ_FOREST_TRIAL_FREESTANDING_KEY,              RG_GANONS_CASTLE_SMALL_KEY,                                                                                                                                                                       true);
    locationTable[RC_GANONS_CASTLE_MQ_DEKU_SCRUB_RIGHT] =                              Location::Base(RC_GANONS_CASTLE_MQ_DEKU_SCRUB_RIGHT,                            RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x00,                               "MQ Deku Scrub Right",                         RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_RIGHT,                           RG_BUY_DEKU_NUTS_5,                                                 SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_MQ_DEKU_SCRUB_RIGHT),                           false, 20);
    locationTable[RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_LEFT] =                        Location::Base(RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_LEFT,                      RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x05,                               "MQ Deku Scrub Center-Left",                   RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_LEFT,                     RG_BUY_BOMBS_535,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_LEFT),                     false, 40);
    locationTable[RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER] =                             Location::Base(RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER,                           RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x03,                               "MQ Deku Scrub Center",                        RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER,                          RG_BUY_ARROWS_30,                                                   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER),                          false, 70);
    locationTable[RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_RIGHT] =                       Location::Base(RC_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_RIGHT,                     RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x07,                               "MQ Deku Scrub Center-Right",                  RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_RIGHT,                    RG_BUY_RED_POTION_40,                                               SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_MQ_DEKU_SCRUB_CENTER_RIGHT),                    false, 40);
    locationTable[RC_GANONS_CASTLE_MQ_DEKU_SCRUB_LEFT] =                               Location::Base(RC_GANONS_CASTLE_MQ_DEKU_SCRUB_LEFT,                             RCQUEST_MQ,      RCTYPE_SCRUB,                                                            ACTOR_EN_DNS,         SCENE_INSIDE_GANONS_CASTLE,         0x08,                               "MQ Deku Scrub Left",                          RHT_GANONS_CASTLE_MQ_DEKU_SCRUB_LEFT,                            RG_BUY_GREEN_POTION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_SCRUBS_PURCHASED_GANONS_CASTLE_MQ_DEKU_SCRUB_LEFT),                            false, 40);

    // Gold Skulltula Tokens

    // Dungeons
    // Deku Tree
    locationTable[RC_DEKU_TREE_GS_BASEMENT_BACK_ROOM] =                             Location::GSToken(RC_DEKU_TREE_GS_BASEMENT_BACK_ROOM,                              RCQUEST_VANILLA,                                                                                                SCENE_DEKU_TREE,                    8193,                         0x01, "GS Basement Back Room",                       RHT_DEKU_TREE_GS_BASEMENT_BACK_ROOM);
    locationTable[RC_DEKU_TREE_GS_BASEMENT_GATE] =                                  Location::GSToken(RC_DEKU_TREE_GS_BASEMENT_GATE,                                   RCQUEST_VANILLA,                                                                                                SCENE_DEKU_TREE,                    8194,                         0x02, "GS Basement Gate",                            RHT_DEKU_TREE_GS_BASEMENT_GATE);
    locationTable[RC_DEKU_TREE_GS_BASEMENT_VINES] =                                 Location::GSToken(RC_DEKU_TREE_GS_BASEMENT_VINES,                                  RCQUEST_VANILLA,                                                                                                SCENE_DEKU_TREE,                    8196,                         0x04, "GS Basement Vines",                           RHT_DEKU_TREE_GS_BASEMENT_VINES);
    locationTable[RC_DEKU_TREE_GS_COMPASS_ROOM] =                                   Location::GSToken(RC_DEKU_TREE_GS_COMPASS_ROOM,                                    RCQUEST_VANILLA,                                                                                                SCENE_DEKU_TREE,                    8200,                         0x08, "GS Compass Room",                             RHT_DEKU_TREE_GS_COMPASS_ROOM);
    locationTable[RC_DEKU_TREE_MQ_GS_LOBBY] =                                       Location::GSToken(RC_DEKU_TREE_MQ_GS_LOBBY,                                        RCQUEST_MQ,                                                                                                     SCENE_DEKU_TREE,                    8194,                         0x02, "MQ GS Lobby",                                 RHT_DEKU_TREE_MQ_GS_LOBBY);
    locationTable[RC_DEKU_TREE_MQ_GS_PAST_BOULDER_VINES] =                          Location::GSToken(RC_DEKU_TREE_MQ_GS_PAST_BOULDER_VINES,                           RCQUEST_MQ,                                                                                                     SCENE_DEKU_TREE,                    8200,                         0x08, "MQ GS Past Boulder Vines",                    RHT_DEKU_TREE_MQ_GS_PAST_BOULDER_VINES);
    locationTable[RC_DEKU_TREE_MQ_GS_BASEMENT_GRAVES_ROOM] =                        Location::GSToken(RC_DEKU_TREE_MQ_GS_BASEMENT_GRAVES_ROOM,                         RCQUEST_MQ,                                                                                                     SCENE_DEKU_TREE,                    8196,                         0x04, "MQ GS Basement Graves Room",                  RHT_DEKU_TREE_MQ_GS_BASEMENT_GRAVES_ROOM);
    locationTable[RC_DEKU_TREE_MQ_GS_BASEMENT_BACK_ROOM] =                          Location::GSToken(RC_DEKU_TREE_MQ_GS_BASEMENT_BACK_ROOM,                           RCQUEST_MQ,                                                                                                     SCENE_DEKU_TREE,                    8193,                         0x01, "MQ GS Basement Back Room",                    RHT_DEKU_TREE_MQ_GS_BASEMENT_BACK_ROOM);
    // Dodongo's Cavern
    locationTable[RC_DODONGOS_CAVERN_GS_VINES_ABOVE_STAIRS] =                       Location::GSToken(RC_DODONGOS_CAVERN_GS_VINES_ABOVE_STAIRS,                        RCQUEST_VANILLA,                                                                                                SCENE_DODONGOS_CAVERN,              8449,                         0x01, "GS Vines Above Stairs",                       RHT_DODONGOS_CAVERN_GS_VINES_ABOVE_STAIRS);
    locationTable[RC_DODONGOS_CAVERN_GS_SCARECROW] =                                Location::GSToken(RC_DODONGOS_CAVERN_GS_SCARECROW,                                 RCQUEST_VANILLA,                                                                                                SCENE_DODONGOS_CAVERN,              8450,                         0x02, "GS Scarecrow",                                RHT_DODONGOS_CAVERN_GS_SCARECROW);
    locationTable[RC_DODONGOS_CAVERN_GS_ALCOVE_ABOVE_STAIRS] =                      Location::GSToken(RC_DODONGOS_CAVERN_GS_ALCOVE_ABOVE_STAIRS,                       RCQUEST_VANILLA,                                                                                                SCENE_DODONGOS_CAVERN,              8452,                         0x04, "GS Alcove Above Stairs",                      RHT_DODONGOS_CAVERN_GS_ALCOVE_ABOVE_STAIRS);
    locationTable[RC_DODONGOS_CAVERN_GS_BACK_ROOM] =                                Location::GSToken(RC_DODONGOS_CAVERN_GS_BACK_ROOM,                                 RCQUEST_VANILLA,                                                                                                SCENE_DODONGOS_CAVERN,              8456,                         0x08, "GS Back Room",                                RHT_DODONGOS_CAVERN_GS_BACK_ROOM);
    locationTable[RC_DODONGOS_CAVERN_GS_SIDE_ROOM_NEAR_LOWER_LIZALFOS] =            Location::GSToken(RC_DODONGOS_CAVERN_GS_SIDE_ROOM_NEAR_LOWER_LIZALFOS,             RCQUEST_VANILLA,                                                                                                SCENE_DODONGOS_CAVERN,              8464,                         0x10, "GS Side Room Near Lower Lizalfos",            RHT_DODONGOS_CAVERN_GS_SIDE_ROOM_NEAR_LOWER_LIZALFOS);
    locationTable[RC_DODONGOS_CAVERN_MQ_GS_SCRUB_ROOM] =                            Location::GSToken(RC_DODONGOS_CAVERN_MQ_GS_SCRUB_ROOM,                             RCQUEST_MQ,                                                                                                     SCENE_DODONGOS_CAVERN,              8450,                         0x02, "MQ GS Scrub Room",                            RHT_DODONGOS_CAVERN_MQ_GS_SCRUB_ROOM);
    locationTable[RC_DODONGOS_CAVERN_MQ_GS_SONG_OF_TIME_BLOCK_ROOM] =               Location::GSToken(RC_DODONGOS_CAVERN_MQ_GS_SONG_OF_TIME_BLOCK_ROOM,                RCQUEST_MQ,                                                                                                     SCENE_DODONGOS_CAVERN,              8456,                         0x08, "MQ GS Song of Time Block Room",               RHT_DODONGOS_CAVERN_MQ_GS_SONG_OF_TIME_BLOCK_ROOM);
    locationTable[RC_DODONGOS_CAVERN_MQ_GS_LIZALFOS_ROOM] =                         Location::GSToken(RC_DODONGOS_CAVERN_MQ_GS_LIZALFOS_ROOM,                          RCQUEST_MQ,                                                                                                     SCENE_DODONGOS_CAVERN,              8452,                         0x04, "MQ GS Lizalfos Room",                         RHT_DODONGOS_CAVERN_MQ_GS_LIZALFOS_ROOM);
    locationTable[RC_DODONGOS_CAVERN_MQ_GS_LARVAE_ROOM] =                           Location::GSToken(RC_DODONGOS_CAVERN_MQ_GS_LARVAE_ROOM,                            RCQUEST_MQ,                                                                                                     SCENE_DODONGOS_CAVERN,              8464,                         0x10, "MQ GS Larvae Room",                           RHT_DODONGOS_CAVERN_MQ_GS_LARVAE_ROOM);
    locationTable[RC_DODONGOS_CAVERN_MQ_GS_BACK_AREA] =                             Location::GSToken(RC_DODONGOS_CAVERN_MQ_GS_BACK_AREA,                              RCQUEST_MQ,                                                                                                     SCENE_DODONGOS_CAVERN,              8449,                         0x01, "MQ GS Back Room",                             RHT_DODONGOS_CAVERN_MQ_GS_BACK_AREA);
    locationTable[RC_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_LOWER] =                    Location::GSToken(RC_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_LOWER,                     RCQUEST_VANILLA,                                                                                                SCENE_JABU_JABU,                    8705,                         0x01, "GS Lobby Basement Lower",                     RHT_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_LOWER);
    locationTable[RC_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_UPPER] =                    Location::GSToken(RC_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_UPPER,                     RCQUEST_VANILLA,                                                                                                SCENE_JABU_JABU,                    8706,                         0x02, "GS Lobby Basement Upper",                     RHT_JABU_JABUS_BELLY_GS_LOBBY_BASEMENT_UPPER);
    locationTable[RC_JABU_JABUS_BELLY_GS_NEAR_BOSS] =                               Location::GSToken(RC_JABU_JABUS_BELLY_GS_NEAR_BOSS,                                RCQUEST_VANILLA,                                                                                                SCENE_JABU_JABU,                    8708,                         0x04, "GS Near Boss",                                RHT_JABU_JABUS_BELLY_GS_NEAR_BOSS);
    locationTable[RC_JABU_JABUS_BELLY_GS_WATER_SWITCH_ROOM] =                       Location::GSToken(RC_JABU_JABUS_BELLY_GS_WATER_SWITCH_ROOM,                        RCQUEST_VANILLA,                                                                                                SCENE_JABU_JABU,                    8712,                         0x08, "GS Water Switch Room",                        RHT_JABU_JABUS_BELLY_GS_WATER_SWITCH_ROOM);
    locationTable[RC_JABU_JABUS_BELLY_MQ_GS_TAILPASARAN_ROOM] =                     Location::GSToken(RC_JABU_JABUS_BELLY_MQ_GS_TAILPASARAN_ROOM,                      RCQUEST_MQ,                                                                                                     SCENE_JABU_JABU,                    8708,                         0x04, "MQ GS Tail Parasan Room",                     RHT_JABU_JABUS_BELLY_MQ_GS_TAILPASARAN_ROOM);
    locationTable[RC_JABU_JABUS_BELLY_MQ_GS_INVISIBLE_ENEMIES_ROOM] =               Location::GSToken(RC_JABU_JABUS_BELLY_MQ_GS_INVISIBLE_ENEMIES_ROOM,                RCQUEST_MQ,                                                                                                     SCENE_JABU_JABU,                    8712,                         0x08, "MQ GS Invisible Enemies Room",                RHT_JABU_JABUS_BELLY_MQ_GS_INVISIBLE_ENEMIES_ROOM);
    locationTable[RC_JABU_JABUS_BELLY_MQ_GS_BOOMERANG_CHEST_ROOM] =                 Location::GSToken(RC_JABU_JABUS_BELLY_MQ_GS_BOOMERANG_CHEST_ROOM,                  RCQUEST_MQ,                                                                                                     SCENE_JABU_JABU,                    8705,                         0x01, "MQ GS Boomerang Chest Room",                  RHT_JABU_JABUS_BELLY_MQ_GS_BOOMERANG_CHEST_ROOM);
    locationTable[RC_JABU_JABUS_BELLY_MQ_GS_NEAR_BOSS] =                            Location::GSToken(RC_JABU_JABUS_BELLY_MQ_GS_NEAR_BOSS,                             RCQUEST_MQ,                                                                                                     SCENE_JABU_JABU,                    8706,                         0x02, "MQ GS Near Boss",                             RHT_JABU_JABUS_BELLY_MQ_GS_NEAR_BOSS);
    locationTable[RC_FOREST_TEMPLE_GS_RAISED_ISLAND_COURTYARD] =                    Location::GSToken(RC_FOREST_TEMPLE_GS_RAISED_ISLAND_COURTYARD,                     RCQUEST_VANILLA,                                                                                                SCENE_FOREST_TEMPLE,                8961,                         0x01, "GS Raised Island Courtyard",                  RHT_FOREST_TEMPLE_GS_RAISED_ISLAND_COURTYARD);
    locationTable[RC_FOREST_TEMPLE_GS_FIRST_ROOM] =                                 Location::GSToken(RC_FOREST_TEMPLE_GS_FIRST_ROOM,                                  RCQUEST_VANILLA,                                                                                                SCENE_FOREST_TEMPLE,                8962,                         0x02, "GS First Room",                               RHT_FOREST_TEMPLE_GS_FIRST_ROOM);
    locationTable[RC_FOREST_TEMPLE_GS_LEVEL_ISLAND_COURTYARD] =                     Location::GSToken(RC_FOREST_TEMPLE_GS_LEVEL_ISLAND_COURTYARD,                      RCQUEST_VANILLA,                                                                                                SCENE_FOREST_TEMPLE,                8964,                         0x04, "GS Level Island Courtyard",                   RHT_FOREST_TEMPLE_GS_LEVEL_ISLAND_COURTYARD);
    locationTable[RC_FOREST_TEMPLE_GS_LOBBY] =                                      Location::GSToken(RC_FOREST_TEMPLE_GS_LOBBY,                                       RCQUEST_VANILLA,                                                                                                SCENE_FOREST_TEMPLE,                8968,                         0x08, "GS Lobby",                                    RHT_FOREST_TEMPLE_GS_LOBBY);
    locationTable[RC_FOREST_TEMPLE_GS_BASEMENT] =                                   Location::GSToken(RC_FOREST_TEMPLE_GS_BASEMENT,                                    RCQUEST_VANILLA,                                                                                                SCENE_FOREST_TEMPLE,                8976,                         0x10, "GS Basement",                                 RHT_FOREST_TEMPLE_GS_BASEMENT);
    locationTable[RC_FOREST_TEMPLE_MQ_GS_FIRST_HALLWAY] =                           Location::GSToken(RC_FOREST_TEMPLE_MQ_GS_FIRST_HALLWAY,                            RCQUEST_MQ,                                                                                                     SCENE_FOREST_TEMPLE,                8962,                         0x02, "MQ GS First Hallway",                         RHT_FOREST_TEMPLE_MQ_GS_FIRST_HALLWAY);
    locationTable[RC_FOREST_TEMPLE_MQ_GS_BLOCK_PUSH_ROOM] =                         Location::GSToken(RC_FOREST_TEMPLE_MQ_GS_BLOCK_PUSH_ROOM,                          RCQUEST_MQ,                                                                                                     SCENE_FOREST_TEMPLE,                8976,                         0x10, "MQ GS Block Push Room",                       RHT_FOREST_TEMPLE_MQ_GS_BLOCK_PUSH_ROOM);
    locationTable[RC_FOREST_TEMPLE_MQ_GS_RAISED_ISLAND_COURTYARD] =                 Location::GSToken(RC_FOREST_TEMPLE_MQ_GS_RAISED_ISLAND_COURTYARD,                  RCQUEST_MQ,                                                                                                     SCENE_FOREST_TEMPLE,                8961,                         0x01, "MQ GS Raised Island Courtyard",               RHT_FOREST_TEMPLE_MQ_GS_RAISED_ISLAND_COURTYARD);
    locationTable[RC_FOREST_TEMPLE_MQ_GS_LEVEL_ISLAND_COURTYARD] =                  Location::GSToken(RC_FOREST_TEMPLE_MQ_GS_LEVEL_ISLAND_COURTYARD,                   RCQUEST_MQ,                                                                                                     SCENE_FOREST_TEMPLE,                8964,                         0x04, "MQ GS Level Island Courtyard",                RHT_FOREST_TEMPLE_MQ_GS_LEVEL_ISLAND_COURTYARD);
    locationTable[RC_FOREST_TEMPLE_MQ_GS_WELL] =                                    Location::GSToken(RC_FOREST_TEMPLE_MQ_GS_WELL,                                     RCQUEST_MQ,                                                                                                     SCENE_FOREST_TEMPLE,                8968,                         0x08, "MQ GS Well",                                  RHT_FOREST_TEMPLE_MQ_GS_WELL);
    // Fire Temple
    locationTable[RC_FIRE_TEMPLE_GS_SONG_OF_TIME_ROOM] =                            Location::GSToken(RC_FIRE_TEMPLE_GS_SONG_OF_TIME_ROOM,                             RCQUEST_VANILLA,                                                                                                SCENE_FIRE_TEMPLE,                  9217,                         0x01, "GS Song of Time Room",                        RHT_FIRE_TEMPLE_GS_SONG_OF_TIME_ROOM);
    locationTable[RC_FIRE_TEMPLE_GS_BOSS_KEY_LOOP] =                                Location::GSToken(RC_FIRE_TEMPLE_GS_BOSS_KEY_LOOP,                                 RCQUEST_VANILLA,                                                                                                SCENE_FIRE_TEMPLE,                  9218,                         0x02, "GS Boss Key Loop",                            RHT_FIRE_TEMPLE_GS_BOSS_KEY_LOOP);
    locationTable[RC_FIRE_TEMPLE_GS_BOULDER_MAZE] =                                 Location::GSToken(RC_FIRE_TEMPLE_GS_BOULDER_MAZE,                                  RCQUEST_VANILLA,                                                                                                SCENE_FIRE_TEMPLE,                  9220,                         0x04, "GS Boulder Maze",                             RHT_FIRE_TEMPLE_GS_BOULDER_MAZE);
    locationTable[RC_FIRE_TEMPLE_GS_SCARECROW_TOP] =                                Location::GSToken(RC_FIRE_TEMPLE_GS_SCARECROW_TOP,                                 RCQUEST_VANILLA,                                                                                                SCENE_FIRE_TEMPLE,                  9224,                         0x08, "GS Scarecrow Top",                            RHT_FIRE_TEMPLE_GS_SCARECROW_TOP);
    locationTable[RC_FIRE_TEMPLE_GS_SCARECROW_CLIMB] =                              Location::GSToken(RC_FIRE_TEMPLE_GS_SCARECROW_CLIMB,                               RCQUEST_VANILLA,                                                                                                SCENE_FIRE_TEMPLE,                  9232,                         0x10, "GS Scarecrow Climb",                          RHT_FIRE_TEMPLE_GS_SCARECROW_CLIMB);
    locationTable[RC_FIRE_TEMPLE_MQ_GS_ABOVE_FIRE_MAZE] =                      Location::GSToken(RC_FIRE_TEMPLE_MQ_GS_ABOVE_FIRE_MAZE,                       RCQUEST_MQ,                                                                                                     SCENE_FIRE_TEMPLE,                  9218,                         0x02, "MQ GS Above Fire Wall Maze",                  RHT_FIRE_TEMPLE_MQ_GS_ABOVE_FIRE_WALL_MAZE);
    locationTable[RC_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_CENTER] =                     Location::GSToken(RC_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_CENTER,                      RCQUEST_MQ,                                                                                                     SCENE_FIRE_TEMPLE,                  9224,                         0x08, "MQ GS Fire Wall Maze Center",                 RHT_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_CENTER);
    locationTable[RC_FIRE_TEMPLE_MQ_GS_BIG_LAVA_ROOM_OPEN_DOOR] =                   Location::GSToken(RC_FIRE_TEMPLE_MQ_GS_BIG_LAVA_ROOM_OPEN_DOOR,                    RCQUEST_MQ,                                                                                                     SCENE_FIRE_TEMPLE,                  9217,                         0x01, "MQ GS Big Lava Room Open Door",               RHT_FIRE_TEMPLE_MQ_GS_BIG_LAVA_ROOM_OPEN_DOOR);
    locationTable[RC_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_SIDE_ROOM] =                  Location::GSToken(RC_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_SIDE_ROOM,                   RCQUEST_MQ,                                                                                                     SCENE_FIRE_TEMPLE,                  9232,                         0x10, "MQ GS Fire Wall Maze Side Room",              RHT_FIRE_TEMPLE_MQ_GS_FIRE_WALL_MAZE_SIDE_ROOM);
    locationTable[RC_FIRE_TEMPLE_MQ_GS_SKULL_ON_FIRE] =                             Location::GSToken(RC_FIRE_TEMPLE_MQ_GS_SKULL_ON_FIRE,                              RCQUEST_MQ,                                                                                                     SCENE_FIRE_TEMPLE,                  9220,                         0x04, "MQ GS Skull on Fire",                         RHT_FIRE_TEMPLE_MQ_GS_SKULL_ON_FIRE);
    // Water Temple
    locationTable[RC_WATER_TEMPLE_GS_BEHIND_GATE] =                                 Location::GSToken(RC_WATER_TEMPLE_GS_BEHIND_GATE,                                  RCQUEST_VANILLA,                                                                                                SCENE_WATER_TEMPLE,                 9473,                         0x01, "GS Behind Gate",                              RHT_WATER_TEMPLE_GS_BEHIND_GATE);
    locationTable[RC_WATER_TEMPLE_GS_FALLING_PLATFORM_ROOM] =                       Location::GSToken(RC_WATER_TEMPLE_GS_FALLING_PLATFORM_ROOM,                        RCQUEST_VANILLA,                                                                                                SCENE_WATER_TEMPLE,                 9474,                         0x02, "GS Falling Platform Room",                    RHT_WATER_TEMPLE_GS_FALLING_PLATFORM_ROOM);
    locationTable[RC_WATER_TEMPLE_GS_CENTRAL_PILLAR] =                              Location::GSToken(RC_WATER_TEMPLE_GS_CENTRAL_PILLAR,                               RCQUEST_VANILLA,                                                                                                SCENE_WATER_TEMPLE,                 9476,                         0x04, "GS Central Pillar",                           RHT_WATER_TEMPLE_GS_CENTRAL_PILLAR);
    locationTable[RC_WATER_TEMPLE_GS_NEAR_BOSS_KEY_CHEST] =                         Location::GSToken(RC_WATER_TEMPLE_GS_NEAR_BOSS_KEY_CHEST,                          RCQUEST_VANILLA,                                                                                                SCENE_WATER_TEMPLE,                 9480,                         0x08, "GS Near Boss Key Chest",                      RHT_WATER_TEMPLE_GS_NEAR_BOSS_KEY_CHEST);
    locationTable[RC_WATER_TEMPLE_GS_RIVER] =                                       Location::GSToken(RC_WATER_TEMPLE_GS_RIVER,                                        RCQUEST_VANILLA,                                                                                                SCENE_WATER_TEMPLE,                 9488,                         0x10, "GS River",                                    RHT_WATER_TEMPLE_GS_RIVER);
    locationTable[RC_WATER_TEMPLE_MQ_GS_BEFORE_UPPER_WATER_SWITCH] =                Location::GSToken(RC_WATER_TEMPLE_MQ_GS_BEFORE_UPPER_WATER_SWITCH,                 RCQUEST_MQ,                                                                                                     SCENE_WATER_TEMPLE,                 9476,                         0x04, "MQ GS Before Upper Water Switch",             RHT_WATER_TEMPLE_MQ_GS_BEFORE_UPPER_WATER_SWITCH);
    locationTable[RC_WATER_TEMPLE_MQ_GS_FREESTANDING_KEY_AREA] =                    Location::GSToken(RC_WATER_TEMPLE_MQ_GS_FREESTANDING_KEY_AREA,                     RCQUEST_MQ,                                                                                                     SCENE_WATER_TEMPLE,                 9480,                         0x08, "MQ GS Freestanding Key Area",                 RHT_WATER_TEMPLE_MQ_GS_FREESTANDING_KEY_AREA);
    locationTable[RC_WATER_TEMPLE_MQ_GS_LIZALFOS_HALLWAY] =                         Location::GSToken(RC_WATER_TEMPLE_MQ_GS_LIZALFOS_HALLWAY,                          RCQUEST_MQ,                                                                                                     SCENE_WATER_TEMPLE,                 9473,                         0x01, "MQ GS Lizalfos Hallway",                      RHT_WATER_TEMPLE_MQ_GS_LIZALFOS_HALLWAY);
    locationTable[RC_WATER_TEMPLE_MQ_GS_RIVER] =                                    Location::GSToken(RC_WATER_TEMPLE_MQ_GS_RIVER,                                     RCQUEST_MQ,                                                                                                     SCENE_WATER_TEMPLE,                 9474,                         0x02, "MQ GS River",                                 RHT_WATER_TEMPLE_MQ_GS_RIVER);
    locationTable[RC_WATER_TEMPLE_MQ_GS_TRIPLE_WALL_TORCH] =                        Location::GSToken(RC_WATER_TEMPLE_MQ_GS_TRIPLE_WALL_TORCH,                         RCQUEST_MQ,                                                                                                     SCENE_WATER_TEMPLE,                 9488,                         0x10, "MQ GS Triple Wall Torch",                     RHT_WATER_TEMPLE_MQ_GS_TRIPLE_WALL_TORCH);
    // Spirit Temple
    locationTable[RC_SPIRIT_TEMPLE_GS_HALL_AFTER_SUN_BLOCK_ROOM] =                  Location::GSToken(RC_SPIRIT_TEMPLE_GS_HALL_AFTER_SUN_BLOCK_ROOM,                   RCQUEST_VANILLA,                                                                                                SCENE_SPIRIT_TEMPLE,                9729,                         0x01, "GS Hall After Sun Block Room",                RHT_SPIRIT_TEMPLE_GS_HALL_AFTER_SUN_BLOCK_ROOM);
    locationTable[RC_SPIRIT_TEMPLE_GS_BOULDER_ROOM] =                               Location::GSToken(RC_SPIRIT_TEMPLE_GS_BOULDER_ROOM,                                RCQUEST_VANILLA,                                                                                                SCENE_SPIRIT_TEMPLE,                9730,                         0x02, "GS Boulder Room",                             RHT_SPIRIT_TEMPLE_GS_BOULDER_ROOM);
    locationTable[RC_SPIRIT_TEMPLE_GS_LOBBY] =                                      Location::GSToken(RC_SPIRIT_TEMPLE_GS_LOBBY,                                       RCQUEST_VANILLA,                                                                                                SCENE_SPIRIT_TEMPLE,                9732,                         0x04, "GS Lobby",                                    RHT_SPIRIT_TEMPLE_GS_LOBBY);
    locationTable[RC_SPIRIT_TEMPLE_GS_SUN_ON_FLOOR_ROOM] =                          Location::GSToken(RC_SPIRIT_TEMPLE_GS_SUN_ON_FLOOR_ROOM,                           RCQUEST_VANILLA,                                                                                                SCENE_SPIRIT_TEMPLE,                9736,                         0x08, "GS Sun on Floor Room",                        RHT_SPIRIT_TEMPLE_GS_SUN_ON_FLOOR_ROOM);
    locationTable[RC_SPIRIT_TEMPLE_GS_METAL_FENCE] =                                Location::GSToken(RC_SPIRIT_TEMPLE_GS_METAL_FENCE,                                 RCQUEST_VANILLA,                                                                                                SCENE_SPIRIT_TEMPLE,                9744,                         0x10, "GS Metal Fence",                              RHT_SPIRIT_TEMPLE_GS_METAL_FENCE);
    locationTable[RC_SPIRIT_TEMPLE_MQ_GS_SYMPHONY_ROOM] =                           Location::GSToken(RC_SPIRIT_TEMPLE_MQ_GS_SYMPHONY_ROOM,                            RCQUEST_MQ,                                                                                                     SCENE_SPIRIT_TEMPLE,                9736,                         0x08, "MQ GS Symphony Room",                         RHT_SPIRIT_TEMPLE_MQ_GS_SYMPHONY_ROOM);
    locationTable[RC_SPIRIT_TEMPLE_MQ_GS_LEEVER_ROOM] =                             Location::GSToken(RC_SPIRIT_TEMPLE_MQ_GS_LEEVER_ROOM,                              RCQUEST_MQ,                                                                                                     SCENE_SPIRIT_TEMPLE,                9730,                         0x02, "MQ GS Leever Room",                           RHT_SPIRIT_TEMPLE_MQ_GS_LEEVER_ROOM);
    locationTable[RC_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_WEST] =                  Location::GSToken(RC_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_WEST,                   RCQUEST_MQ,                                                                                                     SCENE_SPIRIT_TEMPLE,                9732,                         0x04, "MQ GS Nine Thrones Room West",                RHT_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_WEST);
    locationTable[RC_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_NORTH] =                 Location::GSToken(RC_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_NORTH,                  RCQUEST_MQ,                                                                                                     SCENE_SPIRIT_TEMPLE,                9744,                         0x10, "MQ GS Nine Thrones Room North",               RHT_SPIRIT_TEMPLE_MQ_GS_NINE_THRONES_ROOM_NORTH);
    locationTable[RC_SPIRIT_TEMPLE_MQ_GS_SUN_BLOCK_ROOM] =                          Location::GSToken(RC_SPIRIT_TEMPLE_MQ_GS_SUN_BLOCK_ROOM,                           RCQUEST_MQ,                                                                                                     SCENE_SPIRIT_TEMPLE,                9729,                         0x01, "MQ GS Sun Block Room",                        RHT_SPIRIT_TEMPLE_MQ_GS_SUN_BLOCK_ROOM);
    // Shadow Temple
    locationTable[RC_SHADOW_TEMPLE_GS_SINGLE_GIANT_POT] =                           Location::GSToken(RC_SHADOW_TEMPLE_GS_SINGLE_GIANT_POT,                            RCQUEST_VANILLA,                                                                                                SCENE_SHADOW_TEMPLE,                9985,                         0x01, "GS Single Giant Pot",                         RHT_SHADOW_TEMPLE_GS_SINGLE_GIANT_POT);
    locationTable[RC_SHADOW_TEMPLE_GS_FALLING_SPIKES_ROOM] =                        Location::GSToken(RC_SHADOW_TEMPLE_GS_FALLING_SPIKES_ROOM,                         RCQUEST_VANILLA,                                                                                                SCENE_SHADOW_TEMPLE,                9986,                         0x02, "GS Falling Spikes Room",                      RHT_SHADOW_TEMPLE_GS_FALLING_SPIKES_ROOM);
    locationTable[RC_SHADOW_TEMPLE_GS_TRIPLE_GIANT_POT] =                           Location::GSToken(RC_SHADOW_TEMPLE_GS_TRIPLE_GIANT_POT,                            RCQUEST_VANILLA,                                                                                                SCENE_SHADOW_TEMPLE,                9988,                         0x04, "GS Triple Giant Pot",                         RHT_SHADOW_TEMPLE_GS_TRIPLE_GIANT_POT);
    locationTable[RC_SHADOW_TEMPLE_GS_LIKE_LIKE_ROOM] =                             Location::GSToken(RC_SHADOW_TEMPLE_GS_LIKE_LIKE_ROOM,                              RCQUEST_VANILLA,                                                                                                SCENE_SHADOW_TEMPLE,                9992,                         0x08, "GS Like Like Room",                           RHT_SHADOW_TEMPLE_GS_LIKE_LIKE_ROOM);
    locationTable[RC_SHADOW_TEMPLE_GS_NEAR_SHIP] =                                  Location::GSToken(RC_SHADOW_TEMPLE_GS_NEAR_SHIP,                                   RCQUEST_VANILLA,                                                                                                SCENE_SHADOW_TEMPLE,                10000,                        0x10, "GS Near Ship",                                RHT_SHADOW_TEMPLE_GS_NEAR_SHIP);
    locationTable[RC_SHADOW_TEMPLE_MQ_GS_FALLING_SPIKES_ROOM] =                     Location::GSToken(RC_SHADOW_TEMPLE_MQ_GS_FALLING_SPIKES_ROOM,                      RCQUEST_MQ,                                                                                                     SCENE_SHADOW_TEMPLE,                9986,                         0x02, "MQ GS Falling Spikes Room",                   RHT_SHADOW_TEMPLE_MQ_GS_FALLING_SPIKES_ROOM);
    locationTable[RC_SHADOW_TEMPLE_MQ_GS_WIND_HINT_ROOM] =                          Location::GSToken(RC_SHADOW_TEMPLE_MQ_GS_WIND_HINT_ROOM,                           RCQUEST_MQ,                                                                                                     SCENE_SHADOW_TEMPLE,                9985,                         0x01, "MQ GS Wind Hint Room",                        RHT_SHADOW_TEMPLE_MQ_GS_WIND_HINT_ROOM);
    locationTable[RC_SHADOW_TEMPLE_MQ_GS_AFTER_WIND] =                              Location::GSToken(RC_SHADOW_TEMPLE_MQ_GS_AFTER_WIND,                               RCQUEST_MQ,                                                                                                     SCENE_SHADOW_TEMPLE,                9992,                         0x08, "MQ GS After Wind",                            RHT_SHADOW_TEMPLE_MQ_GS_AFTER_WIND);
    locationTable[RC_SHADOW_TEMPLE_MQ_GS_AFTER_SHIP] =                              Location::GSToken(RC_SHADOW_TEMPLE_MQ_GS_AFTER_SHIP,                               RCQUEST_MQ,                                                                                                     SCENE_SHADOW_TEMPLE,                10000,                        0x10, "MQ GS After Ship",                            RHT_SHADOW_TEMPLE_MQ_GS_AFTER_SHIP);
    locationTable[RC_SHADOW_TEMPLE_MQ_GS_NEAR_BOSS] =                               Location::GSToken(RC_SHADOW_TEMPLE_MQ_GS_NEAR_BOSS,                                RCQUEST_MQ,                                                                                                     SCENE_SHADOW_TEMPLE,                9988,                         0x04, "MQ GS Near Boss",                             RHT_SHADOW_TEMPLE_MQ_GS_NEAR_BOSS);
    // Bottom of the Well
    locationTable[RC_BOTTOM_OF_THE_WELL_GS_LIKE_LIKE_CAGE] =                        Location::GSToken(RC_BOTTOM_OF_THE_WELL_GS_LIKE_LIKE_CAGE,                         RCQUEST_VANILLA,                                                                                                SCENE_BOTTOM_OF_THE_WELL,           10241,                        0x01, "GS Like Like Cage",                           RHT_BOTTOM_OF_THE_WELL_GS_LIKE_LIKE_CAGE);
    locationTable[RC_BOTTOM_OF_THE_WELL_GS_EAST_INNER_ROOM] =                       Location::GSToken(RC_BOTTOM_OF_THE_WELL_GS_EAST_INNER_ROOM,                        RCQUEST_VANILLA,                                                                                                SCENE_BOTTOM_OF_THE_WELL,           10242,                        0x02, "GS East Inner Room",                          RHT_BOTTOM_OF_THE_WELL_GS_EAST_INNER_ROOM);
    locationTable[RC_BOTTOM_OF_THE_WELL_GS_WEST_INNER_ROOM] =                       Location::GSToken(RC_BOTTOM_OF_THE_WELL_GS_WEST_INNER_ROOM,                        RCQUEST_VANILLA,                                                                                                SCENE_BOTTOM_OF_THE_WELL,           10244,                        0x04, "GS West Inner Room",                          RHT_BOTTOM_OF_THE_WELL_GS_WEST_INNER_ROOM);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_GS_BASEMENT] =                           Location::GSToken(RC_BOTTOM_OF_THE_WELL_MQ_GS_BASEMENT,                            RCQUEST_MQ,                                                                                                     SCENE_BOTTOM_OF_THE_WELL,           10241,                        0x01, "MQ GS Basement",                              RHT_BOTTOM_OF_THE_WELL_MQ_GS_BASEMENT);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_GS_COFFIN_ROOM] =                        Location::GSToken(RC_BOTTOM_OF_THE_WELL_MQ_GS_COFFIN_ROOM,                         RCQUEST_MQ,                                                                                                     SCENE_BOTTOM_OF_THE_WELL,           10244,                        0x04, "MQ GS Coffin Room",                           RHT_BOTTOM_OF_THE_WELL_MQ_GS_COFFIN_ROOM);
    locationTable[RC_BOTTOM_OF_THE_WELL_MQ_GS_WEST_INNER_ROOM] =                    Location::GSToken(RC_BOTTOM_OF_THE_WELL_MQ_GS_WEST_INNER_ROOM,                     RCQUEST_MQ,                                                                                                     SCENE_BOTTOM_OF_THE_WELL,           10242,                        0x02, "MQ GS West Inner Room",                       RHT_BOTTOM_OF_THE_WELL_MQ_GS_WEST_INNER_ROOM);
    // Ice Cavern
    locationTable[RC_ICE_CAVERN_GS_PUSH_BLOCK_ROOM] =                               Location::GSToken(RC_ICE_CAVERN_GS_PUSH_BLOCK_ROOM,                                RCQUEST_VANILLA,                                                                                                SCENE_ICE_CAVERN,                   10497,                        0x01, "GS Push Block Room",                          RHT_ICE_CAVERN_GS_PUSH_BLOCK_ROOM);
    locationTable[RC_ICE_CAVERN_GS_SPINNING_SCYTHE_ROOM] =                          Location::GSToken(RC_ICE_CAVERN_GS_SPINNING_SCYTHE_ROOM,                           RCQUEST_VANILLA,                                                                                                SCENE_ICE_CAVERN,                   10498,                        0x02, "GS Spinning Scythe Room",                     RHT_ICE_CAVERN_GS_SPINNING_SCYTHE_ROOM);
    locationTable[RC_ICE_CAVERN_GS_HEART_PIECE_ROOM] =                              Location::GSToken(RC_ICE_CAVERN_GS_HEART_PIECE_ROOM,                               RCQUEST_VANILLA,                                                                                                SCENE_ICE_CAVERN,                   10500,                        0x04, "GS Heart Piece Room",                         RHT_ICE_CAVERN_GS_HEART_PIECE_ROOM);
    locationTable[RC_ICE_CAVERN_MQ_GS_SCARECROW] =                                  Location::GSToken(RC_ICE_CAVERN_MQ_GS_SCARECROW,                                   RCQUEST_MQ,                                                                                                     SCENE_ICE_CAVERN,                   10497,                        0x01, "MQ GS Scarecrow",                             RHT_ICE_CAVERN_MQ_GS_SCARECROW);
    locationTable[RC_ICE_CAVERN_MQ_GS_ICE_BLOCK] =                                  Location::GSToken(RC_ICE_CAVERN_MQ_GS_ICE_BLOCK,                                   RCQUEST_MQ,                                                                                                     SCENE_ICE_CAVERN,                   10500,                        0x04, "MQ GS Ice Block",                             RHT_ICE_CAVERN_MQ_GS_ICE_BLOCK);
    locationTable[RC_ICE_CAVERN_MQ_GS_RED_ICE] =                                    Location::GSToken(RC_ICE_CAVERN_MQ_GS_RED_ICE,                                     RCQUEST_MQ,                                                                                                     SCENE_ICE_CAVERN,                   10498,                        0x02, "MQ GS Red Ice",                               RHT_ICE_CAVERN_MQ_GS_RED_ICE);
    // Overworld
    // Kokiri Forest
    locationTable[RC_KF_GS_BEAN_PATCH] =                                            Location::GSToken(RC_KF_GS_BEAN_PATCH,                                             RCQUEST_BOTH,                                                                                                   SCENE_KOKIRI_FOREST,                27649,                        0x01, "GS Bean Patch",                               RHT_KF_GS_BEAN_PATCH,                                                                                  0x0C);
    locationTable[RC_KF_GS_KNOW_IT_ALL_HOUSE] =                                     Location::GSToken(RC_KF_GS_KNOW_IT_ALL_HOUSE,                                      RCQUEST_BOTH,                                                                                                   SCENE_KOKIRI_FOREST,                19458,                        0x02, "GS Know It All House",                        RHT_KF_GS_KNOW_IT_ALL_HOUSE,                                                                           0x0C);
    locationTable[RC_KF_GS_HOUSE_OF_TWINS] =                                        Location::GSToken(RC_KF_GS_HOUSE_OF_TWINS,                                         RCQUEST_BOTH,                                                                                                   SCENE_KOKIRI_FOREST,                19460,                        0x04, "GS House of Twins",                           RHT_KF_GS_HOUSE_OF_TWINS,                                                                              0x0C);
    // Lost Woods
    locationTable[RC_LW_GS_BEAN_PATCH_NEAR_BRIDGE] =                                Location::GSToken(RC_LW_GS_BEAN_PATCH_NEAR_BRIDGE,                                 RCQUEST_BOTH,                                                                                                   SCENE_LOST_WOODS,                   27905,                        0x01, "GS Bean Patch Near Bridge",                   RHT_LW_GS_BEAN_PATCH_NEAR_BRIDGE,                                                                      0x0D);
    locationTable[RC_LW_GS_BEAN_PATCH_NEAR_THEATER] =                               Location::GSToken(RC_LW_GS_BEAN_PATCH_NEAR_THEATER,                                RCQUEST_BOTH,                                                                                                   SCENE_LOST_WOODS,                   27906,                        0x02, "GS Bean Patch Near Theater",                  RHT_LW_GS_BEAN_PATCH_NEAR_THEATER,                                                                     0x0D);
    locationTable[RC_LW_GS_ABOVE_THEATER] =                                         Location::GSToken(RC_LW_GS_ABOVE_THEATER,                                          RCQUEST_BOTH,                                                                                                   SCENE_LOST_WOODS,                   19716,                        0x04, "GS Above Theater",                            RHT_LW_GS_ABOVE_THEATER,                                                                               0x0D);
    locationTable[RC_SFM_GS] =                                                      Location::GSToken(RC_SFM_GS,                                                       RCQUEST_BOTH,                                                                                                   SCENE_SACRED_FOREST_MEADOW,         19720,                        0x08, "GS",                                          RHT_SFM_GS,                                                                                            0x0D);
    // Hyrule Field
    locationTable[RC_HF_GS_COW_GROTTO] =                                            Location::GSToken(RC_HF_GS_COW_GROTTO,                                             RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                       SCENE_GROTTOS,                      10753,                        0x01, "GS Cow Grotto",                               RHT_HF_GS_COW_GROTTO,                                                                                  0x0A);
    locationTable[RC_HF_GS_NEAR_KAK_GROTTO] =                                       Location::GSToken(RC_HF_GS_NEAR_KAK_GROTTO,                                        RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                       SCENE_GROTTOS,                      10754,                        0x02, "GS Near Kak Grotto",                          RHT_HF_GS_NEAR_KAK_GROTTO,                                                                             0x0A);
    // Lake Hylia
    locationTable[RC_LH_GS_BEAN_PATCH] =                                            Location::GSToken(RC_LH_GS_BEAN_PATCH,                                             RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   29185,                        0x01, "GS Bean Patch",                               RHT_LH_GS_BEAN_PATCH,                                                                                  0x12);
    locationTable[RC_LH_GS_SMALL_ISLAND] =                                          Location::GSToken(RC_LH_GS_SMALL_ISLAND,                                           RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   20994,                        0x02, "GS Small Island",                             RHT_LH_GS_SMALL_ISLAND,                                                                                0x12);
    locationTable[RC_LH_GS_LAB_WALL] =                                              Location::GSToken(RC_LH_GS_LAB_WALL,                                               RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   20996,                        0x04, "GS Lab Wall",                                 RHT_LH_GS_LAB_WALL,                                                                                    0x12);
    locationTable[RC_LH_GS_LAB_CRATE] =                                             Location::GSToken(RC_LH_GS_LAB_CRATE,                                              RCQUEST_BOTH,                                                                                                   SCENE_LAKESIDE_LABORATORY,          -28152,                       0x08, "GS Lab Crate",                                RHT_LH_GS_LAB_CRATE,                                                                                   0x12);
    locationTable[RC_LH_GS_TREE] =                                                  Location::GSToken(RC_LH_GS_TREE,                                                   RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   21008,                        0x10, "GS Tree",                                     RHT_LH_GS_TREE,                                                                                        0x12);
    // Gerudo Valley
    locationTable[RC_GV_GS_BEAN_PATCH] =                                            Location::GSToken(RC_GV_GS_BEAN_PATCH,                                             RCQUEST_BOTH,                                                                                                   SCENE_GERUDO_VALLEY,                29441,                        0x01, "GS Bean Patch",                               RHT_GV_GS_BEAN_PATCH,                                                                                  0x13);
    locationTable[RC_GV_GS_SMALL_BRIDGE] =                                          Location::GSToken(RC_GV_GS_SMALL_BRIDGE,                                           RCQUEST_BOTH,                                                                                                   SCENE_GERUDO_VALLEY,                21250,                        0x02, "GS Small Bridge",                             RHT_GV_GS_SMALL_BRIDGE,                                                                                0x13);
    locationTable[RC_GV_GS_PILLAR] =                                                Location::GSToken(RC_GV_GS_PILLAR,                                                 RCQUEST_BOTH,                                                                                                   SCENE_GERUDO_VALLEY,                21252,                        0x04, "GS Pillar",                                   RHT_GV_GS_PILLAR,                                                                                      0x13);
    locationTable[RC_GV_GS_BEHIND_TENT] =                                           Location::GSToken(RC_GV_GS_BEHIND_TENT,                                            RCQUEST_BOTH,                                                                                                   SCENE_GERUDO_VALLEY,                21256,                        0x08, "GS Behind Tent",                              RHT_GV_GS_BEHIND_TENT,                                                                                 0x13);
    // Gerudo Fortress
    locationTable[RC_GF_GS_ARCHERY_RANGE] =                                         Location::GSToken(RC_GF_GS_ARCHERY_RANGE,                                          RCQUEST_BOTH,                                                                                                   SCENE_GERUDOS_FORTRESS,             21505,                        0x01, "GS Archery Range",                            RHT_GF_GS_ARCHERY_RANGE,                                                                               0x14);
    locationTable[RC_GF_GS_TOP_FLOOR] =                                             Location::GSToken(RC_GF_GS_TOP_FLOOR,                                              RCQUEST_BOTH,                                                                                                   SCENE_GERUDOS_FORTRESS,             21506,                        0x02, "GS Top Floor",                                RHT_GF_GS_TOP_FLOOR,                                                                                   0x14);
    // Wasteland & Desert Colossus
    locationTable[RC_WASTELAND_GS] =                                                Location::GSToken(RC_WASTELAND_GS,                                                 RCQUEST_BOTH,                                                                                                   SCENE_HAUNTED_WASTELAND,            13570,                        0x02, "GS",                                          RHT_WASTELAND_GS,                                                                                      0x15);
    locationTable[RC_COLOSSUS_GS_BEAN_PATCH] =                                      Location::GSToken(RC_COLOSSUS_GS_BEAN_PATCH,                                       RCQUEST_BOTH,                                                                                                   SCENE_DESERT_COLOSSUS,              29953,                        0x01, "GS Bean Patch",                               RHT_COLOSSUS_GS_BEAN_PATCH,                                                                            0x15);
    locationTable[RC_COLOSSUS_GS_HILL] =                                            Location::GSToken(RC_COLOSSUS_GS_HILL,                                             RCQUEST_BOTH,                                                                                                   SCENE_DESERT_COLOSSUS,              21764,                        0x04, "GS Hill",                                     RHT_COLOSSUS_GS_HILL,                                                                                  0x15);
    locationTable[RC_COLOSSUS_GS_TREE] =                                            Location::GSToken(RC_COLOSSUS_GS_TREE,                                             RCQUEST_BOTH,                                                                                                   SCENE_DESERT_COLOSSUS,              21768,                        0x08, "GS Tree",                                     RHT_COLOSSUS_GS_TREE,                                                                                  0x15);
    // Hyrule Castle, Market, and Outside Ganon's Castle
    locationTable[RC_OGC_GS] =                                                      Location::GSToken(RC_OGC_GS,                                                       RCQUEST_BOTH,                                                                                                   SCENE_OUTSIDE_GANONS_CASTLE,        11777,                        0x01, "OGC GS",                                      RHT_OGC_GS,                                                                                            0x0E);
    locationTable[RC_HC_GS_STORMS_GROTTO] =                                         Location::GSToken(RC_HC_GS_STORMS_GROTTO,                                          RCQUEST_BOTH,                                        RCAREA_HYRULE_CASTLE,                                      SCENE_GROTTOS,                      11778,                        0x02, "GS Storms Grotto",                            RHT_HC_GS_STORMS_GROTTO,                                                                               0x0E);
    locationTable[RC_HC_GS_TREE] =                                                  Location::GSToken(RC_HC_GS_TREE,                                                   RCQUEST_BOTH,                                                                                                   SCENE_HYRULE_CASTLE,                -29180,                       0x04, "GS Tree",                                     RHT_HC_GS_TREE,                                                                                        0x0E);
    locationTable[RC_MARKET_GS_GUARD_HOUSE] =                                       Location::GSToken(RC_MARKET_GS_GUARD_HOUSE,                                        RCQUEST_BOTH,                                                                                                   SCENE_MARKET_GUARD_HOUSE,           -29176,                       0x08, "Market GS Guard House",                       RHT_MARKET_GS_GUARD_HOUSE,                                                                             0x0E);
    // Kakariko
    locationTable[RC_KAK_GS_HOUSE_UNDER_CONSTRUCTION] =                             Location::GSToken(RC_KAK_GS_HOUSE_UNDER_CONSTRUCTION,                              RCQUEST_BOTH,                                                                                                   SCENE_KAKARIKO_VILLAGE,             20488,                        0x08, "GS House Under Construction",                 RHT_KAK_GS_HOUSE_UNDER_CONSTRUCTION,                                                                   0x10);
    locationTable[RC_KAK_GS_SKULLTULA_HOUSE] =                                      Location::GSToken(RC_KAK_GS_SKULLTULA_HOUSE,                                       RCQUEST_BOTH,                                                                                                   SCENE_KAKARIKO_VILLAGE,             20496,                        0x10, "GS Skulltula House",                          RHT_KAK_GS_SKULLTULA_HOUSE,                                                                            0x10);
    locationTable[RC_KAK_GS_GUARDS_HOUSE] =                                         Location::GSToken(RC_KAK_GS_GUARDS_HOUSE,                                          RCQUEST_BOTH,                                                                                                   SCENE_KAKARIKO_VILLAGE,             20482,                        0x02, "GS Guards House",                             RHT_KAK_GS_GUARDS_HOUSE,                                                                               0x10);
    locationTable[RC_KAK_GS_TREE] =                                                 Location::GSToken(RC_KAK_GS_TREE,                                                  RCQUEST_BOTH,                                                                                                   SCENE_KAKARIKO_VILLAGE,             -28640,                       0x20, "GS Tree",                                     RHT_KAK_GS_TREE,                                                                                       0x10);
    locationTable[RC_KAK_GS_WATCHTOWER] =                                           Location::GSToken(RC_KAK_GS_WATCHTOWER,                                            RCQUEST_BOTH,                                                                                                   SCENE_KAKARIKO_VILLAGE,             20484,                        0x04, "GS Watchtower",                               RHT_KAK_GS_WATCHTOWER,                                                                                 0x10);
    locationTable[RC_KAK_GS_ABOVE_IMPAS_HOUSE] =                                    Location::GSToken(RC_KAK_GS_ABOVE_IMPAS_HOUSE,                                     RCQUEST_BOTH,                                                                                                   SCENE_KAKARIKO_VILLAGE,             20544,                        0x40, "GS Above Impas House",                        RHT_KAK_GS_ABOVE_IMPAS_HOUSE,                                                                          0x10);
    // Graveyard
    locationTable[RC_GRAVEYARD_GS_WALL] =                                           Location::GSToken(RC_GRAVEYARD_GS_WALL,                                            RCQUEST_BOTH,                                                                                                   SCENE_GRAVEYARD,                    20608,                        0x80, "GS Wall",                                     RHT_GRAVEYARD_GS_WALL,                                                                                 0x10);
    locationTable[RC_GRAVEYARD_GS_BEAN_PATCH] =                                     Location::GSToken(RC_GRAVEYARD_GS_BEAN_PATCH,                                      RCQUEST_BOTH,                                                                                                   SCENE_GRAVEYARD,                    28673,                        0x01, "GS Bean Patch",                               RHT_GRAVEYARD_GS_BEAN_PATCH,                                                                           0x10);
    // Death Mountain
    locationTable[RC_DMC_GS_BEAN_PATCH] =                                           Location::GSToken(RC_DMC_GS_BEAN_PATCH,                                            RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_CRATER,        28417,                        0x01, "GS Bean Patch",                               RHT_DMC_GS_BEAN_PATCH,                                                                                 0x0F);
    locationTable[RC_DMC_GS_CRATE] =                                                Location::GSToken(RC_DMC_GS_CRATE,                                                 RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_CRATER,        -28800,                       0x80, "GS Crate",                                    RHT_DMC_GS_CRATE,                                                                                      0x0F);
    locationTable[RC_DMT_GS_BEAN_PATCH] =                                           Location::GSToken(RC_DMT_GS_BEAN_PATCH,                                            RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_TRAIL,         28418,                        0x02, "GS Bean Patch",                               RHT_DMT_GS_BEAN_PATCH,                                                                                 0x0F);
    locationTable[RC_DMT_GS_NEAR_KAK] =                                             Location::GSToken(RC_DMT_GS_NEAR_KAK,                                              RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_TRAIL,         12036,                        0x04, "GS Near Kak",                                 RHT_DMT_GS_NEAR_KAK,                                                                                   0x0F);
    locationTable[RC_DMT_GS_ABOVE_DODONGOS_CAVERN] =                                Location::GSToken(RC_DMT_GS_ABOVE_DODONGOS_CAVERN,                                 RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_TRAIL,         20232,                        0x08, "GS Above Dodongos Cavern",                    RHT_DMT_GS_ABOVE_DODONGOS_CAVERN,                                                                      0x0F);
    locationTable[RC_DMT_GS_FALLING_ROCKS_PATH] =                                   Location::GSToken(RC_DMT_GS_FALLING_ROCKS_PATH,                                    RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_TRAIL,         20240,                        0x10, "GS Falling Rocks Path",                       RHT_DMT_GS_FALLING_ROCKS_PATH,                                                                         0x0F);
    locationTable[RC_GC_GS_CENTER_PLATFORM] =                                       Location::GSToken(RC_GC_GS_CENTER_PLATFORM,                                        RCQUEST_BOTH,                                                                                                   SCENE_GORON_CITY,                   12064,                        0x20, "GS Center Platform",                          RHT_GC_GS_CENTER_PLATFORM,                                                                             0x0F);
    locationTable[RC_GC_GS_BOULDER_MAZE] =                                          Location::GSToken(RC_GC_GS_BOULDER_MAZE,                                           RCQUEST_BOTH,                                                                                                   SCENE_GORON_CITY,                   -28864,                       0x40, "GS Boulder Maze",                             RHT_GC_GS_BOULDER_MAZE,                                                                                0x0F);
    // Zora's River, Domain, and Fountain
    locationTable[RC_ZR_GS_LADDER] =                                                Location::GSToken(RC_ZR_GS_LADDER,                                                 RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_RIVER,                  20737,                        0x01, "GS Ladder",                                   RHT_ZR_GS_LADDER,                                                                                      0x11);
    locationTable[RC_ZR_GS_TREE] =                                                  Location::GSToken(RC_ZR_GS_TREE,                                                   RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_RIVER,                  -28414,                       0x02, "GS Tree",                                     RHT_ZR_GS_TREE,                                                                                        0x11);
    locationTable[RC_ZR_GS_ABOVE_BRIDGE] =                                          Location::GSToken(RC_ZR_GS_ABOVE_BRIDGE,                                           RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_RIVER,                  20744,                        0x08, "GS Above Bridge",                             RHT_ZR_GS_ABOVE_BRIDGE,                                                                                0x11);
    locationTable[RC_ZR_GS_NEAR_RAISED_GROTTOS] =                                   Location::GSToken(RC_ZR_GS_NEAR_RAISED_GROTTOS,                                    RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_RIVER,                  20752,                        0x10, "GS Near Raised Grottos",                      RHT_ZR_GS_NEAR_RAISED_GROTTOS,                                                                         0x11);
    locationTable[RC_ZD_GS_FROZEN_WATERFALL] =                                      Location::GSToken(RC_ZD_GS_FROZEN_WATERFALL,                                       RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_DOMAIN,                 20800,                        0x40, "GS Frozen Waterfall",                         RHT_ZD_GS_FROZEN_WATERFALL,                                                                            0x11);
    locationTable[RC_ZF_GS_ABOVE_THE_LOG] =                                         Location::GSToken(RC_ZF_GS_ABOVE_THE_LOG,                                          RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_FOUNTAIN,               20740,                        0x04, "GS Above The Log",                            RHT_ZF_GS_ABOVE_THE_LOG,                                                                               0x11);
    locationTable[RC_ZF_GS_HIDDEN_CAVE] =                                           Location::GSToken(RC_ZF_GS_HIDDEN_CAVE,                                            RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_FOUNTAIN,               20768,                        0x20, "GS Hidden Cave",                              RHT_ZF_GS_HIDDEN_CAVE,                                                                                 0x11);
    locationTable[RC_ZF_GS_TREE] =                                                  Location::GSToken(RC_ZF_GS_TREE,                                                   RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_FOUNTAIN,               -28288,                       0x80, "GS Tree",                                     RHT_ZF_GS_TREE,                                                                                        0x11);
    // Lon Lon Ranch
    locationTable[RC_LLR_GS_BACK_WALL] =                                            Location::GSToken(RC_LLR_GS_BACK_WALL,                                             RCQUEST_BOTH,                                                                                                   SCENE_LON_LON_RANCH,                11009,                        0x01, "GS Back Wall",                                RHT_LLR_GS_BACK_WALL,                                                                                  0x0B);
    locationTable[RC_LLR_GS_RAIN_SHED] =                                            Location::GSToken(RC_LLR_GS_RAIN_SHED,                                             RCQUEST_BOTH,                                                                                                   SCENE_LON_LON_RANCH,                11010,                        0x02, "GS Rain Shed",                                RHT_LLR_GS_RAIN_SHED,                                                                                  0x0B);
    locationTable[RC_LLR_GS_HOUSE_WINDOW] =                                         Location::GSToken(RC_LLR_GS_HOUSE_WINDOW,                                          RCQUEST_BOTH,                                                                                                   SCENE_LON_LON_RANCH,                11012,                        0x04, "GS House Window",                             RHT_LLR_GS_HOUSE_WINDOW,                                                                               0x0B);
    locationTable[RC_LLR_GS_TREE] =                                                 Location::GSToken(RC_LLR_GS_TREE,                                                  RCQUEST_BOTH,                                                                                                   SCENE_LON_LON_RANCH,                -29944,                       0x08, "GS Tree",                                     RHT_LLR_GS_TREE,                                                                                       0x0B);

    // Bosses
    locationTable[RC_LINKS_POCKET] =                                                   Location::Base(RC_LINKS_POCKET,                                                 RCQUEST_BOTH,    RCTYPE_LINKS_POCKET,                RCAREA_KOKIRI_FOREST,                ACTOR_ID_MAX,         SCENE_ID_MAX,                       0x00,                               "Link's Pocket",   "Link's Pocket",            RHT_LINKS_POCKET,                                                RG_NONE,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_LINKS_POCKET),                                                                 true);
    locationTable[RC_QUEEN_GOHMA] =                                                    Location::Base(RC_QUEEN_GOHMA,                                                  RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_DEKU_TREE_BOSS,               0x00,                               "Queen Gohma",     "Queen Gohma",              RHT_QUEEN_GOHMA,                                                 RG_KOKIRI_EMERALD,                                                  SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_USED_DEKU_TREE_BLUE_WARP),                                                    true);
    locationTable[RC_KING_DODONGO] =                                                   Location::Base(RC_KING_DODONGO,                                                 RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_DODONGOS_CAVERN_BOSS,         0x00,                               "King Dodongo",    "King Dodongo",             RHT_KING_DODONGO,                                                RG_GORON_RUBY,                                                      SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_USED_DODONGOS_CAVERN_BLUE_WARP),                                              true);
    locationTable[RC_BARINADE] =                                                       Location::Base(RC_BARINADE,                                                     RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_JABU_JABU_BOSS,               0x00,                               "Barinade",        "Barinade",                 RHT_BARINADE,                                                    RG_ZORA_SAPPHIRE,                                                   SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_USED_JABU_JABUS_BELLY_BLUE_WARP),                                             true);
    locationTable[RC_PHANTOM_GANON] =                                                  Location::Base(RC_PHANTOM_GANON,                                                RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_FOREST_TEMPLE_BOSS,           0x00,                               "Phantom Ganon",   "Phantom Ganon",            RHT_PHANTOM_GANON,                                               RG_FOREST_MEDALLION,                                                SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_USED_FOREST_TEMPLE_BLUE_WARP),                                                true);
    locationTable[RC_VOLVAGIA] =                                                       Location::Base(RC_VOLVAGIA,                                                     RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_FIRE_TEMPLE_BOSS,             0x00,                               "Volvagia",        "Volvagia",                 RHT_VOLVAGIA,                                                    RG_FIRE_MEDALLION,                                                  SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_USED_FIRE_TEMPLE_BLUE_WARP),                                                  true);
    locationTable[RC_MORPHA] =                                                         Location::Base(RC_MORPHA,                                                       RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_WATER_TEMPLE_BOSS,            0x00,                               "Morpha",          "Morpha",                   RHT_MORPHA,                                                      RG_WATER_MEDALLION,                                                 SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_USED_WATER_TEMPLE_BLUE_WARP),                                                 true);
    locationTable[RC_TWINROVA] =                                                       Location::Base(RC_TWINROVA,                                                     RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_SPIRIT_TEMPLE_BOSS,           0x00,                               "Twinrova",        "Twinrova",                 RHT_TWINROVA,                                                    RG_SPIRIT_MEDALLION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_DUNGEONS_DONE_SPIRIT_TEMPLE),                                                  true);
    locationTable[RC_BONGO_BONGO] =                                                    Location::Base(RC_BONGO_BONGO,                                                  RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_SHADOW_TEMPLE_BOSS,           0x00,                               "Bongo Bongo",     "Bongo Bongo",              RHT_BONGO_BONGO,                                                 RG_SHADOW_MEDALLION,                                                SpoilerCollectionCheck::RandomizerInf(RAND_INF_DUNGEONS_DONE_SHADOW_TEMPLE),                                                  true);
    locationTable[RC_GANON] =                                                          Location::Base(RC_GANON,                                                        RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,                                                   ACTOR_DOOR_WARP1,     SCENE_GANON_BOSS,                   0x00,                               "Ganon",           "Ganon",                    RHT_NONE,                                                        RG_TRIFORCE);
    locationTable[RC_GIFT_FROM_RAURU] =                                                Location::Base(RC_GIFT_FROM_RAURU,                                              RCQUEST_BOTH,    RCTYPE_DUNGEON_REWARD,              RCAREA_MARKET,                       ACTOR_ID_MAX,         SCENE_ID_MAX,                       0x00,                               "Gift from Rauru", "Gift from Rauru",          RHT_GIFT_FROM_RAURU,                                             RG_LIGHT_MEDALLION,                                                 SpoilerCollectionCheck::EventChkInf(0xC9),                                                                                    true);

    // Heart Containers
    locationTable[RC_DEKU_TREE_QUEEN_GOHMA_HEART] =                                    Location::Base(RC_DEKU_TREE_QUEEN_GOHMA_HEART,                                  RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_DEKU_TREE_BOSS,               0x00,                               "Queen Gohma Heart Container",                 RHT_DEKU_TREE_QUEEN_GOHMA_HEART,                                 RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x11, 0x1F),                                                                              true);
    locationTable[RC_DODONGOS_CAVERN_KING_DODONGO_HEART] =                             Location::Base(RC_DODONGOS_CAVERN_KING_DODONGO_HEART,                           RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_DODONGOS_CAVERN_BOSS,         0x00,                               "King Dodongo Heart Container",                RHT_DODONGOS_CAVERN_KING_DODONGO_HEART,                          RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x12, 0x1F),                                                                              true);
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_HEART] =                                Location::Base(RC_JABU_JABUS_BELLY_BARINADE_HEART,                              RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_JABU_JABU_BOSS,               0x00,                               "Barinade Heart Container",                    RHT_JABU_JABUS_BELLY_BARINADE_HEART,                             RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x13, 0x1F),                                                                              true);
    locationTable[RC_FOREST_TEMPLE_PHANTOM_GANON_HEART] =                              Location::Base(RC_FOREST_TEMPLE_PHANTOM_GANON_HEART,                            RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_FOREST_TEMPLE_BOSS,           0x00,                               "Phantom Ganon Heart Container",               RHT_FOREST_TEMPLE_PHANTOM_GANON_HEART,                           RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x14, 0x1F),                                                                              true);
    locationTable[RC_FIRE_TEMPLE_VOLVAGIA_HEART] =                                     Location::Base(RC_FIRE_TEMPLE_VOLVAGIA_HEART,                                   RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_FIRE_TEMPLE_BOSS,             0x00,                               "Volvagia Heart Container",                    RHT_FIRE_TEMPLE_VOLVAGIA_HEART,                                  RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x15, 0x1F),                                                                              true);
    locationTable[RC_WATER_TEMPLE_MORPHA_HEART] =                                      Location::Base(RC_WATER_TEMPLE_MORPHA_HEART,                                    RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_WATER_TEMPLE_BOSS,            0x00,                               "Morpha Heart Container",                      RHT_WATER_TEMPLE_MORPHA_HEART,                                   RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x16, 0x1F),                                                                              true);
    locationTable[RC_SPIRIT_TEMPLE_TWINROVA_HEART] =                                   Location::Base(RC_SPIRIT_TEMPLE_TWINROVA_HEART,                                 RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_SPIRIT_TEMPLE_BOSS,           0x00,                               "Twinrova Heart Container",                    RHT_SPIRIT_TEMPLE_TWINROVA_HEART,                                RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x17, 0x1F),                                                                              true);
    locationTable[RC_SHADOW_TEMPLE_BONGO_BONGO_HEART] =                                Location::Base(RC_SHADOW_TEMPLE_BONGO_BONGO_HEART,                              RCQUEST_BOTH,    RCTYPE_BOSS_HEART_OR_OTHER_REWARD,                                       ACTOR_ITEM_B_HEART,   SCENE_SHADOW_TEMPLE_BOSS,           0x00,                               "Bongo Bongo Heart Container",                 RHT_SHADOW_TEMPLE_BONGO_BONGO_HEART,                             RG_HEART_CONTAINER,                                                 SpoilerCollectionCheck::Collectable(0x18, 0x1F),                                                                              true);

    // Cutscenes
    locationTable[RC_TOT_MASTER_SWORD] =                                               Location::Base(RC_TOT_MASTER_SWORD,                                             RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_TEMPLE_OF_TIME,               0x00,                               "ToT Master Sword",                            RHT_TOT_MASTER_SWORD,                                            RG_MASTER_SWORD,                                                    SpoilerCollectionCheck::EventChkInf(0x45),                                                                                    true);
    locationTable[RC_TOT_LIGHT_ARROWS_CUTSCENE] =                                      Location::Base(RC_TOT_LIGHT_ARROWS_CUTSCENE,                                    RCQUEST_BOTH,    RCTYPE_STANDARD,                                                         ACTOR_ID_MAX,         SCENE_TEMPLE_OF_TIME,               0x00,                               "ToT Light Arrow Cutscene",                    RHT_TOT_LIGHT_ARROWS_CUTSCENE,                                   RG_LIGHT_ARROWS,                                                    SpoilerCollectionCheck::EventChkInf(0xC4),                                                                                    true);
    locationTable[RC_LW_GIFT_FROM_SARIA] =                                             Location::Base(RC_LW_GIFT_FROM_SARIA,                                           RCQUEST_BOTH,    RCTYPE_OCARINA,                                                          ACTOR_ID_MAX,         SCENE_LOST_WOODS,                   0x00,                               "Gift From Saria",                             RHT_LW_GIFT_FROM_SARIA,                                          RG_PROGRESSIVE_OCARINA,                                             SpoilerCollectionCheck::EventChkInf(0xC1),                                                                                    true);
    locationTable[RC_ZF_GREAT_FAIRY_REWARD] =                                          Location::Base(RC_ZF_GREAT_FAIRY_REWARD,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_ZORAS_FOUNTAIN,               ACTOR_BG_DY_YOSEIZO,  SCENE_GREAT_FAIRYS_FOUNTAIN_SPELLS, 1,                                  "Great Fairy Reward",                          RHT_ZF_GREAT_FAIRY_REWARD,                                       RG_FARORES_WIND,                                                    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_GREAT_FAIRY_REWARD),                                                        true);
    locationTable[RC_HC_GREAT_FAIRY_REWARD] =                                          Location::Base(RC_HC_GREAT_FAIRY_REWARD,                                        RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_HYRULE_CASTLE,                ACTOR_BG_DY_YOSEIZO,  SCENE_GREAT_FAIRYS_FOUNTAIN_SPELLS, 2,                                  "Great Fairy Reward",                          RHT_HC_GREAT_FAIRY_REWARD,                                       RG_DINS_FIRE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_GREAT_FAIRY_REWARD),                                                        true);
    locationTable[RC_COLOSSUS_GREAT_FAIRY_REWARD] =                                    Location::Base(RC_COLOSSUS_GREAT_FAIRY_REWARD,                                  RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_DESERT_COLOSSUS,              ACTOR_BG_DY_YOSEIZO,  SCENE_GREAT_FAIRYS_FOUNTAIN_SPELLS, 3,                                  "Great Fairy Reward",                          RHT_COLOSSUS_GREAT_FAIRY_REWARD,                                 RG_NAYRUS_LOVE,                                                     SpoilerCollectionCheck::RandomizerInf(RAND_INF_COLOSSUS_GREAT_FAIRY_REWARD),                                                  true);
    locationTable[RC_DMT_GREAT_FAIRY_REWARD] =                                         Location::Base(RC_DMT_GREAT_FAIRY_REWARD,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_DEATH_MOUNTAIN_TRAIL,         ACTOR_BG_DY_YOSEIZO,  SCENE_GREAT_FAIRYS_FOUNTAIN_MAGIC,  1,                                  "Great Fairy Reward",                          RHT_DMT_GREAT_FAIRY_REWARD,                                      RG_PROGRESSIVE_MAGIC_METER,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMT_GREAT_FAIRY_REWARD),                                                       true);
    locationTable[RC_DMC_GREAT_FAIRY_REWARD] =                                         Location::Base(RC_DMC_GREAT_FAIRY_REWARD,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_BG_DY_YOSEIZO,  SCENE_GREAT_FAIRYS_FOUNTAIN_MAGIC,  2,                                  "Great Fairy Reward",                          RHT_DMC_GREAT_FAIRY_REWARD,                                      RG_PROGRESSIVE_MAGIC_METER,                                         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_GREAT_FAIRY_REWARD),                                                       true);
    locationTable[RC_OGC_GREAT_FAIRY_REWARD] =                                         Location::Base(RC_OGC_GREAT_FAIRY_REWARD,                                       RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_HYRULE_CASTLE,                ACTOR_BG_DY_YOSEIZO,  SCENE_GREAT_FAIRYS_FOUNTAIN_MAGIC,  3,                                  "OGC Great Fairy Reward", "OGC Great Fairy Reward", RHT_OGC_GREAT_FAIRY_REWARD,                                 RG_DOUBLE_DEFENSE,                                                  SpoilerCollectionCheck::RandomizerInf(RAND_INF_OGC_GREAT_FAIRY_REWARD),                                                       true);

    // Songs
    locationTable[RC_SHEIK_IN_FOREST] =                                                Location::Base(RC_SHEIK_IN_FOREST,                                              RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_SACRED_FOREST_MEADOW,         0x00,                               "Sheik in Forest",           "Sheik in Forest",           RHT_SHEIK_IN_FOREST,                                  RG_MINUET_OF_FOREST,                                                SpoilerCollectionCheck::EventChkInf(0x50),                                                                                    true);
    locationTable[RC_SHEIK_IN_CRATER] =                                                Location::Base(RC_SHEIK_IN_CRATER,                                              RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_DEATH_MOUNTAIN_CRATER,        0x00,                               "Sheik in Crater",           "Sheik in Crater",           RHT_SHEIK_IN_CRATER,                                  RG_BOLERO_OF_FIRE,                                                  SpoilerCollectionCheck::EventChkInf(0x51),                                                                                    true);
    locationTable[RC_SHEIK_IN_ICE_CAVERN] =                                            Location::Base(RC_SHEIK_IN_ICE_CAVERN,                                          RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_ICE_CAVERN,                   0x00,                               "Sheik in Ice Cavern",       "Sheik in Ice Cavern",       RHT_SHEIK_IN_ICE_CAVERN,                              RG_SERENADE_OF_WATER,                                               SpoilerCollectionCheck::EventChkInf(0x52),                                                                                    true);
    locationTable[RC_SHEIK_AT_COLOSSUS] =                                              Location::Base(RC_SHEIK_AT_COLOSSUS,                                            RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_DESERT_COLOSSUS,              0x00,                               "Sheik at Colossus",         "Sheik at Colossus",         RHT_SHEIK_AT_COLOSSUS,                                RG_REQUIEM_OF_SPIRIT,                                               SpoilerCollectionCheck::EventChkInf(0xAC),                                                                                    true);
    locationTable[RC_SHEIK_IN_KAKARIKO] =                                              Location::Base(RC_SHEIK_IN_KAKARIKO,                                            RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_KAKARIKO_VILLAGE,             0x00,                               "Sheik in Kakariko",         "Sheik in Kakariko",         RHT_SHEIK_IN_KAKARIKO,                                RG_NOCTURNE_OF_SHADOW,                                              SpoilerCollectionCheck::EventChkInf(0xAA),                                                                                    true);
    locationTable[RC_SHEIK_AT_TEMPLE] =                                                Location::Base(RC_SHEIK_AT_TEMPLE,                                              RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_TEMPLE_OF_TIME,               0x00,                               "Sheik at Temple",           "Sheik at Temple",           RHT_SHEIK_AT_TEMPLE,                                  RG_PRELUDE_OF_LIGHT,                                                SpoilerCollectionCheck::EventChkInf(0x55),                                                                                    true);
    locationTable[RC_SONG_FROM_IMPA] =                                                 Location::Base(RC_SONG_FROM_IMPA,                                               RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,               RCAREA_HYRULE_CASTLE,                ACTOR_ID_MAX,         SCENE_ID_MAX,                       0x00,                               "Song from Impa",            "Song from Impa",            RHT_SONG_FROM_IMPA,                                   RG_ZELDAS_LULLABY,                                                  SpoilerCollectionCheck::EventChkInf(0x59),                                                                                    true);
    locationTable[RC_SONG_FROM_MALON] =                                                Location::Base(RC_SONG_FROM_MALON,                                              RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_LON_LON_RANCH,                0x00,                               "Song from Malon",           "Song from Malon",           RHT_SONG_FROM_MALON,                                  RG_EPONAS_SONG,                                                     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LEARNED_EPONA_SONG),                                                           true);
    locationTable[RC_SONG_FROM_SARIA] =                                                Location::Base(RC_SONG_FROM_SARIA,                                              RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_SACRED_FOREST_MEADOW,         0x00,                               "Song from Saria",           "Song from Saria",           RHT_SONG_FROM_SARIA,                                  RG_SARIAS_SONG,                                                     SpoilerCollectionCheck::EventChkInf(0x57),                                                                                    true);
    locationTable[RC_SONG_FROM_ROYAL_FAMILYS_TOMB] =                                   Location::Base(RC_SONG_FROM_ROYAL_FAMILYS_TOMB,                                 RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_ROYAL_FAMILYS_TOMB,           0x00,                               "Song from Composers Grave", "Song from Composers Grave", RHT_SONG_FROM_ROYAL_FAMILYS_TOMB,                     RG_SUNS_SONG,                                                       SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_LEARNED_SUNS_SONG),                                                           true);
    locationTable[RC_SONG_FROM_OCARINA_OF_TIME] =                                      Location::Base(RC_SONG_FROM_OCARINA_OF_TIME,                                    RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,                                                    ACTOR_ID_MAX,         SCENE_HYRULE_FIELD,                 0x00,                               "Song from Ocarina of Time", "Song from Ocarina of Time", RHT_SONG_FROM_OCARINA_OF_TIME,                        RG_SONG_OF_TIME,                                                    SpoilerCollectionCheck::EventChkInf(0xA9),                                                                                    true);
    locationTable[RC_SONG_FROM_WINDMILL] =                                             Location::Base(RC_SONG_FROM_WINDMILL,                                           RCQUEST_BOTH,    RCTYPE_SONG_LOCATION,               RCAREA_KAKARIKO_VILLAGE,             ACTOR_ID_MAX,         SCENE_WINDMILL_AND_DAMPES_GRAVE,    0x00,                               "Song from Windmill",        "Song from Windmill",        RHT_SONG_FROM_WINDMILL,                               RG_SONG_OF_STORMS,                                                  SpoilerCollectionCheck::EventChkInf(EVENTCHKINF_LEARNED_SONG_OF_STORMS),                                                      true);

    //Beehives
    locationTable[RC_KF_STORMS_GROTTO_BEEHIVE_LEFT] =                                  Location::Base(RC_KF_STORMS_GROTTO_BEEHIVE_LEFT,                                RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_KOKIRI_FOREST,                ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x2C),       "Storms Grotto Beehive Left",                  RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_KF_STORMS_GROTTO_LEFT));
    locationTable[RC_KF_STORMS_GROTTO_BEEHIVE_RIGHT] =                                 Location::Base(RC_KF_STORMS_GROTTO_BEEHIVE_RIGHT,                               RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_KOKIRI_FOREST,                ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x2C),       "Storms Grotto Beehive Right",                 RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_KF_STORMS_GROTTO_RIGHT));
    locationTable[RC_LW_NEAR_SHORTCUTS_GROTTO_BEEHIVE_LEFT] =                          Location::Base(RC_LW_NEAR_SHORTCUTS_GROTTO_BEEHIVE_LEFT,                        RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_LOST_WOODS,                   ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x14),       "Near Shortcuts Grotto Beehive Left",          RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_LW_NEAR_SHORTCUTS_GROTTO_LEFT));
    locationTable[RC_LW_NEAR_SHORTCUTS_GROTTO_BEEHIVE_RIGHT] =                         Location::Base(RC_LW_NEAR_SHORTCUTS_GROTTO_BEEHIVE_RIGHT,                       RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_LOST_WOODS,                   ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x14),       "Near Shortcuts Grotto Beehive Right",         RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_LW_NEAR_SHORTCUTS_GROTTO_RIGHT));
    locationTable[RC_LW_DEKU_SCRUB_GROTTO_BEEHIVE] =                                   Location::Base(RC_LW_DEKU_SCRUB_GROTTO_BEEHIVE,                                 RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_LOST_WOODS,                   ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(747,  0xF5),       "Deku Scrub Grotto Beehive",                   RHT_BEEHIVE_SCRUB_PAIR_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_LW_DEKU_SCRUB_GROTTO));
    locationTable[RC_SFM_STORMS_GROTTO_BEEHIVE] =                                      Location::Base(RC_SFM_STORMS_GROTTO_BEEHIVE,                                    RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_SACRED_FOREST_MEADOW,         ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(2262, 0xEE),       "Storms Grotto Beehive",                       RHT_BEEHIVE_SCRUB_PAIR_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_SFM_STORMS_GROTTO));
    locationTable[RC_HF_NEAR_MARKET_GROTTO_BEEHIVE_LEFT] =                             Location::Base(RC_HF_NEAR_MARKET_GROTTO_BEEHIVE_LEFT,                           RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x00),       "Near Market Grotto Beehive Left",             RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_NEAR_MARKET_GROTTO_LEFT));
    locationTable[RC_HF_NEAR_MARKET_GROTTO_BEEHIVE_RIGHT] =                            Location::Base(RC_HF_NEAR_MARKET_GROTTO_BEEHIVE_RIGHT,                          RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x00),       "Near Market Grotto Beehive Right",            RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_NEAR_MARKET_GROTTO_RIGHT));
    locationTable[RC_HF_OPEN_GROTTO_BEEHIVE_LEFT] =                                    Location::Base(RC_HF_OPEN_GROTTO_BEEHIVE_LEFT,                                  RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x03),       "Open Grotto Beehive Left",                    RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_OPEN_GROTTO_LEFT));
    locationTable[RC_HF_OPEN_GROTTO_BEEHIVE_RIGHT] =                                   Location::Base(RC_HF_OPEN_GROTTO_BEEHIVE_RIGHT,                                 RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x03),       "Open Grotto Beehive Right",                   RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_OPEN_GROTTO_RIGHT));
    locationTable[RC_HF_SOUTHEAST_GROTTO_BEEHIVE_LEFT] =                               Location::Base(RC_HF_SOUTHEAST_GROTTO_BEEHIVE_LEFT,                             RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x22),       "Southeast Grotto Beehive Left",               RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_SOUTHEAST_GROTTO_LEFT));
    locationTable[RC_HF_SOUTHEAST_GROTTO_BEEHIVE_RIGHT] =                              Location::Base(RC_HF_SOUTHEAST_GROTTO_BEEHIVE_RIGHT,                            RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x22),       "Southeast Grotto Beehive Right",              RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_SOUTHEAST_GROTTO_RIGHT));
    locationTable[RC_HF_INSIDE_FENCE_GROTTO_BEEHIVE] =                                 Location::Base(RC_HF_INSIDE_FENCE_GROTTO_BEEHIVE,                               RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_HYRULE_FIELD,                 ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(1410, 0xE6),       "Inside Fence Grotto Beehive",                 RHT_BEEHIVE_LONELY_SCRUB_GROTTO,                                 RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_HF_INSIDE_FENCE_GROTTO));
    locationTable[RC_LLR_GROTTO_BEEHIVE] =                                             Location::Base(RC_LLR_GROTTO_BEEHIVE,                                           RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_LON_LON_RANCH,                ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(5144, 0xFC),       "Grotto Beehive",                              RHT_BEEHIVE_SCRUB_TRIO_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_LLR_GROTTO));
    locationTable[RC_KAK_OPEN_GROTTO_BEEHIVE_LEFT] =                                   Location::Base(RC_KAK_OPEN_GROTTO_BEEHIVE_LEFT,                                 RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_KAKARIKO_VILLAGE,             ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x28),       "Open Grotto Beehive Left",                    RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_KAK_OPEN_GROTTO_LEFT));
    locationTable[RC_KAK_OPEN_GROTTO_BEEHIVE_RIGHT] =                                  Location::Base(RC_KAK_OPEN_GROTTO_BEEHIVE_RIGHT,                                RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_KAKARIKO_VILLAGE,             ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x28),       "Open Grotto Beehive Right",                   RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_KAK_OPEN_GROTTO_RIGHT));
    locationTable[RC_DMT_COW_GROTTO_BEEHIVE] =                                         Location::Base(RC_DMT_COW_GROTTO_BEEHIVE,                                       RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DEATH_MOUNTAIN_TRAIL,         ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(2617, 0xF8),       "Cow Grotto Beehive",                          RHT_BEEHIVE_COW_GROTTO,                                          RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_DMT_COW_GROTTO));
    locationTable[RC_DMT_STORMS_GROTTO_BEEHIVE_LEFT] =                                 Location::Base(RC_DMT_STORMS_GROTTO_BEEHIVE_LEFT,                               RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DEATH_MOUNTAIN_TRAIL,         ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x57),       "Storms Grotto Beehive Left",                  RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_DMT_STORMS_GROTTO_LEFT));
    locationTable[RC_DMT_STORMS_GROTTO_BEEHIVE_RIGHT] =                                Location::Base(RC_DMT_STORMS_GROTTO_BEEHIVE_RIGHT,                              RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DEATH_MOUNTAIN_TRAIL,         ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x57),       "Storms Grotto Beehive Right",                 RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_DMT_STORMS_GROTTO_RIGHT));
    locationTable[RC_GC_GROTTO_BEEHIVE] =                                              Location::Base(RC_GC_GROTTO_BEEHIVE,                                            RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_GORON_CITY,                   ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(5144, 0xFB),       "Grotto Beehive",                              RHT_BEEHIVE_SCRUB_TRIO_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_GC_GROTTO));
    locationTable[RC_DMC_UPPER_GROTTO_BEEHIVE_LEFT] =                                  Location::Base(RC_DMC_UPPER_GROTTO_BEEHIVE_LEFT,                                RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x7A),       "Upper Grotto Beehive Left",                   RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_DMC_UPPER_GROTTO_LEFT));
    locationTable[RC_DMC_UPPER_GROTTO_BEEHIVE_RIGHT] =                                 Location::Base(RC_DMC_UPPER_GROTTO_BEEHIVE_RIGHT,                               RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x7A),       "Upper Grotto Beehive Right",                  RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_DMC_UPPER_GROTTO_RIGHT));
    locationTable[RC_DMC_HAMMER_GROTTO_BEEHIVE] =                                      Location::Base(RC_DMC_HAMMER_GROTTO_BEEHIVE,                                    RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DEATH_MOUNTAIN_CRATER,        ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(5144, 0xF9),       "Hammer Grotto Beehive",                       RHT_BEEHIVE_SCRUB_TRIO_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_DMC_HAMMER_GROTTO));
    locationTable[RC_ZR_OPEN_GROTTO_BEEHIVE_LEFT] =                                    Location::Base(RC_ZR_OPEN_GROTTO_BEEHIVE_LEFT,                                  RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_ZORAS_RIVER,                  ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(-144, 0x29),       "Open Grotto Beehive Left",                    RHT_BEEHIVE_CHEST_GROTTO,                                        RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_ZR_OPEN_GROTTO_LEFT));
    locationTable[RC_ZR_OPEN_GROTTO_BEEHIVE_RIGHT] =                                   Location::Base(RC_ZR_OPEN_GROTTO_BEEHIVE_RIGHT,                                 RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_ZORAS_RIVER,                  ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(121,  0x29),       "Open Grotto Beehive Right",                   RHT_BEEHIVE_CHEST_GROTTO,                                        RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_ZR_OPEN_GROTTO_RIGHT));
    locationTable[RC_ZR_STORMS_GROTTO_BEEHIVE] =                                       Location::Base(RC_ZR_STORMS_GROTTO_BEEHIVE,                                     RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_ZORAS_RIVER,                  ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(2262, 0xEB),       "Storms Grotto Beehive",                       RHT_BEEHIVE_SCRUB_PAIR_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_ZR_STORMS_GROTTO));
    locationTable[RC_ZD_IN_FRONT_OF_KING_ZORA_BEEHIVE_LEFT] =                          Location::Base(RC_ZD_IN_FRONT_OF_KING_ZORA_BEEHIVE_LEFT,                        RCQUEST_BOTH,    RCTYPE_BEEHIVE,                                                          ACTOR_OBJ_COMB,       SCENE_ZORAS_DOMAIN,                 TWO_ACTOR_PARAMS(382,  0x00),       "In Front of King Zora Beehive Left",          RHT_BEEHIVE_IN_FRONT_OF_KING_ZORA,                               RG_BLUE_RUPEE,                                                      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_ZD_IN_FRONT_OF_KING_ZORA_LEFT));
    locationTable[RC_ZD_IN_FRONT_OF_KING_ZORA_BEEHIVE_RIGHT] =                         Location::Base(RC_ZD_IN_FRONT_OF_KING_ZORA_BEEHIVE_RIGHT,                       RCQUEST_BOTH,    RCTYPE_BEEHIVE,                                                          ACTOR_OBJ_COMB,       SCENE_ZORAS_DOMAIN,                 TWO_ACTOR_PARAMS(948,  0x00),       "In Front of King Zora Beehive Right",         RHT_BEEHIVE_IN_FRONT_OF_KING_ZORA,                               RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_ZD_IN_FRONT_OF_KING_ZORA_RIGHT));
    locationTable[RC_ZD_BEHIND_KING_ZORA_BEEHIVE] =                                    Location::Base(RC_ZD_BEHIND_KING_ZORA_BEEHIVE,                                  RCQUEST_BOTH,    RCTYPE_BEEHIVE,                                                          ACTOR_OBJ_COMB,       SCENE_ZORAS_DOMAIN,                 TWO_ACTOR_PARAMS(701,  0x00),       "Behind King Zora Beehive",                    RHT_BEEHIVE_BEHIND_KING_ZORA,                                    RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_ZD_BEHIND_KING_ZORA));
    locationTable[RC_LH_GROTTO_BEEHIVE] =                                              Location::Base(RC_LH_GROTTO_BEEHIVE,                                            RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_LAKE_HYLIA,                   ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(5144, 0xEF),       "Grotto Beehive",                              RHT_BEEHIVE_SCRUB_TRIO_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_LH_GROTTO));
    locationTable[RC_GV_DEKU_SCRUB_GROTTO_BEEHIVE] =                                   Location::Base(RC_GV_DEKU_SCRUB_GROTTO_BEEHIVE,                                 RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_GERUDO_VALLEY,                ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(2262, 0xF0),       "Deku Scrub Grotto Beehive",                   RHT_BEEHIVE_SCRUB_PAIR_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_GV_DEKU_SCRUB_GROTTO));
    locationTable[RC_COLOSSUS_GROTTO_BEEHIVE] =                                        Location::Base(RC_COLOSSUS_GROTTO_BEEHIVE,                                      RCQUEST_BOTH,    RCTYPE_BEEHIVE,                     RCAREA_DESERT_COLOSSUS,              ACTOR_OBJ_COMB,       SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(2262, 0xFD),       "Grotto Beehive",                              RHT_BEEHIVE_SCRUB_PAIR_GROTTO,                                   RG_RED_RUPEE,                                                       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BEEHIVE_COLOSSUS_GROTTO));

    // Cows
    locationTable[RC_KF_LINKS_HOUSE_COW] =                                             Location::Base(RC_KF_LINKS_HOUSE_COW,                                           RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_LINKS_HOUSE,                  0x00,                               "Links House Cow",                             RHT_KF_LINKS_HOUSE_COW,                                          RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_KF_LINKS_HOUSE_COW));
    locationTable[RC_HF_COW_GROTTO_COW] =                                              Location::Base(RC_HF_COW_GROTTO_COW,                                            RCQUEST_BOTH,    RCTYPE_COW,                         RCAREA_HYRULE_FIELD,                 ACTOR_EN_COW,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(3485, -291),       "Cow Grotto Cow",                              RHT_HF_COW_GROTTO_COW,                                           RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_HF_COW_GROTTO_COW));
    locationTable[RC_LLR_STABLES_LEFT_COW] =                                           Location::Base(RC_LLR_STABLES_LEFT_COW,                                         RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_STABLE,                       TWO_ACTOR_PARAMS(-122, -254),       "Stables Left Cow",                            RHT_LLR_STABLES_LEFT_COW,                                        RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_LLR_STABLES_LEFT_COW));
    locationTable[RC_LLR_STABLES_RIGHT_COW] =                                          Location::Base(RC_LLR_STABLES_RIGHT_COW,                                        RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_STABLE,                       TWO_ACTOR_PARAMS(116, -254),        "Stables Right Cow",                           RHT_LLR_STABLES_RIGHT_COW,                                       RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_LLR_STABLES_RIGHT_COW));
    locationTable[RC_LLR_TOWER_LEFT_COW] =                                             Location::Base(RC_LLR_TOWER_LEFT_COW,                                           RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_LON_LON_BUILDINGS,            TWO_ACTOR_PARAMS(-229, 157),        "Tower Left Cow",                              RHT_LLR_TOWER_LEFT_COW,                                          RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_LLR_TOWER_LEFT_COW));
    locationTable[RC_LLR_TOWER_RIGHT_COW] =                                            Location::Base(RC_LLR_TOWER_RIGHT_COW,                                          RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_LON_LON_BUILDINGS,            TWO_ACTOR_PARAMS(-142, -140),       "Tower Right Cow",                             RHT_LLR_TOWER_RIGHT_COW,                                         RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_LLR_TOWER_RIGHT_COW));
    locationTable[RC_KAK_IMPAS_HOUSE_COW] =                                            Location::Base(RC_KAK_IMPAS_HOUSE_COW,                                          RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_IMPAS_HOUSE,                  0x00,                               "Impas House Cow",                             RHT_KAK_IMPAS_HOUSE_COW,                                         RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_KAK_IMPAS_HOUSE_COW));
    locationTable[RC_DMT_COW_GROTTO_COW] =                                             Location::Base(RC_DMT_COW_GROTTO_COW,                                           RCQUEST_BOTH,    RCTYPE_COW,                         RCAREA_DEATH_MOUNTAIN_TRAIL,         ACTOR_EN_COW,         SCENE_GROTTOS,                      TWO_ACTOR_PARAMS(2444, -471),       "Cow Grotto Cow",                              RHT_DMT_COW_GROTTO_COW,                                          RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_DMT_COW_GROTTO_COW));
    locationTable[RC_GV_COW] =                                                         Location::Base(RC_GV_COW,                                                       RCQUEST_BOTH,    RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_GERUDO_VALLEY,                0x00,                               "Cow",                                         RHT_GV_COW,                                                      RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_GV_COW));
    locationTable[RC_JABU_JABUS_BELLY_MQ_COW] =                                        Location::Base(RC_JABU_JABUS_BELLY_MQ_COW,                                      RCQUEST_MQ,      RCTYPE_COW,                                                              ACTOR_EN_COW,         SCENE_JABU_JABU,                    0x00,                               "MQ Cow",                                      RHT_JABU_JABUS_BELLY_MQ_COW,                                     RG_MILK,                                                            SpoilerCollectionCheck::RandomizerInf(RAND_INF_COWS_MILKED_JABU_JABUS_BELLY_MQ_COW));

    /*-------------------------------
              --- SHOPS ---
      8     6               2     4

      7     5               1     3
      -------------------------------*/
    // Kokiri Forest
    locationTable[RC_KF_SHOP_ITEM_1] =                                                 Location::Base(RC_KF_SHOP_ITEM_1,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x00,                               "Shop Item 1",                                 RHT_KF_SHOP_ITEM_1,                                              RG_BUY_DEKU_SHIELD,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_1), true,  40);
    locationTable[RC_KF_SHOP_ITEM_2] =                                                 Location::Base(RC_KF_SHOP_ITEM_2,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x01,                               "Shop Item 2",                                 RHT_KF_SHOP_ITEM_2,                                              RG_BUY_DEKU_NUTS_5,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_2), false, 15);
    locationTable[RC_KF_SHOP_ITEM_3] =                                                 Location::Base(RC_KF_SHOP_ITEM_3,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x02,                               "Shop Item 3",                                 RHT_KF_SHOP_ITEM_3,                                              RG_BUY_DEKU_NUTS_10,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_3), false, 30);
    locationTable[RC_KF_SHOP_ITEM_4] =                                                 Location::Base(RC_KF_SHOP_ITEM_4,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x03,                               "Shop Item 4",                                 RHT_KF_SHOP_ITEM_4,                                              RG_BUY_DEKU_STICK_1,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_4), false, 10);
    locationTable[RC_KF_SHOP_ITEM_5] =                                                 Location::Base(RC_KF_SHOP_ITEM_5,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x04,                               "Shop Item 5",                                 RHT_KF_SHOP_ITEM_5,                                              RG_BUY_DEKU_SEEDS_30, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_5), false, 30);
    locationTable[RC_KF_SHOP_ITEM_6] =                                                 Location::Base(RC_KF_SHOP_ITEM_6,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x05,                               "Shop Item 6",                                 RHT_KF_SHOP_ITEM_6,                                              RG_BUY_ARROWS_10,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_6), false, 20);
    locationTable[RC_KF_SHOP_ITEM_7] =                                                 Location::Base(RC_KF_SHOP_ITEM_7,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x06,                               "Shop Item 7",                                 RHT_KF_SHOP_ITEM_7,                                              RG_BUY_ARROWS_30,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_7), false, 60);
    locationTable[RC_KF_SHOP_ITEM_8] =                                                 Location::Base(RC_KF_SHOP_ITEM_8,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_KOKIRI_SHOP,                  0x07,                               "Shop Item 8",                                 RHT_KF_SHOP_ITEM_8,                                              RG_BUY_HEART,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KF_SHOP_ITEM_8), false, 10);

    // Kakariko Village
    locationTable[RC_KAK_POTION_SHOP_ITEM_1] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_1,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x00,                               "Potion Shop Item 1",                          RHT_KAK_POTION_SHOP_ITEM_1,                                      RG_BUY_GREEN_POTION,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_1), false, 30);
    locationTable[RC_KAK_POTION_SHOP_ITEM_2] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_2,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x01,                               "Potion Shop Item 2",                          RHT_KAK_POTION_SHOP_ITEM_2,                                      RG_BUY_BLUE_FIRE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_2), false, 300);
    locationTable[RC_KAK_POTION_SHOP_ITEM_3] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_3,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x02,                               "Potion Shop Item 3",                          RHT_KAK_POTION_SHOP_ITEM_3,                                      RG_BUY_RED_POTION_30, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_3), false, 30);
    locationTable[RC_KAK_POTION_SHOP_ITEM_4] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_4,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x03,                               "Potion Shop Item 4",                          RHT_KAK_POTION_SHOP_ITEM_4,                                      RG_BUY_FAIRYS_SPIRIT, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_4), false, 50);
    locationTable[RC_KAK_POTION_SHOP_ITEM_5] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_5,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x04,                               "Potion Shop Item 5",                          RHT_KAK_POTION_SHOP_ITEM_5,                                      RG_BUY_DEKU_NUTS_5,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_5), false, 15);
    locationTable[RC_KAK_POTION_SHOP_ITEM_6] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_6,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x05,                               "Potion Shop Item 6",                          RHT_KAK_POTION_SHOP_ITEM_6,                                      RG_BUY_BOTTLE_BUG,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_6), false, 50);
    locationTable[RC_KAK_POTION_SHOP_ITEM_7] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_7,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x06,                               "Potion Shop Item 7",                          RHT_KAK_POTION_SHOP_ITEM_7,                                      RG_BUY_POE,           SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_7), false, 30);
    locationTable[RC_KAK_POTION_SHOP_ITEM_8] =                                         Location::Base(RC_KAK_POTION_SHOP_ITEM_8,                                       RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_KAKARIKO,         0x07,                               "Potion Shop Item 8",                          RHT_KAK_POTION_SHOP_ITEM_8,                                      RG_BUY_FISH,          SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_POTION_SHOP_ITEM_8), false, 200);
    locationTable[RC_KAK_BAZAAR_ITEM_1] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_1,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x00,                               "Bazaar Item 1",                               RHT_KAK_BAZAAR_ITEM_1,                                           RG_BUY_HYLIAN_SHIELD, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_1), false, 80);
    locationTable[RC_KAK_BAZAAR_ITEM_2] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_2,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x01,                               "Bazaar Item 2",                               RHT_KAK_BAZAAR_ITEM_2,                                           RG_BUY_BOMBS_535,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_2), false, 35);
    locationTable[RC_KAK_BAZAAR_ITEM_3] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_3,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x02,                               "Bazaar Item 3",                               RHT_KAK_BAZAAR_ITEM_3,                                           RG_BUY_DEKU_NUTS_5,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_3), false, 10);
    locationTable[RC_KAK_BAZAAR_ITEM_4] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_4,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x03,                               "Bazaar Item 4",                               RHT_KAK_BAZAAR_ITEM_4,                                           RG_BUY_HEART,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_4), false, 20);
    locationTable[RC_KAK_BAZAAR_ITEM_5] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_5,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x04,                               "Bazaar Item 5",                               RHT_KAK_BAZAAR_ITEM_5,                                           RG_BUY_ARROWS_10,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_5), false, 20);
    locationTable[RC_KAK_BAZAAR_ITEM_6] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_6,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x05,                               "Bazaar Item 6",                               RHT_KAK_BAZAAR_ITEM_6,                                           RG_BUY_ARROWS_50,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_6), false, 90);
    locationTable[RC_KAK_BAZAAR_ITEM_7] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_7,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x06,                               "Bazaar Item 7",                               RHT_KAK_BAZAAR_ITEM_7,                                           RG_BUY_DEKU_STICK_1,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_7), false, 10);
    locationTable[RC_KAK_BAZAAR_ITEM_8] =                                              Location::Base(RC_KAK_BAZAAR_ITEM_8,                                            RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_TEST01,                       0x07,                               "Bazaar Item 8",                               RHT_KAK_BAZAAR_ITEM_8,                                           RG_BUY_ARROWS_30,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_KAK_BAZAAR_ITEM_8), false, 60);

    // Market
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_1] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_1,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x00,                               "Bombchu Shop Item 1",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_1,                                  RG_BUY_BOMBCHUS_10,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_1), false, 99);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_2] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_2,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x01,                               "Bombchu Shop Item 2",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_2,                                  RG_BUY_BOMBCHUS_10,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_2), false, 99);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_3] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_3,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x02,                               "Bombchu Shop Item 3",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_3,                                  RG_BUY_BOMBCHUS_10,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_3), false, 99);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_4] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_4,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x03,                               "Bombchu Shop Item 4",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_4,                                  RG_BUY_BOMBCHUS_10,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_4), false, 99);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_5] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_5,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x04,                               "Bombchu Shop Item 5",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_5,                                  RG_BUY_BOMBCHUS_20,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_5), false, 180);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_6] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_6,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x05,                               "Bombchu Shop Item 6",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_6,                                  RG_BUY_BOMBCHUS_20,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_6), false, 180);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_7] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_7,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x06,                               "Bombchu Shop Item 7",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_7,                                  RG_BUY_BOMBCHUS_20,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_7), false, 180);
    locationTable[RC_MARKET_BOMBCHU_SHOP_ITEM_8] =                                     Location::Base(RC_MARKET_BOMBCHU_SHOP_ITEM_8,                                   RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_BOMBCHU_SHOP,                 0x07,                               "Bombchu Shop Item 8",                         RHT_MARKET_BOMBCHU_SHOP_ITEM_8,                                  RG_BUY_BOMBCHUS_20,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BOMBCHU_SHOP_ITEM_8), false, 180);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_1] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_1,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x00,                               "Potion Shop Item 1",                          RHT_MARKET_POTION_SHOP_ITEM_1,                                   RG_BUY_GREEN_POTION,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_1), false, 30);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_2] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_2,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x01,                               "Potion Shop Item 2",                          RHT_MARKET_POTION_SHOP_ITEM_2,                                   RG_BUY_BLUE_FIRE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_2), false, 300);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_3] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_3,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x02,                               "Potion Shop Item 3",                          RHT_MARKET_POTION_SHOP_ITEM_3,                                   RG_BUY_RED_POTION_30, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_3), false, 30);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_4] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_4,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x03,                               "Potion Shop Item 4",                          RHT_MARKET_POTION_SHOP_ITEM_4,                                   RG_BUY_FAIRYS_SPIRIT, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_4), false, 50);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_5] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_5,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x04,                               "Potion Shop Item 5",                          RHT_MARKET_POTION_SHOP_ITEM_5,                                   RG_BUY_DEKU_NUTS_5,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_5), false, 15);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_6] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_6,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x05,                               "Potion Shop Item 6",                          RHT_MARKET_POTION_SHOP_ITEM_6,                                   RG_BUY_BOTTLE_BUG,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_6), false, 50);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_7] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_7,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x06,                               "Potion Shop Item 7",                          RHT_MARKET_POTION_SHOP_ITEM_7,                                   RG_BUY_POE,           SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_7), false, 30);
    locationTable[RC_MARKET_POTION_SHOP_ITEM_8] =                                      Location::Base(RC_MARKET_POTION_SHOP_ITEM_8,                                    RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_POTION_SHOP_MARKET,           0x07,                               "Potion Shop Item 8",                          RHT_MARKET_POTION_SHOP_ITEM_8,                                   RG_BUY_FISH,          SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_POTION_SHOP_ITEM_8), false, 200);
    locationTable[RC_MARKET_BAZAAR_ITEM_1] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_1,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x00,                               "Bazaar Item 1",                               RHT_MARKET_BAZAAR_ITEM_1,                                        RG_BUY_HYLIAN_SHIELD, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_1), false, 80);
    locationTable[RC_MARKET_BAZAAR_ITEM_2] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_2,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x01,                               "Bazaar Item 2",                               RHT_MARKET_BAZAAR_ITEM_2,                                        RG_BUY_BOMBS_535,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_2), false, 35);
    locationTable[RC_MARKET_BAZAAR_ITEM_3] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_3,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x02,                               "Bazaar Item 3",                               RHT_MARKET_BAZAAR_ITEM_3,                                        RG_BUY_DEKU_NUTS_5,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_3), false, 15);
    locationTable[RC_MARKET_BAZAAR_ITEM_4] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_4,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x03,                               "Bazaar Item 4",                               RHT_MARKET_BAZAAR_ITEM_4,                                        RG_BUY_HEART,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_4), false, 10);
    locationTable[RC_MARKET_BAZAAR_ITEM_5] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_5,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x04,                               "Bazaar Item 5",                               RHT_MARKET_BAZAAR_ITEM_5,                                        RG_BUY_ARROWS_10,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_5), false, 20);
    locationTable[RC_MARKET_BAZAAR_ITEM_6] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_6,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x05,                               "Bazaar Item 6",                               RHT_MARKET_BAZAAR_ITEM_6,                                        RG_BUY_ARROWS_50,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_6), false, 90);
    locationTable[RC_MARKET_BAZAAR_ITEM_7] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_7,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x06,                               "Bazaar Item 7",                               RHT_MARKET_BAZAAR_ITEM_7,                                        RG_BUY_DEKU_STICK_1,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_7), false, 10);
    locationTable[RC_MARKET_BAZAAR_ITEM_8] =                                           Location::Base(RC_MARKET_BAZAAR_ITEM_8,                                         RCQUEST_BOTH,    RCTYPE_SHOP,                        RCAREA_MARKET,                       ACTOR_EN_GIRLA,       SCENE_BAZAAR,                       0x07,                               "Bazaar Item 8",                               RHT_MARKET_BAZAAR_ITEM_8,                                        RG_BUY_ARROWS_30,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_MARKET_BAZAAR_ITEM_8), false, 60);

    // Zora's Domain
    locationTable[RC_ZD_SHOP_ITEM_1] =                                                 Location::Base(RC_ZD_SHOP_ITEM_1,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x00,                               "Shop Item 1",                                 RHT_ZD_SHOP_ITEM_1,                                              RG_BUY_ZORA_TUNIC,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_1), false, 200);
    locationTable[RC_ZD_SHOP_ITEM_2] =                                                 Location::Base(RC_ZD_SHOP_ITEM_2,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x01,                               "Shop Item 2",                                 RHT_ZD_SHOP_ITEM_2,                                              RG_BUY_ARROWS_10,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_2), false, 20);
    locationTable[RC_ZD_SHOP_ITEM_3] =                                                 Location::Base(RC_ZD_SHOP_ITEM_3,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x02,                               "Shop Item 3",                                 RHT_ZD_SHOP_ITEM_3,                                              RG_BUY_HEART,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_3), false, 10);
    locationTable[RC_ZD_SHOP_ITEM_4] =                                                 Location::Base(RC_ZD_SHOP_ITEM_4,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x03,                               "Shop Item 4",                                 RHT_ZD_SHOP_ITEM_4,                                              RG_BUY_ARROWS_30,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_4), false, 60);
    locationTable[RC_ZD_SHOP_ITEM_5] =                                                 Location::Base(RC_ZD_SHOP_ITEM_5,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x04,                               "Shop Item 5",                                 RHT_ZD_SHOP_ITEM_5,                                              RG_BUY_DEKU_NUTS_5,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_5), false, 15);
    locationTable[RC_ZD_SHOP_ITEM_6] =                                                 Location::Base(RC_ZD_SHOP_ITEM_6,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x05,                               "Shop Item 6",                                 RHT_ZD_SHOP_ITEM_6,                                              RG_BUY_ARROWS_50,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_6), false, 90);
    locationTable[RC_ZD_SHOP_ITEM_7] =                                                 Location::Base(RC_ZD_SHOP_ITEM_7,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x06,                               "Shop Item 7",                                 RHT_ZD_SHOP_ITEM_7,                                              RG_BUY_FISH,          SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_7), false, 200);
    locationTable[RC_ZD_SHOP_ITEM_8] =                                                 Location::Base(RC_ZD_SHOP_ITEM_8,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_ZORA_SHOP,                    0x07,                               "Shop Item 8",                                 RHT_ZD_SHOP_ITEM_8,                                              RG_BUY_RED_POTION_50, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_ZD_SHOP_ITEM_8), false, 50);

    // Goron City
    locationTable[RC_GC_SHOP_ITEM_1] =                                                 Location::Base(RC_GC_SHOP_ITEM_1,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x00,                               "Shop Item 1",                                 RHT_GC_SHOP_ITEM_1,                                              RG_BUY_BOMBS_525,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_1), false, 25);
    locationTable[RC_GC_SHOP_ITEM_2] =                                                 Location::Base(RC_GC_SHOP_ITEM_2,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x01,                               "Shop Item 2",                                 RHT_GC_SHOP_ITEM_2,                                              RG_BUY_BOMBS_10,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_2), false, 50);
    locationTable[RC_GC_SHOP_ITEM_3] =                                                 Location::Base(RC_GC_SHOP_ITEM_3,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x02,                               "Shop Item 3",                                 RHT_GC_SHOP_ITEM_3,                                              RG_BUY_BOMBS_20,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_3), false, 80);
    locationTable[RC_GC_SHOP_ITEM_4] =                                                 Location::Base(RC_GC_SHOP_ITEM_4,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x03,                               "Shop Item 4",                                 RHT_GC_SHOP_ITEM_4,                                              RG_BUY_BOMBS_30,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_4), false, 120);
    locationTable[RC_GC_SHOP_ITEM_5] =                                                 Location::Base(RC_GC_SHOP_ITEM_5,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x04,                               "Shop Item 5",                                 RHT_GC_SHOP_ITEM_5,                                              RG_BUY_GORON_TUNIC,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_5), false, 200);
    locationTable[RC_GC_SHOP_ITEM_6] =                                                 Location::Base(RC_GC_SHOP_ITEM_6,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x05,                               "Shop Item 6",                                 RHT_GC_SHOP_ITEM_6,                                              RG_BUY_HEART,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_6), false, 10);
    locationTable[RC_GC_SHOP_ITEM_7] =                                                 Location::Base(RC_GC_SHOP_ITEM_7,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x06,                               "Shop Item 7",                                 RHT_GC_SHOP_ITEM_7,                                              RG_BUY_RED_POTION_40, SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_7), false, 40);
    locationTable[RC_GC_SHOP_ITEM_8] =                                                 Location::Base(RC_GC_SHOP_ITEM_8,                                               RCQUEST_BOTH,    RCTYPE_SHOP,                                                             ACTOR_EN_GIRLA,       SCENE_GORON_SHOP,                   0x07,                               "Shop Item 8",                                 RHT_GC_SHOP_ITEM_8,                                              RG_BUY_HEART,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHOP_ITEMS_GC_SHOP_ITEM_8), false, 10);

    /* +--------------+
       |  FISHSANITY  |
       +--------------+ */

    // Fishing Pond
    locationTable[RC_LH_CHILD_FISH_1] =                                                Location::Fish(RC_LH_CHILD_FISH_1,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 100,      RAND_INF_CHILD_FISH_1,                         "Child Pond Fish 1",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_2] =                                                Location::Fish(RC_LH_CHILD_FISH_2,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 101,      RAND_INF_CHILD_FISH_2,                         "Child Pond Fish 2",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_3] =                                                Location::Fish(RC_LH_CHILD_FISH_3,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 102,      RAND_INF_CHILD_FISH_3,                         "Child Pond Fish 3",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_4] =                                                Location::Fish(RC_LH_CHILD_FISH_4,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 103,      RAND_INF_CHILD_FISH_4,                         "Child Pond Fish 4",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_5] =                                                Location::Fish(RC_LH_CHILD_FISH_5,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 104,      RAND_INF_CHILD_FISH_5,                         "Child Pond Fish 5",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_6] =                                                Location::Fish(RC_LH_CHILD_FISH_6,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 105,      RAND_INF_CHILD_FISH_6,                         "Child Pond Fish 6",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_7] =                                                Location::Fish(RC_LH_CHILD_FISH_7,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 106,      RAND_INF_CHILD_FISH_7,                         "Child Pond Fish 7",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_8] =                                                Location::Fish(RC_LH_CHILD_FISH_8,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 107,      RAND_INF_CHILD_FISH_8,                         "Child Pond Fish 8",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_9] =                                                Location::Fish(RC_LH_CHILD_FISH_9,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 108,      RAND_INF_CHILD_FISH_9,                         "Child Pond Fish 9",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_10] =                                               Location::Fish(RC_LH_CHILD_FISH_10,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 109,      RAND_INF_CHILD_FISH_10,                        "Child Pond Fish 10",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_11] =                                               Location::Fish(RC_LH_CHILD_FISH_11,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 110,      RAND_INF_CHILD_FISH_11,                        "Child Pond Fish 11",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_12] =                                               Location::Fish(RC_LH_CHILD_FISH_12,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 111,      RAND_INF_CHILD_FISH_12,                        "Child Pond Fish 12",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_13] =                                               Location::Fish(RC_LH_CHILD_FISH_13,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 112,      RAND_INF_CHILD_FISH_13,                        "Child Pond Fish 13",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_14] =                                               Location::Fish(RC_LH_CHILD_FISH_14,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 113,      RAND_INF_CHILD_FISH_14,                        "Child Pond Fish 14",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_FISH_15] =                                               Location::Fish(RC_LH_CHILD_FISH_15,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 114,      RAND_INF_CHILD_FISH_15,                        "Child Pond Fish 15",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_CHILD_LOACH_1] =                                               Location::Fish(RC_LH_CHILD_LOACH_1,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 115,      RAND_INF_CHILD_LOACH_1,                        "Child Pond Loach 1",          RHT_LH_HYRULE_LOACH,               RG_NONE);
    locationTable[RC_LH_CHILD_LOACH_2] =                                               Location::Fish(RC_LH_CHILD_LOACH_2,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 116,      RAND_INF_CHILD_LOACH_2,                        "Child Pond Loach 2",          RHT_LH_HYRULE_LOACH,               RG_NONE);
    locationTable[RC_LH_ADULT_FISH_1] =                                                Location::Fish(RC_LH_ADULT_FISH_1,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 100,      RAND_INF_ADULT_FISH_1,                         "Adult Pond Fish 1",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_2] =                                                Location::Fish(RC_LH_ADULT_FISH_2,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 101,      RAND_INF_ADULT_FISH_2,                         "Adult Pond Fish 2",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_3] =                                                Location::Fish(RC_LH_ADULT_FISH_3,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 102,      RAND_INF_ADULT_FISH_3,                         "Adult Pond Fish 3",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_4] =                                                Location::Fish(RC_LH_ADULT_FISH_4,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 103,      RAND_INF_ADULT_FISH_4,                         "Adult Pond Fish 4",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_5] =                                                Location::Fish(RC_LH_ADULT_FISH_5,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 104,      RAND_INF_ADULT_FISH_5,                         "Adult Pond Fish 5",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_6] =                                                Location::Fish(RC_LH_ADULT_FISH_6,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 105,      RAND_INF_ADULT_FISH_6,                         "Adult Pond Fish 6",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_7] =                                                Location::Fish(RC_LH_ADULT_FISH_7,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 106,      RAND_INF_ADULT_FISH_7,                         "Adult Pond Fish 7",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_8] =                                                Location::Fish(RC_LH_ADULT_FISH_8,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 107,      RAND_INF_ADULT_FISH_8,                         "Adult Pond Fish 8",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_9] =                                                Location::Fish(RC_LH_ADULT_FISH_9,                                              RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 108,      RAND_INF_ADULT_FISH_9,                         "Adult Pond Fish 9",           RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_10] =                                               Location::Fish(RC_LH_ADULT_FISH_10,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 109,      RAND_INF_ADULT_FISH_10,                        "Adult Pond Fish 10",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_11] =                                               Location::Fish(RC_LH_ADULT_FISH_11,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 110,      RAND_INF_ADULT_FISH_11,                        "Adult Pond Fish 11",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_12] =                                               Location::Fish(RC_LH_ADULT_FISH_12,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 111,      RAND_INF_ADULT_FISH_12,                        "Adult Pond Fish 12",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_13] =                                               Location::Fish(RC_LH_ADULT_FISH_13,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 112,      RAND_INF_ADULT_FISH_13,                        "Adult Pond Fish 13",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_14] =                                               Location::Fish(RC_LH_ADULT_FISH_14,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 113,      RAND_INF_ADULT_FISH_14,                        "Adult Pond Fish 14",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_FISH_15] =                                               Location::Fish(RC_LH_ADULT_FISH_15,                                             RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 114,      RAND_INF_ADULT_FISH_15,                        "Adult Pond Fish 15",          RHT_LH_POND_FISH,                  RG_NONE);
    locationTable[RC_LH_ADULT_LOACH] =                                                 Location::Fish(RC_LH_ADULT_LOACH,                                               RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,                 115,      RAND_INF_ADULT_LOACH,                          "Adult Pond Loach",            RHT_LH_HYRULE_LOACH,               RG_NONE);
    // Grotto fish
    locationTable[RC_KF_STORMS_GROTTO_FISH] =                                          Location::GrottoFish(RC_KF_STORMS_GROTTO_FISH,                                  RCQUEST_BOTH,                                        RCAREA_KOKIRI_FOREST,                                                                          0x12C,    RAND_INF_GROTTO_FISH_KF_STORMS_GROTTO,         "Storms Grotto Fish",          RHT_KF_STORMS_GROTTO_FISH);
    locationTable[RC_LW_NEAR_SHORTCUTS_GROTTO_FISH] =                                  Location::GrottoFish(RC_LW_NEAR_SHORTCUTS_GROTTO_FISH,                          RCQUEST_BOTH,                                        RCAREA_LOST_WOODS,                                                                             0x114,    RAND_INF_GROTTO_FISH_LW_NEAR_SHORTCUTS_GROTTO, "Near Shortcuts Grotto Fish",  RHT_LW_NEAR_SHORTCUTS_GROTTO_FISH);
    locationTable[RC_HF_SOUTHEAST_GROTTO_FISH] =                                       Location::GrottoFish(RC_HF_SOUTHEAST_GROTTO_FISH,                               RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                                                           0x122,    RAND_INF_GROTTO_FISH_HF_SOUTHEAST_GROTTO,      "Southeast Grotto Fish",       RHT_HF_SOUTHEAST_GROTTO_FISH);
    locationTable[RC_HF_OPEN_GROTTO_FISH] =                                            Location::GrottoFish(RC_HF_OPEN_GROTTO_FISH,                                    RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                                                           0x103,    RAND_INF_GROTTO_FISH_HF_OPEN_GROTTO,           "Open Grotto Fish",            RHT_HF_OPEN_GROTTO_FISH);
    locationTable[RC_HF_NEAR_MARKET_GROTTO_FISH] =                                     Location::GrottoFish(RC_HF_NEAR_MARKET_GROTTO_FISH,                             RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                                                           0x100,    RAND_INF_GROTTO_FISH_HF_NEAR_MARKET_GROTTO,    "Near Market Grotto Fish",     RHT_HF_NEAR_MARKET_GROTTO_FISH);
    locationTable[RC_KAK_OPEN_GROTTO_FISH] =                                           Location::GrottoFish(RC_KAK_OPEN_GROTTO_FISH,                                   RCQUEST_BOTH,                                        RCAREA_KAKARIKO_VILLAGE,                                                                       0x128,    RAND_INF_GROTTO_FISH_KAK_OPEN_GROTTO,          "Open Grotto Fish",            RHT_KAK_OPEN_GROTTO_FISH);
    locationTable[RC_DMT_STORMS_GROTTO_FISH] =                                         Location::GrottoFish(RC_DMT_STORMS_GROTTO_FISH,                                 RCQUEST_BOTH,                                        RCAREA_DEATH_MOUNTAIN_TRAIL,                                                                   0x157,    RAND_INF_GROTTO_FISH_DMT_STORMS_GROTTO,        "Storms Grotto Fish",          RHT_DMT_STORMS_GROTTO_FISH);
    locationTable[RC_DMC_UPPER_GROTTO_FISH] =                                          Location::GrottoFish(RC_DMC_UPPER_GROTTO_FISH,                                  RCQUEST_BOTH,                                        RCAREA_DEATH_MOUNTAIN_CRATER,                                                                  0x17A,    RAND_INF_GROTTO_FISH_DMC_UPPER_GROTTO,         "Upper Grotto Fish",           RHT_DMC_UPPER_GROTTO_FISH);
    locationTable[RC_ZR_OPEN_GROTTO_FISH] =                                            Location::GrottoFish(RC_ZR_OPEN_GROTTO_FISH,                                    RCQUEST_BOTH,                                        RCAREA_ZORAS_RIVER,                                                                            0x129,    RAND_INF_GROTTO_FISH_ZR_OPEN_GROTTO,           "Open Grotto Fish",            RHT_ZR_OPEN_GROTTO_FISH);
    // Zora's Domain fish
    locationTable[RC_ZD_FISH_1] =                                                      Location::Fish(RC_ZD_FISH_1,                                                    RCQUEST_BOTH,                                                                             ACTOR_EN_FISH,        SCENE_ZORAS_DOMAIN,                 -1 ^ 0,   RAND_INF_ZD_FISH_1,                            "Fish 1",                      RHT_ZD_FISH,                       RG_FISH);
    locationTable[RC_ZD_FISH_2] =                                                      Location::Fish(RC_ZD_FISH_2,                                                    RCQUEST_BOTH,                                                                             ACTOR_EN_FISH,        SCENE_ZORAS_DOMAIN,                 -1 ^ 1,   RAND_INF_ZD_FISH_2,                            "Fish 2",                      RHT_ZD_FISH,                       RG_FISH);
    locationTable[RC_ZD_FISH_3] =                                                      Location::Fish(RC_ZD_FISH_3,                                                    RCQUEST_BOTH,                                                                             ACTOR_EN_FISH,        SCENE_ZORAS_DOMAIN,                 -1 ^ 2,   RAND_INF_ZD_FISH_3,                            "Fish 3",                      RHT_ZD_FISH,                       RG_FISH);
    locationTable[RC_ZD_FISH_4] =                                                      Location::Fish(RC_ZD_FISH_4,                                                    RCQUEST_BOTH,                                                                             ACTOR_EN_FISH,        SCENE_ZORAS_DOMAIN,                 -1 ^ 3,   RAND_INF_ZD_FISH_4,                            "Fish 4",                      RHT_ZD_FISH,                       RG_FISH);
    locationTable[RC_ZD_FISH_5] =                                                      Location::Fish(RC_ZD_FISH_5,                                                    RCQUEST_BOTH,                                                                             ACTOR_EN_FISH,        SCENE_ZORAS_DOMAIN,                 -1 ^ 4,   RAND_INF_ZD_FISH_5,                            "Fish 5",                      RHT_ZD_FISH,                       RG_FISH);


    /* +----------------+
       |  SHUFFLE POTS  |
       +----------------+ */

    // Overworld Pots
    //            Randomizer Check                                 	Randomizer Check                                           Quest         Area                          Scene ID                           Params                          Short Name                    	  Spoiler Name                                Hint Text Key                     Vanilla             Spoiler Collection Check
    locationTable[RC_KF_LINKS_HOUSE_POT]                    	=	Location::Pot(RC_KF_LINKS_HOUSE_POT,                       RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_LINKS_HOUSE,                 TWO_ACTOR_PARAMS(-118, 51),     "Links House Pot",                  "KF Links House Pot",                       RHT_POT_KOKIRI_FOREST,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_LINKS_HOUSE_POT));
    locationTable[RC_KF_TWINS_HOUSE_POT_2]                  	=	Location::Pot(RC_KF_TWINS_HOUSE_POT_2,                     RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_TWINS_HOUSE,                 TWO_ACTOR_PARAMS(35, 57),       "Twins House Pot 2",                "KF Twins House Pot 2",                     RHT_POT_KOKIRI_FOREST,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_TWINS_HOUSE_POT_2));
    locationTable[RC_KF_TWINS_HOUSE_POT_1]                  	=	Location::Pot(RC_KF_TWINS_HOUSE_POT_1,                     RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_TWINS_HOUSE,                 TWO_ACTOR_PARAMS(33, -55),      "Twins House Pot 1",                "KF Twins House Pot 1",                     RHT_POT_KOKIRI_FOREST,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_TWINS_HOUSE_POT_1));
    locationTable[RC_KF_BROTHERS_HOUSE_POT_1]               	=	Location::Pot(RC_KF_BROTHERS_HOUSE_POT_1,                  RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_KNOW_IT_ALL_BROS_HOUSE,      TWO_ACTOR_PARAMS(-134, -29),    "Brothers House Pot 1",             "KF Brothers House Pot 1",                  RHT_POT_KOKIRI_FOREST,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_BROTHERS_HOUSE_POT_1));
    locationTable[RC_KF_BROTHERS_HOUSE_POT_2]               	=	Location::Pot(RC_KF_BROTHERS_HOUSE_POT_2,                  RCQUEST_BOTH, RCAREA_KOKIRI_FOREST,         SCENE_KNOW_IT_ALL_BROS_HOUSE,      TWO_ACTOR_PARAMS(-68, 114),     "Brothers House Pot 2",             "KF Brothers House Pot 2",                  RHT_POT_KOKIRI_FOREST,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_KF_BROTHERS_HOUSE_POT_2));
    locationTable[RC_GF_BREAK_ROOM_POT_1]                   	=	Location::Pot(RC_GF_BREAK_ROOM_POT_1,                      RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1438, -3629),  "Break Room Pot 1",                 "GF Break Room Pot 1",                      RHT_POT_GERUDO_FORTRESS,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_BREAK_ROOM_POT_1));
    locationTable[RC_GF_BREAK_ROOM_POT_2]                   	=	Location::Pot(RC_GF_BREAK_ROOM_POT_2,                      RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1403, -3679),  "Break Room Pot 2",                 "GF Break Room Pot 2",                      RHT_POT_GERUDO_FORTRESS,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_BREAK_ROOM_POT_2));
    locationTable[RC_GF_KITCHEN_POT_1]                      	=	Location::Pot(RC_GF_KITCHEN_POT_1,                         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1908, -789),   "Kitchen Pot 1",                    "GF Kitchen Pot 1",                         RHT_POT_GERUDO_FORTRESS,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_KITCHEN_POT_1));
    locationTable[RC_GF_KITCHEN_POT_2]                      	=	Location::Pot(RC_GF_KITCHEN_POT_2,                         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(1951, -850),   "Kitchen Pot 2",                    "GF Kitchen Pot 2",                         RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_KITCHEN_POT_2));
    locationTable[RC_GF_NORTH_F1_CARPENTER_POT_1]           	=	Location::Pot(RC_GF_NORTH_F1_CARPENTER_POT_1,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(-475, -2622),  "North F1 Carpenter Pot 1",         "GF North F1 Carpenter Pot 1",              RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F1_CARPENTER_POT_1));
    locationTable[RC_GF_NORTH_F1_CARPENTER_POT_2]           	=	Location::Pot(RC_GF_NORTH_F1_CARPENTER_POT_2,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(-512, -2621),  "North F1 Carpenter Pot 2",         "GF North F1 Carpenter Pot 2",              RHT_POT_GERUDO_FORTRESS,	        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F1_CARPENTER_POT_2));
    locationTable[RC_GF_NORTH_F1_CARPENTER_POT_3]           	=	Location::Pot(RC_GF_NORTH_F1_CARPENTER_POT_3,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(-511, -2582),  "North F1 Carpenter Pot 3",         "GF North F1 Carpenter Pot 3",              RHT_POT_GERUDO_FORTRESS,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F1_CARPENTER_POT_3));
    locationTable[RC_GF_NORTH_F2_CARPENTER_POT_1]           	=	Location::Pot(RC_GF_NORTH_F2_CARPENTER_POT_1,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(67, -1975),    "North F2 Carpenter Pot 1",         "GF North F2 Carpenter Pot 1",              RHT_POT_GERUDO_FORTRESS,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F2_CARPENTER_POT_1));
    locationTable[RC_GF_NORTH_F2_CARPENTER_POT_2]           	=	Location::Pot(RC_GF_NORTH_F2_CARPENTER_POT_2,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(39, -1943),    "North F2 Carpenter Pot 2",         "GF North F2 Carpenter Pot 2",              RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_NORTH_F2_CARPENTER_POT_2));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_POT_1]           	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_POT_1,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(691, 48),      "South F1 Carpenter Pot 1",         "GF South F1 Carpenter Pot 1",              RHT_POT_GERUDO_FORTRESS,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_POT_1));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_POT_2]           	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_POT_2,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(691, 16),      "South F1 Carpenter Pot 2",         "GF South F1 Carpenter Pot 2",              RHT_POT_GERUDO_FORTRESS,	        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_POT_2));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_POT_3]           	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_POT_3,              RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(661, 16),      "South F1 Carpenter Pot 3",         "GF South F1 Carpenter Pot 3",              RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_POT_3));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_1]      	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_1,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(26, 524),      "South F1 Carpenter Cell Pot 1",    "GF South F1 Carpenter Cell Pot 1",         RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_1));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_2]      	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_2,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(61, 549),      "South F1 Carpenter Cell Pot 2",    "GF South F1 Carpenter Cell Pot 2",         RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_2));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_3]      	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_3,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(215, 549),     "South F1 Carpenter Cell Pot 3",    "GF South F1 Carpenter Cell Pot 3",         RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_3));
    locationTable[RC_GF_SOUTH_F1_CARPENTER_CELL_POT_4]      	=	Location::Pot(RC_GF_SOUTH_F1_CARPENTER_CELL_POT_4,         RCQUEST_BOTH, RCAREA_GERUDO_FORTRESS,       SCENE_THIEVES_HIDEOUT,             TWO_ACTOR_PARAMS(254, 529),     "South F1 Carpenter Cell Pot 4",    "GF South F1 Carpenter Cell Pot 4",         RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GF_SOUTH_F1_CARPENTER_CELL_POT_4));
    locationTable[RC_WASTELAND_NEAR_GS_POT_1]               	=	Location::Pot(RC_WASTELAND_NEAR_GS_POT_1,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(488, -2424),   "Near GS Pot 1",                    "Wasteland Near GS Pot 1",                  RHT_POT_GERUDO_FORTRESS,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_1));
    locationTable[RC_WASTELAND_NEAR_GS_POT_2]               	=	Location::Pot(RC_WASTELAND_NEAR_GS_POT_2,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(485, -2463),   "Near GS Pot 2",                    "Wasteland Near GS Pot 2",                  RHT_POT_GERUDO_FORTRESS,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_2));
    locationTable[RC_WASTELAND_NEAR_GS_POT_3]               	=	Location::Pot(RC_WASTELAND_NEAR_GS_POT_3,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(806, -2426),   "Near GS Pot 3",                    "Wasteland Near GS Pot 3",                  RHT_POT_GERUDO_FORTRESS,	        RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_3));
    locationTable[RC_WASTELAND_NEAR_GS_POT_4]               	=	Location::Pot(RC_WASTELAND_NEAR_GS_POT_4,                  RCQUEST_BOTH, RCAREA_WASTELAND,             SCENE_HAUNTED_WASTELAND,           TWO_ACTOR_PARAMS(801, -2460),   "Near GS Pot 4",                    "Wasteland Near GS Pot 4",                  RHT_POT_GERUDO_FORTRESS,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WASTELAND_NEAR_GS_POT_4));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_1]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_1,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-60, 27),      "Guard House Child Pot 1",          "MK Guard House Child Pot 1",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_1));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_2]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_2,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-89, 28),      "Guard House Child Pot 2",          "MK Guard House Child Pot 2",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_2));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_3]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_3,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-110, 6),      "Guard House Child Pot 3",          "MK Guard House Child Pot 3",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_3));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_4]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_4,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-58, -7),      "Guard House Child Pot 4",          "MK Guard House Child Pot 4",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_4));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_5]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_5,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-80, -7),      "Guard House Child Pot 5",          "MK Guard House Child Pot 5",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_5));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_6]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_6,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-65, -45),     "Guard House Child Pot 6",          "MK Guard House Child Pot 6",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_6));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_7]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_7,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-85, -41),     "Guard House Child Pot 7",          "MK Guard House Child Pot 7",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_7));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_8]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_8,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-107, -45),    "Guard House Child Pot 8",          "MK Guard House Child Pot 8",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_8));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_9]            	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_9,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-66, -79),     "Guard House Child Pot 9",          "MK Guard House Child Pot 9",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_9));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_10]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_10,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(-88, -84),     "Guard House Child Pot 10",         "MK Guard House Child Pot 10",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_10));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_11]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_11,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(70, 215),      "Guard House Child Pot 11",         "MK Guard House Child Pot 11",              RHT_POT_MARKET,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_11));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_12]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_12,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(68, 148),      "Guard House Child Pot 12",         "MK Guard House Child Pot 12",              RHT_POT_MARKET,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_12));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_13]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_13,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(73, 117),      "Guard House Child Pot 13",         "MK Guard House Child Pot 13",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_13));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_14]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_14,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(40, 123),      "Guard House Child Pot 14",         "MK Guard House Child Pot 14",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_14));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_15]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_15,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(43, 89),       "Guard House Child Pot 15",         "MK Guard House Child Pot 15",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_15));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_16]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_16,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(68, 81),       "Guard House Child Pot 16",         "MK Guard House Child Pot 16",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_16));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_17]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_17,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(21, 73),       "Guard House Child Pot 17",         "MK Guard House Child Pot 17",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_17));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_18]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_18,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(64, 45),       "Guard House Child Pot 18",         "MK Guard House Child Pot 18",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_18));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_19]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_19,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(84, 31),       "Guard House Child Pot 19",         "MK Guard House Child Pot 19",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_19));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_20]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_20,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(42, 26),       "Guard House Child Pot 20",         "MK Guard House Child Pot 20",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_20));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_21]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_21,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(20, 34),       "Guard House Child Pot 21",         "MK Guard House Child Pot 21",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_21));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_22]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_22,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(89, -2),       "Guard House Child Pot 22",         "MK Guard House Child Pot 22",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_22));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_23]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_23,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(70, -12),      "Guard House Child Pot 23",         "MK Guard House Child Pot 23",              RHT_POT_MARKET,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_23));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_24]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_24,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(42, -5),       "Guard House Child Pot 24",         "MK Guard House Child Pot 24",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_24));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_25]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_25,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(16, -6),       "Guard House Child Pot 25",         "MK Guard House Child Pot 25",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_25));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_26]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_26,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(68, -44),      "Guard House Child Pot 26",         "MK Guard House Child Pot 26",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_26));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_27]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_27,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(37, -40),      "Guard House Child Pot 27",         "MK Guard House Child Pot 27",              RHT_POT_MARKET,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_27));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_28]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_28,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(70, -80),      "Guard House Child Pot 28",         "MK Guard House Child Pot 28",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_28));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_29]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_29,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(50, -74),      "Guard House Child Pot 29",         "MK Guard House Child Pot 29",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_29));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_30]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_30,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(28, -79),      "Guard House Child Pot 30",         "MK Guard House Child Pot 30",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_30));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_31]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_31,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(39, -111),     "Guard House Child Pot 31",         "MK Guard House Child Pot 31",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_31));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_32]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_32,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(169, 216),     "Guard House Child Pot 32",         "MK Guard House Child Pot 32",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_32));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_33]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_33,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(168, 166),     "Guard House Child Pot 33",         "MK Guard House Child Pot 33",              RHT_POT_MARKET,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_33));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_34]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_34,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, 120),     "Guard House Child Pot 34",         "MK Guard House Child Pot 34",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_34));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_35]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_35,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(177, 85),      "Guard House Child Pot 35",         "MK Guard House Child Pot 35",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_35));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_36]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_36,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(155, 39),      "Guard House Child Pot 36",         "MK Guard House Child Pot 36",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_36));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_37]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_37,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(184, 13),      "Guard House Child Pot 37",         "MK Guard House Child Pot 37",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_37));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_38]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_38,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(156, -1),      "Guard House Child Pot 38",         "MK Guard House Child Pot 38",              RHT_POT_MARKET,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_38));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_39]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_39,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(181, -33),     "Guard House Child Pot 39",         "MK Guard House Child Pot 39",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_39));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_40]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_40,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(156, -45),     "Guard House Child Pot 40",         "MK Guard House Child Pot 40",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_40));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_41]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_41,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(172, -82),     "Guard House Child Pot 41",         "MK Guard House Child Pot 41",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_41));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_42]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_42,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, -120),    "Guard House Child Pot 42",         "MK Guard House Child Pot 42",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_42));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_43]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_43,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, -166),    "Guard House Child Pot 43",         "MK Guard House Child Pot 43",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_43));
    locationTable[RC_MK_GUARD_HOUSE_CHILD_POT_44]           	=	Location::Pot(RC_MK_GUARD_HOUSE_CHILD_POT_44,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(170, -216),    "Guard House Child Pot 44",         "MK Guard House Child Pot 44",              RHT_POT_MARKET,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_CHILD_POT_44));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_1]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_1,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(61, 204),      "Guard House Adult Pot 1",          "MK Guard House Adult Pot 1",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_1));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_2]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_2,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(71, 132),      "Guard House Adult Pot 2",          "MK Guard House Adult Pot 2",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_2));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_3]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_3,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(74, 23),       "Guard House Adult Pot 3",          "MK Guard House Adult Pot 3",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_3));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_4]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_4,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(40, 4),        "Guard House Adult Pot 4",          "MK Guard House Adult Pot 4",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_4));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_5]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_5,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(71, -22),      "Guard House Adult Pot 5",          "MK Guard House Adult Pot 5",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_5));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_6]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_6,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(44, -151),     "Guard House Adult Pot 6",          "MK Guard House Adult Pot 6",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_6));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_7]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_7,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(79, -182),     "Guard House Adult Pot 7",          "MK Guard House Adult Pot 7",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_7));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_8]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_8,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(44, -198),     "Guard House Adult Pot 8",          "MK Guard House Adult Pot 8",               RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_8));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_9]            	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_9,               RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(168, 210),     "Guard House Adult Pot 9",          "MK Guard House Adult Pot 9",               RHT_POT_MARKET,	                RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_9));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_10]           	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_10,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(167, -122),    "Guard House Adult Pot 10",         "MK Guard House Adult Pot 10",              RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_10));
    locationTable[RC_MK_GUARD_HOUSE_ADULT_POT_11]           	=	Location::Pot(RC_MK_GUARD_HOUSE_ADULT_POT_11,              RCQUEST_BOTH, RCAREA_MARKET,                SCENE_MARKET_GUARD_HOUSE,          TWO_ACTOR_PARAMS(167, -210),    "Guard House Adult Pot 11",         "MK Guard House Adult Pot 11",              RHT_POT_MARKET,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_GUARD_HOUSE_ADULT_POT_11));
    locationTable[RC_MK_BACK_ALLEY_HOUSE_POT_1]             	=	Location::Pot(RC_MK_BACK_ALLEY_HOUSE_POT_1,                RCQUEST_BOTH, RCAREA_MARKET,                SCENE_BACK_ALLEY_HOUSE,            TWO_ACTOR_PARAMS(100, 45),      "Back Alley House Pot 1",           "MK Back Alley House Pot 1",                RHT_POT_MARKET,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_BACK_ALLEY_HOUSE_POT_1));
    locationTable[RC_MK_BACK_ALLEY_HOUSE_POT_2]             	=	Location::Pot(RC_MK_BACK_ALLEY_HOUSE_POT_2,                RCQUEST_BOTH, RCAREA_MARKET,                SCENE_BACK_ALLEY_HOUSE,            TWO_ACTOR_PARAMS(12, -180),     "Back Alley House Pot 2",           "MK Back Alley House Pot 2",                RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_BACK_ALLEY_HOUSE_POT_2));
    locationTable[RC_MK_BACK_ALLEY_HOUSE_POT_3]             	=	Location::Pot(RC_MK_BACK_ALLEY_HOUSE_POT_3,                RCQUEST_BOTH, RCAREA_MARKET,                SCENE_BACK_ALLEY_HOUSE,            TWO_ACTOR_PARAMS(-54, -180),    "Back Alley House Pot 3",           "MK Back Alley House Pot 3",                RHT_POT_MARKET,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_MK_BACK_ALLEY_HOUSE_POT_3));
    locationTable[RC_KAK_NEAR_POTION_SHOP_POT_1]            	=	Location::Pot(RC_KAK_NEAR_POTION_SHOP_POT_1,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(222, -377),    "Near Potion Shop Pot 1",           "Kak Near Potion Shop Pot 1",               RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_POTION_SHOP_POT_1));
    locationTable[RC_KAK_NEAR_POTION_SHOP_POT_2]            	=	Location::Pot(RC_KAK_NEAR_POTION_SHOP_POT_2,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(255, -366),    "Near Potion Shop Pot 2",           "Kak Near Potion Shop Pot 2",               RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_POTION_SHOP_POT_2));
    locationTable[RC_KAK_NEAR_POTION_SHOP_POT_3]            	=	Location::Pot(RC_KAK_NEAR_POTION_SHOP_POT_3,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(284, -356),    "Near Potion Shop Pot 3",           "Kak Near Potion Shop Pot 3",               RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_POTION_SHOP_POT_3));
    locationTable[RC_KAK_NEAR_IMPAS_HOUSE_POT_1]            	=	Location::Pot(RC_KAK_NEAR_IMPAS_HOUSE_POT_1,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-384, 1568),   "Near Impas House Pot 1",           "Kak Near Impas House Pot 1",               RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_IMPAS_HOUSE_POT_1));
    locationTable[RC_KAK_NEAR_IMPAS_HOUSE_POT_2]            	=	Location::Pot(RC_KAK_NEAR_IMPAS_HOUSE_POT_2,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-384, 1539),   "Near Impas House Pot 2",           "Kak Near Impas House Pot 2",               RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_IMPAS_HOUSE_POT_2));
    locationTable[RC_KAK_NEAR_IMPAS_HOUSE_POT_3]            	=	Location::Pot(RC_KAK_NEAR_IMPAS_HOUSE_POT_3,               RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-384, 1510),   "Near Impas House Pot 3",           "Kak Near Impas House Pot 3",               RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_IMPAS_HOUSE_POT_3));
    locationTable[RC_KAK_NEAR_GUARDS_HOUSE_POT_1]           	=	Location::Pot(RC_KAK_NEAR_GUARDS_HOUSE_POT_1,              RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-392, -870),   "Near Guards House Pot 1",          "Kak Near Guards House Pot 1",              RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_GUARDS_HOUSE_POT_1));
    locationTable[RC_KAK_NEAR_GUARDS_HOUSE_POT_2]           	=	Location::Pot(RC_KAK_NEAR_GUARDS_HOUSE_POT_2,              RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-422, -883),   "Near Guards House Pot 2",          "Kak Near Guards House Pot 2",              RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_GUARDS_HOUSE_POT_2));
    locationTable[RC_KAK_NEAR_GUARDS_HOUSE_POT_3]           	=	Location::Pot(RC_KAK_NEAR_GUARDS_HOUSE_POT_3,              RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(-450, -895),   "Near Guards House Pot 3",          "Kak Near Guards House Pot 3",              RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_GUARDS_HOUSE_POT_3));
    locationTable[RC_KAK_NEAR_MEDICINE_SHOP_POT_1]          	=	Location::Pot(RC_KAK_NEAR_MEDICINE_SHOP_POT_1,             RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(781, 89),      "Near Medicine Shop Pot 1",         "Kak Near Medicine Shop Pot 1",             RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_MEDICINE_SHOP_POT_1));
    locationTable[RC_KAK_NEAR_MEDICINE_SHOP_POT_2]          	=	Location::Pot(RC_KAK_NEAR_MEDICINE_SHOP_POT_2,             RCQUEST_BOTH, RCAREA_KAKARIKO_VILLAGE,      SCENE_KAKARIKO_VILLAGE,            TWO_ACTOR_PARAMS(815, 89),      "Near Medicine Shop Pot 2",         "Kak Near Medicine Shop Pot 2",             RHT_POT_KAKARIKO,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_KAK_NEAR_MEDICINE_SHOP_POT_2));
    locationTable[RC_GY_DAMPES_GRAVE_POT_1]                 	=	Location::Pot(RC_GY_DAMPES_GRAVE_POT_1,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(-319, -1542),  "Dampes Grave Pot 1",               "GY Dampes Grave Pot 1",                    RHT_POT_GRAVEYARD,	            RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_1));
    locationTable[RC_GY_DAMPES_GRAVE_POT_2]                 	=	Location::Pot(RC_GY_DAMPES_GRAVE_POT_2,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(-319, -1600),  "Dampes Grave Pot 2",               "GY Dampes Grave Pot 2",                    RHT_POT_GRAVEYARD,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_2));
    locationTable[RC_GY_DAMPES_GRAVE_POT_3]                 	=	Location::Pot(RC_GY_DAMPES_GRAVE_POT_3,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(-364, -1571),  "Dampes Grave Pot 3",               "GY Dampes Grave Pot 3",                    RHT_POT_GRAVEYARD,	            RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_3));
    locationTable[RC_GY_DAMPES_GRAVE_POT_4]                 	=	Location::Pot(RC_GY_DAMPES_GRAVE_POT_4,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(198, -1540),   "Dampes Grave Pot 4",               "GY Dampes Grave Pot 4",                    RHT_POT_GRAVEYARD,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_4));
    locationTable[RC_GY_DAMPES_GRAVE_POT_5]                 	=	Location::Pot(RC_GY_DAMPES_GRAVE_POT_5,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(198, -1608),   "Dampes Grave Pot 5",               "GY Dampes Grave Pot 5",                    RHT_POT_GRAVEYARD,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_5));
    locationTable[RC_GY_DAMPES_GRAVE_POT_6]                 	=	Location::Pot(RC_GY_DAMPES_GRAVE_POT_6,                    RCQUEST_BOTH, RCAREA_GRAVEYARD,             SCENE_WINDMILL_AND_DAMPES_GRAVE,   TWO_ACTOR_PARAMS(239, -1577),   "Dampes Grave Pot 6",               "GY Dampes Grave Pot 6",                    RHT_POT_GRAVEYARD,	            RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GY_DAMPES_GRAVE_POT_6));
    locationTable[RC_GC_LOWER_STAIRCASE_POT_1]              	=	Location::Pot(RC_GC_LOWER_STAIRCASE_POT_1,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-189, 866),    "Lower Staircase Pot 1",            "GC Lower Staircase Pot 1",                 RHT_POT_GORON_CITY,	            RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_LOWER_STAIRCASE_POT_1));
    locationTable[RC_GC_LOWER_STAIRCASE_POT_2]              	=	Location::Pot(RC_GC_LOWER_STAIRCASE_POT_2,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-271, 825),    "Lower Staircase Pot 2",            "GC Lower Staircase Pot 2",                 RHT_POT_GORON_CITY,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_LOWER_STAIRCASE_POT_2));
    locationTable[RC_GC_UPPER_STAIRCASE_POT_1]              	=	Location::Pot(RC_GC_UPPER_STAIRCASE_POT_1,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-1170, 60),    "Upper Staircase Pot 1",            "GC Upper Staircase Pot 1",                 RHT_POT_GORON_CITY,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_UPPER_STAIRCASE_POT_1));
    locationTable[RC_GC_UPPER_STAIRCASE_POT_2]              	=	Location::Pot(RC_GC_UPPER_STAIRCASE_POT_2,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-1185, 95),    "Upper Staircase Pot 2",            "GC Upper Staircase Pot 2",                 RHT_POT_GORON_CITY,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_UPPER_STAIRCASE_POT_2));
    locationTable[RC_GC_UPPER_STAIRCASE_POT_3]              	=	Location::Pot(RC_GC_UPPER_STAIRCASE_POT_3,                 RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-1200, 63),    "Upper Staircase Pot 3",            "GC Upper Staircase Pot 3",                 RHT_POT_GORON_CITY,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_UPPER_STAIRCASE_POT_3));
    locationTable[RC_GC_MEDIGORON_POT_1]                    	=	Location::Pot(RC_GC_MEDIGORON_POT_1,                       RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(-694, 1196),   "Medigoron Pot 1",                  "GC Medigoron Pot 1",                       RHT_POT_GORON_CITY,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_MEDIGORON_POT_1));
    locationTable[RC_GC_DARUNIA_POT_1]                      	=	Location::Pot(RC_GC_DARUNIA_POT_1,                         RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(262, -1210),   "Darunia Pot 1",                    "GC Darunia Pot 1",                         RHT_POT_GORON_CITY,	            RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_DARUNIA_POT_1));
    locationTable[RC_GC_DARUNIA_POT_2]                      	=	Location::Pot(RC_GC_DARUNIA_POT_2,                         RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(261, -1254),   "Darunia Pot 2",                    "GC Darunia Pot 2",                         RHT_POT_GORON_CITY,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_DARUNIA_POT_2));
    locationTable[RC_GC_DARUNIA_POT_3]                      	=	Location::Pot(RC_GC_DARUNIA_POT_3,                         RCQUEST_BOTH, RCAREA_GORON_CITY,            SCENE_GORON_CITY,                  TWO_ACTOR_PARAMS(262, -1386),   "Darunia Pot 3",                    "GC Darunia Pot 3",                         RHT_POT_GORON_CITY,	            RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GC_DARUNIA_POT_3));
    locationTable[RC_DMC_NEAR_GC_POT_2]                     	=	Location::Pot(RC_DMC_NEAR_GC_POT_2,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1546, 141),   "Near GC Pot 2",                    "DMC Near GC Pot 2",                        RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_2));
    locationTable[RC_DMC_NEAR_GC_POT_3]                     	=	Location::Pot(RC_DMC_NEAR_GC_POT_3,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1641, -127),  "Near GC Pot 3",                    "DMC Near GC Pot 3",                        RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_3));
    locationTable[RC_DMC_NEAR_GC_POT_1]                     	=	Location::Pot(RC_DMC_NEAR_GC_POT_1,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1590, 132),   "Near GC Pot 1",                    "DMC Near GC Pot 1",                        RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_1));
    locationTable[RC_DMC_NEAR_GC_POT_4]                     	=	Location::Pot(RC_DMC_NEAR_GC_POT_4,                        RCQUEST_BOTH, RCAREA_DEATH_MOUNTAIN_CRATER, SCENE_DEATH_MOUNTAIN_CRATER,       TWO_ACTOR_PARAMS(-1606, -166),  "Near GC Pot 4",                    "DMC Near GC Pot 4",                        RHT_POT_DEATH_MOUNTAIN_CRATER,    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DMC_NEAR_GC_POT_4));
    locationTable[RC_ZD_NEAR_SHOP_POT_1]                    	=	Location::Pot(RC_ZD_NEAR_SHOP_POT_1,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(676, 377),     "Near Shop Pot 1",                  "ZD Near Shop Pot 1",                       RHT_POT_ZORAS_DOMAIN,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_1));
    locationTable[RC_ZD_NEAR_SHOP_POT_2]                    	=	Location::Pot(RC_ZD_NEAR_SHOP_POT_2,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(614, 419),     "Near Shop Pot 2",                  "ZD Near Shop Pot 2",                       RHT_POT_ZORAS_DOMAIN,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_2));
    locationTable[RC_ZD_NEAR_SHOP_POT_3]                    	=	Location::Pot(RC_ZD_NEAR_SHOP_POT_3,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(289, 415),     "Near Shop Pot 3",                  "ZD Near Shop Pot 3",                       RHT_POT_ZORAS_DOMAIN,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_3));
    locationTable[RC_ZD_NEAR_SHOP_POT_4]                    	=	Location::Pot(RC_ZD_NEAR_SHOP_POT_4,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(289, 289),     "Near Shop Pot 4",                  "ZD Near Shop Pot 4",                       RHT_POT_ZORAS_DOMAIN,	            RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_4));
    locationTable[RC_ZD_NEAR_SHOP_POT_5]                    	=	Location::Pot(RC_ZD_NEAR_SHOP_POT_5,                       RCQUEST_BOTH, RCAREA_ZORAS_DOMAIN,          SCENE_ZORAS_DOMAIN,                TWO_ACTOR_PARAMS(220, 384),     "Near Shop Pot 5",                  "ZD Near Shop Pot 5",                       RHT_POT_ZORAS_DOMAIN,	            RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZD_NEAR_SHOP_POT_5));
    locationTable[RC_ZF_HIDDEN_CAVE_POT_1]                  	=	Location::Pot(RC_ZF_HIDDEN_CAVE_POT_1,                     RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(385, 2587),    "Hidden Cave Pot 1",                "ZF Hidden Cave Pot 1",                     RHT_POT_ZORAS_FOUNTAIN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_HIDDEN_CAVE_POT_1));
    locationTable[RC_ZF_HIDDEN_CAVE_POT_2]                  	=	Location::Pot(RC_ZF_HIDDEN_CAVE_POT_2,                     RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(418, 2609),    "Hidden Cave Pot 2",                "ZF Hidden Cave Pot 2",                     RHT_POT_ZORAS_FOUNTAIN,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_HIDDEN_CAVE_POT_2));
    locationTable[RC_ZF_HIDDEN_CAVE_POT_3]                  	=	Location::Pot(RC_ZF_HIDDEN_CAVE_POT_3,                     RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(430, 2568),    "Hidden Cave Pot 3",                "ZF Hidden Cave Pot 3",                     RHT_POT_ZORAS_FOUNTAIN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_HIDDEN_CAVE_POT_3));
    locationTable[RC_ZF_NEAR_JABU_POT_1]                    	=	Location::Pot(RC_ZF_NEAR_JABU_POT_1,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1630, 170),   "Near Jabu Pot 1",                  "ZF Near Jabu Pot 1",                       RHT_POT_ZORAS_FOUNTAIN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_1));
    locationTable[RC_ZF_NEAR_JABU_POT_2]                    	=	Location::Pot(RC_ZF_NEAR_JABU_POT_2,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1550, 170),   "Near Jabu Pot 2",                  "ZF Near Jabu Pot 2",                       RHT_POT_ZORAS_FOUNTAIN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_2));
    locationTable[RC_ZF_NEAR_JABU_POT_3]                    	=	Location::Pot(RC_ZF_NEAR_JABU_POT_3,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1630, -260),  "Near Jabu Pot 3",                  "ZF Near Jabu Pot 3",                       RHT_POT_ZORAS_FOUNTAIN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_3));
    locationTable[RC_ZF_NEAR_JABU_POT_4]                    	=	Location::Pot(RC_ZF_NEAR_JABU_POT_4,                       RCQUEST_BOTH, RCAREA_ZORAS_FOUNTAIN,        SCENE_ZORAS_FOUNTAIN,              TWO_ACTOR_PARAMS(-1550, -260),  "Near Jabu Pot 4",                  "ZF Near Jabu Pot 4",                       RHT_POT_ZORAS_FOUNTAIN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_ZF_NEAR_JABU_POT_4));
    locationTable[RC_LLR_FRONT_POT_1]                       	=	Location::Pot(RC_LLR_FRONT_POT_1,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3218),   "Front Pot 1",                      "LLR Front Pot 1",                          RHT_POT_LON_LON_RANCH,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_1));
    locationTable[RC_LLR_FRONT_POT_2]                       	=	Location::Pot(RC_LLR_FRONT_POT_2,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3186),   "Front Pot 2",                      "LLR Front Pot 2",                          RHT_POT_LON_LON_RANCH,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_2));
    locationTable[RC_LLR_FRONT_POT_3]                       	=	Location::Pot(RC_LLR_FRONT_POT_3,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3154),   "Front Pot 3",                      "LLR Front Pot 3",                          RHT_POT_LON_LON_RANCH,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_3));
    locationTable[RC_LLR_FRONT_POT_4]                       	=	Location::Pot(RC_LLR_FRONT_POT_4,                          RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(667, -3122),   "Front Pot 4",                      "LLR Front Pot 4",                          RHT_POT_LON_LON_RANCH,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_FRONT_POT_4));
    locationTable[RC_LLR_RAIN_SHED_POT_1]                   	=	Location::Pot(RC_LLR_RAIN_SHED_POT_1,                      RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(852, 172),     "Rain Shed Pot 1",                  "LLR Rain Shed Pot 1",                      RHT_POT_LON_LON_RANCH,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_RAIN_SHED_POT_1));
    locationTable[RC_LLR_RAIN_SHED_POT_2]                   	=	Location::Pot(RC_LLR_RAIN_SHED_POT_2,                      RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(840, 212),     "Rain Shed Pot 2",                  "LLR Rain Shed Pot 2",                      RHT_POT_LON_LON_RANCH,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_RAIN_SHED_POT_2));
    locationTable[RC_LLR_RAIN_SHED_POT_3]                   	=	Location::Pot(RC_LLR_RAIN_SHED_POT_3,                      RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_RANCH,               TWO_ACTOR_PARAMS(872, 219),     "Rain Shed Pot 3",                  "LLR Rain Shed Pot 3",                      RHT_POT_LON_LON_RANCH,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_RAIN_SHED_POT_3));
    locationTable[RC_LLR_TALONS_HOUSE_POT_1]                	=	Location::Pot(RC_LLR_TALONS_HOUSE_POT_1,                   RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_BUILDINGS,           TWO_ACTOR_PARAMS(1255, 47),     "Talons House Pot 1",               "LLR Talons House Pot 1",                   RHT_POT_LON_LON_RANCH,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_TALONS_HOUSE_POT_1));
    locationTable[RC_LLR_TALONS_HOUSE_POT_2]                	=	Location::Pot(RC_LLR_TALONS_HOUSE_POT_2,                   RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_BUILDINGS,           TWO_ACTOR_PARAMS(1256, -51),    "Talons House Pot 2",               "LLR Talons House Pot 2",                   RHT_POT_LON_LON_RANCH,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_TALONS_HOUSE_POT_2));
    locationTable[RC_LLR_TALONS_HOUSE_POT_3]                	=	Location::Pot(RC_LLR_TALONS_HOUSE_POT_3,                   RCQUEST_BOTH, RCAREA_LON_LON_RANCH,         SCENE_LON_LON_BUILDINGS,           TWO_ACTOR_PARAMS(1256, -78),    "Talons House Pot 3",               "LLR Talons House Pot 3",                   RHT_POT_LON_LON_RANCH, 	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_LLR_TALONS_HOUSE_POT_3));
    locationTable[RC_HF_COW_GROTTO_POT_1]                   	=	Location::Pot(RC_HF_COW_GROTTO_POT_1,                      RCQUEST_BOTH, RCAREA_HYRULE_FIELD,          SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(3410, -223),   "Cow Grotto Pot 1",                 "HF Cow Grotto Pot 1",                      RHT_POT_HYRULE_FIELD,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_HF_COW_GROTTO_POT_1));
    locationTable[RC_HF_COW_GROTTO_POT_2]                   	=	Location::Pot(RC_HF_COW_GROTTO_POT_2,                      RCQUEST_BOTH, RCAREA_HYRULE_FIELD,          SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(3390, -258),   "Cow Grotto Pot 2",                 "HF Cow Grotto Pot 2",                      RHT_POT_HYRULE_FIELD,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_HF_COW_GROTTO_POT_2));
    locationTable[RC_HC_STORMS_GROTTO_POT_1]                	=	Location::Pot(RC_HC_STORMS_GROTTO_POT_1,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1843, 1014),   "Storms Grotto Pot 1",              "HC Storms Grotto Pot 1",                   RHT_POT_HYRULE_CASTLE,	        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_1));
    locationTable[RC_HC_STORMS_GROTTO_POT_2]                	=	Location::Pot(RC_HC_STORMS_GROTTO_POT_2,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1769, 954),    "Storms Grotto Pot 2",              "HC Storms Grotto Pot 2",                   RHT_POT_HYRULE_CASTLE,	        RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_2));
    locationTable[RC_HC_STORMS_GROTTO_POT_3]                	=	Location::Pot(RC_HC_STORMS_GROTTO_POT_3,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1857, 897),    "Storms Grotto Pot 3",              "HC Storms Grotto Pot 3",                   RHT_POT_HYRULE_CASTLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_3));
    locationTable[RC_HC_STORMS_GROTTO_POT_4]                	=	Location::Pot(RC_HC_STORMS_GROTTO_POT_4,                   RCQUEST_BOTH, RCAREA_HYRULE_CASTLE,         SCENE_GROTTOS,                     TWO_ACTOR_PARAMS(1764, 847),    "Storms Grotto Pot 4",              "HC Storms Grotto Pot 4",                   RHT_POT_HYRULE_CASTLE,	        RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_HC_STORMS_GROTTO_POT_4));
    
    // Dungeon Pots
    //            Randomizer Check                                 	Randomizer Check                                           Quest            Area                       Scene ID                           Params                          Short Name                          Spoiler Name                                Hint Text Key                 Vanilla             Spoiler Collection Check
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_3]          	=	Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_3,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2673, -2060),  "Lizalfos Pot 3",                   "Dodongos Cavern Lizalfos Pot 3",           RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_3));
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_2]          	=	Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_2,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3526, -2574),  "Lizalfos Pot 2",                   "Dodongos Cavern Lizalfos Pot 2",           RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_2));
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_1]        	=	Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_1,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3579, -2574),  "Lizalfos Pot 1",                   "Dodongos Cavern Lizalfos Pot 1",           RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_1));
    locationTable[RC_DODONGOS_CAVERN_LIZALFOS_POT_4]          	=	Location::Pot(RC_DODONGOS_CAVERN_LIZALFOS_POT_4,           RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2653, -2031),  "Lizalfos Pot 4",                   "Dodongos Cavern Lizalfos Pot 4",           RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_LIZALFOS_POT_4));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_1]         	=	Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_1,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1708, -471),   "Side Room Pot 1",                  "Dodongos Cavern Side Room Pot 1",          RHT_POT_DODONGOS_CAVERN,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_2]         	=	Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_2,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1746, -424),   "Side Room Pot 2",                  "Dodongos Cavern Side Room Pot 2",          RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_3]         	=	Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_3,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2573, -489),   "Side Room Pot 3",                  "Dodongos Cavern Side Room Pot 3",          RHT_POT_DODONGOS_CAVERN,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_4]         	=	Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_4,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2622, -489),   "Side Room Pot 4",                  "Dodongos Cavern Side Room Pot 4",          RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_4));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_5]         	=	Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_5,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3507, -964),   "Side Room Pot 5",                  "Dodongos Cavern Side Room Pot 5",          RHT_POT_DODONGOS_CAVERN,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_5));
    locationTable[RC_DODONGOS_CAVERN_SIDE_ROOM_POT_6]         	=	Location::Pot(RC_DODONGOS_CAVERN_SIDE_ROOM_POT_6,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3617, -911),   "Side Room Pot 6",                  "Dodongos Cavern Side Room Pot 6",          RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SIDE_ROOM_POT_6));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_1]        	=	Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_1,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1846, -1879),  "Torch Room Pot 1",                 "Dodongos Cavern Torch Room Pot 1",         RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_2]        	=	Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_2,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2050, -1964),  "Torch Room Pot 2",                 "Dodongos Cavern Torch Room Pot 2",         RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_3]        	=	Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_3,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2281, -1799),  "Torch Room Pot 3",                 "Dodongos Cavern Torch Room Pot 3",         RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_TORCH_ROOM_POT_4]        	=	Location::Pot(RC_DODONGOS_CAVERN_TORCH_ROOM_POT_4,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1836, -1239),  "Torch Room Pot 4",                 "Dodongos Cavern Torch Room Pot 4",         RHT_POT_DODONGOS_CAVERN,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_TORCH_ROOM_POT_4));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_1]         	=	Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_1,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-2414, -1823), "Staircase Pot 1",                  "Dodongos Cavern Staircase Pot 1",          RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_2]         	=	Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_2,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-2368, -1827), "Staircase Pot 2",                  "Dodongos Cavern Staircase Pot 2",          RHT_POT_DODONGOS_CAVERN,	    RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_3]         	=	Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_3,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-1574, -1831), "Staircase Pot 3",                  "Dodongos Cavern Staircase Pot 3",          RHT_POT_DODONGOS_CAVERN,	    RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_3));
    locationTable[RC_DODONGOS_CAVERN_STAIRCASE_POT_4]         	=	Location::Pot(RC_DODONGOS_CAVERN_STAIRCASE_POT_4,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(-1528, -1827), "Staircase Pot 4",                  "Dodongos Cavern Staircase Pot 4",          RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_STAIRCASE_POT_4));
    locationTable[RC_DODONGOS_CAVERN_SINGLE_EYE_POT_1]        	=	Location::Pot(RC_DODONGOS_CAVERN_SINGLE_EYE_POT_1,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2704, -483),   "Single Eye Pot 1",                 "Dodongos Cavern Single Eye Pot 1",         RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SINGLE_EYE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_SINGLE_EYE_POT_2]        	=	Location::Pot(RC_DODONGOS_CAVERN_SINGLE_EYE_POT_2,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3195, -155),   "Single Eye Pot 2",                 "Dodongos Cavern Single Eye Pot 2",         RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_SINGLE_EYE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_BLADE_POT_1]             	=	Location::Pot(RC_DODONGOS_CAVERN_BLADE_POT_1,              RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1656, -531),   "Blade Pot 1",                      "Dodongos Cavern Blade Pot 1",              RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BLADE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_BLADE_POT_2]             	=	Location::Pot(RC_DODONGOS_CAVERN_BLADE_POT_2,              RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1831, -593),   "Blade Pot 2",                      "Dodongos Cavern Blade Pot 2",              RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BLADE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_1]        	=	Location::Pot(RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_1,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(3329, -753),   "Double Eye Pot 1",                 "Dodongos Cavern Double Eye Pot 1",         RHT_POT_DODONGOS_CAVERN,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_DOUBLE_EYE_POT_1));
    locationTable[RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_2]        	=	Location::Pot(RC_DODONGOS_CAVERN_DOUBLE_EYE_POT_2,         RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(2705, -1086),  "Double Eye Pot 2",                 "Dodongos Cavern Double Eye Pot 2",         RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_DOUBLE_EYE_POT_2));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_1]         	=	Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_1,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(1027, -3704),  "Back Room Pot 1",                  "Dodongos Cavern Back Room Pot 1",          RHT_POT_DODONGOS_CAVERN,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_2]         	=	Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_2,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(966, -3704),   "Back Room Pot 2",                  "Dodongos Cavern Back Room Pot 2",          RHT_POT_DODONGOS_CAVERN,	    RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_2));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_3]         	=	Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_3,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(125, -3854),   "Back Room Pot 3",                  "Dodongos Cavern Back Room Pot 3",          RHT_POT_DODONGOS_CAVERN,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_3));
    locationTable[RC_DODONGOS_CAVERN_BACK_ROOM_POT_4]         	=	Location::Pot(RC_DODONGOS_CAVERN_BACK_ROOM_POT_4,          RCQUEST_VANILLA, RCAREA_DODONGOS_CAVERN,    SCENE_DODONGOS_CAVERN,             TWO_ACTOR_PARAMS(78, -3835),    "Back Room Pot 4",                  "Dodongos Cavern Back Room Pot 4",          RHT_POT_DODONGOS_CAVERN,	    RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_BACK_ROOM_POT_4));
    locationTable[RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1]   	=	Location::Pot(RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1,    RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1295, -3612), "Above Big Octo Pot 1",             "Jabu Jabus Belly Above Big Octo Pot 1",    RHT_POT_JABU_JABUS_BELLY,	    RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2]   	=	Location::Pot(RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2,    RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1355, -3612), "Above Big Octo Pot 2",             "Jabu Jabus Belly Above Big Octo Pot 2",    RHT_POT_JABU_JABUS_BELLY,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3]   	=	Location::Pot(RC_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3,    RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1422, -3612), "Above Big Octo Pot 3",             "Jabu Jabus Belly Above Big Octo Pot 3",    RHT_POT_JABU_JABUS_BELLY,	    RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_ABOVE_BIG_OCTO_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_1]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_1,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(-551, 33),     "Barinade Pot 1",                   "Jabu Jabus Belly Barinade Pot 1",          RHT_POT_JABU_JABUS_BELLY,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_2]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_2,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(551, 36),      "Barinade Pot 2",                   "Jabu Jabus Belly Barinade Pot 2",          RHT_POT_JABU_JABUS_BELLY,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_3]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_3,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(554, -493),    "Barinade Pot 3",                   "Jabu Jabus Belly Barinade Pot 3",          RHT_POT_JABU_JABUS_BELLY,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_4]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_4,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(-543, -496),   "Barinade Pot 4",                   "Jabu Jabus Belly Barinade Pot 4",          RHT_POT_JABU_JABUS_BELLY,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_4));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_5]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_5,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(-268, -786),   "Barinade Pot 5",                   "Jabu Jabus Belly Barinade Pot 5",          RHT_POT_JABU_JABUS_BELLY,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_5));
    locationTable[RC_JABU_JABUS_BELLY_BARINADE_POT_6]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BARINADE_POT_6,          RCQUEST_BOTH, RCAREA_JABU_JABUS_BELLY,      SCENE_JABU_JABU_BOSS,              TWO_ACTOR_PARAMS(279, -761),    "Barinade Pot 6",                   "Jabu Jabus Belly Barinade Pot 6",          RHT_POT_JABU_JABUS_BELLY,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BARINADE_POT_6));
    locationTable[RC_JABU_JABUS_BELLY_BASEMENT_POT_1]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BASEMENT_POT_1,          RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(703, -2371),   "Basement Pot 1",                   "Jabu Jabus Belly Basement Pot 1",          RHT_POT_JABU_JABUS_BELLY,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BASEMENT_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_BASEMENT_POT_2]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BASEMENT_POT_2,          RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(645, -2408),   "Basement Pot 2",                   "Jabu Jabus Belly Basement Pot 2",          RHT_POT_JABU_JABUS_BELLY,	    RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BASEMENT_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_BASEMENT_POT_3]         	=	Location::Pot(RC_JABU_JABUS_BELLY_BASEMENT_POT_3,          RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(650, -2344),   "Basement Pot 3",                   "Jabu Jabus Belly Basement Pot 3",          RHT_POT_JABU_JABUS_BELLY,	    RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_BASEMENT_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1]      	=	Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1127, -2271), "Two Octorok Pot 1",                "Jabu Jabus Belly Two Octorok Pot 1",       RHT_POT_JABU_JABUS_BELLY,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_1));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2]      	=	Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1178, -2272), "Two Octorok Pot 2",                "Jabu Jabus Belly Two Octorok Pot 2",       RHT_POT_JABU_JABUS_BELLY,	    RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_2));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3]      	=	Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1150, -2248), "Two Octorok Pot 3",                "Jabu Jabus Belly Two Octorok Pot 3",       RHT_POT_JABU_JABUS_BELLY,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_3));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4]      	=	Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1131, -2221), "Two Octorok Pot 4",                "Jabu Jabus Belly Two Octorok Pot 4",       RHT_POT_JABU_JABUS_BELLY,	    RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_4));
    locationTable[RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5]      	=	Location::Pot(RC_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5,       RCQUEST_VANILLA, RCAREA_JABU_JABUS_BELLY,   SCENE_JABU_JABU,                   TWO_ACTOR_PARAMS(-1173, -2227), "Two Octorok Pot 5",                "Jabu Jabus Belly Two Octorok Pot 5",       RHT_POT_JABU_JABUS_BELLY,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_TWO_OCTOROK_POT_5));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_1]               	=	Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_1,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(720, -1037),   "Lobby Pot 1",                      "Forest Temple Lobby Pot 1",                RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_1));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_2]               	=	Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_2,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(675, -989),    "Lobby Pot 2",                      "Forest Temple Lobby Pot 2",                RHT_POT_FOREST_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_2));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_3]               	=	Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_3,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(630, -943),    "Lobby Pot 3",                      "Forest Temple Lobby Pot 3",                RHT_POT_FOREST_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_3));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_4]               	=	Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_4,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(-384, -937),   "Lobby Pot 4",                      "Forest Temple Lobby Pot 4",                RHT_POT_FOREST_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_4));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_5]               	=	Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_5,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(-429, -981),   "Lobby Pot 5",                      "Forest Temple Lobby Pot 5",                RHT_POT_FOREST_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_5));
    locationTable[RC_FOREST_TEMPLE_LOBBY_POT_6]               	=	Location::Pot(RC_FOREST_TEMPLE_LOBBY_POT_6,                RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(-477, -1034),  "Lobby Pot 6",                      "Forest Temple Lobby Pot 6",                RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOBBY_POT_6));
    locationTable[RC_FOREST_TEMPLE_LOWER_STALFOS_POT_1]       	=	Location::Pot(RC_FOREST_TEMPLE_LOWER_STALFOS_POT_1,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(90, -3583),    "Lower Stalfos Pot 1",              "Forest Temple Lower Stalfos Pot 1",        RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOWER_STALFOS_POT_1));
    locationTable[RC_FOREST_TEMPLE_LOWER_STALFOS_POT_2]       	=	Location::Pot(RC_FOREST_TEMPLE_LOWER_STALFOS_POT_2,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(142, -3583),   "Lower Stalfos Pot 2",              "Forest Temple Lower Stalfos Pot 2",        RHT_POT_FOREST_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_LOWER_STALFOS_POT_2));
    locationTable[RC_FOREST_TEMPLE_GREEN_POE_POT_1]           	=	Location::Pot(RC_FOREST_TEMPLE_GREEN_POE_POT_1,            RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(1647, -1529),  "Green Poe Pot 1",                  "Forest Temple Green Poe Pot 1",            RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_GREEN_POE_POT_1));
    locationTable[RC_FOREST_TEMPLE_GREEN_POE_POT_2]           	=	Location::Pot(RC_FOREST_TEMPLE_GREEN_POE_POT_2,            RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(1645, -1588),  "Green Poe Pot 2",                  "Forest Temple Green Poe Pot 2",            RHT_POT_FOREST_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_GREEN_POE_POT_2));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_1]       	=	Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_1,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(32, -3012),    "Upper Stalfos Pot 1",              "Forest Temple Upper Stalfos Pot 1",        RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_1));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_2]       	=	Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_2,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(202, -3012),   "Upper Stalfos Pot 2",              "Forest Temple Upper Stalfos Pot 2",        RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_2));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_3]       	=	Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_3,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(30, -3631),    "Upper Stalfos Pot 3",              "Forest Temple Upper Stalfos Pot 3",        RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_3));
    locationTable[RC_FOREST_TEMPLE_UPPER_STALFOS_POT_4]       	=	Location::Pot(RC_FOREST_TEMPLE_UPPER_STALFOS_POT_4,        RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(206, -3631),   "Upper Stalfos Pot 4",              "Forest Temple Upper Stalfos Pot 4",        RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_UPPER_STALFOS_POT_4));
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_POT_1]            	=	Location::Pot(RC_FOREST_TEMPLE_BLUE_POE_POT_1,             RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(830, -3639),   "Blue Poe Pot 1",                   "Forest Temple Blue Poe Pot 1",             RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_BLUE_POE_POT_1));
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_POT_2]            	=	Location::Pot(RC_FOREST_TEMPLE_BLUE_POE_POT_2,             RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(883, -3640),   "Blue Poe Pot 2",                   "Forest Temple Blue Poe Pot 2",             RHT_POT_FOREST_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_BLUE_POE_POT_2));
    locationTable[RC_FOREST_TEMPLE_BLUE_POE_POT_3]            	=	Location::Pot(RC_FOREST_TEMPLE_BLUE_POE_POT_3,             RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(941, -3640),   "Blue Poe Pot 3",                   "Forest Temple Blue Poe Pot 3",             RHT_POT_FOREST_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_BLUE_POE_POT_3));
    locationTable[RC_FOREST_TEMPLE_FROZEN_EYE_POT_1]          	=	Location::Pot(RC_FOREST_TEMPLE_FROZEN_EYE_POT_1,           RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(2312, -874),   "Frozen Eye Pot 1",                 "Forest Temple Frozen Eye Pot 1",           RHT_POT_FOREST_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_FROZEN_EYE_POT_1));
    locationTable[RC_FOREST_TEMPLE_FROZEN_EYE_POT_2]          	=	Location::Pot(RC_FOREST_TEMPLE_FROZEN_EYE_POT_2,           RCQUEST_VANILLA, RCAREA_FOREST_TEMPLE,      SCENE_FOREST_TEMPLE,               TWO_ACTOR_PARAMS(2364, -873),   "Frozen Eye Pot 2",                 "Forest Temple Frozen Eye Pot 2",           RHT_POT_FOREST_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_FROZEN_EYE_POT_2));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_1]             	=	Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_1,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1086, -714),  "Near Boss Pot 1",                  "Fire Temple Near Boss Pot 1",              RHT_POT_FIRE_TEMPLE,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_1));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_2]             	=	Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_2,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1138, -713),  "Near Boss Pot 2",                  "Fire Temple Near Boss Pot 2",              RHT_POT_FIRE_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_2));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_3]             	=	Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_3,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1087, -765),  "Near Boss Pot 3",                  "Fire Temple Near Boss Pot 3",              RHT_POT_FIRE_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_3));
    locationTable[RC_FIRE_TEMPLE_NEAR_BOSS_POT_4]             	=	Location::Pot(RC_FIRE_TEMPLE_NEAR_BOSS_POT_4,              RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-1139, -766),  "Near Boss Pot 4",                  "Fire Temple Near Boss Pot 4",              RHT_POT_FIRE_TEMPLE,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_NEAR_BOSS_POT_4));
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_POT_1]              	=	Location::Pot(RC_FIRE_TEMPLE_BIG_LAVA_POT_1,               RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(2226, -754),   "Big Lava Pot 1",                   "Fire Temple Big Lava Pot 1",               RHT_POT_FIRE_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_BIG_LAVA_POT_1));
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_POT_2]              	=	Location::Pot(RC_FIRE_TEMPLE_BIG_LAVA_POT_2,               RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(2226, -809),   "Big Lava Pot 2",                   "Fire Temple Big Lava Pot 2",               RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_BIG_LAVA_POT_2));
    locationTable[RC_FIRE_TEMPLE_BIG_LAVA_POT_3]              	=	Location::Pot(RC_FIRE_TEMPLE_BIG_LAVA_POT_3,               RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(2227, -862),   "Big Lava Pot 3",                   "Fire Temple Big Lava Pot 3",               RHT_POT_FIRE_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_BIG_LAVA_POT_3));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1]       	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2388, 953),   "Flame Maze Left Pot 1",            "Fire Temple Flame Maze Left Pot 1",        RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_1));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2]       	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2423, 920),   "Flame Maze Left Pot 2",            "Fire Temple Flame Maze Left Pot 2",        RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_2));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3]       	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2442, 715),   "Flame Maze Left Pot 3",            "Fire Temple Flame Maze Left Pot 3",        RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_3));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4]       	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4,        RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2408, 680),   "Flame Maze Left Pot 4",            "Fire Temple Flame Maze Left Pot 4",        RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_LEFT_POT_4));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1]      	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2208, -168),  "Flame Maze Right Pot 1",           "Fire Temple Flame Maze Right Pot 1",       RHT_POT_FIRE_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_1));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2]      	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2253, -167),  "Flame Maze Right Pot 2",           "Fire Temple Flame Maze Right Pot 2",       RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_2));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3]      	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2657, -591),  "Flame Maze Right Pot 3",           "Fire Temple Flame Maze Right Pot 3",       RHT_POT_FIRE_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_3));
    locationTable[RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4]      	=	Location::Pot(RC_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4,       RCQUEST_VANILLA, RCAREA_FIRE_TEMPLE,        SCENE_FIRE_TEMPLE,                 TWO_ACTOR_PARAMS(-2657, -635),  "Flame Maze Right Pot 4",           "Fire Temple Flame Maze Right Pot 4",       RHT_POT_FIRE_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_FLAME_MAZE_RIGHT_POT_4));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_1]         	=	Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_1,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(330, -198),    "Main Level 2 Pot 1",               "Water Temple Main Level 2 Pot 1",          RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_2_POT_1));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_2]         	=	Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_2_POT_2,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(331, -168),    "Main Level 2 Pot 2",               "Water Temple Main Level 2 Pot 2",          RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_2_POT_2));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_1]         	=	Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_1,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-241, -1193),  "Main Level 1 Pot 1",               "Water Temple Main Level 1 Pot 1",          RHT_POT_WATER_TEMPLE,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_1_POT_1));
    locationTable[RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_2]         	=	Location::Pot(RC_WATER_TEMPLE_MAIN_LEVEL_1_POT_2,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-117, -1192),  "Main Level 1 Pot 2",               "Water Temple Main Level 1 Pot 2",          RHT_POT_WATER_TEMPLE,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MAIN_LEVEL_1_POT_2));
    locationTable[RC_WATER_TEMPLE_TORCH_POT_1]                	=	Location::Pot(RC_WATER_TEMPLE_TORCH_POT_1,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1123, 62),     "Torch Pot 1",                      "Water Temple Torch Pot 1",                 RHT_POT_WATER_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_TORCH_POT_1));
    locationTable[RC_WATER_TEMPLE_TORCH_POT_2]                	=	Location::Pot(RC_WATER_TEMPLE_TORCH_POT_2,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1123, 293),    "Torch Pot 2",                      "Water Temple Torch Pot 2",                 RHT_POT_WATER_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_TORCH_POT_2));
    locationTable[RC_WATER_TEMPLE_NEAR_COMPASS_POT_1]         	=	Location::Pot(RC_WATER_TEMPLE_NEAR_COMPASS_POT_1,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1222, -617),   "Near Compass Pot 1",               "Water Temple Near Compass Pot 1",          RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_NEAR_COMPASS_POT_1));
    locationTable[RC_WATER_TEMPLE_NEAR_COMPASS_POT_2]         	=	Location::Pot(RC_WATER_TEMPLE_NEAR_COMPASS_POT_2,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1172, -617),   "Near Compass Pot 2",               "Water Temple Near Compass Pot 2",          RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_NEAR_COMPASS_POT_2));
    locationTable[RC_WATER_TEMPLE_NEAR_COMPASS_POT_3]         	=	Location::Pot(RC_WATER_TEMPLE_NEAR_COMPASS_POT_3,          RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(1122, -617),   "Near Compass Pot 3",               "Water Temple Near Compass Pot 3",          RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_NEAR_COMPASS_POT_3));
    locationTable[RC_WATER_TEMPLE_CENTRAL_BOW_POT_1]          	=	Location::Pot(RC_WATER_TEMPLE_CENTRAL_BOW_POT_1,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(688, 1008),    "Central Bow Pot 1",                "Water Temple Central Bow Pot 1",           RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_CENTRAL_BOW_POT_1));
    locationTable[RC_WATER_TEMPLE_CENTRAL_BOW_POT_2]          	=	Location::Pot(RC_WATER_TEMPLE_CENTRAL_BOW_POT_2,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(631, 1008),    "Central Bow Pot 2",                "Water Temple Central Bow Pot 2",           RHT_POT_WATER_TEMPLE,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_CENTRAL_BOW_POT_2));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_1]          	=	Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_1,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2313, 944),   "Behind Gate Pot 1",                "Water Temple Behind Gate Pot 1",           RHT_POT_WATER_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_1));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_2]          	=	Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_2,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2313, 905),   "Behind Gate Pot 2",                "Water Temple Behind Gate Pot 2",           RHT_POT_WATER_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_2));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_3]          	=	Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_3,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2314, 808),   "Behind Gate Pot 3",                "Water Temple Behind Gate Pot 3",           RHT_POT_WATER_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_3));
    locationTable[RC_WATER_TEMPLE_BEHIND_GATE_POT_4]          	=	Location::Pot(RC_WATER_TEMPLE_BEHIND_GATE_POT_4,           RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2314, 770),   "Behind Gate Pot 4",                "Water Temple Behind Gate Pot 4",           RHT_POT_WATER_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BEHIND_GATE_POT_4));
    locationTable[RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1]	=	Location::Pot(RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1, RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-28, -3817),   "Basement Block Puzzle Pot 1",      "Water Temple Basement Block Puzzle Pot 1", RHT_POT_WATER_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_1));
    locationTable[RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2] 	=	Location::Pot(RC_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2, RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(28, -3817),    "Basement Block Puzzle Pot 2",      "Water Temple Basement Block Puzzle Pot 2", RHT_POT_WATER_TEMPLE,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BASEMENT_BLOCK_PUZZLE_POT_2));
    locationTable[RC_WATER_TEMPLE_RIVER_POT_1]                	=	Location::Pot(RC_WATER_TEMPLE_RIVER_POT_1,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2226, -2487), "River Pot 1",                      "Water Temple River Pot 1",                 RHT_POT_WATER_TEMPLE,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_RIVER_POT_1));
    locationTable[RC_WATER_TEMPLE_RIVER_POT_2]                	=	Location::Pot(RC_WATER_TEMPLE_RIVER_POT_2,                 RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-2184, -2456), "River Pot 2",                      "Water Temple River Pot 2",                 RHT_POT_WATER_TEMPLE,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_RIVER_POT_2));
    locationTable[RC_WATER_TEMPLE_LIKE_LIKE_POT_1]            	=	Location::Pot(RC_WATER_TEMPLE_LIKE_LIKE_POT_1,             RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-3188, -1628), "Like Like Pot 1",                  "Water Temple Like Like Pot 1",             RHT_POT_WATER_TEMPLE,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_LIKE_LIKE_POT_1));
    locationTable[RC_WATER_TEMPLE_LIKE_LIKE_POT_2]            	=	Location::Pot(RC_WATER_TEMPLE_LIKE_LIKE_POT_2,             RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-3056, -1626), "Like Like Pot 2",                  "Water Temple Like Like Pot 2",             RHT_POT_WATER_TEMPLE,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_LIKE_LIKE_POT_2));
    locationTable[RC_WATER_TEMPLE_BOSS_KEY_POT_1]            	=	Location::Pot(RC_WATER_TEMPLE_BOSS_KEY_POT_1,              RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-1150, -3031), "Boss Key Pot 1",                   "Water Temple Boss Key Pot 1",              RHT_POT_WATER_TEMPLE,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BOSS_KEY_POT_1));
    locationTable[RC_WATER_TEMPLE_BOSS_KEY_POT_2]            	=	Location::Pot(RC_WATER_TEMPLE_BOSS_KEY_POT_2,              RCQUEST_VANILLA, RCAREA_WATER_TEMPLE,       SCENE_WATER_TEMPLE,                TWO_ACTOR_PARAMS(-1417, -3025), "Boss Key Pot 2",                   "Water Temple Boss Key Pot 2",              RHT_POT_WATER_TEMPLE,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_BOSS_KEY_POT_2));
    locationTable[RC_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1]      	=	Location::Pot(RC_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1806, -344),  "Near Dead Hand Pot 1",             "Shadow Temple Near Dead Hand Pot 1",       RHT_POT_SHADOW_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_NEAR_DEAD_HAND_POT_1));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1]    	=	Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1721, 61),    "Whispering Walls Pot 1",           "Shadow Temple Whispering Walls Pot 1",     RHT_POT_SHADOW_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_1));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2]    	=	Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1407, 61),    "Whispering Walls Pot 2",           "Shadow Temple Whispering Walls Pot 2",     RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_2));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3]    	=	Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1614, 61),    "Whispering Walls Pot 3",           "Shadow Temple Whispering Walls Pot 3",     RHT_POT_SHADOW_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_3));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4]    	=	Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1055, -257),  "Whispering Walls Pot 4",           "Shadow Temple Whispering Walls Pot 4",     RHT_POT_SHADOW_TEMPLE,	    RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_4));
    locationTable[RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5]    	=	Location::Pot(RC_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5,     RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-1050, -130),  "Whispering Walls Pot 5",           "Shadow Temple Whispering Walls Pot 5",     RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_WHISPERING_WALLS_POT_5));
    locationTable[RC_SHADOW_TEMPLE_MAP_CHEST_POT_1]           	=	Location::Pot(RC_SHADOW_TEMPLE_MAP_CHEST_POT_1,            RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-665, -760),   "Map Chest Pot 1",                  "Shadow Temple Map Chest Pot 1",            RHT_POT_SHADOW_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MAP_CHEST_POT_1));
    locationTable[RC_SHADOW_TEMPLE_MAP_CHEST_POT_2]           	=	Location::Pot(RC_SHADOW_TEMPLE_MAP_CHEST_POT_2,            RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-929, -748),   "Map Chest Pot 2",                  "Shadow Temple Map Chest Pot 2",            RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MAP_CHEST_POT_2));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_1]      	=	Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_1,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(614, 3579),    "Falling Spikes Pot 1",             "Shadow Temple Falling Spikes Pot 1",       RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_1));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_2]      	=	Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_2,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(798, 3574),    "Falling Spikes Pot 2",             "Shadow Temple Falling Spikes Pot 2",       RHT_POT_SHADOW_TEMPLE,	    RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_2));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_3]      	=	Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_3,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(1173, 3707),   "Falling Spikes Pot 3",             "Shadow Temple Falling Spikes Pot 3",       RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_3));
    locationTable[RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_4]      	=	Location::Pot(RC_SHADOW_TEMPLE_FALLING_SPIKES_POT_4,       RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(1173, 3605),   "Falling Spikes Pot 4",             "Shadow Temple Falling Spikes Pot 4",       RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FALLING_SPIKES_POT_4));
    locationTable[RC_SHADOW_TEMPLE_AFTER_WIND_POT_1]          	=	Location::Pot(RC_SHADOW_TEMPLE_AFTER_WIND_POT_1,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(4222, -916),   "After Wind Pot 1",                 "Shadow Temple After Wind Pot 1",           RHT_POT_SHADOW_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_WIND_POT_1));
    locationTable[RC_SHADOW_TEMPLE_AFTER_WIND_POT_2]          	=	Location::Pot(RC_SHADOW_TEMPLE_AFTER_WIND_POT_2,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(4549, -922),   "After Wind Pot 2",                 "Shadow Temple After Wind Pot 2",           RHT_POT_SHADOW_TEMPLE,	    RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_WIND_POT_2));
    locationTable[RC_SHADOW_TEMPLE_SPIKE_WALLS_POT_1]         	=	Location::Pot(RC_SHADOW_TEMPLE_SPIKE_WALLS_POT_1,          RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-4303, -2657), "Spike Walls Pot 1",                "Shadow Temple Spike Walls Pot 1",          RHT_POT_SHADOW_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_SPIKE_WALLS_POT_1));
    locationTable[RC_SHADOW_TEMPLE_FLOORMASTER_POT_1]         	=	Location::Pot(RC_SHADOW_TEMPLE_FLOORMASTER_POT_1,          RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-4575, -811),  "Floormaster Pot 1",                "Shadow Temple Floormaster Pot 1",          RHT_POT_SHADOW_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FLOORMASTER_POT_1));
    locationTable[RC_SHADOW_TEMPLE_FLOORMASTER_POT_2]         	=	Location::Pot(RC_SHADOW_TEMPLE_FLOORMASTER_POT_2,          RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-4036, -811),  "Floormaster Pot 2",                "Shadow Temple Floormaster Pot 2",          RHT_POT_SHADOW_TEMPLE,	    RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_FLOORMASTER_POT_2));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_1]          	=	Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_1,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2400, -1470), "After Boat Pot 1",                 "Shadow Temple After Boat Pot 1",           RHT_POT_SHADOW_TEMPLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_1));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_2]          	=	Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_2,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2230, -1470), "After Boat Pot 2",                 "Shadow Temple After Boat Pot 2",           RHT_POT_SHADOW_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_2));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_3]          	=	Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_3,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2859, -497),  "After Boat Pot 3",                 "Shadow Temple After Boat Pot 3",           RHT_POT_SHADOW_TEMPLE,	    RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_3));
    locationTable[RC_SHADOW_TEMPLE_AFTER_BOAT_POT_4]          	=	Location::Pot(RC_SHADOW_TEMPLE_AFTER_BOAT_POT_4,           RCQUEST_VANILLA, RCAREA_SHADOW_TEMPLE,      SCENE_SHADOW_TEMPLE,               TWO_ACTOR_PARAMS(-2624, -492),  "After Boat Pot 4",                 "Shadow Temple After Boat Pot 4",           RHT_POT_SHADOW_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_AFTER_BOAT_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_LOBBY_POT_1]               	=	Location::Pot(RC_SPIRIT_TEMPLE_LOBBY_POT_1,                RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-181, 233),    "Lobby Pot 1",                      "Spirit Temple Lobby Pot 1",                RHT_POT_SPIRIT_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_LOBBY_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_LOBBY_POT_2]               	=	Location::Pot(RC_SPIRIT_TEMPLE_LOBBY_POT_2,                RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(184, 231),     "Lobby Pot 2",                      "Spirit Temple Lobby Pot 2",                RHT_POT_SPIRIT_TEMPLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_LOBBY_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_1]              	=	Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_1,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-975, -1751),  "Anubis Pot 1",                     "Spirit Temple Anubis Pot 1",               RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_2]              	=	Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_2,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-928, -1827),  "Anubis Pot 2",                     "Spirit Temple Anubis Pot 2",               RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_3]              	=	Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_3,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1211, -1530), "Anubis Pot 3",                     "Spirit Temple Anubis Pot 3",               RHT_POT_SPIRIT_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_ANUBIS_POT_4]              	=	Location::Pot(RC_SPIRIT_TEMPLE_ANUBIS_POT_4,               RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1055, -1680), "Anubis Pot 4",                     "Spirit Temple Anubis Pot 4",               RHT_POT_SPIRIT_TEMPLE,	    RG_DEKU_SHIELD,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_ANUBIS_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1]         	=	Location::Pot(RC_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1,          RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1101, -1305), "Child Climb Pot 1",                "Spirit Temple Child Climb Pot 1",          RHT_POT_SPIRIT_TEMPLE,	    RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CHILD_CLIMB_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1]     	=	Location::Pot(RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1781, 830),   "After Sun Block Pot 1",            "Spirit Temple After Sun Block Pot 1",      RHT_POT_SPIRIT_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2]     	=	Location::Pot(RC_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-1617, 830),   "After Sun Block Pot 2",            "Spirit Temple After Sun Block Pot 2",      RHT_POT_SPIRIT_TEMPLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_AFTER_SUN_BLOCK_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1]     	=	Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-600, -1213),  "Central Chamber Pot 1",            "Spirit Temple Central Chamber Pot 1",      RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_1));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2]     	=	Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-564, -1180),  "Central Chamber Pot 2",            "Spirit Temple Central Chamber Pot 2",      RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_2));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3]     	=	Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(-526, -1213),  "Central Chamber Pot 3",            "Spirit Temple Central Chamber Pot 3",      RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_3));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4]     	=	Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(639, -1224),   "Central Chamber Pot 4",            "Spirit Temple Central Chamber Pot 4",      RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_4));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5]     	=	Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(671, -1179),   "Central Chamber Pot 5",            "Spirit Temple Central Chamber Pot 5",      RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_5));
    locationTable[RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6]     	=	Location::Pot(RC_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6,      RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(704, -1224),   "Central Chamber Pot 6",            "Spirit Temple Central Chamber Pot 6",      RHT_POT_SPIRIT_TEMPLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_CENTRAL_CHAMBER_POT_6));
    locationTable[RC_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1]         	=	Location::Pot(RC_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1,          RCQUEST_VANILLA, RCAREA_SPIRIT_TEMPLE,      SCENE_SPIRIT_TEMPLE,               TWO_ACTOR_PARAMS(819, -333),    "Beamos Hall Pot 1",                "Spirit Temple Beamos Hall Pot 1",          RHT_POT_SPIRIT_TEMPLE,	    RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_BEAMOS_HALL_POT_1));
    locationTable[RC_GANONS_CASTLE_FOREST_TRIAL_POT_1]        	=	Location::Pot(RC_GANONS_CASTLE_FOREST_TRIAL_POT_1,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1725, 2336),   "Forest Trial Pot 1",               "Ganons Castle Forest Trial Pot 1",         RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FOREST_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_FOREST_TRIAL_POT_2]        	=	Location::Pot(RC_GANONS_CASTLE_FOREST_TRIAL_POT_2,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1892, 2241),   "Forest Trial Pot 2",               "Ganons Castle Forest Trial Pot 2",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FOREST_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_FIRE_TRIAL_POT_2]          	=	Location::Pot(RC_GANONS_CASTLE_FIRE_TRIAL_POT_2,           RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-1898, -3942), "Fire Trial Pot 2",                 "Ganons Castle Fire Trial Pot 2",           RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FIRE_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_FIRE_TRIAL_POT_1]          	=	Location::Pot(RC_GANONS_CASTLE_FIRE_TRIAL_POT_1,           RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-2051, -3855), "Fire Trial Pot 1",                 "Ganons Castle Fire Trial Pot 1",           RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_FIRE_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_POT_1]         	=	Location::Pot(RC_GANONS_CASTLE_WATER_TRIAL_POT_1,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(3558, -920),   "Water Trial Pot 1",                "Ganons Castle Water Trial Pot 1",          RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_WATER_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_POT_2]         	=	Location::Pot(RC_GANONS_CASTLE_WATER_TRIAL_POT_2,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(3558, -761),   "Water Trial Pot 2",                "Ganons Castle Water Trial Pot 2",          RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_WATER_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_WATER_TRIAL_POT_3]         	=	Location::Pot(RC_GANONS_CASTLE_WATER_TRIAL_POT_3,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(2822, -507),   "Water Trial Pot 3",                "Ganons Castle Water Trial Pot 3",          RHT_POT_GANONS_CASTLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_WATER_TRIAL_POT_3));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_1]        	=	Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_1,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1234, -3418),  "Shadow Trial Pot 1",               "Ganons Castle Shadow Trial Pot 1",         RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_2]        	=	Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_2,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(1621, -3203),  "Shadow Trial Pot 2",               "Ganons Castle Shadow Trial Pot 2",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_3]        	=	Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_3,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(2067, -4592),  "Shadow Trial Pot 3",               "Ganons Castle Shadow Trial Pot 3",         RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_3));
    locationTable[RC_GANONS_CASTLE_SHADOW_TRIAL_POT_4]        	=	Location::Pot(RC_GANONS_CASTLE_SHADOW_TRIAL_POT_4,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(2224, -4506),  "Shadow Trial Pot 4",               "Ganons Castle Shadow Trial Pot 4",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SHADOW_TRIAL_POT_4));
    locationTable[RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_1]        	=	Location::Pot(RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_1,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-1398, 1402),  "Spirit Trial Pot 1",               "Ganons Castle Spirit Trial Pot 1",         RHT_POT_GANONS_CASTLE,	    RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SPIRIT_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_2]        	=	Location::Pot(RC_GANONS_CASTLE_SPIRIT_TRIAL_POT_2,         RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-1237, 1494),  "Spirit Trial Pot 2",               "Ganons Castle Spirit Trial Pot 2",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_SPIRIT_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1] 	=	Location::Pot(RC_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1,  RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-2581, -839),  "Light Trial Boulder Pot 1",        "Ganons Castle Light Trial Boulder Pot 1",  RHT_POT_GANONS_CASTLE,	    RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_LIGHT_TRIAL_BOULDER_POT_1));
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_POT_1]         	=	Location::Pot(RC_GANONS_CASTLE_LIGHT_TRIAL_POT_1,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-3840, -923),  "Light Trial Pot 1",                "Ganons Castle Light Trial Pot 1",          RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_LIGHT_TRIAL_POT_1));
    locationTable[RC_GANONS_CASTLE_LIGHT_TRIAL_POT_2]         	=	Location::Pot(RC_GANONS_CASTLE_LIGHT_TRIAL_POT_2,          RCQUEST_VANILLA, RCAREA_GANONS_CASTLE,      SCENE_INSIDE_GANONS_CASTLE,        TWO_ACTOR_PARAMS(-3840, -755),  "Light Trial Pot 2",                "Ganons Castle Light Trial Pot 2",          RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_LIGHT_TRIAL_POT_2));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_1]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_1,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-422, -258),   "Ganons Tower Pot 1",               "Ganons Castle Ganons Tower Pot 1",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_1));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_2]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_2,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-382, -300),   "Ganons Tower Pot 2",               "Ganons Castle Ganons Tower Pot 2",         RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_2));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_3]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_3,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-342, -341),   "Ganons Tower Pot 3",               "Ganons Castle Ganons Tower Pot 3",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_3));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_4]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_4,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-298, -383),   "Ganons Tower Pot 4",               "Ganons Castle Ganons Tower Pot 4",         RHT_POT_GANONS_CASTLE,	    RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_4));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_5]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_5,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-257, -424),   "Ganons Tower Pot 5",               "Ganons Castle Ganons Tower Pot 5",         RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_5));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_6]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_6,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-382, -259),   "Ganons Tower Pot 6",               "Ganons Castle Ganons Tower Pot 6",         RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_6));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_7]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_7,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-342, -300),   "Ganons Tower Pot 7",               "Ganons Castle Ganons Tower Pot 7",         RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_7));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_8]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_8,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-301, -341),   "Ganons Tower Pot 8",               "Ganons Castle Ganons Tower Pot 8",         RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_8));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_9]        	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_9,         RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(-258, -383),   "Ganons Tower Pot 9",               "Ganons Castle Ganons Tower Pot 9",         RHT_POT_GANONS_CASTLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_9));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_10]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_10,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(418, -261),    "Ganons Tower Pot 10",              "Ganons Castle Ganons Tower Pot 10",        RHT_POT_GANONS_CASTLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_10));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_11]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_11,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(377, -301),    "Ganons Tower Pot 11",              "Ganons Castle Ganons Tower Pot 11",        RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_11));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_12]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_12,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(338, -340),    "Ganons Tower Pot 12",              "Ganons Castle Ganons Tower Pot 12",        RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_12));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_13]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_13,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(300, -380),    "Ganons Tower Pot 13",              "Ganons Castle Ganons Tower Pot 13",        RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_13));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_14]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_14,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(260, -420),    "Ganons Tower Pot 14",              "Ganons Castle Ganons Tower Pot 14",        RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_14));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_15]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_15,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(378, -261),    "Ganons Tower Pot 15",              "Ganons Castle Ganons Tower Pot 15",        RHT_POT_GANONS_CASTLE,	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_15));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_16]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_16,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(337, -300),    "Ganons Tower Pot 16",              "Ganons Castle Ganons Tower Pot 16",        RHT_POT_GANONS_CASTLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_16));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_17]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_17,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(299, -340),    "Ganons Tower Pot 17",              "Ganons Castle Ganons Tower Pot 17",        RHT_POT_GANONS_CASTLE,	    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_17));
    locationTable[RC_GANONS_CASTLE_GANONS_TOWER_POT_18]       	=	Location::Pot(RC_GANONS_CASTLE_GANONS_TOWER_POT_18,        RCQUEST_BOTH, RCAREA_GANONS_CASTLE,         SCENE_GANONS_TOWER,                TWO_ACTOR_PARAMS(260, -380),    "Ganons Tower Pot 18",              "Ganons Castle Ganons Tower Pot 18",        RHT_POT_GANONS_CASTLE,	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_GANONS_TOWER_POT_18));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_1]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_1,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-95, -673),    "Basement Pot 1",                   "Bottom Of The Well Basement Pot 1",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_2]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_2,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(75, -598),     "Basement Pot 2",                   "Bottom Of The Well Basement Pot 2",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_3]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_3,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(270, -1151),   "Basement Pot 3",                   "Bottom Of The Well Basement Pot 3",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_3));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_4]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_4,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(566, -1254),   "Basement Pot 4",                   "Bottom Of The Well Basement Pot 4",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_4));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_5]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_5,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(378, -1089),   "Basement Pot 5",                   "Bottom Of The Well Basement Pot 5",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_5));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_6]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_6,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(665, -1252),   "Basement Pot 6",                   "Bottom Of The Well Basement Pot 6",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_6));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_7]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_7,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(566, -1345),   "Basement Pot 7",                   "Bottom Of The Well Basement Pot 7",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_7));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_8]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_8,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(562, -1304),   "Basement Pot 8",                   "Bottom Of The Well Basement Pot 8",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_8));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_9]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_9,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(313, -1086),   "Basement Pot 9",                   "Bottom Of The Well Basement Pot 9",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_9));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_10]      	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_10,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(618, -1254),   "Basement Pot 10",                  "Bottom Of The Well Basement Pot 10",       RHT_POT_BOTTOM_OF_THE_WELL,	RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_10));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_11]      	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_11,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(257, -1099),   "Basement Pot 11",                  "Bottom Of The Well Basement Pot 11",       RHT_POT_BOTTOM_OF_THE_WELL,	RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_11));
    locationTable[RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_12]      	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_BASEMENT_POT_12,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(296, -1206),   "Basement Pot 12",                  "Bottom Of The Well Basement Pot 12",       RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_BASEMENT_POT_12));
    locationTable[RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1]      	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-969, 55),     "Left Side Pot 1",                  "Bottom Of The Well Left Side Pot 1",       RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2]      	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-966, 9),      "Left Side Pot 2",                  "Bottom Of The Well Left Side Pot 2",       RHT_POT_BOTTOM_OF_THE_WELL,	RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3]      	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3,       RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-965, -32),    "Left Side Pot 3",                  "Bottom Of The Well Left Side Pot 3",       RHT_POT_BOTTOM_OF_THE_WELL,	RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_LEFT_SIDE_POT_3));
    locationTable[RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1]  	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1,   RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(-41, 389),     "Near Entrance Pot 1",              "Bottom Of The Well Near Entrance Pot 1",   RHT_POT_BOTTOM_OF_THE_WELL,	RG_DEKU_STICK_1,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2]  	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2,   RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(36, 389),      "Near Entrance Pot 2",              "Bottom Of The Well Near Entrance Pot 2",   RHT_POT_BOTTOM_OF_THE_WELL,	RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_NEAR_ENTRANCE_POT_2));
    locationTable[RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1]     	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1,      RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(874, -1294),   "Fire Keese Pot 1",                 "Bottom Of The Well Fire Keese Pot 1",      RHT_POT_BOTTOM_OF_THE_WELL,	RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_FIRE_KEESE_POT_1));
    locationTable[RC_BOTTOM_OF_THE_WELL_UNDERWATER_POT]       	=	Location::Pot(RC_BOTTOM_OF_THE_WELL_UNDERWATER_POT,        RCQUEST_VANILLA, RCAREA_BOTTOM_OF_THE_WELL, SCENE_BOTTOM_OF_THE_WELL,          TWO_ACTOR_PARAMS(3, -1641),     "Underwater Pot",                   "Bottom Of The Well Underwater Pot",        RHT_POT_BOTTOM_OF_THE_WELL,	RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_UNDERWATER_POT));
    locationTable[RC_ICE_CAVERN_HALL_POT_1]                   	=	Location::Pot(RC_ICE_CAVERN_HALL_POT_1,                    RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-206, 449),    "Hall Pot 1",                       "Ice Cavern Hall Pot 1",                    RHT_POT_ICE_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_HALL_POT_1));
    locationTable[RC_ICE_CAVERN_HALL_POT_2]                   	=	Location::Pot(RC_ICE_CAVERN_HALL_POT_2,                    RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-203, 492),    "Hall Pot 2",                       "Ice Cavern Hall Pot 2",                    RHT_POT_ICE_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_HALL_POT_2));
    locationTable[RC_ICE_CAVERN_SPINNING_BLADE_POT_1]         	=	Location::Pot(RC_ICE_CAVERN_SPINNING_BLADE_POT_1,          RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(433, -732),    "Spinning Blade Pot 1",             "Ice Cavern Spinning Blade Pot 1",          RHT_POT_ICE_CAVERN,	        RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_SPINNING_BLADE_POT_1));
    locationTable[RC_ICE_CAVERN_SPINNING_BLADE_POT_2]         	=	Location::Pot(RC_ICE_CAVERN_SPINNING_BLADE_POT_2,          RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(569, -175),    "Spinning Blade Pot 2",             "Ice Cavern Spinning Blade Pot 2",          RHT_POT_ICE_CAVERN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_SPINNING_BLADE_POT_2));
    locationTable[RC_ICE_CAVERN_SPINNING_BLADE_POT_3]         	=	Location::Pot(RC_ICE_CAVERN_SPINNING_BLADE_POT_3,          RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(521, -131),    "Spinning Blade Pot 3",             "Ice Cavern Spinning Blade Pot 3",          RHT_POT_ICE_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_SPINNING_BLADE_POT_3));
    locationTable[RC_ICE_CAVERN_NEAR_END_POT_1]               	=	Location::Pot(RC_ICE_CAVERN_NEAR_END_POT_1,                RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-1422, 586),   "Near End Pot 1",                   "Ice Cavern Near End Pot 1",                RHT_POT_ICE_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_NEAR_END_POT_1));
    locationTable[RC_ICE_CAVERN_NEAR_END_POT_2]               	=	Location::Pot(RC_ICE_CAVERN_NEAR_END_POT_2,                RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(-1488, 676),   "Near End Pot 2",                   "Ice Cavern Near End Pot 2",                RHT_POT_ICE_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_NEAR_END_POT_2));
    locationTable[RC_ICE_CAVERN_FROZEN_POT_1]                 	=	Location::Pot(RC_ICE_CAVERN_FROZEN_POT_1,                  RCQUEST_VANILLA, RCAREA_ICE_CAVERN,         SCENE_ICE_CAVERN,                  TWO_ACTOR_PARAMS(366, -2036),   "Frozen Pot 1",                     "Ice Cavern Frozen Pot 1",                  RHT_POT_ICE_CAVERN,	        RG_PURPLE_RUPEE,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_FROZEN_POT_1));
	
	// MQ Dungeon Pots
	//            Randomizer Check                                    Randomizer Check                                                  Quest       Area                            Scene ID                        Params                          Short Name                      Spoiler Name                                        Hint Text Key                       Vanilla             Spoiler Collection Check
	locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1, 		        RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(-2391, -1804), "MQ Staircase Pot 1", 			"Dodongos Cavern MQ Staircase Pot 1", 	    	    RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2, 		        RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(-2361, -1161), "MQ Staircase Pot 2", 			"Dodongos Cavern MQ Staircase Pot 2", 	    	    RHT_POT_DODONGOS_CAVERN,	        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3, 		        RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(-1525, -1800), "MQ Staircase Pot 3", 			"Dodongos Cavern MQ Staircase Pot 3", 	    	    RHT_POT_DODONGOS_CAVERN,	        RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_3));
	locationTable[RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4, 		        RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(-1558, -1169), "MQ Staircase Pot 4", 			"Dodongos Cavern MQ Staircase Pot 4", 	    	    RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_STAIRCASE_POT_4));
	locationTable[RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_MIDDLE_POT] 	= Location::Pot(RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_MIDDLE_POT,      RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1532, -1225), 	"MQ Torch Puzzle Middle Pot", 	"Dodongos Cavern MQ Torch Puzzle Middle Pot", 	    RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BLOCK_ROOM_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_1] 	        = Location::Pot(RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_1, 		        RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3195, -155), 	"MQ Big Block Pot 1", 	     	"Dodongos Cavern MQ Big Block Pot 1",   	        RHT_POT_DODONGOS_CAVERN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_SILVER_BLOCK_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_2] 	        = Location::Pot(RC_DODONGOS_CAVERN_MQ_BIG_BLOCK_POT_2, 		        RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(2704, -483), 	"MQ Big Block Pot 2", 	    	"Dodongos Cavern MQ Big Block Pot 2",   	        RHT_POT_DODONGOS_CAVERN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_SILVER_BLOCK_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1] 	    = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1, 	    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3869, -811), 	"MQ Upper Lizalfos Pot 1", 		"Dodongos Cavern MQ Upper Lizalfos Pot 1",  	    RHT_POT_DODONGOS_CAVERN, 	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2] 	    = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2, 	    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3867, -969), 	"MQ Upper Lizalfos Pot 2", 		"Dodongos Cavern MQ Upper Lizalfos Pot 2",  	    RHT_POT_DODONGOS_CAVERN, 	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3] 	    = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3, 	    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(4687, -1071), 	"MQ Upper Lizalfos Pot 3", 		"Dodongos Cavern MQ Upper Lizalfos Pot 3",      	RHT_POT_DODONGOS_CAVERN, 	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_3));
	locationTable[RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4] 	    = Location::Pot(RC_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4, 	    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(4693, -1194), 	"MQ Upper Lizalfos Pot 4", 		"Dodongos Cavern MQ Upper Lizalfos Pot 4",      	RHT_POT_DODONGOS_CAVERN, 	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_UPPER_LIZALFOS_POT_4));
	locationTable[RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3327, -805), 	"MQ Two Flames Pot 1", 			"Dodongos Cavern MQ Two Flames Pot 1", 		        RHT_POT_DODONGOS_CAVERN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2]   		= Location::Pot(RC_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3326, -753), 	"MQ Two Flames Pot 2", 			"Dodongos Cavern MQ Two Flames Pot 2", 		        RHT_POT_DODONGOS_CAVERN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_TWO_FLAMES_POT_2));
    locationTable[RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_CORNER_POT]    = Location::Pot(RC_DODONGOS_CAVERN_MQ_TORCH_PUZZLE_CORNER_POT,      RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1827, -587), 	"MQ Torch Puzzle Corner Pot", 	"Dodongos Cavern MQ Torch Puzzle Corner Pot",   	RHT_POT_DODONGOS_CAVERN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BLOCK_ROOM_POT_1));
    locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1727, -151), 	"MQ Right Side Pot 1", 			"Dodongos Cavern MQ Right Side Pot 1", 		        RHT_POT_DODONGOS_CAVERN,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1815, -154), 	"MQ Right Side Pot 2", 			"Dodongos Cavern MQ Right Side Pot 2", 		        RHT_POT_DODONGOS_CAVERN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3442, -469), 	"MQ Right Side Pot 3", 			"Dodongos Cavern MQ Right Side Pot 3", 		        RHT_POT_DODONGOS_CAVERN,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_3));
	locationTable[RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(3682, -466), 	"MQ Right Side Pot 4", 			"Dodongos Cavern MQ Right Side Pot 4", 		        RHT_POT_DODONGOS_CAVERN,	        RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_RIGHT_SIDE_POT_4));
	locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1, 			    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1844, -1867), 	"MQ Poe Room Pot 1", 			"Dodongos Cavern MQ Poe Room Pot 1", 		        RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2, 			    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(2272, -2213), 	"MQ Poe Room Pot 2", 			"Dodongos Cavern MQ Poe Room Pot 2", 		        RHT_POT_DODONGOS_CAVERN,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3, 			    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(2240, -1369), 	"MQ Poe Room Pot 3", 			"Dodongos Cavern MQ Poe Room Pot 3", 		        RHT_POT_DODONGOS_CAVERN,	        RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_3));
	locationTable[RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4, 			    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1535, -1299), 	"MQ Poe Room Pot 4", 			"Dodongos Cavern MQ Poe Room Pot 4", 		        RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_POE_ROOM_POT_4));
	locationTable[RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_SW_POT] 	    = Location::Pot(RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_SW_POT, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(258, -3055), 	"MQ Before Boss SW Pot", 		"Dodongos Cavern MQ Before Boss SW Pot", 	        RHT_POT_DODONGOS_CAVERN,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BEFORE_BOSS_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_NE_POT] 		= Location::Pot(RC_DODONGOS_CAVERN_MQ_BEFORE_BOSS_NE_POT, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(-265, -2499), 	"MQ Before Boss NE Pot", 		"Dodongos Cavern MQ Before Boss NE Pot", 	        RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BEFORE_BOSS_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SE_POT] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SE_POT, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1758, -3194), 	"MQ Armos Room SE Pot", 		"Dodongos Cavern MQ Armos SE Pot", 		    	    RHT_POT_DODONGOS_CAVERN,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_3));
	locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SW_POT] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_SW_POT, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1223, -3197), 	"MQ Armos Room SW Pot", 		"Dodongos Cavern MQ Armos SW Pot", 		    	    RHT_POT_DODONGOS_CAVERN,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_4));
	locationTable[RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_1] 	    	= Location::Pot(RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_1, 			    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1663, -4311), 	"MQ Backroom Pot 1", 			"Dodongos Cavern MQ Backroom Pot 1", 		        RHT_POT_DODONGOS_CAVERN,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BACKROOM_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_2] 	    	= Location::Pot(RC_DODONGOS_CAVERN_MQ_BACKROOM_POT_2, 			    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1648, -4694), 	"MQ Backroom Pot 2", 			"Dodongos Cavern MQ Backroom Pot 2", 		        RHT_POT_DODONGOS_CAVERN,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_BACKROOM_POT_2));
	locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NW_POT] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NW_POT, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1005, -3711), 	"MQ Armos Room NW Pot", 		"Dodongos Cavern MQ Armos NW Pot", 		    	    RHT_POT_DODONGOS_CAVERN,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_1));
	locationTable[RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NE_POT] 		    = Location::Pot(RC_DODONGOS_CAVERN_MQ_ARMOS_ROOM_NE_POT, 		    RCQUEST_MQ, RCAREA_DODONGOS_CAVERN, 	    SCENE_DODONGOS_CAVERN, 		    TWO_ACTOR_PARAMS(1066, -3711), 	"MQ Armos Room NE Pot", 		"Dodongos Cavern MQ Armos NE Pot", 		    	    RHT_POT_DODONGOS_CAVERN,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_DODONGOS_CAVERN_MQ_ARMOS_POT_2));
	locationTable[RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1] 	    	= Location::Pot(RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1, 		        RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(-183, -449), 	"MQ Entrance Pot 1", 			"Jabu Jabus Belly MQ Entrance Pot 1", 	    	    RHT_POT_JABU_JABUS_BELLY,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_1));
	locationTable[RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2] 	    	= Location::Pot(RC_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2, 		        RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(202, -62), 	"MQ Entrance Pot 2", 			"Jabu Jabus Belly MQ Entrance Pot 2", 	    	    RHT_POT_JABU_JABUS_BELLY,	        RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_ENTRANCE_POT_2));
	locationTable[RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_1] 			    = Location::Pot(RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_1, 			    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(-689, -1800), 	"MQ Geyser Pot 1", 				"Jabu Jabus Belly MQ Geyser Pot 1", 	    	    RHT_POT_JABU_JABUS_BELLY,	        RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_GEYSER_POT_1));
	locationTable[RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_2] 			    = Location::Pot(RC_JABU_JABUS_BELLY_MQ_GEYSER_POT_2, 			    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(-693, -1614), 	"MQ Geyser Pot 2", 				"Jabu Jabus Belly MQ Geyser Pot 2", 	    	    RHT_POT_JABU_JABUS_BELLY,	        RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_GEYSER_POT_2));
	locationTable[RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1] 	    	= Location::Pot(RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1, 		    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(697, -2318), 	"MQ Time Block Pot 1", 			"Jabu Jabus Belly MQ Time Block Pot 1",     	    RHT_POT_JABU_JABUS_BELLY,	        RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_1));
	locationTable[RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2] 	    	= Location::Pot(RC_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2, 		    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(699, -2445), 	"MQ Time Block Pot 2", 			"Jabu Jabus Belly MQ Time Block Pot 2",     	    RHT_POT_JABU_JABUS_BELLY,	        RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_TIME_BLOCK_POT_2));
	locationTable[RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1] 		    = Location::Pot(RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1, 		    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(612, -5932), 	"MQ Like Likes Pot 1", 			"Jabu Jabus Belly MQ Like Likes Pot 1",     	    RHT_POT_JABU_JABUS_BELLY,	        RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_1));
	locationTable[RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2] 	    	= Location::Pot(RC_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2, 		    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(718, -5929), 	"MQ Like Likes Pot 2", 			"Jabu Jabus Belly MQ Like Likes Pot 2",     	    RHT_POT_JABU_JABUS_BELLY,	        RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_LIKE_LIKES_POT_2));
	locationTable[RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1]     	= Location::Pot(RC_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1, 		    RCQUEST_MQ, RCAREA_JABU_JABUS_BELLY, 	    SCENE_JABU_JABU, 			    TWO_ACTOR_PARAMS(1305, -2005), 	"MQ Before Boss Pot 1", 		"Jabu Jabus Belly MQ Before Boss Pot 1",    	    RHT_POT_JABU_JABUS_BELLY,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_JABU_JABUS_BELLY_MQ_BEFORE_BOSS_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_1] 			    	= Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_1, 				    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(675, -989), 	"MQ Lobby Pot 1", 				"Forest Temple MQ Lobby Pot 1", 			        RHT_POT_FOREST_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_2] 			    	= Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_2, 				    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-429, -981), 	"MQ Lobby Pot 2", 				"Forest Temple MQ Lobby Pot 2", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_2));
	locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_3] 			    	= Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_3, 				    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(720, -1037), 	"MQ Lobby Pot 3", 				"Forest Temple MQ Lobby Pot 3", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_3));
	locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_4] 			    	= Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_4, 				    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-477, -1034), 	"MQ Lobby Pot 4", 				"Forest Temple MQ Lobby Pot 4", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_4));
	locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_5] 			    	= Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_5, 				    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(630, -943), 	"MQ Lobby Pot 5", 				"Forest Temple MQ Lobby Pot 5", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_5));
	locationTable[RC_FOREST_TEMPLE_MQ_LOBBY_POT_6] 			    	= Location::Pot(RC_FOREST_TEMPLE_MQ_LOBBY_POT_6, 				    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-384, -937), 	"MQ Lobby Pot 6", 				"Forest Temple MQ Lobby Pot 6", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOBBY_POT_6));
	locationTable[RC_FOREST_TEMPLE_MQ_WOLFOS_POT_1] 		        = Location::Pot(RC_FOREST_TEMPLE_MQ_WOLFOS_POT_1, 		            RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(90, -3583), 	"MQ Wolfos Pot 1", 		        "Forest Temple MQ Wolfos Pot 1",              	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOWER_STALFOS_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_WOLFOS_POT_2] 		        = Location::Pot(RC_FOREST_TEMPLE_MQ_WOLFOS_POT_2, 	        	    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(142, -3583), 	"MQ Wolfos Pot 2", 		        "Forest Temple MQ Wolfos Pot 2",             	    RHT_POT_FOREST_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_LOWER_STALFOS_POT_2));
	locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1] 	    	= Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1, 		    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(206, -3631), 	"MQ Upper Stalfos Pot 1", 		"Forest Temple MQ Upper Stalfos Pot 1",     	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2] 	    	= Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2, 		    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(32, -3012), 	"MQ Upper Stalfos Pot 2", 		"Forest Temple MQ Upper Stalfos Pot 2",     	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_2));
	locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3] 	    	= Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3, 		    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(30, -3631), 	"MQ Upper Stalfos Pot 3", 		"Forest Temple MQ Upper Stalfos Pot 3",     	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_3));
	locationTable[RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4] 	    	= Location::Pot(RC_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4, 		    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(202, -3012), 	"MQ Upper Stalfos Pot 4", 		"Forest Temple MQ Upper Stalfos Pot 4",     	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_UPPER_STALFOS_POT_4));
	locationTable[RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_1] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_1, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(830, -3639), 	"MQ Blue Poe Pot 1", 			"Forest Temple MQ Blue Poe Pot 1", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BLUE_POE_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_2] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_2, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(941, -3640), 	"MQ Blue Poe Pot 2", 			"Forest Temple MQ Blue Poe Pot 2", 			        RHT_POT_FOREST_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BLUE_POE_POT_2));
	locationTable[RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_3] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BLUE_POE_POT_3, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(883, -3640), 	"MQ Blue Poe Pot 3", 			"Forest Temple MQ Blue Poe Pot 3", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BLUE_POE_POT_3));
	locationTable[RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_1] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_1, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(1647, -1529), 	"MQ Green Poe Pot 1", 			"Forest Temple MQ Green Poe Pot 1", 	    	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_GREEN_POE_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_2] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_GREEN_POE_POT_2, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(1645, -1588), 	"MQ Green Poe Pot 2", 			"Forest Temple MQ Green Poe Pot 2", 	    	    RHT_POT_FOREST_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_GREEN_POE_POT_2));
	locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_1] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_1, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-461, -1095), 	"MQ Basement Pot 1", 			"Forest Temple MQ Basement Pot 1", 		       	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_1));
	locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_2] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_2, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-351, -979), 	"MQ Basement Pot 2", 			"Forest Temple MQ Basement Pot 2", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_2));
	locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_3] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_3, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-387, -1018), 	"MQ Basement Pot 3", 			"Forest Temple MQ Basement Pot 3", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_3));
	locationTable[RC_FOREST_TEMPLE_MQ_BASEMENT_POT_4] 		    	= Location::Pot(RC_FOREST_TEMPLE_MQ_BASEMENT_POT_4, 			    RCQUEST_MQ, RCAREA_FOREST_TEMPLE, 		    SCENE_FOREST_TEMPLE, 		    TWO_ACTOR_PARAMS(-424, -1057), 	"MQ Basement Pot 4", 			"Forest Temple MQ Basement Pot 4", 		    	    RHT_POT_FOREST_TEMPLE,	            RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FOREST_TEMPLE_MQ_BASEMENT_POT_4));
	locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1] 		    = Location::Pot(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1, 		    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(1873, 2252), 	"MQ Forest Trial Pot 1", 		"Ganons Castle MQ Forest Trial Pot 1", 	    	    RHT_POT_GANONS_CASTLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_1));
	locationTable[RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2, 		    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(1726, 2326), 	"MQ Forest Trial Pot 2", 		"Ganons Castle MQ Forest Trial Pot 2", 	    	    RHT_POT_GANONS_CASTLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FOREST_TRIAL_POT_2));
	locationTable[RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1, 		        RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(3544, -931), 	"MQ Water Trial Pot 1", 		"Ganons Castle MQ Water Trial Pot 1", 	    	    RHT_POT_GANONS_CASTLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_WATER_TRIAL_POT_1));
	locationTable[RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2, 		        RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(3541, -755), 	"MQ Water Trial Pot 2", 		"Ganons Castle MQ Water Trial Pot 2", 	    	    RHT_POT_GANONS_CASTLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_WATER_TRIAL_POT_2));
	locationTable[RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1, 		    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(2073, -4592), 	"MQ Shadow Trial Pot 1", 		"Ganons Castle MQ Shadow Trial Pot 1", 	    	    RHT_POT_GANONS_CASTLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_1));
	locationTable[RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2] 		    = Location::Pot(RC_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2, 		    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(2217, -4516), 	"MQ Shadow Trial Pot 2", 		"Ganons Castle MQ Shadow Trial Pot 2", 	    	    RHT_POT_GANONS_CASTLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SHADOW_TRIAL_POT_2));
	locationTable[RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1] 		    = Location::Pot(RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1, 			    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-1918, -3957), "MQ Fire Trial Pot 1", 			"Ganons Castle MQ Fire Trial Pot 1", 	    	    RHT_POT_GANONS_CASTLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_1));
	locationTable[RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2, 			    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-2052, -3888), "MQ Fire Trial Pot 2", 			"Ganons Castle MQ Fire Trial Pot 2", 	    	    RHT_POT_GANONS_CASTLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_FIRE_TRIAL_POT_2));
	locationTable[RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1] 		    = Location::Pot(RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1, 		        RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-3842, -759), 	"MQ Light Trial Pot 1", 		"Ganons Castle MQ Light Trial Pot 1", 	    	    RHT_POT_GANONS_CASTLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_1));
	locationTable[RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2, 		        RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-3845, -933), 	"MQ Light Trial Pot 2", 		"Ganons Castle MQ Light Trial Pot 2", 	    	    RHT_POT_GANONS_CASTLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_LIGHT_TRIAL_POT_2));
	locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1] 	    	= Location::Pot(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1, 		    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-1402, 1414), 	"MQ Spirit Trial Pot 1", 		"Ganons Castle MQ Spirit Trial Pot 1", 	    	    RHT_POT_GANONS_CASTLE,	            RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_1));
	locationTable[RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2] 		    = Location::Pot(RC_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2, 		    RCQUEST_MQ, RCAREA_GANONS_CASTLE, 		    SCENE_INSIDE_GANONS_CASTLE,     TWO_ACTOR_PARAMS(-1256, 1500), 	"MQ Spirit Trial Pot 2", 		"Ganons Castle MQ Spirit Trial Pot 2", 		        RHT_POT_GANONS_CASTLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_GANONS_CASTLE_MQ_SPIRIT_TRIAL_POT_2));
	locationTable[RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1]   	= Location::Pot(RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1, 	    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-1045, -130), 	"MQ Whispering Walls Pot 1", 	"Shadow Temple MQ Whispering Walls Pot 1",          RHT_POT_SHADOW_TEMPLE, 	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_1));
	locationTable[RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2]   	= Location::Pot(RC_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2, 	    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-1045, -240), 	"MQ Whispering Walls Pot 2", 	"Shadow Temple MQ Whispering Walls Pot 2",          RHT_POT_SHADOW_TEMPLE, 	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_WHISPERING_WALLS_POT_2));
	locationTable[RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1, 	        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-657, -949), 	"MQ Entrance Redead Pot 1", 	"Shadow Temple MQ Entrance Redead Pot 1", 	        RHT_POT_SHADOW_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_1));
	locationTable[RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2, 	        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-713, -1016), 	"MQ Entrance Redead Pot 2", 	"Shadow Temple MQ Entrance Redead Pot 2", 	        RHT_POT_SHADOW_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_ENTRANCE_REDEAD_POT_2));
	locationTable[RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_WEST_POT] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_WEST_POT, 	    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(614, 3579), 	"MQ Lower Umbrella W Pot", 		"Shadow Temple MQ Stone Umbrella Lower West Pot",   RHT_POT_SHADOW_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_1));
	locationTable[RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_EAST_POT] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_LOWER_UMBRELLA_EAST_POT, 	    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(798, 3574), 	"MQ Lower Umbrella E Pot", 		"Shadow Temple MQ Stone Umbrella Lower East Pot", 	RHT_POT_SHADOW_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_2));
	locationTable[RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_SOUTH_POT]     = Location::Pot(RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_SOUTH_POT, 	    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(1173, 3707), 	"MQ Upper Umbrella S Pot", 		"Shadow Temple MQ Stone Umbrella Upper South Pot", 	RHT_POT_SHADOW_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_3));
	locationTable[RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_NORTH_POT]     = Location::Pot(RC_SHADOW_TEMPLE_MQ_UPPER_UMBRELLA_NORTH_POT, 	    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(1173, 3605), 	"MQ Upper Umbrella N Pot", 		"Shadow Temple MQ Stone Umbrella Upper North Pot", 	RHT_POT_SHADOW_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_FALLING_SPIKES_POT_4));
	locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1] 	    	= Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1, 		        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(4222, -916), 	"MQ Before Boat Pot 1", 		"Shadow Temple MQ Before Boat Pot 1", 		        RHT_POT_SHADOW_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_1));
	locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2] 	    	= Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2, 		        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(4549, -922), 	"MQ Before Boat Pot 2", 		"Shadow Temple MQ Before Boat Pot 2", 	    	    RHT_POT_SHADOW_TEMPLE,	            RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_BEFORE_BOAT_POT_2));
	locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_WEST_POT] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_WEST_POT, 	        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-2400, -1470), "MQ Before Chasm W Pot", 		"Shadow Temple MQ Before Chasm West Pot", 		    RHT_POT_SHADOW_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_1));
	locationTable[RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_EAST_POT] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_BEFORE_CHASM_EAST_POT, 	        RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-2230, -1470), "MQ Before Chasm E Pot", 		"Shadow Temple MQ Before Chasm East Pot", 	    	RHT_POT_SHADOW_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_2));
	locationTable[RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_WEST_POT] 	    = Location::Pot(RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_WEST_POT, 		    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-2859, -497), 	"MQ After Chasm W Pot", 		"Shadow Temple MQ After Chasm West Pot", 	    	RHT_POT_SHADOW_TEMPLE,	            RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_3));
	locationTable[RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_EAST_POT] 		= Location::Pot(RC_SHADOW_TEMPLE_MQ_AFTER_CHASM_EAST_POT, 		    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-2624, -492), 	"MQ After Chasm E Pot", 		"Shadow Temple MQ After Chasm East Pot", 	    	RHT_POT_SHADOW_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_AFTER_BOAT_POT_4));
	locationTable[RC_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT] 		    = Location::Pot(RC_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT, 		    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-4303, -2657), "MQ Spike Baricade Pot", 		"Shadow Temple MQ Spike Baricade Pot", 	    	    RHT_POT_SHADOW_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_SPIKE_BARICADE_POT));
	locationTable[RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1] 			    = Location::Pot(RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1, 			    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-4575, -811), 	"MQ Dead Hand Pot 1", 			"Shadow Temple MQ Dead Hand Pot 1", 		        RHT_POT_SHADOW_TEMPLE,	            RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_1));
	locationTable[RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2] 			    = Location::Pot(RC_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2, 			    RCQUEST_MQ, RCAREA_SHADOW_TEMPLE, 		    SCENE_SHADOW_TEMPLE, 		    TWO_ACTOR_PARAMS(-4036, -811), 	"MQ Dead Hand Pot 2", 			"Shadow Temple MQ Dead Hand Pot 2", 		        RHT_POT_SHADOW_TEMPLE,	            RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_SHADOW_TEMPLE_MQ_DEAD_HAND_POT_2));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1]   	= Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1, 	    RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(339, -377), 	"MQ Inner Lobby Pot 1", 		"Bottom Of The Well MQ Inner Lobby Pot 1",  	    RHT_POT_BOTTOM_OF_THE_WELL, 	    RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_1));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2] 	    = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2, 	    RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(416, -377), 	"MQ Inner Lobby Pot 2", 		"Bottom Of The Well MQ Inner Lobby Pot 2",  	    RHT_POT_BOTTOM_OF_THE_WELL, 	    RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_2));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3] 	    = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3, 	    RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(269, -376), 	"MQ Inner Lobby Pot 3", 		"Bottom Of The Well MQ Inner Lobby Pot 3",  	    RHT_POT_BOTTOM_OF_THE_WELL, 	    RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_INNER_LOBBY_POT_3));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT]     	= Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT, 		    RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(421, -174), 	"MQ Outer Lobby Pot", 			"Bottom Of The Well MQ Outer Lobby Pot",    	    RHT_POT_BOTTOM_OF_THE_WELL,	        RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_OUTER_LOBBY_POT));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_1]  	= Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_1,     RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(288, -1240), 	"MQ East Inner Pot 1", 			"Bottom Of The Well MQ East Inner Room Pot 1", 	    RHT_POT_BOTTOM_OF_THE_WELL,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_SOUTH_KEY_POT_1));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_2]   = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_2,     RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(438, -1234), 	"MQ East Inner Pot 2", 			"Bottom Of The Well MQ East Inner Room Pot 2",      RHT_POT_BOTTOM_OF_THE_WELL,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_SOUTH_KEY_POT_2));
	locationTable[RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_3]   = Location::Pot(RC_BOTTOM_OF_THE_WELL_MQ_EAST_INNER_ROOM_POT_3,     RCQUEST_MQ, RCAREA_BOTTOM_OF_THE_WELL, 	    SCENE_BOTTOM_OF_THE_WELL, 	    TWO_ACTOR_PARAMS(444, -1114), 	"MQ East Inner Pot 3", 			"Bottom Of The Well MQ East Inner Room Pot 3",      RHT_POT_BOTTOM_OF_THE_WELL,	        RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_BOTTOM_OF_THE_WELL_MQ_SOUTH_KEY_POT_3));
	locationTable[RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_1] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_1, 				    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-357, 957), 	"MQ Entrance Pot 1", 			"Fire Temple MQ Entrance Pot 1", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ENTRANCE_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_2] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_ENTRANCE_POT_2, 				    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(356, 959), 	"MQ Entrance Pot 2", 			"Fire Temple MQ Entrance Pot 2", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ENTRANCE_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1]     	= Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1,           RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(187, -1449), 	"MQ Before Mini Boss Pot 1", 	"Fire Temple MQ Before Mini Boss Pot 1", 	        RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2]     	= Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(188, -1514), 	"MQ Before Mini Boss Pot 2", 	"Fire Temple MQ Before Mini Boss Pot 2",    	    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3]     	= Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(487, -1450), 	"MQ Before Mini Boss Pot 3", 	"Fire Temple MQ Before Mini Boss Pot 3",    	    RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_3));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4]     	= Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(486, -1515), 	"MQ Before Mini Boss Pot 4", 	"Fire Temple MQ Before Mini Boss Pot 4",    	    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_4));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5] 	    = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(38, -1179), 	"MQ Before Mini Boss Pot 5", 	"Fire Temple MQ Before Mini Boss Pot 5", 	        RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_5));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6]     	= Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(42, -1298), 	"MQ Before Mini Boss Pot 6", 	"Fire Temple MQ Before Mini Boss Pot 6",    	    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_6));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7] 	    = Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(520, -943), 	"MQ Before Mini Boss Pot 7", 	"Fire Temple MQ Before Mini Boss Pot 7",    	    RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_7));
	locationTable[RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8]     	= Location::Pot(RC_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8, 		    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(35, -946), 	"MQ Before Mini Boss Pot 8", 	"Fire Temple MQ Before Mini Boss Pot 8",    	    RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_BEFORE_MINI_BOSS_POT_8));
	locationTable[RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1] 	    	= Location::Pot(RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-1426, -711), 	"MQ Outside Boss Pot 1", 		"Fire Temple MQ Outside Boss Pot 1", 	    	    RHT_POT_FIRE_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2] 	    	= Location::Pot(RC_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-1425, -660), 	"MQ Outside Boss Pot 2", 		"Fire Temple MQ Outside Boss Pot 2", 	    	    RHT_POT_FIRE_TEMPLE,	            RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_OUTSIDE_BOSS_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_LAVA_ROOM_NORTH_POT] 			= Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_ROOM_NORTH_POT, 		        RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1646, -1260), 	"MQ Lava Room North Pot", 		"Fire Temple MQ Lava Room North Pot", 			    RHT_POT_FIRE_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_LAVA_ROOM_HIGH_POT] 			= Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_ROOM_HIGH_POT, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(2226, -809), 	"MQ Lava Room High Pot", 		"Fire Temple MQ Lava Room High Pot", 			    RHT_POT_FIRE_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_LAVA_ROOM_SOUTH_POT] 			= Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_ROOM_SOUTH_POT, 		        RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1634, 1475), 	"MQ Lava Room South Pot", 		"Fire Temple MQ Lava Room South Pot", 		        RHT_POT_FIRE_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_POT_3));
	locationTable[RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(845, -1591), 	"MQ Lava Torch Pot 1", 			"Fire Temple MQ Lava Torch Pot 1", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(901, -1593), 	"MQ Lava Torch Pot 2", 			"Fire Temple MQ Lava Torch Pot 2", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_LAVA_TORCH_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1756, -155), 	"MQ Above Lava Pot 1", 			"Fire Temple MQ Above Lava Pot 1", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1819, -82), 	"MQ Above Lava Pot 2", 			"Fire Temple MQ Above Lava Pot 2", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1789, -122), 	"MQ Above Lava Pot 3", 			"Fire Temple MQ Above Lava Pot 3", 		    	    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_ABOVE_LAVA_POT_3));
	locationTable[RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1493, 386), 	"MQ Flame Wall Pot 1", 			"Fire Temple MQ Flame Wall Pot 1", 	    		    RHT_POT_FIRE_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FLAME_WALL_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2] 		    	= Location::Pot(RC_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2, 			    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(1445, 445), 	"MQ Flame Wall Pot 2", 			"Fire Temple MQ Flame Wall Pot 2", 	    		    RHT_POT_FIRE_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FLAME_WALL_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_SOUTH_POT] 		= Location::Pot(RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_SOUTH_POT, 	    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-2217, -163), 	"MQ Past Fire Maze North Pot", 	"Fire Temple MQ Past Fire Maze North Pot",     	    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_1));
	locationTable[RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_NORTH_POT] 		= Location::Pot(RC_FIRE_TEMPLE_MQ_PAST_FIRE_MAZE_NORTH_POT, 	    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-2657, -591), 	"MQ Past Fire Maze South Pot", 	"Fire Temple MQ Past Fire Maze South Pot", 	   	    RHT_POT_FIRE_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_2));
	locationTable[RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHMOST_POT] 		= Location::Pot(RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHMOST_POT, 	        RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-1849, -1114), "MQ Fire Maze Northmost Pot", 	"Fire Temple MQ Fire Maze Northmost Pot", 		    RHT_POT_FIRE_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_3));
	locationTable[RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHWEST_POT] 	    = Location::Pot(RC_FIRE_TEMPLE_MQ_FIRE_MAZE_NORTHWEST_POT, 	        RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-2110, -646), 	"MQ Fire Maze North West Pot", 	"Fire Temple MQ Fire Maze North West Pot", 		    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_4));
	locationTable[RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_WEST_POT] 	    = Location::Pot(RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_WEST_POT, 	    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-2319, 838), 	"MQ South Fire Maze W Pot", 	"Fire Temple MQ South Fire Maze West Pot", 		    RHT_POT_FIRE_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_5));
	locationTable[RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_EAST_POT] 		= Location::Pot(RC_FIRE_TEMPLE_MQ_SOUTH_FIRE_MAZE_EAST_POT, 	    RCQUEST_MQ, RCAREA_FIRE_TEMPLE, 		    SCENE_FIRE_TEMPLE, 			    TWO_ACTOR_PARAMS(-2285, 803), 	"MQ South Fire Maze E Pot", 	"Fire Temple MQ South Fire Maze East Pot", 		    RHT_POT_FIRE_TEMPLE,	            RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_FIRE_TEMPLE_MQ_FIRE_MAZE_POT_6));
	locationTable[RC_ICE_CAVERN_MQ_ENTRANCE_POT] 			    	= Location::Pot(RC_ICE_CAVERN_MQ_ENTRANCE_POT, 				        RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(248, 2287), 	"MQ Entrance Pot", 				"Ice Cavern MQ Entrance Pot", 			    	    RHT_POT_ICE_CAVERN,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_ENTRANCE_POT));
	locationTable[RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1] 		    = Location::Pot(RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1, 			    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(51, 718), 		"MQ First Crystal Pot 1", 		"Ice Cavern MQ First Crystal Pot 1", 	    	    RHT_POT_ICE_CAVERN,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_1));
	locationTable[RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2] 		    = Location::Pot(RC_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2, 			    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(52, 768), 		"MQ First Crystal Pot 2", 		"Ice Cavern MQ First Crystal Pot 2", 	    	    RHT_POT_ICE_CAVERN,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_FIRST_CRYSTAL_POT_2));
	locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1] 		    	= Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1, 			    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(433, -732), 	"MQ Early Wolfos Pot 1", 		"Ice Cavern MQ Early Wolfos Pot 1", 	    	    RHT_POT_ICE_CAVERN,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_1));
	locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2] 		    	= Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2, 			    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(569, -175), 	"MQ Early Wolfos Pot 2", 		"Ice Cavern MQ Early Wolfos Pot 2", 	    	    RHT_POT_ICE_CAVERN,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_2));
	locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3] 		    	= Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3, 			    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(521, -131), 	"MQ Early Wolfos Pot 3", 		"Ice Cavern MQ Early Wolfos Pot 3", 	    	    RHT_POT_ICE_CAVERN,	                RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_3));
	locationTable[RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4] 		    	= Location::Pot(RC_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4, 			    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(138, -672), 	"MQ Early Wolfos Pot 4", 		"Ice Cavern MQ Early Wolfos Pot 4", 	    	    RHT_POT_ICE_CAVERN,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_EARLY_WOLFOS_POT_4));
	locationTable[RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1] 		    	= Location::Pot(RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1, 			        RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(-1352, 639), 	"MQ Push Block Pot 1", 			"Ice Cavern MQ Push Block Pot 1", 		    	    RHT_POT_ICE_CAVERN,	                RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_PUSH_BLOCK_POT_1));
	locationTable[RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2] 		    	= Location::Pot(RC_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2, 			        RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(-1396, 596), 	"MQ Push Block Pot 2", 			"Ice Cavern MQ Push Block Pot 2", 		    	    RHT_POT_ICE_CAVERN,	                RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_PUSH_BLOCK_POT_2));
	locationTable[RC_ICE_CAVERN_MQ_COMPASS_POT_1] 			    	= Location::Pot(RC_ICE_CAVERN_MQ_COMPASS_POT_1, 				    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(902, -2720), 	"MQ Compass Pot 1", 			"Ice Cavern MQ Compass Pot 1", 			    	    RHT_POT_ICE_CAVERN,	                RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_COMPASS_POT_1));
	locationTable[RC_ICE_CAVERN_MQ_COMPASS_POT_2] 			    	= Location::Pot(RC_ICE_CAVERN_MQ_COMPASS_POT_2, 				    RCQUEST_MQ, RCAREA_ICE_CAVERN, 			    SCENE_ICE_CAVERN, 			    TWO_ACTOR_PARAMS(451, -2726), 	"MQ Compass Pot 2", 			"Ice Cavern MQ Compass Pot 2", 		    		    RHT_POT_ICE_CAVERN,	                RG_BOMBS_10,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_ICE_CAVERN_MQ_COMPASS_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1] 		    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-255, 742), 	"MQ Entrance Pot 1", 			"Spirit Temple MQ Entrance Pot 1", 		    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2] 		    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(306, 682), 	"MQ Entrance Pot 2", 			"Spirit Temple MQ Entrance Pot 2", 		    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3] 		    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-297, 687), 	"MQ Entrance Pot 3", 			"Spirit Temple MQ Entrance Pot 3", 		    	    RHT_POT_SPIRIT_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_3));
	locationTable[RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4] 		    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(260, 735), 	"MQ Entrance Pot 4", 			"Spirit Temple MQ Entrance Pot 4", 		    	    RHT_POT_SPIRIT_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_ENTRANCE_POT_4));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-894, -50), 	"MQ Child Slugma Pot", 			"Spirit Temple MQ Child Slugma Pot", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_SLUGMA_POT));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1, 		        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-382, -1256), 	"MQ Child Gibdo Pot 1", 		"Spirit Temple MQ Child Gibdo Pot 1", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2, 		        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-381, -1207), 	"MQ Child Gibdo Pot 2", 		"Spirit Temple MQ Child Gibdo Pot 2", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_GIBDO_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-1035, -1240), "MQ Child Like Like Pot", 		"Spirit Temple MQ Child Like Like Pot",     	    RHT_POT_SPIRIT_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_LIKE_LIKE_POT));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-1166, -1931), "MQ Child Stalfos Pot 1", 		"Spirit Temple MQ Child Stalfos Pot 1",     	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2] 		    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-959, -1930), 	"MQ Child Stalfos Pot 2", 		"Spirit Temple MQ Child Stalfos Pot 2",     	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3] 		    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-853, -1837), 	"MQ Child Stalfos Pot 3", 		"Spirit Temple MQ Child Stalfos Pot 3",     	    RHT_POT_SPIRIT_TEMPLE,	            RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_3));
	locationTable[RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4] 		    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-1250, -1852), "MQ Child Stalfos Pot 4", 		"Spirit Temple MQ Child Stalfos Pot 4", 	        RHT_POT_SPIRIT_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CHILD_STALFOS_POT_4));
	locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_2F_CENTER_EAST_POT]    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_2F_CENTER_EAST_POT,      RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(639, -1224), 	"MQ Statue Room 2F CE Pot", 	"Spirit Temple MQ Statue 2F Center East Pot",       RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_3F_EAST_POT]    	    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_3F_EAST_POT, 	        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-521, -1576), 	"MQ Statue Room 3F E Pot", 	    "Spirit Temple MQ Statue Room 3F East Pot",   	    RHT_POT_SPIRIT_TEMPLE,	            RG_ARROWS_5,        SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_3F_WEST_POT]    	    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_3F_WEST_POT, 	        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-595, -1576), 	"MQ Statue Room 3F W Pot", 	    "Spirit Temple MQ Statue Room 3F West Pot",   	    RHT_POT_SPIRIT_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_3));
	locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_2F_WEST_POT] 	        = Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_2F_WEST_POT, 	        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-559, -1209), 	"MQ Statue Room 2F W Pot", 	    "Spirit Temple MQ Statue 2F West Pot",   	        RHT_POT_SPIRIT_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_4));
	locationTable[RC_SPIRIT_TEMPLE_MQ_STATUE_2F_EASTMOST_POT]    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_STATUE_2F_EASTMOST_POT, 	    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(704, -1224), 	"MQ Statue Room 2F E Pot", 	    "Spirit Temple MQ Statue 2F Eastmost Pot",          RHT_POT_SPIRIT_TEMPLE,	            RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_CENTRAL_CHAMBER_POT_5));
	locationTable[RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-1276, 432), 	"MQ Sun Blocks Pot 1", 			"Spirit Temple MQ Sun Blocks Pot 1", 		        RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2] 		    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-1635, -446), 	"MQ Sun Blocks Pot 2", 			"Spirit Temple MQ Sun Blocks Pot 2", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_DEKU_SEEDS_30,   SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_SUN_BLOCKS_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(829, 250), 	"MQ Long Climb Pot 1", 			"Spirit Temple MQ Long Climb Pot 1", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(531, 249), 	"MQ Long Climb Pot 2", 			"Spirit Temple MQ Long Climb Pot 2", 		        RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_LONG_CLIMB_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(244, -893), 	"MQ Big Mirror Pot 1", 			"Spirit Temple MQ Big Mirror Pot 1", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-122, -741), 	"MQ Big Mirror Pot 2", 			"Spirit Temple MQ Big Mirror Pot 2", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(250, -741), 	"MQ Big Mirror Pot 3", 			"Spirit Temple MQ Big Mirror Pot 3", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_3));
	locationTable[RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4] 		    = Location::Pot(RC_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4, 			    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(-123, -892), 	"MQ Big Mirror Pot 4", 			"Spirit Temple MQ Big Mirror Pot 4", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BIG_MIRROR_POT_4));
	locationTable[RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(781, -841), 	"MQ Before Mirror Pot 1", 		"Spirit Temple MQ Before Mirror Pot 1",     	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2, 		    RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(587, -844), 	"MQ Before Mirror Pot 2", 		"Spirit Temple MQ Before Mirror Pot 2",     	    RHT_POT_SPIRIT_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_BEFORE_MIRROR_POT_2));
	locationTable[RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1, 		        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(1204, -910), 	"MQ Early Adult Pot 1", 		"Spirit Temple MQ Early Adult Pot 1", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_1));
	locationTable[RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2] 	    	= Location::Pot(RC_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2, 		        RCQUEST_MQ, RCAREA_SPIRIT_TEMPLE, 		    SCENE_SPIRIT_TEMPLE, 		    TWO_ACTOR_PARAMS(1204, -1028), 	"MQ Early Adult Pot 2", 		"Spirit Temple MQ Early Adult Pot 2", 	    	    RHT_POT_SPIRIT_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_SPIRIT_TEMPLE_MQ_EARLY_ADULT_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1] 		    = Location::Pot(RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(1123, 62), 	"MQ Lower Torches Pot 1", 		"Water Temple MQ Lower Torches Pot 1", 		        RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2] 		    = Location::Pot(RC_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(1123, 293), 	"MQ Lower Torches Pot 2", 		"Water Temple MQ Lower Torches Pot 2", 	    	    RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWER_TORCHES_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1] 	    	= Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(1222, -617), 	"MQ Storage Room A Pot 1", 		"Water Temple MQ Storage Room A Pot 1",     	    RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2] 	    	= Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(1122, -617), 	"MQ Storage Room A Pot 2", 		"Water Temple MQ Storage Room A Pot 2",     	    RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3] 	    	= Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(1172, -617), 	"MQ Storage Room A Pot 3", 		"Water Temple MQ Storage Room A Pot 3",     	    RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_A_POT_3));
	locationTable[RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-989, 255), 	"MQ GS Storage Room Pot 1", 	"Water Temple MQ GS Storage Room Pot 1",    	    RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-1026, 257), 	"MQ GS Storage Room Pot 2", 	"Water Temple MQ GS Storage Room Pot 2",    	    RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-1062, 256), 	"MQ GS Storage Room Pot 3", 	"Water Temple MQ GS Storage Room Pot 3",    	    RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_GS_STORAGE_ROOM_POT_3));
	locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_WEST_POT] 	= Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_WEST_POT,       RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(512, 579), 	"MQ Lizalfos Hall W Pot", 		"Water Temple MQ Lizalfos Hallway West Pot", 	    RHT_POT_WATER_TEMPLE,	            RG_RED_RUPEE,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SOUTH_POT] 	= Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SOUTH_POT,      RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(602, 1022), 	"MQ Lizalfos Hall S Pot", 		"Water Temple MQ Lizalfos Hallway South Pot", 	    RHT_POT_WATER_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SE_POT] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_HALLWAY_SE_POT, 	    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(789, 947), 	"MQ Lizalfos Hall SE Pot", 		"Water Temple MQ Lizalfos Hallway SE Pot", 	        RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_3));
	locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_SOUTH_POT] 		= Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_SOUTH_POT, 	    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(768, 565), 	"MQ Lizalfos Cage N Pot", 		"Water Temple MQ Lizalfos Cage North Pot", 	        RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_4));
	locationTable[RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_NORTH_POT] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_LIZALFOS_CAGE_NORTH_POT, 	    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(767, 517), 	"MQ Lizalfos Cage S Pot", 		"Water Temple MQ Lizalfos Cage South Pot", 	        RHT_POT_WATER_TEMPLE,	            RG_RECOVERY_HEART,  SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_CENTRAL_GATE_POT_5));
	locationTable[RC_WATER_TEMPLE_MQ_STALFOS_PIT_MIDDLE_POT]    	= Location::Pot(RC_WATER_TEMPLE_MQ_STALFOS_PIT_MIDDLE_POT, 	        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3500, -751), 	"MQ Stalfos Pit Middle Pot", 	"Water Temple MQ Stalfos Pit Middle Pot",   	    RHT_POT_WATER_TEMPLE,	            RG_DEKU_NUTS_5,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_3));
	locationTable[RC_WATER_TEMPLE_MQ_STALFOS_PIT_SOUTH_POT]    	    = Location::Pot(RC_WATER_TEMPLE_MQ_STALFOS_PIT_SOUTH_POT, 	        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3500, -686), 	"MQ Stalfos Pit South Pot", 	"Water Temple MQ Stalfos Pit South Pot",    	    RHT_POT_WATER_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_4));
	locationTable[RC_WATER_TEMPLE_MQ_STALFOS_PIT_NORTH_POT] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_STALFOS_PIT_NORTH_POT,  	        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3500, -810), 	"MQ Stalfos Pit North Pot", 	"Water Temple MQ Stalfos Pit North Pot",    	    RHT_POT_WATER_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_5));
	locationTable[RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1]    	= Location::Pot(RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1, 	        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3188, -1628), "MQ Before Dark Link Pot 1", 	"Water Temple MQ Before Dark Link Pot 1",   	    RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2]    	= Location::Pot(RC_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2, 	        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3056, -1626), "MQ Before Dark Link Pot 2", 	"Water Temple MQ Before Dark Link Pot 2",   	    RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BEFORE_DARK_LINK_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1]     	= Location::Pot(RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3072, -4024), "MQ After Dark Link Pot 1", 	"Water Temple MQ After Dark Link Pot 1", 	        RHT_POT_WATER_TEMPLE,	            RG_ARROWS_30,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2] 	    = Location::Pot(RC_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-3169, -4023), "MQ After Dark Link Pot 2", 	"Water Temple MQ After Dark Link Pot 2", 	        RHT_POT_WATER_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_AFTER_DARK_LINK_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_RIVER_POT_1] 			    	= Location::Pot(RC_WATER_TEMPLE_MQ_RIVER_POT_1, 				    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-2226, -2487), "MQ River Pot 1", 				"Water Temple MQ River Pot 1", 				        RHT_POT_WATER_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_RIVER_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_RIVER_POT_2] 			    	= Location::Pot(RC_WATER_TEMPLE_MQ_RIVER_POT_2, 				    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-2184, -2456), "MQ River Pot 2", 				"Water Temple MQ River Pot 2", 				        RHT_POT_WATER_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_RIVER_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_BOSS_KEY_POT] 				    = Location::Pot(RC_WATER_TEMPLE_MQ_BOSS_KEY_POT, 				    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-1033, -1569), "MQ Boss Key Pot", 				"Water Temple MQ Boss Key Pot", 		    	    RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_BOSS_KEY_POT));
    locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_1] 			    = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_1, 			    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-2314, 770), 	"MQ Lowest GS Pot 1", 			"Water Temple MQ Lowest GS Pot 1", 		    	    RHT_POT_WATER_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_2] 			    = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_2, 			    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-2314, 808), 	"MQ Lowest GS Pot 2", 			"Water Temple MQ Lowest GS Pot 2", 		    	    RHT_POT_WATER_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_3] 			    = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_3, 			    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-2313, 944), 	"MQ Lowest GS Pot 3", 			"Water Temple MQ Lowest GS Pot 3", 		    	    RHT_POT_WATER_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_3));
	locationTable[RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_4] 			    = Location::Pot(RC_WATER_TEMPLE_MQ_LOWEST_GS_POT_4, 			    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-2313, 905), 	"MQ Lowest GS Pot 4", 			"Water Temple MQ Lowest GS Pot 4", 		    	    RHT_POT_WATER_TEMPLE,	            RG_ARROWS_10,       SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_LOWEST_GS_POT_4));
	locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1] 		    = Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-1417, -3025), "MQ Storage Room B Pot 1", 		"Water Temple MQ Storage Room B Pot 1", 	        RHT_POT_WATER_TEMPLE,	            RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2] 	    	= Location::Pot(RC_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2, 		    RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-1150, -3031), "MQ Storage Room B Pot 2", 		"Water Temple MQ Storage Room B Pot 2",     	    RHT_POT_WATER_TEMPLE,	            RG_BLUE_RUPEE,      SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_STORAGE_ROOM_B_POT_2));
	locationTable[RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1] 	    	= Location::Pot(RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1, 		        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(28, -3817), 	"MQ Mini Dodongo Pot 1", 		"Water Temple MQ Mini Dodongo Pot 1", 	    	    RHT_POT_WATER_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_MINI_DODONGO_POT_1));
	locationTable[RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2] 	    	= Location::Pot(RC_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2, 		        RCQUEST_MQ, RCAREA_WATER_TEMPLE, 		    SCENE_WATER_TEMPLE, 		    TWO_ACTOR_PARAMS(-28, -3817), 	"MQ Mini Dodongo Pot 2", 		"Water Temple MQ Mini Dodongo Pot 2", 	    	    RHT_POT_WATER_TEMPLE,	            RG_BOMBS_5,         SpoilerCollectionCheck::RandomizerInf(RAND_INF_WATER_TEMPLE_MQ_MINI_DODONGO_POT_2));
	locationTable[RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_LEFT_POT_1]   = Location::Pot(RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_LEFT_POT_1,     RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,	SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(-324, -72),    "MQ Lobby Left Pot 1",          "Gerudo Training Grounds MQ Lobby Left Pot 1", 	    RHT_POT_GERUDO_TRAINING_GROUNDS,    RG_MAGIC_SINGLE,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_LEFT_POT_1));
    locationTable[RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_LEFT_POT_2]   = Location::Pot(RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_LEFT_POT_2,     RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,	SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(-324, -177),   "MQ Lobby Left Pot 2",          "Gerudo Training Grounds MQ Lobby Left Pot 2", 	    RHT_POT_GERUDO_TRAINING_GROUNDS,    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_LEFT_POT_2));
    locationTable[RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_RIGHT_POT_1]  = Location::Pot(RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_RIGHT_POT_1,    RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,	SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(199, -79),     "MQ Lobby Right Pot 1",         "Gerudo Training Grounds MQ Lobby Right Pot 1",     RHT_POT_GERUDO_TRAINING_GROUNDS,    RG_GREEN_RUPEE,     SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_RIGHT_POT_1));
    locationTable[RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_RIGHT_POT_2]  = Location::Pot(RC_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_RIGHT_POT_2,    RCQUEST_MQ, RCAREA_GERUDO_TRAINING_GROUND,	SCENE_GERUDO_TRAINING_GROUND,   TWO_ACTOR_PARAMS(197, -179),    "MQ Lobby Right Pot 2",         "Gerudo Training Grounds MQ Lobby Right Pot 2",     RHT_POT_GERUDO_TRAINING_GROUNDS,    RG_MAGIC_DOUBLE,    SpoilerCollectionCheck::RandomizerInf(RAND_INF_GERUDO_TRAINING_GROUNDS_MQ_LOBBY_RIGHT_POT_2));


    // Gossip Stones
    locationTable[RC_DMC_GOSSIP_STONE] =                                          Location::HintStone(RC_DMC_GOSSIP_STONE,                                             RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_CRATER,        14341,                              "Gossip Stone");
    locationTable[RC_DMT_GOSSIP_STONE] =                                          Location::HintStone(RC_DMT_GOSSIP_STONE,                                             RCQUEST_BOTH,                                                                                                   SCENE_DEATH_MOUNTAIN_TRAIL,         14340,                              "Gossip Stone");
    locationTable[RC_COLOSSUS_GOSSIP_STONE] =                                     Location::HintStone(RC_COLOSSUS_GOSSIP_STONE,                                        RCQUEST_BOTH,                                                                                                   SCENE_DESERT_COLOSSUS,              14362,                              "Gossip Stone");
    locationTable[RC_DODONGOS_CAVERN_GOSSIP_STONE] =                              Location::HintStone(RC_DODONGOS_CAVERN_GOSSIP_STONE,                                 RCQUEST_VANILLA,                                                                                                SCENE_DODONGOS_CAVERN,              4372,                               "Gossip Stone");
    locationTable[RC_GV_GOSSIP_STONE] =                                           Location::HintStone(RC_GV_GOSSIP_STONE,                                              RCQUEST_BOTH,                                                                                                   SCENE_GERUDO_VALLEY,                14353,                              "Gossip Stone");
    locationTable[RC_GC_MAZE_GOSSIP_STONE] =                                      Location::HintStone(RC_GC_MAZE_GOSSIP_STONE,                                         RCQUEST_BOTH,                                                                                                   SCENE_GORON_CITY,                   14357,                              "Maze Gossip Stone");
    locationTable[RC_GC_MEDIGORON_GOSSIP_STONE] =                                 Location::HintStone(RC_GC_MEDIGORON_GOSSIP_STONE,                                    RCQUEST_BOTH,                                                                                                   SCENE_GORON_CITY,                   14873,                              "Medigoron Gossip Stone");
    locationTable[RC_GRAVEYARD_GOSSIP_STONE] =                                    Location::HintStone(RC_GRAVEYARD_GOSSIP_STONE,                                       RCQUEST_BOTH,                                                                                                   SCENE_GRAVEYARD,                    14346,                              "Gossip Stone");
    locationTable[RC_HC_MALON_GOSSIP_STONE] =                                     Location::HintStone(RC_HC_MALON_GOSSIP_STONE,                                        RCQUEST_BOTH,                                                                                                   SCENE_HYRULE_CASTLE,                14610,                              "Malon Gossip Stone");
    locationTable[RC_HC_ROCK_WALL_GOSSIP_STONE] =                                 Location::HintStone(RC_HC_ROCK_WALL_GOSSIP_STONE,                                    RCQUEST_BOTH,                                                                                                   SCENE_HYRULE_CASTLE,                14347,                              "Rock Wall Gossip Stone");
    locationTable[RC_HC_STORMS_GROTTO_GOSSIP_STONE] =                             Location::HintStone(RC_HC_STORMS_GROTTO_GOSSIP_STONE,                                RCQUEST_BOTH,                                        RCAREA_HYRULE_CASTLE,                                      SCENE_GROTTOS,                      14355,                              "Storms Grotto Gossip Stone");
    locationTable[RC_KF_DEKU_TREE_LEFT_GOSSIP_STONE] =                            Location::HintStone(RC_KF_DEKU_TREE_LEFT_GOSSIP_STONE,                               RCQUEST_BOTH,                                                                                                   SCENE_KOKIRI_FOREST,                14623,                              "Deku Tree Left Gossip Stone");
    locationTable[RC_KF_DEKU_TREE_RIGHT_GOSSIP_STONE] =                           Location::HintStone(RC_KF_DEKU_TREE_RIGHT_GOSSIP_STONE,                              RCQUEST_BOTH,                                                                                                   SCENE_KOKIRI_FOREST,                14880,                              "Deku Tree Right Gossip Stone");
    locationTable[RC_KF_GOSSIP_STONE] =                                           Location::HintStone(RC_KF_GOSSIP_STONE,                                              RCQUEST_BOTH,                                                                                                   SCENE_KOKIRI_FOREST,                14366,                              "Gossip Stone");
    locationTable[RC_KF_STORMS_GROTTO_GOSSIP_STONE] =                             Location::HintStone(RC_KF_STORMS_GROTTO_GOSSIP_STONE,                                RCQUEST_BOTH,                                        RCAREA_KOKIRI_FOREST,                                      SCENE_GROTTOS,                      -22988,                             "Storms Gossip Stone");
    locationTable[RC_LH_LAB_GOSSIP_STONE] =                                       Location::HintStone(RC_LH_LAB_GOSSIP_STONE,                                          RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   14339,                              "Lab Gossip Stone");
    locationTable[RC_LH_SOUTHEAST_GOSSIP_STONE] =                                 Location::HintStone(RC_LH_SOUTHEAST_GOSSIP_STONE,                                    RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   14863,                              "Southeast Gossip Stone");
    locationTable[RC_LH_SOUTHWEST_GOSSIP_STONE] =                                 Location::HintStone(RC_LH_SOUTHWEST_GOSSIP_STONE,                                    RCQUEST_BOTH,                                                                                                   SCENE_LAKE_HYLIA,                   14600,                              "Southwest Gossip Stone");
    locationTable[RC_LW_GOSSIP_STONE] =                                           Location::HintStone(RC_LW_GOSSIP_STONE,                                              RCQUEST_BOTH,                                                                                                   SCENE_LOST_WOODS,                   14365,                              "Gossip Stone");
    locationTable[RC_SFM_MAZE_LOWER_GOSSIP_STONE] =                               Location::HintStone(RC_SFM_MAZE_LOWER_GOSSIP_STONE,                                  RCQUEST_BOTH,                                                                                                   SCENE_SACRED_FOREST_MEADOW,         14358,                              "Maze Lower Gossip Stone");
    locationTable[RC_SFM_MAZE_UPPER_GOSSIP_STONE] =                               Location::HintStone(RC_SFM_MAZE_UPPER_GOSSIP_STONE,                                  RCQUEST_BOTH,                                                                                                   SCENE_SACRED_FOREST_MEADOW,         14615,                              "Maze Upper Gossip Stone");
    locationTable[RC_SFM_SARIA_GOSSIP_STONE] =                                    Location::HintStone(RC_SFM_SARIA_GOSSIP_STONE,                                       RCQUEST_BOTH,                                                                                                   SCENE_SACRED_FOREST_MEADOW,         14876,                              "Saria Gossip Stone");
    locationTable[RC_TOT_LEFTMOST_GOSSIP_STONE] =                                 Location::HintStone(RC_TOT_LEFTMOST_GOSSIP_STONE,                                    RCQUEST_BOTH,                                        RCAREA_MARKET,                                             SCENE_ID_MAX,                       0x00,                               "ToT Left Gossip Stone");
    locationTable[RC_TOT_RIGHTMOST_GOSSIP_STONE] =                                Location::HintStone(RC_TOT_RIGHTMOST_GOSSIP_STONE,                                   RCQUEST_BOTH,                                        RCAREA_MARKET,                                             SCENE_ID_MAX,                       0x00,                               "ToT Right Gossip Stone");
    locationTable[RC_TOT_RIGHT_CENTER_GOSSIP_STONE] =                             Location::HintStone(RC_TOT_RIGHT_CENTER_GOSSIP_STONE,                                RCQUEST_BOTH,                                        RCAREA_MARKET,                                             SCENE_ID_MAX,                       0x00,                               "ToT Right Center Gossip Stone");
    locationTable[RC_TOT_LEFT_CENTER_GOSSIP_STONE] =                              Location::HintStone(RC_TOT_LEFT_CENTER_GOSSIP_STONE,                                 RCQUEST_BOTH,                                        RCAREA_MARKET,                                             SCENE_ID_MAX,                       0x00,                               "ToT Left Center Gossip Stone");
    locationTable[RC_ZD_GOSSIP_STONE] =                                           Location::HintStone(RC_ZD_GOSSIP_STONE,                                              RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_DOMAIN,                 14345,                              "Gossip Stone");
    locationTable[RC_ZF_FAIRY_GOSSIP_STONE] =                                     Location::HintStone(RC_ZF_FAIRY_GOSSIP_STONE,                                        RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_FOUNTAIN,               0x00,                               "Fairy Gossip Stone");
    locationTable[RC_ZF_JABU_GOSSIP_STONE] =                                      Location::HintStone(RC_ZF_JABU_GOSSIP_STONE,                                         RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_FOUNTAIN,               0x00,                               "Jabu Gossip Stone");
    locationTable[RC_ZR_NEAR_GROTTOS_GOSSIP_STONE] =                              Location::HintStone(RC_ZR_NEAR_GROTTOS_GOSSIP_STONE,                                 RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_RIVER,                  14605,                              "Near Grottos Gossip Stone");
    locationTable[RC_ZR_NEAR_DOMAIN_GOSSIP_STONE] =                               Location::HintStone(RC_ZR_NEAR_DOMAIN_GOSSIP_STONE,                                  RCQUEST_BOTH,                                                                                                   SCENE_ZORAS_RIVER,                  14860,                              "Near Domain Gossip Stone");
    locationTable[RC_HF_COW_GROTTO_GOSSIP_STONE] =                                Location::HintStone(RC_HF_COW_GROTTO_GOSSIP_STONE,                                   RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                       SCENE_GROTTOS,                      14363,                              "Cow Grotto Gossip Stone");
    locationTable[RC_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE] =                        Location::HintStone(RC_HF_NEAR_MARKET_GROTTO_GOSSIP_STONE,                           RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                       SCENE_GROTTOS,                      -22944,                             "Near Market Gossip Stone");
    locationTable[RC_HF_SOUTHEAST_GROTTO_GOSSIP_STONE] =                          Location::HintStone(RC_HF_SOUTHEAST_GROTTO_GOSSIP_STONE,                             RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                       SCENE_GROTTOS,                      -22978,                             "Southeast Gossip Stone");
    locationTable[RC_HF_OPEN_GROTTO_GOSSIP_STONE] =                               Location::HintStone(RC_HF_OPEN_GROTTO_GOSSIP_STONE,                                  RCQUEST_BOTH,                                        RCAREA_HYRULE_FIELD,                                       SCENE_GROTTOS,                      -22947,                             "Open Grotto Gossip Stone");
    locationTable[RC_KAK_OPEN_GROTTO_GOSSIP_STONE] =                              Location::HintStone(RC_KAK_OPEN_GROTTO_GOSSIP_STONE,                                 RCQUEST_BOTH,                                        RCAREA_KAKARIKO_VILLAGE,                                   SCENE_GROTTOS,                      -22984,                             "Open Grotto Gossip Stone");
    locationTable[RC_ZR_OPEN_GROTTO_GOSSIP_STONE] =                               Location::HintStone(RC_ZR_OPEN_GROTTO_GOSSIP_STONE,                                  RCQUEST_BOTH,                                        RCAREA_ZORAS_RIVER,                                        SCENE_GROTTOS,                      -22985,                             "Open Grotto Gossip Stone");
    locationTable[RC_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE] =                     Location::HintStone(RC_LW_NEAR_SHORTCUTS_GROTTO_GOSSIP_STONE,                        RCQUEST_BOTH,                                        RCAREA_LOST_WOODS,                                         SCENE_GROTTOS,                      -22964,                             "Near Shortcuts Gossip Stone");
    locationTable[RC_DMT_STORMS_GROTTO_GOSSIP_STONE] =                            Location::HintStone(RC_DMT_STORMS_GROTTO_GOSSIP_STONE,                               RCQUEST_BOTH,                                        RCAREA_DEATH_MOUNTAIN_TRAIL,                               SCENE_GROTTOS,                      -23255,                             "Storms Grotto Gossip Stone");
    locationTable[RC_DMC_UPPER_GROTTO_GOSSIP_STONE] =                             Location::HintStone(RC_DMC_UPPER_GROTTO_GOSSIP_STONE,                                RCQUEST_BOTH,                                        RCAREA_DEATH_MOUNTAIN_CRATER,                              SCENE_GROTTOS,                      -23802,                             "Upper Grotto Gossip Stone");

    // Other Hints
    locationTable[RC_GANONDORF_HINT]    =                                         Location::OtherHint(RC_GANONDORF_HINT,                                               RCQUEST_BOTH,                                                                             ACTOR_EN_GANON_MANT,  SCENE_GANON_BOSS,           "Ganondorf Hint");
    locationTable[RC_SHEIK_HINT_GC]     =                                         Location::OtherHint(RC_SHEIK_HINT_GC,                                                RCQUEST_VANILLA,                                                                          ACTOR_EN_XC,          SCENE_INSIDE_GANONS_CASTLE, "Sheik Hint");
    locationTable[RC_SHEIK_HINT_MQ_GC]  =                                         Location::OtherHint(RC_SHEIK_HINT_MQ_GC,                                             RCQUEST_MQ,                                                                               ACTOR_EN_XC,          SCENE_INSIDE_GANONS_CASTLE, "Sheik Hint");
    locationTable[RC_DAMPE_HINT]        =                                         Location::OtherHint(RC_DAMPE_HINT,                                                   RCQUEST_BOTH,                                                                             ACTOR_ID_MAX,         SCENE_GRAVEKEEPERS_HUT,     "Diary Hint");
    locationTable[RC_GREG_HINT]         =                                         Location::OtherHint(RC_GREG_HINT,                                                    RCQUEST_BOTH,                                        RCAREA_MARKET,                       ACTOR_EN_TAKARA_MAN,  SCENE_TREASURE_BOX_SHOP,    "Greg Hint");
    locationTable[RC_SARIA_SONG_HINT]   =                                         Location::OtherHint(RC_SARIA_SONG_HINT,                                              RCQUEST_BOTH,                                        RCAREA_KOKIRI_FOREST,                ACTOR_ID_MAX,         SCENE_ID_MAX,               "Sarias Song Hint", "Magic Hint Via Saria's Song");
    locationTable[RC_ALTAR_HINT_CHILD]  =                                         Location::OtherHint(RC_ALTAR_HINT_CHILD,                                             RCQUEST_BOTH,                                                                             ACTOR_ID_MAX,         SCENE_TEMPLE_OF_TIME,       "ToT Child Altar Hint");
    locationTable[RC_ALTAR_HINT_ADULT]  =                                         Location::OtherHint(RC_ALTAR_HINT_ADULT,                                             RCQUEST_BOTH,                                                                             ACTOR_ID_MAX,         SCENE_TEMPLE_OF_TIME,       "ToT Adult Altar Hint");
    locationTable[RC_FISHING_POLE_HINT] =                                         Location::OtherHint(RC_FISHING_POLE_HINT,                                            RCQUEST_BOTH,                                                                             ACTOR_FISHING,        SCENE_FISHING_POND,         "Fishing Pole Hint");
    locationTable[RC_TOT_SHEIK_HINT] =                                            Location::OtherHint(RC_TOT_SHEIK_HINT,                                               RCQUEST_BOTH,                                                                             ACTOR_EN_XC,          SCENE_TEMPLE_OF_TIME,       "Ocarina of Time Hint");
    locationTable[RC_MASK_SHOP_HINT] =                                            Location::OtherHint(RC_MASK_SHOP_HINT,                                               RCQUEST_BOTH,                                                                             ACTOR_ID_MAX,         SCENE_HAPPY_MASK_SHOP,      "Mask Shop Hint");

    locationTable[RC_TRIFORCE_COMPLETED] =                                        Location::Base(RC_TRIFORCE_COMPLETED,                                                RCQUEST_BOTH,    RCTYPE_STANDARD,                    RCAREA_MARKET,                       ACTOR_ID_MAX,         SCENE_ID_MAX,                       0x00,                               "Completed Triforce", "Completed Triforce",    RHT_NONE,                                                        RG_NONE);
    // clang-format on

    // Init locationNameToEnum
    locationNameToEnum["Invalid Location"] = RC_UNKNOWN_CHECK;
    locationNameToEnum["Link's Pocket"] = RC_LINKS_POCKET;

    for (auto& location : locationTable) {
        locationNameToEnum[location.GetName()] = location.GetRandomizerCheck();
        CheckFromActorMultimap.emplace(std::make_tuple((int16_t)location.GetActorID(), (int16_t)location.GetScene(), location.GetActorParams()), location.GetRandomizerCheck());
    }
}

Location* Rando::StaticData::GetLocation(RandomizerCheck locKey) {
    return &(locationTable[locKey]);
}

std::array<Location, RC_MAX>& Rando::StaticData::GetLocationTable() {
    return locationTable;
}
